﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Articulos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos
{
  /// <summary>
  /// Implementación de IGestionArticuloService
  /// </summary>
  public class GestionArticuloService : IAplicacionService, IGestionArticuloService
  {
    #region Atributos

    private readonly IArticuloRepository _articuloRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="articuloRepository">Dependencia al repositorio de articulos, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionArticuloService(IArticuloRepository articuloRepository)
    {
      if (articuloRepository == null) throw new ArgumentNullException("articuloRepository");

      _articuloRepository = articuloRepository;
    }

    #endregion

    #region Miembros de IGestionArticuloService

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="articulo"><see cref="IGestionArticuloService"/></param>
    public void AddArticulo(Articulo articulo)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _articuloRepository.UnitOfWork;

      _articuloRepository.Add(articulo);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    public void AddArticulo(string nombre, string descripcion, Guid? idProveedor, string idCategoriaArticulo, decimal? costeActualPorUnidad, decimal? costeAnteriorPorUnidad)
    {
      var articulo = new Articulo
      {
        Id = Guid.NewGuid(),
        Nombre = nombre,
        Descripcion = descripcion,
        IdProveedor = idProveedor,
        IdCategoria = idCategoriaArticulo,
        CosteActual = costeActualPorUnidad,
        CosteAnterior = costeAnteriorPorUnidad
      };

      AddArticulo(articulo);
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionArticuloService"/></param>    
    public void AddArticulo(Guid idGuid)
    {
      var articulo = new Articulo
                {
                  Id = idGuid
                };

      AddArticulo(articulo);
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionArticuloService"/></param>
    /// <param name="nombre"><see cref="IGestionArticuloService"/></param>
    /// <param name="descripcion"><see cref="IGestionArticuloService"/></param>
    /// <param name="idProveedor"><see cref="IGestionArticuloService"/></param>
    /// <param name="idCategoriaArticulo"><see cref="IGestionArticuloService"/></param>
    /// <param name="costeActualPorUnidad"><see cref="IGestionArticuloService"/></param>    
    /// <param name="costeAnteriorPorUnidad"><see cref="IGestionArticuloService"/></param>
    public void ChangeArticulo(Guid idGuid, string nombre, string descripcion, Guid? idProveedor,
                               string idCategoriaArticulo, decimal? costeActualPorUnidad,
                               decimal? costeAnteriorPorUnidad)
    {
      var articulo = BuscarArticuloPorId(idGuid).FirstOrDefault();
      if (articulo != null)
      {
        articulo.Nombre = nombre;
        articulo.Descripcion = descripcion;
        articulo.IdProveedor = idProveedor;
        articulo.IdCategoria = idCategoriaArticulo;
        articulo.CosteActual = costeActualPorUnidad;
        articulo.CosteAnterior = costeAnteriorPorUnidad;

        ChangeArticulo(articulo);
      }
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="articulo"><see cref="IGestionArticuloService"/></param>
    public void ChangeArticulo(Articulo articulo)
    {
      if (articulo == null) throw new ArgumentNullException("articulo");

      var unitOfWork = _articuloRepository.UnitOfWork;

      _articuloRepository.Modify(articulo);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionArticuloService"/></param>
    public void RemoveArticulo(Guid idGuid)
    {
      var articulo = BuscarArticuloPorId(idGuid).FirstOrDefault();
      if (articulo != null)
      {
        RemoveArticulo(articulo);
      }
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="articulo"><see cref="IGestionArticuloService"/></param>
    public void RemoveArticulo(Articulo articulo)
    {
      if (articulo == null) throw new ArgumentNullException("articulo");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _articuloRepository.UnitOfWork;

      _articuloRepository.Remove(articulo);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionArticuloService"/> </param>
    /// <returns><see cref="IGestionArticuloService"/></returns>
    public List<Articulo> BuscarArticuloPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del articulo en el repositorio
      var spec = new IdArticuloSpecification(id);

      return _articuloRepository.BuscarArticulo(spec);
    }

    /// <summary>
    /// <see cref="IGestionArticuloService"/>
    /// </summary>
    /// <param name="nombre"><see cref="IGestionArticuloService"/> </param>
    /// <returns><see cref="IGestionArticuloService"/></returns>
    public List<Articulo> BuscarArticuloPorNombre(string nombre)
    {
      //Crear especificación para la búsqueda del articulo en el repositorio
      var spec = new NombreArticuloSpecification(nombre);

      return _articuloRepository.BuscarArticulo(spec);
    }

    #endregion
  }
}