﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.ArticulosAlmacen;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulosAlmacen
{
  /// <summary>
  /// Implementación de IGestionArticuloAlmacenService
  /// </summary>
  public class GestionArticuloAlmacenService : IAplicacionService, IGestionArticuloAlmacenService
  {
    #region Atributos

    private readonly IArticuloAlmacenRepository _articuloAlmacenRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="articuloAlmacenRepository">Dependencia al repositorio de articuloAlmacens, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionArticuloAlmacenService(IArticuloAlmacenRepository articuloAlmacenRepository)
    {
      if (articuloAlmacenRepository == null) throw new ArgumentNullException("articuloAlmacenRepository");

      _articuloAlmacenRepository = articuloAlmacenRepository;
    }

    #endregion

    #region Miembros de IGestionArticuloAlmacenService

    /// <summary>
    /// <see cref="IGestionArticuloAlmacenService"/>
    /// </summary>
    /// <param name="articuloAlmacen"><see cref="IGestionArticuloAlmacenService"/></param>
    public void AddArticuloAlmacen(ArticuloAlmacen articuloAlmacen)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _articuloAlmacenRepository.UnitOfWork;

      _articuloAlmacenRepository.Add(articuloAlmacen);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    ///   <see cref="IGestionArticuloAlmacenService" />
    /// </summary>
    /// <param name="idArticuloAlmacen"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="idEmpleado"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="fechaEntrada"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="stockMinimo"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="stockMaximo"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="stockActual"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="ubicacion"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="idArticulo"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="codigoTaller"><see cref="IGestionArticuloAlmacenService"/></param>
    public void AddArticuloAlmacen(Guid idArticuloAlmacen, Guid? idEmpleado, DateTime? fechaEntrada,
                                   decimal? stockMinimo, decimal? stockMaximo, decimal? stockActual, string ubicacion,
                                   Guid idArticulo, string codigoTaller)
    {
      var articuloAlmacen = new ArticuloAlmacen
                {
                  Id = idArticuloAlmacen,
                  IdEmpleado = idEmpleado,
                  Fecha = fechaEntrada,
                  StockMinimo = stockMinimo,
                  StockMaximo = stockMaximo,
                  StockActual = stockActual,
                  Ubicacion = ubicacion,
                  IdArticulo = idArticulo,
                  CodigoTaller = codigoTaller
                };

      AddArticuloAlmacen(articuloAlmacen);
    }

    /// <summary>
    /// <see cref="IGestionArticuloAlmacenService"/>
    /// </summary>
    /// <param name="idArticuloAlmacen"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="idEmpleado"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="fechaEntrada"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="stockMinimo"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="stockMaximo"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="stockActual"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="ubicacion"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="idArticulo"><see cref="IGestionArticuloAlmacenService"/></param>
    /// <param name="codigoTaller"><see cref="IGestionArticuloAlmacenService"/></param>
    public void ChangeArticuloAlmacen(Guid idArticuloAlmacen, Guid? idEmpleado, DateTime? fechaEntrada,
                                   decimal? stockMinimo, decimal? stockMaximo, decimal? stockActual, string ubicacion,
                                   Guid idArticulo, string codigoTaller)
    {
      var articuloAlmacen = BuscarArticuloAlmacenPorId(idArticuloAlmacen).FirstOrDefault();
      if (articuloAlmacen != null)
      {
        articuloAlmacen.Id = idArticuloAlmacen;
        articuloAlmacen.IdEmpleado = idEmpleado;
        articuloAlmacen.Fecha = fechaEntrada;
        articuloAlmacen.StockMinimo = stockMinimo;
        articuloAlmacen.StockMaximo = stockMaximo;
        articuloAlmacen.StockActual = stockActual;
        articuloAlmacen.Ubicacion = ubicacion;
        articuloAlmacen.IdArticulo = idArticulo;
        articuloAlmacen.CodigoTaller = codigoTaller;

        ChangeArticuloAlmacen(articuloAlmacen);
      }
    }

    /// <summary>
    /// <see cref="IGestionArticuloAlmacenService"/>
    /// </summary>
    /// <param name="articuloAlmacen"><see cref="IGestionArticuloAlmacenService"/></param>
    public void ChangeArticuloAlmacen(ArticuloAlmacen articuloAlmacen)
    {
      if (articuloAlmacen == null) throw new ArgumentNullException("articuloAlmacen");

      var unitOfWork = _articuloAlmacenRepository.UnitOfWork;

      _articuloAlmacenRepository.Modify(articuloAlmacen);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionArticuloAlmacenService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionArticuloAlmacenService"/></param>
    public void RemoveArticuloAlmacen(Guid idGuid)
    {
      var articuloAlmacen = BuscarArticuloAlmacenPorId(idGuid).FirstOrDefault();
      if (articuloAlmacen != null)
      {
        RemoveArticuloAlmacen(articuloAlmacen);
      }
    }

    /// <summary>
    /// <see cref="IGestionArticuloAlmacenService"/>
    /// </summary>
    /// <param name="articuloAlmacen"><see cref="IGestionArticuloAlmacenService"/></param>
    public void RemoveArticuloAlmacen(ArticuloAlmacen articuloAlmacen)
    {
      if (articuloAlmacen == null) throw new ArgumentNullException("articuloAlmacen");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _articuloAlmacenRepository.UnitOfWork;

      _articuloAlmacenRepository.Remove(articuloAlmacen);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionArticuloAlmacenService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionArticuloAlmacenService"/> </param>
    /// <returns><see cref="IGestionArticuloAlmacenService"/></returns>
    public List<ArticuloAlmacen> BuscarArticuloAlmacenPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del articuloAlmacen en el repositorio
      var spec = new IdArticuloAlmacenSpecification(id);

      return _articuloAlmacenRepository.BuscarArticuloAlmacen(spec);
    }

    /// <summary>
    /// <see cref="IGestionArticuloAlmacenService"/>
    /// </summary>
    /// <param name="idArticulo"><see cref="IGestionArticuloAlmacenService"/> </param>
    /// <returns><see cref="IGestionArticuloAlmacenService"/></returns>
    public List<ArticuloAlmacen> BuscarArticuloAlmacenPorIdArticulo(Guid? idArticulo)
    {
      //Crear especificación para la búsqueda del articuloAlmacen en el repositorio
      var spec = new IdArticuloArticuloAlmacenSpecification(idArticulo);

      return _articuloAlmacenRepository.BuscarArticuloAlmacen(spec);
    }

    #endregion
  }
}