﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Clientes;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes
{
  /// <summary>
  /// Implementación de IGestionClienteService
  /// </summary>
  public class GestionClienteService : IAplicacionService, IGestionClienteService
  {
    #region Atributos

    readonly IClienteRepository _clienteRepository;
    
    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="clienteRepository">Dependencia al repositorio de clientes, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionClienteService(IClienteRepository clienteRepository)
    {
      if (clienteRepository == null) throw new ArgumentNullException("clienteRepository");

      _clienteRepository = clienteRepository;
    }

    #endregion

    #region Miembros de IGestionVehiculoService

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="cliente"><see cref="IGestionClienteService"/></param>
    public void AddCliente(Cliente cliente)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _clienteRepository.UnitOfWork;

      _clienteRepository.Add(cliente);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionClienteService"/></param>
    /// <param name="nombre"><see cref="IGestionClienteService"/> </param>
    /// <param name="primerApellido"><see cref="IGestionClienteService"/> </param>
    /// <param name="segundoApellido"><see cref="IGestionClienteService"/> </param>
    /// <param name="correoElectronico"><see cref="IGestionClienteService"/> </param>
    /// <param name="cuentaBancaria"><see cref="IGestionClienteService"/> </param>
    /// <param name="direccion"><see cref="IGestionClienteService"/> </param>
    /// <param name="listaTelefonos"><see cref="IGestionClienteService"/> </param>
    /// <param name="nifCif"><see cref="IGestionClienteService"/> </param>
    /// <param name="nombreComercial"><see cref="IGestionClienteService"/> </param>
    /// <param name="observaciones"><see cref="IGestionClienteService"/> </param>
    /// <param name="formaDePago"><see cref="IGestionClienteService"/> </param>
    /// <param name="idCodigoPostal"><see cref="IGestionClienteService"/> </param>
    public void AddCliente(Guid idGuid, string nombre, string primerApellido, string segundoApellido,
                           string correoElectronico,
                           string cuentaBancaria, string direccion, string listaTelefonos, string nifCif,
                           string nombreComercial, string observaciones,
                           int? formaDePago, int? idCodigoPostal)
    {
      var cliente = new Cliente
                      {
                        Id = idGuid,
                        Nombre = nombre,
                        PrimerApellido = primerApellido,
                        SegundoApellido = segundoApellido,
                        CorreoElectronico = correoElectronico,
                        CuentaBancaria = cuentaBancaria,
                        Direccion = direccion,
                        FormaDePago = formaDePago,
                        IdCodigoPostal = idCodigoPostal,
                        Telefonos = listaTelefonos,
                        NifCif = nifCif,
                        NombreComercial = nombreComercial,
                        Observaciones = observaciones
                      };
      
      AddCliente(cliente);
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionClienteService"/></param>
    /// <param name="nombre"><see cref="IGestionClienteService"/> </param>
    /// <param name="primerApellido"><see cref="IGestionClienteService"/> </param>
    /// <param name="segundoApellido"><see cref="IGestionClienteService"/> </param>
    /// <param name="correoElectronico"><see cref="IGestionClienteService"/> </param>
    /// <param name="cuentaBancaria"><see cref="IGestionClienteService"/> </param>
    /// <param name="direccion"><see cref="IGestionClienteService"/> </param>
    /// <param name="listaTelefonos"><see cref="IGestionClienteService"/> </param>
    /// <param name="nifCif"><see cref="IGestionClienteService"/> </param>
    /// <param name="nombreComercial"><see cref="IGestionClienteService"/> </param>
    /// <param name="observaciones"><see cref="IGestionClienteService"/> </param>
    /// <param name="formaDePago"><see cref="IGestionClienteService"/> </param>
    /// <param name="idCodigoPostal"><see cref="IGestionClienteService"/> </param>
    public void ChangeCliente(Guid idGuid, string nombre, string primerApellido, string segundoApellido,
                              string correoElectronico,
                              string cuentaBancaria, string direccion, string listaTelefonos, string nifCif,
                              string nombreComercial, string observaciones,
                              int? formaDePago, int? idCodigoPostal)
    {
      var cliente = BuscarClientePorId(idGuid).FirstOrDefault();
      if (cliente != null)
      {
        cliente.Nombre = nombre;
        cliente.PrimerApellido = primerApellido;
        cliente.SegundoApellido = segundoApellido;
        cliente.CorreoElectronico = correoElectronico;
        cliente.CuentaBancaria = cuentaBancaria;
        cliente.Direccion = direccion;
        cliente.FormaDePago = formaDePago;
        cliente.IdCodigoPostal = idCodigoPostal;
        cliente.Telefonos = listaTelefonos;
        cliente.NifCif = nifCif;
        cliente.NombreComercial = nombreComercial;
        cliente.Observaciones = observaciones;
        
        ChangeCliente(cliente);
      }
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="cliente"><see cref="IGestionClienteService"/></param>
    public void ChangeCliente(Cliente cliente)
    {
      if (cliente == null) throw new ArgumentNullException("cliente");

      var unitOfWork = _clienteRepository.UnitOfWork;

      _clienteRepository.Modify(cliente);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionClienteService"/></param>
    public void RemoveCliente(Guid idGuid)
    {
      var cliente = BuscarClientePorId(idGuid).FirstOrDefault();
      if (cliente != null)
      {
        RemoveCliente(cliente);
      }
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="cliente"><see cref="IGestionClienteService"/></param>
    public void RemoveCliente(Cliente cliente)
    {
      if (cliente == null) throw new ArgumentNullException("cliente");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _clienteRepository.UnitOfWork;
      
      _clienteRepository.Remove(cliente);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="nombre"><see cref="IGestionClienteService"/> </param>
    /// <param name="primerApellido"><see cref="IGestionClienteService"/> </param>
    /// <param name="segundoApellido"><see cref="IGestionClienteService"/> </param>
    /// <returns><see cref="IGestionClienteService"/></returns>
    public List<Cliente> BuscarCliente(string nombre, string primerApellido, string segundoApellido)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new ClienteSpecification(nombre, primerApellido, segundoApellido);

      return _clienteRepository.BuscarCliente(spec);
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="nifCif"><see cref="IGestionClienteService"/> </param>
    /// <returns><see cref="IGestionClienteService"/></returns>
    public List<Cliente> BuscarClientePorNifCif(string nifCif)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new NifCifClienteSpecification(nifCif);

      return _clienteRepository.BuscarCliente(spec);
    }

    /// <summary>
    /// <see cref="IGestionClienteService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionClienteService"/> </param>
    /// <returns><see cref="IGestionClienteService"/></returns>
    public List<Cliente> BuscarClientePorId(Guid? id)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new IdClienteSpecification(id);

      return _clienteRepository.BuscarCliente(spec);
    }
    #endregion
  }
}

