﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Empleados;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionEmpleados
{
  /// <summary>
  /// Implementación de IGestionEmpleadoService
  /// </summary>
  public class GestionEmpleadoService : IAplicacionService, IGestionEmpleadoService
  {
    #region Atributos

    private readonly IEmpleadoRepository _empleadoRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="empleadoRepository">Dependencia al repositorio de empleados, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionEmpleadoService(IEmpleadoRepository empleadoRepository)
    {
      if (empleadoRepository == null) throw new ArgumentNullException("empleadoRepository");

      _empleadoRepository = empleadoRepository;
    }

    #endregion

    #region Miembros de IGestionEmpleadoService

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <param name="empleado"><see cref="IGestionEmpleadoService"/></param>
    public void AddEmpleado(Empleado empleado)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _empleadoRepository.UnitOfWork;

      _empleadoRepository.Add(empleado);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionEmpleadoService"/></param>    
    public void AddEmpleado(Guid idGuid)
    {
      var empleado = new Empleado
                {
                  Id = idGuid
                };

      AddEmpleado(empleado);
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    public void AddEmpleado()
    {
      throw new NotImplementedException();
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionEmpleadoService"/></param>    
    public void ChangeEmpleado(Guid idGuid)
    {
      var empleado = BuscarEmpleadoPorId(idGuid).FirstOrDefault();
      if (empleado != null)
      {
        // TODO: Falta poner propiedades a actualizar	
        //cliente.Nombre = nombre;

        ChangeEmpleado(empleado);
      }
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <param name="empleado"><see cref="IGestionEmpleadoService"/></param>
    public void ChangeEmpleado(Empleado empleado)
    {
      if (empleado == null) throw new ArgumentNullException("empleado");

      var unitOfWork = _empleadoRepository.UnitOfWork;

      _empleadoRepository.Modify(empleado);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionEmpleadoService"/></param>
    public void RemoveEmpleado(Guid idGuid)
    {
      var empleado = BuscarEmpleadoPorId(idGuid).FirstOrDefault();
      if (empleado != null)
      {
        RemoveEmpleado(empleado);
      }
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <param name="empleado"><see cref="IGestionEmpleadoService"/></param>
    public void RemoveEmpleado(Empleado empleado)
    {
      if (empleado == null) throw new ArgumentNullException("empleado");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _empleadoRepository.UnitOfWork;

      _empleadoRepository.Remove(empleado);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionEmpleadoService"/> </param>
    /// <returns><see cref="IGestionEmpleadoService"/></returns>
    public List<Empleado> BuscarEmpleadoPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del empleado en el repositorio
      var spec = new IdEmpleadoSpecification(id);

      return _empleadoRepository.BuscarEmpleado(spec);
    }

    /// <summary>
    /// <see cref="IGestionEmpleadoService"/>
    /// </summary>
    /// <returns><see cref="Empleado"/></returns>
    public List<Empleado> BuscarTodosLosEmpleados()
    {
      return _empleadoRepository.BuscarTodosLosEmpleados();
    }
    #endregion
  }
}