﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.ManosDeObra;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionManosDeObra
{
  /// <summary>
  /// Implementación de IGestionManoDeObraService
  /// </summary>
  public class GestionManoDeObraService : IAplicacionService, IGestionManoDeObraService
  {
    #region Atributos

    private readonly IManoDeObraRepository _manoDeObraRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="manoDeObraRepository">Dependencia al repositorio de manosDeObra, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionManoDeObraService(IManoDeObraRepository manoDeObraRepository)
    {
      if (manoDeObraRepository == null) throw new ArgumentNullException("manoDeObraRepository");

      _manoDeObraRepository = manoDeObraRepository;
    }

    #endregion

    #region Miembros de IGestionManoDeObraService

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="manoDeObra"><see cref="IGestionManoDeObraService"/></param>
    public void AddManoDeObra(ManoDeObra manoDeObra)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _manoDeObraRepository.UnitOfWork;

      _manoDeObraRepository.Add(manoDeObra);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionManoDeObraService"/></param>
    /// <param name="descripcion"><see cref="IGestionManoDeObraService"/></param>
    /// <param name="duracion"><see cref="IGestionManoDeObraService"/></param>
    /// <param name="precioPorHora"><see cref="IGestionManoDeObraService"/></param>
    public void AddManoDeObra(Guid idGuid, string descripcion, decimal? duracion, decimal? precioPorHora)
    {
      var manoDeObra = new ManoDeObra
                {
                  Id = idGuid,
                  Descripcion = descripcion,
                  Duracion = duracion,
                  PrecioPorHora = precioPorHora
                };

      AddManoDeObra(manoDeObra);
    }

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionManoDeObraService"/></param>
    /// <param name="descripcion"><see cref="IGestionManoDeObraService"/></param>
    /// <param name="duracion"><see cref="IGestionManoDeObraService"/></param>
    /// <param name="precioPorHora"><see cref="IGestionManoDeObraService"/></param>
    public void ChangeManoDeObra(Guid idGuid, string descripcion, decimal? duracion, decimal? precioPorHora)
    {
      var manoDeObra = BuscarManoDeObraPorId(idGuid).FirstOrDefault();
      if (manoDeObra != null)
      {
        manoDeObra.Descripcion = descripcion;
        manoDeObra.Duracion = duracion;
        manoDeObra.PrecioPorHora = precioPorHora;

        ChangeManoDeObra(manoDeObra);
      }
    }

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="manoDeObra"><see cref="IGestionManoDeObraService"/></param>
    public void ChangeManoDeObra(ManoDeObra manoDeObra)
    {
      if (manoDeObra == null) throw new ArgumentNullException("manoDeObra");

      var unitOfWork = _manoDeObraRepository.UnitOfWork;

      _manoDeObraRepository.Modify(manoDeObra);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionManoDeObraService"/></param>
    public void RemoveManoDeObra(Guid idGuid)
    {
      var manoDeObra = BuscarManoDeObraPorId(idGuid).FirstOrDefault();
      if (manoDeObra != null)
      {
        RemoveManoDeObra(manoDeObra);
      }
    }

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="manoDeObra"><see cref="IGestionManoDeObraService"/></param>
    public void RemoveManoDeObra(ManoDeObra manoDeObra)
    {
      if (manoDeObra == null) throw new ArgumentNullException("manoDeObra");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _manoDeObraRepository.UnitOfWork;

      _manoDeObraRepository.Remove(manoDeObra);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionManoDeObraService"/> </param>
    /// <returns><see cref="IGestionManoDeObraService"/></returns>
    public List<ManoDeObra> BuscarManoDeObraPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del manoDeObra en el repositorio
      var spec = new IdManoDeObraSpecification(id);

      return _manoDeObraRepository.BuscarManosDeObra(spec);
    }

    /// <summary>
    /// <see cref="IGestionManoDeObraService"/>
    /// </summary>
    /// <param name="descripcion"><see cref="IGestionManoDeObraService"/> </param>
    /// <returns><see cref="IGestionManoDeObraService"/></returns>
    public List<ManoDeObra> BuscarManoDeObraPorDescripcion(string descripcion)
    {
      //Crear especificación para la búsqueda del manoDeObra en el repositorio
      var spec = new DescripcionManoDeObraSpecification(descripcion);

      return _manoDeObraRepository.BuscarManosDeObra(spec);
    }

    #endregion
  }
}