﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.OrdenesReparacion;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion
{
  /// <summary>
  /// Implementación de IGestionOrdenReparacionService
  /// </summary>
  public class GestionOrdenDeReparacionService : IAplicacionService, IGestionOrdenDeReparacionService
  {
    #region Atributos

    private readonly IOrdenReparacionRepository _ordenReparacionRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="ordenReparacionRepository">Dependencia al repositorio de ordenes de Reparacion, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionOrdenDeReparacionService(IOrdenReparacionRepository ordenReparacionRepository)
    {
      if (ordenReparacionRepository == null) throw new ArgumentNullException("ordenReparacionRepository");

      _ordenReparacionRepository = ordenReparacionRepository;
    }

    #endregion

    #region Miembros de IGestionOrdenReparacionService

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="ordenReparacion"><see cref="IGestionOrdenDeReparacionService"/></param>
    public void AddOrdenReparacion(OrdenReparacion ordenReparacion)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _ordenReparacionRepository.UnitOfWork;

      _ordenReparacionRepository.Add(ordenReparacion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionOrdenDeReparacionService"/></param>
    /// <param name="numeroOrden"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="fechaHoraEntrega"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="estado"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="idPersonalAsignado"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="subTotal"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="importeIva"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="diagnostico"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="idRecepcion"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="idVehiculo"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionOrdenDeReparacionService"/> </param>
    public void AddOrdenReparacion(Guid idGuid, string numeroOrden, DateTime? fechaHoraEntrega, int? estado, Guid? idPersonalAsignado, decimal? subTotal, decimal? importeIva, string diagnostico, Guid? idRecepcion, Guid? idVehiculo, string codigoTaller)
    {
      var ordenReparacion = new OrdenReparacion
                {
                  Id = idGuid,
                  NumeroOrden = numeroOrden,
                  FechaHoraEntrega = fechaHoraEntrega,
                  Estado = estado,
                  IdPersonalAsignado = idPersonalAsignado,
                  SubTotal = subTotal,
                  ImporteIva = importeIva,
                  Diagnostico = diagnostico,
                  IdRecepcion = idRecepcion,
                  IdVehiculo = idVehiculo,
                  CodigoTaller = codigoTaller
                };

      AddOrdenReparacion(ordenReparacion);
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionOrdenDeReparacionService"/></param>
    /// <param name="numeroOrden"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="fechaHoraEntrega"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="estado"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="idPersonalAsignado"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="subTotal"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="importeIva"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="diagnostico"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="idRecepcion"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="idVehiculo"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionOrdenDeReparacionService"/> </param>
    public void ChangeOrdenReparacion(Guid idGuid, string numeroOrden, DateTime? fechaHoraEntrega, int? estado, Guid? idPersonalAsignado, decimal? subTotal, decimal? importeIva, string diagnostico, Guid? idRecepcion, Guid? idVehiculo, string codigoTaller)
    {
      var ordenReparacion = BuscarOrdenReparacionPorId(idGuid).FirstOrDefault();
      if (ordenReparacion != null)
      {
        ordenReparacion.NumeroOrden = numeroOrden;
        ordenReparacion.FechaHoraEntrega = fechaHoraEntrega;
        ordenReparacion.Estado = estado;
        ordenReparacion.IdPersonalAsignado = idPersonalAsignado;
        ordenReparacion.SubTotal = subTotal;
        ordenReparacion.ImporteIva = importeIva;
        ordenReparacion.Diagnostico = diagnostico;
        ordenReparacion.IdRecepcion = idRecepcion;
        ordenReparacion.IdVehiculo = idVehiculo;
        ordenReparacion.CodigoTaller = codigoTaller;

        ChangeOrdenReparacion(ordenReparacion);
      }
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="ordenReparacion"><see cref="IGestionOrdenDeReparacionService"/></param>
    public void ChangeOrdenReparacion(OrdenReparacion ordenReparacion)
    {
      if (ordenReparacion == null) throw new ArgumentNullException("ordenReparacion");

      var unitOfWork = _ordenReparacionRepository.UnitOfWork;

      _ordenReparacionRepository.Modify(ordenReparacion);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionOrdenDeReparacionService"/></param>
    public void RemoveOrdenReparacion(Guid idGuid)
    {
      var ordenReparacion = BuscarOrdenReparacionPorId(idGuid).FirstOrDefault();
      if (ordenReparacion != null)
      {
        RemoveOrdenReparacion(ordenReparacion);
      }
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="ordenReparacion"><see cref="IGestionOrdenDeReparacionService"/></param>
    public void RemoveOrdenReparacion(OrdenReparacion ordenReparacion)
    {
      if (ordenReparacion == null) throw new ArgumentNullException("ordenReparacion");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _ordenReparacionRepository.UnitOfWork;

      _ordenReparacionRepository.Remove(ordenReparacion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <returns><see cref="IGestionOrdenDeReparacionService"/></returns>
    public List<OrdenReparacion> BuscarOrdenReparacionPorId(Guid? id)
    {
      //Crear especificación para la búsqueda de la ordenReparacion en el repositorio
      var spec = new IdOrdenReparacionSpecification(id);

      return _ordenReparacionRepository.BuscarOrdenReparacion(spec);
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="numeroOrden"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <returns><see cref="IGestionOrdenDeReparacionService"/></returns>
    public List<OrdenReparacion> BuscarOrdenDeReparacionPorNumeroOrden(string numeroOrden)
    {
      //Crear especificación para la búsqueda de la ordenReparacion en el repositorio
      var spec = new NumeroOrdenReparacionSpecification(numeroOrden);

      return _ordenReparacionRepository.BuscarOrdenReparacion(spec);
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="matricula"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <returns><see cref="IGestionOrdenDeReparacionService"/></returns>
    public List<OrdenReparacion> BuscarOrdenDeReparacionPorMatricula(string matricula)
    {
      //Crear especificación para la búsqueda de la ordenReparacion en el repositorio
      var spec = new MatriculaOrdenReparacionSpecification(matricula);

      return _ordenReparacionRepository.BuscarOrdenReparacion(spec);
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="nombreCliente"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <returns><see cref="IGestionOrdenDeReparacionService"/></returns>
    public List<OrdenReparacion> BuscarOrdenDeReparacionPorNombreCliente(string nombreCliente)
    {
      //Crear especificación para la búsqueda de la ordenReparacion en el repositorio
      var spec = new NombreClienteOrdenReparacionSpecification(nombreCliente);

      return _ordenReparacionRepository.BuscarOrdenReparacion(spec);
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="idEmpleado"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <param name="estado"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <returns><see cref="IGestionOrdenDeReparacionService"/></returns>
    public List<OrdenReparacion> BuscarPorEmpleadoYEstado(Guid? idEmpleado, int estado)
    {
      //Crear especificación para la búsqueda de la ordenReparacion en el repositorio
      var spec = new IdEmpleadoYEstadoOrdenReparacionSpecification(idEmpleado, estado);

      return _ordenReparacionRepository.BuscarOrdenReparacion(spec);
    }

    /// <summary>
    /// <see cref="IGestionOrdenDeReparacionService"/>
    /// </summary>
    /// <param name="recepcionId"><see cref="IGestionOrdenDeReparacionService"/> </param>
    /// <returns><see cref="IGestionOrdenDeReparacionService"/></returns>
    public List<OrdenReparacion> BuscarOrdenReparacionPorRecepcionId(Guid recepcionId)
    {
      //Crear especificación para la búsqueda de la ordenReparacion en el repositorio
      var spec = new RecepcionIdOrdenReparacionSpecification(recepcionId);

      return _ordenReparacionRepository.BuscarOrdenReparacion(spec);
    }

    #endregion
  }
}