﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Paquetes;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetes
{
  /// <summary>
  /// Implementación de IGestionPaqueteService
  /// </summary>
  public class GestionPaqueteService : IAplicacionService, IGestionPaqueteService
  {
    #region Atributos

    private readonly IPaqueteRepository _paqueteRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="paqueteRepository">Dependencia al repositorio de paquetes, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionPaqueteService(IPaqueteRepository paqueteRepository)
    {
      if (paqueteRepository == null) throw new ArgumentNullException("paqueteRepository");

      _paqueteRepository = paqueteRepository;
    }

    #endregion

    #region Miembros de IGestionPaqueteService

    /// <summary>
    /// <see cref="IGestionPaqueteService"/>
    /// </summary>
    /// <param name="paquete"><see cref="IGestionPaqueteService"/></param>
    public void AddPaquete(Paquete paquete)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _paqueteRepository.UnitOfWork;

      _paqueteRepository.Add(paquete);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionPaqueteService"/>
    /// </summary>
    /// <param name="idArticulo"><see cref="IGestionPaqueteService"/></param>    
    /// <param name="idManoDeObra"><see cref="IGestionPaqueteService"/></param>
    /// <param name="idPaqueteDeServicio"><see cref="IGestionPaqueteService"/></param>
    public void AddPaquete(Guid idArticulo, Guid idManoDeObra, Guid idPaqueteDeServicio)
    {
      var paquete = new Paquete
                {
                  IdArticulo = idArticulo,
                  IdManoDeObra = idManoDeObra,
                  IdPaqueteDeServicio = idPaqueteDeServicio
                };

      AddPaquete(paquete);
    }

    /// <summary>
    /// <see cref="IGestionPaqueteService"/>
    /// </summary>
    /// <param name="idArticulo"><see cref="IGestionPaqueteService"/></param>    
    /// <param name="idManoDeObra"><see cref="IGestionPaqueteService"/></param>
    /// <param name="idPaqueteDeServicio"><see cref="IGestionPaqueteService"/></param>
    public void ChangePaquete(Guid? idArticulo, Guid? idManoDeObra, Guid? idPaqueteDeServicio)
    {
      var paquete = BuscarPaquetePorId(idArticulo, idManoDeObra, idPaqueteDeServicio).FirstOrDefault();
      if (paquete != null)
      {
        // TODO: Falta poner propiedades a actualizar	
        //cliente.Nombre = nombre;

        ChangePaquete(paquete);
      }
    }

    /// <summary>
    /// <see cref="IGestionPaqueteService"/>
    /// </summary>
    /// <param name="paquete"><see cref="IGestionPaqueteService"/></param>
    public void ChangePaquete(Paquete paquete)
    {
      if (paquete == null) throw new ArgumentNullException("paquete");

      var unitOfWork = _paqueteRepository.UnitOfWork;

      _paqueteRepository.Modify(paquete);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionPaqueteService"/>
    /// </summary>
    /// <param name="idArticulo"><see cref="IGestionPaqueteService"/></param>    
    /// <param name="idManoDeObra"><see cref="IGestionPaqueteService"/></param>
    /// <param name="idPaqueteDeServicio"><see cref="IGestionPaqueteService"/></param>
    public void RemovePaquete(Guid? idArticulo, Guid? idManoDeObra, Guid? idPaqueteDeServicio)
    {
      var paquete = BuscarPaquetePorId(idArticulo, idManoDeObra, idPaqueteDeServicio).FirstOrDefault();
      if (paquete != null)
      {
        RemovePaquete(paquete);
      }
    }

    /// <summary>
    /// <see cref="IGestionPaqueteService"/>
    /// </summary>
    /// <param name="paquete"><see cref="IGestionPaqueteService"/></param>
    public void RemovePaquete(Paquete paquete)
    {
      if (paquete == null) throw new ArgumentNullException("paquete");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _paqueteRepository.UnitOfWork;

      _paqueteRepository.Remove(paquete);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionPaqueteService"/>
    /// </summary>
    /// <param name="idArticulo"><see cref="IGestionPaqueteService"/> </param>
    /// <param name="idManoDeObra"><see cref="IGestionPaqueteService"/> </param>
    /// <param name="idPaqueteDeServicio"><see cref="IGestionPaqueteService"/> </param>
    /// <returns><see cref="IGestionPaqueteService"/></returns>
    public List<Paquete> BuscarPaquetePorId(Guid? idArticulo, Guid? idManoDeObra, Guid? idPaqueteDeServicio)
    {
      //Crear especificación para la búsqueda del paquete en el repositorio
      var spec = new IdPaqueteSpecification(idArticulo, idManoDeObra, idPaqueteDeServicio);

      return _paqueteRepository.BuscarPaquete(spec);
    }

    #endregion
  }
}