﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.PaquetesDeServicio;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetesDeServicio
{
  /// <summary>
  /// Implementación de IGestionPaqueteDeServicioService
  /// </summary>
  public class GestionPaqueteDeServicioService : IAplicacionService, IGestionPaqueteDeServicioService
  {
    #region Atributos

    private readonly IPaqueteDeServicioRepository _paqueteDeServicioRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="paqueteDeServicioRepository">Dependencia al repositorio de paqueteDeServicios, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionPaqueteDeServicioService(IPaqueteDeServicioRepository paqueteDeServicioRepository)
    {
      if (paqueteDeServicioRepository == null) throw new ArgumentNullException("paqueteDeServicioRepository");

      _paqueteDeServicioRepository = paqueteDeServicioRepository;
    }

    #endregion

    #region Miembros de IGestionPaqueteDeServicioService

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="paqueteDeServicio"><see cref="IGestionPaqueteDeServicioService"/></param>
    public void AddPaqueteDeServicio(PaqueteDeServicio paqueteDeServicio)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _paqueteDeServicioRepository.UnitOfWork;

      _paqueteDeServicioRepository.Add(paqueteDeServicio);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionPaqueteDeServicioService"/></param>    
    /// <param name="descripcion"><see cref="IGestionPaqueteDeServicioService"/></param>
    public void AddPaqueteDeServicio(Guid idGuid, string descripcion)
    {
      var paqueteDeServicio = new PaqueteDeServicio
                {
                  Id = idGuid,
                  Descripcion = descripcion
                };

      AddPaqueteDeServicio(paqueteDeServicio);
    }

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionPaqueteDeServicioService"/></param>    
    /// <param name="descripcion"><see cref="IGestionPaqueteDeServicioService"/></param>
    public void ChangePaqueteDeServicio(Guid idGuid, string descripcion)
    {
      var paqueteDeServicio = BuscarPaqueteDeServicioPorId(idGuid).FirstOrDefault();
      if (paqueteDeServicio != null)
      {
        paqueteDeServicio.Descripcion = descripcion;
        ChangePaqueteDeServicio(paqueteDeServicio);
      }
    }

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="paqueteDeServicio"><see cref="IGestionPaqueteDeServicioService"/></param>
    public void ChangePaqueteDeServicio(PaqueteDeServicio paqueteDeServicio)
    {
      if (paqueteDeServicio == null) throw new ArgumentNullException("paqueteDeServicio");

      var unitOfWork = _paqueteDeServicioRepository.UnitOfWork;

      _paqueteDeServicioRepository.Modify(paqueteDeServicio);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionPaqueteDeServicioService"/></param>
    public void RemovePaqueteDeServicio(Guid idGuid)
    {
      var paqueteDeServicio = BuscarPaqueteDeServicioPorId(idGuid).FirstOrDefault();
      if (paqueteDeServicio != null)
      {
        RemovePaqueteDeServicio(paqueteDeServicio);
      }
    }

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="paqueteDeServicio"><see cref="IGestionPaqueteDeServicioService"/></param>
    public void RemovePaqueteDeServicio(PaqueteDeServicio paqueteDeServicio)
    {
      if (paqueteDeServicio == null) throw new ArgumentNullException("paqueteDeServicio");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _paqueteDeServicioRepository.UnitOfWork;

      _paqueteDeServicioRepository.Remove(paqueteDeServicio);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionPaqueteDeServicioService"/> </param>
    /// <returns><see cref="IGestionPaqueteDeServicioService"/></returns>
    public List<PaqueteDeServicio> BuscarPaqueteDeServicioPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del paqueteDeServicio en el repositorio
      var spec = new IdPaqueteDeServicioSpecification(id);

      return _paqueteDeServicioRepository.BuscarPaqueteDeServicio(spec);
    }

    /// <summary>
    /// <see cref="IGestionPaqueteDeServicioService"/>
    /// </summary>
    /// <param name="descripcion"><see cref="IGestionPaqueteDeServicioService"/> </param>
    /// <returns><see cref="IGestionPaqueteDeServicioService"/></returns>
    public List<PaqueteDeServicio> BuscarPaqueteDeServicioPorDescripcion(string descripcion)
    {
      //Crear especificación para la búsqueda del paqueteDeServicio en el repositorio
      var spec = new DescripcionPaqueteDeServicioSpecification(descripcion);

      return _paqueteDeServicioRepository.BuscarPaqueteDeServicio(spec);
    }

    #endregion
  }
}