﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Presupuestos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPresupuestos
{
  /// <summary>
  /// Implementación de IGestionPresupuestoService
  /// </summary>
  public class GestionPresupuestoService : IAplicacionService, IGestionPresupuestoService
  {
    #region Atributos

    private readonly IPresupuestoRepository _presupuestoRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="presupuestoRepository">Dependencia al repositorio de presupuestos, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionPresupuestoService(IPresupuestoRepository presupuestoRepository)
    {
      if (presupuestoRepository == null) throw new ArgumentNullException("presupuestoRepository");

      _presupuestoRepository = presupuestoRepository;
    }

    #endregion

    #region Miembros de IGestionPresupuestoService

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="presupuesto"><see cref="IGestionPresupuestoService"/></param>
    public void AddPresupuesto(Presupuesto presupuesto)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _presupuestoRepository.UnitOfWork;

      _presupuestoRepository.Add(presupuesto);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionPresupuestoService"/></param>
    /// <param name="numero"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="fecha"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="baseImponible"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="porcentajeIva"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="idEmpleado"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="idCliente"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionPresupuestoService"/> </param>    
    public void AddPresupuesto(Guid idGuid, string numero, DateTime? fecha, decimal baseImponible, decimal porcentajeIva,
                               Guid? idEmpleado, Guid? idCliente, string codigoTaller)
    {
      var presupuesto = new Presupuesto
                {
                  Id = idGuid,
                  BaseImponible = baseImponible,
                  IdCliente = idCliente,
                  Numero = numero,
                  Fecha = fecha,
                  PorcentajeIva = porcentajeIva,
                  IdEmpleado = idEmpleado,
                  CodigoTaller = codigoTaller
                };

      AddPresupuesto(presupuesto);
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionPresupuestoService"/></param>
    /// <param name="numero"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="fecha"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="baseImponible"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="porcentajeIva"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="idEmpleado"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="idCliente"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionPresupuestoService"/> </param>    
    public void ChangePresupuesto(Guid idGuid, string numero, DateTime? fecha, decimal baseImponible,
                                  decimal porcentajeIva, Guid? idEmpleado, Guid? idCliente, string codigoTaller)
    {
      var presupuesto = BuscarPresupuestoPorId(idGuid).FirstOrDefault();
      if (presupuesto != null)
      {
        presupuesto.Id = idGuid;
        presupuesto.Numero = numero;
        presupuesto.Fecha = fecha;
        presupuesto.BaseImponible = baseImponible;
        presupuesto.PorcentajeIva = porcentajeIva;
        presupuesto.IdEmpleado = idEmpleado;
        presupuesto.IdCliente = idCliente;
        presupuesto.CodigoTaller = codigoTaller;

        ChangePresupuesto(presupuesto);
      }
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="presupuesto"><see cref="IGestionPresupuestoService"/></param>
    public void ChangePresupuesto(Presupuesto presupuesto)
    {
      if (presupuesto == null) throw new ArgumentNullException("presupuesto");

      var unitOfWork = _presupuestoRepository.UnitOfWork;

      _presupuestoRepository.Modify(presupuesto);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionPresupuestoService"/></param>
    public void RemovePresupuesto(Guid idGuid)
    {
      var presupuesto = BuscarPresupuestoPorId(idGuid).FirstOrDefault();
      if (presupuesto != null)
      {
        RemovePresupuesto(presupuesto);
      }
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="presupuesto"><see cref="IGestionPresupuestoService"/></param>
    public void RemovePresupuesto(Presupuesto presupuesto)
    {
      if (presupuesto == null) throw new ArgumentNullException("presupuesto");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _presupuestoRepository.UnitOfWork;

      _presupuestoRepository.Remove(presupuesto);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionPresupuestoService"/> </param>
    /// <returns><see cref="IGestionPresupuestoService"/></returns>
    public List<Presupuesto> BuscarPresupuestoPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del presupuesto en el repositorio
      var spec = new IdPresupuestoSpecification(id);

      return _presupuestoRepository.BuscarPresupuesto(spec);
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="numero"><see cref="IGestionPresupuestoService"/> </param>
    /// <returns><see cref="IGestionPresupuestoService"/></returns>
    public List<Presupuesto> BuscarPresupuestoPorNumero(string numero)
    {
      //Crear especificación para la búsqueda del presupuesto en el repositorio
      var spec = new NumeroPresupuestoSpecification(numero);

      return _presupuestoRepository.BuscarPresupuesto(spec);
    }

    /// <summary>
    /// <see cref="IGestionPresupuestoService"/>
    /// </summary>
    /// <param name="cliente"><see cref="IGestionPresupuestoService"/> </param>
    /// <param name="fecha"><see cref="IGestionPresupuestoService"/> </param>
    /// <returns><see cref="IGestionPresupuestoService"/></returns>
    public List<Presupuesto> BuscarPresupuestoPorClienteYFecha(Guid? cliente, DateTime? fecha)
    {
      var spec = new ClienteYFechaDelPresupuestoSpecification(cliente, fecha);

      return _presupuestoRepository.BuscarPresupuesto(spec);
    }

    #endregion
  }
}