﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Proveedores;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProveedores
{
  /// <summary>
  /// Implementación de IGestionProveedorService
  /// </summary>
  public class GestionProveedorService : IAplicacionService, IGestionProveedorService
  {
    #region Atributos

    private readonly IProveedorRepository _proveedorRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="proveedorRepository">Dependencia al repositorio de proveedores, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionProveedorService(IProveedorRepository proveedorRepository)
    {
      if (proveedorRepository == null) throw new ArgumentNullException("proveedorRepository");

      _proveedorRepository = proveedorRepository;
    }

    #endregion

    #region Miembros de IGestionProveedorService

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="proveedor"><see cref="IGestionProveedorService"/></param>
    public void AddProveedor(Proveedor proveedor)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _proveedorRepository.UnitOfWork;

      _proveedorRepository.Add(proveedor);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionProveedorService"/></param>
    /// <param name="codigo"><see cref="IGestionProveedorService"/> </param>
    /// <param name="razonSocial"><see cref="IGestionProveedorService"/> </param>
    /// <param name="direccion"><see cref="IGestionProveedorService"/> </param>
    /// <param name="idCodigoPostal"><see cref="IGestionProveedorService"/> </param>
    /// <param name="cifDni"><see cref="IGestionProveedorService"/> </param>
    /// <param name="telefonos"><see cref="IGestionProveedorService"/> </param>
    /// <param name="observaciones"><see cref="IGestionProveedorService"/> </param>
    /// <param name="correoElectronico"><see cref="IGestionProveedorService"/> </param>
    /// <param name="paginaWeb"><see cref="IGestionProveedorService"/> </param>    
    public void AddProveedor(Guid idGuid, string codigo, string razonSocial, string direccion, int? idCodigoPostal,
                             string cifDni, string telefonos, string observaciones, string correoElectronico,
                             string paginaWeb)
    {
      var proveedor = new Proveedor
                {
                  Id = idGuid,
                  Codigo = codigo,
                  RazonSocial = razonSocial,
                  Direccion = direccion,
                  IdCodigoPostal = idCodigoPostal,
                  CifDni = cifDni,
                  Telefonos = telefonos,
                  Observaciones = observaciones,
                  CorreoElectronico = correoElectronico,
                  PaginaWeb = paginaWeb
                };

      AddProveedor(proveedor);
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionProveedorService"/></param>  
    /// <param name="codigo"><see cref="IGestionProveedorService"/> </param>
    /// <param name="razonSocial"><see cref="IGestionProveedorService"/> </param>
    /// <param name="direccion"><see cref="IGestionProveedorService"/> </param>
    /// <param name="idCodigoPostal"><see cref="IGestionProveedorService"/> </param>
    /// <param name="cifDni"><see cref="IGestionProveedorService"/> </param>
    /// <param name="telefonos"><see cref="IGestionProveedorService"/> </param>
    /// <param name="observaciones"><see cref="IGestionProveedorService"/> </param>
    /// <param name="correoElectronico"><see cref="IGestionProveedorService"/> </param>
    /// <param name="paginaWeb"><see cref="IGestionProveedorService"/> </param>      
    public void ChangeProveedor(Guid idGuid, string codigo, string razonSocial, string direccion, int? idCodigoPostal,
                             string cifDni, string telefonos, string observaciones, string correoElectronico,
                             string paginaWeb)
    {
      var proveedor = BuscarProveedorPorId(idGuid).FirstOrDefault();
      if (proveedor != null)
      {
        proveedor.Codigo = codigo;
        proveedor.RazonSocial = razonSocial;
        proveedor.Direccion = direccion;
        proveedor.IdCodigoPostal = idCodigoPostal;
        proveedor.CifDni = cifDni;
        proveedor.Telefonos = telefonos;
        proveedor.Observaciones = observaciones;
        proveedor.CorreoElectronico = correoElectronico;
        proveedor.PaginaWeb = paginaWeb;

        ChangeProveedor(proveedor);
      }
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="proveedor"><see cref="IGestionProveedorService"/></param>
    public void ChangeProveedor(Proveedor proveedor)
    {
      if (proveedor == null) throw new ArgumentNullException("proveedor");

      var unitOfWork = _proveedorRepository.UnitOfWork;

      _proveedorRepository.Modify(proveedor);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionProveedorService"/></param>
    public void RemoveProveedor(Guid idGuid)
    {
      var proveedor = BuscarProveedorPorId(idGuid).FirstOrDefault();
      if (proveedor != null)
      {
        RemoveProveedor(proveedor);
      }
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="proveedor"><see cref="IGestionProveedorService"/></param>
    public void RemoveProveedor(Proveedor proveedor)
    {
      if (proveedor == null) throw new ArgumentNullException("proveedor");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _proveedorRepository.UnitOfWork;

      _proveedorRepository.Remove(proveedor);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionProveedorService"/> </param>
    /// <returns><see cref="IGestionProveedorService"/></returns>
    public List<Proveedor> BuscarProveedorPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del proveedor en el repositorio
      var spec = new IdProveedorSpecification(id);

      return _proveedorRepository.BuscarProveedor(spec);
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <returns><see cref="IGestionProveedorService"/></returns>
    public List<Proveedor> BuscarTodosLosProveedores()
    {
      return _proveedorRepository.BuscarProveedor();
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="codigo"><see cref="IGestionProveedorService"/> </param>
    /// <returns><see cref="IGestionProveedorService"/></returns>
    public List<Proveedor> BuscarProveedorPorCodigo(string codigo)
    {
      //Crear especificación para la búsqueda del proveedor en el repositorio
      var spec = new CodigoProveedorSpecification(codigo);

      return _proveedorRepository.BuscarProveedor(spec);
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="cifDni"><see cref="IGestionProveedorService"/> </param>
    /// <returns><see cref="IGestionProveedorService"/></returns>
    public List<Proveedor> BuscarProveedorPorCifDni(string cifDni)
    {
      //Crear especificación para la búsqueda del proveedor en el repositorio
      var spec = new CifDniProveedorSpecification(cifDni);

      return _proveedorRepository.BuscarProveedor(spec);
    }

    /// <summary>
    /// <see cref="IGestionProveedorService"/>
    /// </summary>
    /// <param name="razonSocial"><see cref="IGestionProveedorService"/> </param>
    /// <returns><see cref="IGestionProveedorService"/></returns>
    public List<Proveedor> BuscarProveedorPorRazonSocial(string razonSocial)
    {
      //Crear especificación para la búsqueda del proveedor en el repositorio
      var spec = new RazonSocialProveedorSpecification(razonSocial);

      return _proveedorRepository.BuscarProveedor(spec);
    }

    #endregion
  }
}