﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Reparaciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionReparaciones
{
  /// <summary>
  /// Implementación de IGestionReparacionService
  /// </summary>
  public class GestionReparacionService : IAplicacionService, IGestionReparacionService
  {
    #region Atributos

    private readonly IReparacionRepository _reparacionRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="reparacionRepository">Dependencia al repositorio de reparaciones, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionReparacionService(IReparacionRepository reparacionRepository)
    {
      if (reparacionRepository == null) throw new ArgumentNullException("reparacionRepository");

      _reparacionRepository = reparacionRepository;
    }

    #endregion

    #region Miembros de IGestionReparacionService

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    /// <param name="reparacion"><see cref="IGestionReparacionService"/></param>
    public void AddReparacion(Reparacion reparacion)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _reparacionRepository.UnitOfWork;

      _reparacionRepository.Add(reparacion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionReparacionService"/></param>    
    public void AddReparacion(Guid idGuid)
    {
      var reparacion = new Reparacion
                {
                  Id = idGuid
                };

      AddReparacion(reparacion);
    }

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    public void AddReparacion()
    {
      throw new NotImplementedException();
    }

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    /// <param name="idReparacion"><see cref="IGestionReparacionService"/></param>
    /// <param name="duracionSugerida"><see cref="IGestionReparacionService"/> </param>
    /// <param name="duracionReal"><see cref="IGestionReparacionService"/> </param>
    /// <param name="estado"><see cref="IGestionReparacionService"/> </param>
    /// <param name="idEmpleado"><see cref="IGestionReparacionService"/> </param>
    /// <param name="idOrdenDeReparacion"><see cref="IGestionReparacionService"/> </param>    
    public void ChangeReparacion(Guid idReparacion, decimal? duracionSugerida, decimal? duracionReal, int? estado,
                                 Guid? idEmpleado, Guid idOrdenDeReparacion)
    {
      var reparacion = BuscarReparacionPorId(idReparacion).FirstOrDefault();
      if (reparacion != null)
      {
        reparacion.Id = idReparacion;
        reparacion.DuracionSugerida = duracionSugerida;
        reparacion.DuracionReal = duracionReal;
        reparacion.Estado = estado;
        reparacion.IdEmpleado = idEmpleado;
        reparacion.IdOrdenReparacion = idOrdenDeReparacion;

        ChangeReparacion(reparacion);
      }
    }

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    /// <param name="reparacion"><see cref="IGestionReparacionService"/></param>
    public void ChangeReparacion(Reparacion reparacion)
    {
      if (reparacion == null) throw new ArgumentNullException("reparacion");

      var unitOfWork = _reparacionRepository.UnitOfWork;

      _reparacionRepository.Modify(reparacion);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionReparacionService"/></param>
    public void RemoveReparacion(Guid idGuid)
    {
      var reparacion = BuscarReparacionPorId(idGuid).FirstOrDefault();
      if (reparacion != null)
      {
        RemoveReparacion(reparacion);
      }
    }

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    /// <param name="reparacion"><see cref="IGestionReparacionService"/></param>
    public void RemoveReparacion(Reparacion reparacion)
    {
      if (reparacion == null) throw new ArgumentNullException("reparacion");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _reparacionRepository.UnitOfWork;

      _reparacionRepository.Remove(reparacion);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionReparacionService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionReparacionService"/> </param>
    /// <returns><see cref="IGestionReparacionService"/></returns>
    public List<Reparacion> BuscarReparacionPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del reparacion en el repositorio
      var spec = new IdReparacionSpecification(id);

      return _reparacionRepository.BuscarReparacion(spec);
    }

    #endregion
  }
}