﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVentas;
using GestionConcesionarios.Dominio.ModuloPrincipal.VehiculosVenta;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta
{
  /// <summary>
  /// Implementación de IGestionVehiculoService
  /// </summary>
  public class GestionVehiculoVentaService : IAplicacionService, IGestionVehiculoVentaService
  {
    #region Atributos

    readonly IVehiculoVentaRepository _vehiculoVentaRepository;
    private readonly IGestionVehiculoService _gestionVehiculoService;
    private readonly IGestionVentaService _gestionVentaService;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="vehiculoVentaRepository">Dependencia al repositorio de vehiculos de venta, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    /// <param name="gestionVehiculoService">Dependencia al servicio de aplicación de gestión de vehiculos </param>
    /// <param name="gestionVentaService">Dependencia al servicio de aplicación de gestión de ventas </param>
    public GestionVehiculoVentaService(IVehiculoVentaRepository vehiculoVentaRepository, 
                                       IGestionVehiculoService gestionVehiculoService, 
                                       IGestionVentaService gestionVentaService)
    {
      if (vehiculoVentaRepository == null) throw new ArgumentNullException("vehiculoVentaRepository");

      _vehiculoVentaRepository = vehiculoVentaRepository;
      _gestionVehiculoService = gestionVehiculoService;
      _gestionVentaService = gestionVentaService;
    }

    #endregion

    #region Miembros de IGestionVehiculoVentaService

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="vehiculoVenta"><see cref="IGestionVehiculoVentaService"/></param>
    public void AddVehiculoVenta(VehiculoVenta vehiculoVenta)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _vehiculoVentaRepository.UnitOfWork;

      _vehiculoVentaRepository.Add(vehiculoVenta);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="codigo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaDeEntrada"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="estado"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="matricula"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="marca"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="modelo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="kilometros"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="color"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="tipo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaMatriculacion"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="cilindrada"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="potencia"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="combustible"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="bastidor"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="itv"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaUltimaInspeccion"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaProximaInspeccion"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="observaciones"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="precio"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaInicioGarantia"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="mesesGarantia"><see cref="IGestionVehiculoVentaService"/> </param>
    public void AddVehiculoVenta(Guid idGuid, string codigo, DateTime? fechaDeEntrada, int? estado, string matricula, string marca, string modelo, decimal? kilometros, string color, string tipo, DateTime? fechaMatriculacion, decimal? cilindrada, decimal? potencia, string combustible, string bastidor, int? itv, DateTime? fechaUltimaInspeccion, DateTime? fechaProximaInspeccion, string observaciones, decimal? precio, DateTime? fechaInicioGarantia, int? mesesGarantia)
    {
      var vehiculoVenta = new VehiculoVenta
      {
        Id = idGuid,
        Codigo = codigo,
        FechaDeEntrada = fechaDeEntrada,
        Estado = estado,
        Matricula = matricula,
        Marca = marca,
        Modelo = modelo,
        Kilometros = kilometros,
        Color = color,
        Tipo = tipo,
        FechaMatriculacion = fechaMatriculacion,
        Cilindrada = cilindrada,
        Potencia = potencia,
        Combustible = combustible,
        Bastidor = bastidor,
        Itv = itv,
        UltimaInspeccion = fechaUltimaInspeccion,
        ProximaInspeccion = fechaProximaInspeccion,
        Observaciones = observaciones,
        Precio = precio,
        FechaInicioGarantia = fechaInicioGarantia,
        MesesGarantia = mesesGarantia
      };
      AddVehiculoVenta(vehiculoVenta);
    }

    /// <summary>
    /// <see cref="GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos.IGestionVehiculoService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="codigo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaDeEntrada"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="estado"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="matricula"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="marca"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="modelo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="kilometros"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="color"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="tipo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaMatriculacion"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="cilindrada"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="potencia"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="combustible"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="bastidor"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="itv"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaUltimaInspeccion"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaProximaInspeccion"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="observaciones"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="precio"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="fechaInicioGarantia"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="mesesGarantia"><see cref="IGestionVehiculoVentaService"/> </param>
    public void ChangeVehiculoVenta(Guid idGuid, string codigo, DateTime? fechaDeEntrada, int? estado, string matricula, string marca, string modelo, decimal? kilometros, string color, string tipo, DateTime? fechaMatriculacion, decimal? cilindrada, decimal? potencia, string combustible, string bastidor, int? itv, DateTime? fechaUltimaInspeccion, DateTime? fechaProximaInspeccion, string observaciones, decimal? precio, DateTime? fechaInicioGarantia, int? mesesGarantia)
    {
      var vehiculoVenta = BuscarVehiculoVentaPorId(idGuid).FirstOrDefault();
      if (vehiculoVenta != null)
      {
        vehiculoVenta.Codigo = codigo;
        vehiculoVenta.FechaDeEntrada = fechaDeEntrada;
        vehiculoVenta.Estado = estado;
        vehiculoVenta.Matricula = matricula;
        vehiculoVenta.Marca = marca;
        vehiculoVenta.Modelo = modelo;
        vehiculoVenta.Kilometros = kilometros;
        vehiculoVenta.Color = color;
        vehiculoVenta.Tipo = tipo;
        vehiculoVenta.FechaMatriculacion = fechaMatriculacion;
        vehiculoVenta.Cilindrada = cilindrada;
        vehiculoVenta.Potencia = potencia;
        vehiculoVenta.Combustible = combustible;
        vehiculoVenta.Bastidor = bastidor;
        vehiculoVenta.Itv = itv;
        vehiculoVenta.UltimaInspeccion = fechaUltimaInspeccion;
        vehiculoVenta.ProximaInspeccion = fechaProximaInspeccion;
        vehiculoVenta.Observaciones = observaciones;
        vehiculoVenta.Precio = precio;
        vehiculoVenta.FechaInicioGarantia = fechaInicioGarantia;
        vehiculoVenta.MesesGarantia = mesesGarantia;

        ChangeVehiculoVenta(vehiculoVenta);
      }
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="vehiculoVenta"><see cref="IGestionVehiculoVentaService"/></param>
    public void ChangeVehiculoVenta(VehiculoVenta vehiculoVenta)
    {
      if (vehiculoVenta == null) throw new ArgumentNullException("vehiculoVenta");

      var unitOfWork = _vehiculoVentaRepository.UnitOfWork;

      _vehiculoVentaRepository.Modify(vehiculoVenta);

      if (EstaVendido(vehiculoVenta) && !HayUnVehiculoParaElVehiculoDeVenta(vehiculoVenta))
      {
        var vehiculo = CrearVehiculo(vehiculoVenta);
        AsociarVehiculoConClienteDeVehiculoVenta(vehiculo, vehiculoVenta.IdVenta);
        _gestionVehiculoService.AddVehiculo(vehiculo);
      }

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionVehiculoVentaService"/></param>
    public void RemoveVehiculoVenta(Guid idGuid)
    {
      var vehiculoVenta = BuscarVehiculoVentaPorId(idGuid).FirstOrDefault();
      if (vehiculoVenta != null)
      {
        RemoveVehiculoVenta(vehiculoVenta);
      }
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="vehiculoVenta"><see cref="IGestionVehiculoVentaService"/></param>
    public void RemoveVehiculoVenta(VehiculoVenta vehiculoVenta)
    {
      if (vehiculoVenta == null) throw new ArgumentNullException("vehiculoVenta");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _vehiculoVentaRepository.UnitOfWork;

      _vehiculoVentaRepository.Remove(vehiculoVenta);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="marca"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <param name="modelo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <returns><see cref="IGestionVehiculoVentaService"/></returns>
    public List<VehiculoVenta> BuscarVehiculoVentaPorMarcaYModelo(string marca, string modelo)
    {
      //Crear especificación para la búsqueda del vehículo de venta en el repositorio
      var spec = new MarcaModeloVehiculoVentaSpecification(marca, modelo);

      return _vehiculoVentaRepository.BuscarVehiculoVenta(spec);
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="matricula"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <returns><see cref="IGestionVehiculoVentaService"/></returns>
    public List<VehiculoVenta> BuscarVehiculoVentaPorMatricula(string matricula)
    {
      //Crear especificación para la búsqueda del vehículo de venta en el repositorio
      var spec = new MatriculaVehiculoVentaSpecification(matricula);

      return _vehiculoVentaRepository.BuscarVehiculoVenta(spec);
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="codigo"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <returns><see cref="IGestionVehiculoVentaService"/></returns>
    public List<VehiculoVenta> BuscarVehiculoVentaPorCodigo(string codigo)
    {
      //Crear especificación para la búsqueda del vehículo de venta en el repositorio
      var spec = new CodigoVehiculoVentaSpecification(codigo);

      return _vehiculoVentaRepository.BuscarVehiculoVenta(spec);
    }

    /// <summary>
    /// <see cref="IGestionVehiculoVentaService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionVehiculoVentaService"/> </param>
    /// <returns><see cref="IGestionVehiculoVentaService"/></returns>
    public List<VehiculoVenta> BuscarVehiculoVentaPorId(Guid? id)
    {
      //Crear especificación para la búsqueda del vehículo en el repositorio
      var spec = new IdVehiculoVentaSpecification(id);

      return _vehiculoVentaRepository.BuscarVehiculoVenta(spec);
    }
    #endregion

    #region Métodos Privados

    private void AsociarVehiculoConClienteDeVehiculoVenta(Vehiculo vehiculo, Guid? idVenta)
    {
      if (idVenta.HasValue)
      {
        var venta = _gestionVentaService.BuscarVentaPorId(idVenta);
        if (venta.Count > 0)
          vehiculo.IdCliente = venta[0].IdCliente;
      }
    }

    private Vehiculo CrearVehiculo(VehiculoVenta vehiculoVenta)
    {
      var vehiculo = new Vehiculo
      {
        Matricula = vehiculoVenta.Matricula,
        Marca = vehiculoVenta.Marca,
        Modelo = vehiculoVenta.Modelo,
        Color = vehiculoVenta.Color,
        Fecha = vehiculoVenta.FechaMatriculacion,
        Bastidor = vehiculoVenta.Bastidor
      };

      return vehiculo;
    }

    private bool HayUnVehiculoParaElVehiculoDeVenta(VehiculoVenta vehiculoVenta)
    {
      if (vehiculoVenta == null) throw new ArgumentNullException("vehiculoVenta");

      var vehiculos = _gestionVehiculoService.BuscarVehiculoPorMatricula(vehiculoVenta.Matricula);

      return vehiculos.Count > 0;
    }

    private bool EstaVendido(VehiculoVenta vehiculoVenta)
    {
      return vehiculoVenta.IdVenta != Guid.Empty && vehiculoVenta.IdVenta != null;
    }

    #endregion
  }
}
