﻿using System;
using System.Collections.Generic;
using System.Linq;
using GestionConcesionarios.Aplicacion.Core.Container;
using GestionConcesionarios.Dominio.ModuloPrincipal.Ventas;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;

namespace GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVentas
{
  /// <summary>
  /// Implementación de IGestionVentaService
  /// </summary>
  public class GestionVentaService : IAplicacionService, IGestionVentaService
  {
    #region Atributos

    private readonly IVentaRepository _ventaRepository;

    #endregion

    #region Constructor

    /// <summary>
    /// Crear una nueva instancia 
    /// </summary>
    /// <param name="ventaRepository">Dependencia al repositorio de ventas, será resuelta automáticamente por el contenedor IoC (Autofac) con la inyección de dependencia</param>
    public GestionVentaService(IVentaRepository ventaRepository)
    {
      if (ventaRepository == null) throw new ArgumentNullException("ventaRepository");

      _ventaRepository = ventaRepository;
    }

    #endregion

    #region Miembros de IGestionVentaService

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="venta"><see cref="IGestionVentaService"/></param>
    public void AddVenta(Venta venta)
    {
      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _ventaRepository.UnitOfWork;

      _ventaRepository.Add(venta);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.Commit();
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionVentaService"/></param>
    /// <param name="numero"><see cref="IGestionVentaService"/> </param>
    /// <param name="fecha"><see cref="IGestionVentaService"/> </param>
    /// <param name="baseImponible"><see cref="IGestionVentaService"/> </param>
    /// <param name="porcentajeIva"><see cref="IGestionVentaService"/> </param>
    /// <param name="idEmpleado"><see cref="IGestionVentaService"/> </param>
    /// <param name="idCliente"><see cref="IGestionVentaService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionVentaService"/> </param>    
    /// <param name="formaDePago"><see cref="IGestionVentaService"/> </param>
    /// <param name="cuentaBancaria"><see cref="IGestionVentaService"/> </param>    
    public void AddVenta(Guid idGuid, string numero, DateTime? fecha, decimal baseImponible, decimal porcentajeIva,
                         Guid? idEmpleado, Guid? idCliente, string codigoTaller, int? formaDePago, string cuentaBancaria)
    {
      var venta = new Venta
                {
                  Id = idGuid,
                  BaseImponible = baseImponible,
                  IdCliente = idCliente,
                  Numero = numero,
                  Fecha = fecha,
                  PorcentajeIva = (double?) porcentajeIva,
                  IdEmpleado = idEmpleado,
                  CodigoTaller = codigoTaller,
                  FormaDePago = formaDePago,
                  CuentaBancaria = cuentaBancaria
                };

      AddVenta(venta);
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionVentaService"/></param>
    /// <param name="numero"><see cref="IGestionVentaService"/> </param>
    /// <param name="fecha"><see cref="IGestionVentaService"/> </param>
    /// <param name="baseImponible"><see cref="IGestionVentaService"/> </param>
    /// <param name="porcentajeIva"><see cref="IGestionVentaService"/> </param>
    /// <param name="idEmpleado"><see cref="IGestionVentaService"/> </param>
    /// <param name="idCliente"><see cref="IGestionVentaService"/> </param>
    /// <param name="codigoTaller"><see cref="IGestionVentaService"/> </param>
    /// <param name="formaDePago"><see cref="IGestionVentaService"/> </param>
    /// <param name="cuentaBancaria"><see cref="IGestionVentaService"/> </param>    
    public void ChangeVenta(Guid idGuid, string numero, DateTime? fecha, decimal baseImponible,
                            decimal porcentajeIva, Guid? idEmpleado, Guid? idCliente, string codigoTaller,
                            int? formaDePago, string cuentaBancaria)
    {
      var venta = BuscarVentaPorId(idGuid).FirstOrDefault();
      if (venta != null)
      {
        venta.Id = idGuid;
        venta.Numero = numero;
        venta.Fecha = fecha;
        venta.BaseImponible = baseImponible;
        venta.PorcentajeIva = (double?) porcentajeIva;
        venta.IdEmpleado = idEmpleado;
        venta.IdCliente = idCliente;
        venta.CodigoTaller = codigoTaller;
        venta.FormaDePago = formaDePago;
        venta.CuentaBancaria = cuentaBancaria;

        ChangeVenta(venta);
      }
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="venta"><see cref="IGestionVentaService"/></param>
    public void ChangeVenta(Venta venta)
    {
      if (venta == null) throw new ArgumentNullException("venta");

      var unitOfWork = _ventaRepository.UnitOfWork;

      _ventaRepository.Modify(venta);

      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="idGuid"><see cref="IGestionVentaService"/></param>
    public void RemoveVenta(Guid idGuid)
    {
      var venta = BuscarVentaPorId(idGuid).FirstOrDefault();
      if (venta != null)
      {
        RemoveVenta(venta);
      }
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="venta"><see cref="IGestionVentaService"/></param>
    public void RemoveVenta(Venta venta)
    {
      if (venta == null) throw new ArgumentNullException("venta");

      // Iniciar unidad de trabajo (en caso de necesitar una transacción iniciariamos aquí un 
      // nuevo elemento TransactionScope)
      var unitOfWork = _ventaRepository.UnitOfWork;

      _ventaRepository.Remove(venta);

      // Completar los cambios en esta unidad de trabajo
      unitOfWork.CommitAndRefreshChanges();
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="id"><see cref="IGestionVentaService"/> </param>
    /// <returns><see cref="IGestionVentaService"/></returns>
    public List<Venta> BuscarVentaPorId(Guid? id)
    {
      //Crear especificación para la búsqueda de la venta en el repositorio
      var spec = new IdVentaSpecification(id);

      return _ventaRepository.BuscarVenta(spec);
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="numero"><see cref="IGestionVentaService"/> </param>
    /// <returns><see cref="IGestionVentaService"/></returns>
    public List<Venta> BuscarVentaPorNumero(string numero)
    {
      //Crear especificación para la búsqueda de la venta en el repositorio
      var spec = new NumeroVentaSpecification(numero);

      return _ventaRepository.BuscarVenta(spec);
    }

    /// <summary>
    /// <see cref="IGestionVentaService"/>
    /// </summary>
    /// <param name="cliente"><see cref="IGestionVentaService"/> </param>
    /// <param name="fecha"><see cref="IGestionVentaService"/> </param>
    /// <returns><see cref="IGestionVentaService"/></returns>
    public List<Venta> BuscarVentaPorClienteYFecha(Guid? cliente, DateTime? fecha)
    {
      var spec = new ClienteYFechaDeLaVentaSpecification(cliente, fecha);

      return _ventaRepository.BuscarVenta(spec);
    }

    #endregion
  }
}