﻿using System;
using System.Collections;
using System.Linq.Expressions;
using System.Linq;

namespace GestionConcesionarios.Infraestructura.Helpers
{
  public static class ExpressionHelper
  {
    public static string GetPropertyName<TSource>(Expression<Func<TSource, object>> exceptionProperty)
    {
      if (exceptionProperty == null) throw new ArgumentNullException("exceptionProperty");

      if (exceptionProperty.Body is MemberExpression)
        return ((MemberExpression)exceptionProperty.Body).Member.Name;
      return exceptionProperty.Body is UnaryExpression
                     ? ((MemberExpression)((UnaryExpression)exceptionProperty.Body).Operand).Member.Name
               : string.Empty;
    }

    /// <summary>
    ///
    /// </summary>
    /// <typeparam name="TSource"></typeparam>
    /// <typeparam name="TResult"></typeparam>
    /// <param name="source">Root Object - it must be a reference type or a sub class of IEnumerable</param>
    /// <param name="expression">Labmda expression to set the property value returned</param>
    /// <param name="defaultValue">The default value in the case the property is not reachable </param>
    /// <returns></returns>
    public static TResult NullSafeGetValue<TSource, TResult>(this TSource source, Expression<Func<TSource, TResult>> expression, TResult defaultValue)
    {
      var value = GetValue(expression, source);
      return value == null ? defaultValue : (TResult)value;
    }

    /// <summary>
    ///
    /// </summary>
    /// <typeparam name="TSource"></typeparam>
    /// <typeparam name="TResult"></typeparam>
    /// <typeparam name="TCastResultType"></typeparam>
    /// <param name="source">Root Object</param>
    /// <param name="expression">Labmda expression to set the property value returned</param>
    /// <param name="defaultValue">The default value in the case the property is not reachable</param>
    /// <param name="convertToResultToAction">An action to cast the returned value</param>
    /// <returns></returns>
    public static TCastResultType NullSafeGetValue<TSource, TResult, TCastResultType>(this TSource source, Expression<Func<TSource, TResult>> expression, TCastResultType defaultValue, Func<object, TCastResultType> convertToResultToAction)
    {
      var value = GetValue(expression, source);
      return value == null ? defaultValue : convertToResultToAction.Invoke(value);
    }
    public static string GetFullPropertyPath<TSource, TResult>(Expression<Func<TSource, TResult>> expression)
    {
      return expression.Body.ToString().Replace(expression.Parameters[0] + ".", string.Empty);
    }

    public static string GetPropertyParentPath<TSource>(Expression<Func<TSource, object>> property)
    {
      var propertyPathArray = GetFullPropertyPath(property).Split('.').ToArray();
      var count = propertyPathArray.Count();
      var propertyPathArrayTakingLastElement = propertyPathArray.Take(count - 1);
      var fullPath = String.Join(".", propertyPathArrayTakingLastElement);
      return fullPath;
    }

    private static object GetValue<TSource, TResult>(Expression<Func<TSource, TResult>> expression, TSource source)
    {
      string fullPropertyPathName = GetFullPropertyPath(expression);
      return GetNestedPropertyValue(fullPropertyPathName, source);
    }

    private static object GetNestedPropertyValue(string name, object obj)
    {
      foreach (var part in name.Split('.'))
      {
        if (obj == null)
        {
          return null;
        }
        var type = obj.GetType();
        if (obj is IEnumerable)
        {
          type = (obj as IEnumerable).GetType();
          var methodInfo = type.GetMethod("get_Item");
          var index = int.Parse(part.Split('(')[1].Replace(")", string.Empty));
          try
          {
            obj = methodInfo.Invoke(obj, new object[] { index });
          }
          catch (Exception)
          {
            obj = null;
          }
        }
        else
        {
          var info = type.GetProperty(part);
          if (info == null)
          {
            return null;
          }
          obj = info.GetValue(obj, null);
        }
      }
      return obj;
    }


    public static LambdaExpression GetNestedPropertyTargetExpression<TSource, TProperty>(Expression<Func<TSource, TProperty>> targetExpression)
    {
      MemberExpression memberExpression = null;

      if (targetExpression.Body.NodeType == ExpressionType.Convert)
      {
        memberExpression = ((UnaryExpression)targetExpression.Body).Operand as MemberExpression;
      }

      if (targetExpression.Body.NodeType == ExpressionType.MemberAccess)
      {
        memberExpression = targetExpression.Body as MemberExpression;
      }

      if (memberExpression.Expression.NodeType != ExpressionType.Parameter)
      {
        ParameterExpression parameter = GetParameterExpression(memberExpression.Expression);
        if (parameter != null)
        {
          return Expression.Lambda(memberExpression.Expression, parameter);
        }
      }
      throw new ArgumentException("Invalid target expression");

    }


    private static ParameterExpression GetParameterExpression(Expression expression)
    {
      while (expression.NodeType == ExpressionType.MemberAccess)
      {
        expression = ((MemberExpression)expression).Expression;
      }
      if (expression.NodeType == ExpressionType.Parameter)
      {
        return (ParameterExpression)expression;
      }
      return null;
    }

  }
}