﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

namespace GestionConcesionarios.Infraestructura.Helpers
{

  public static class TypeExtensions
  {
    private static readonly Dictionary<Type, PropertyInfo[]> PropertyInfoCache = new Dictionary<Type, PropertyInfo[]>();
    private static readonly Dictionary<Type, FieldInfo[]> FieldInfoCache = new Dictionary<Type, FieldInfo[]>();

    private static IEnumerable<Type> GetGenericInterfaces(Type type)
    {
      if (type == null) throw new ArgumentNullException("type");

      return type.GetInterfaces().Where(x => x.IsGenericType);
    }

    private static bool MethodHasAParameterOfType(MethodInfo methodInfo, Type parameterType)
    {
      if (methodInfo == null) throw new ArgumentNullException("methodInfo");
      if (parameterType == null) throw new ArgumentNullException("parameterType");

      return methodInfo.GetParameters().Any(p => p.ParameterType == parameterType);
    }


    public static T Cast<T>(object o)
    {
      return (T) o;
    }

    public static bool IsNullable(this Type type)
    {
      return type.IsGenericType && type.GetGenericTypeDefinition() == typeof (Nullable<>);
    }

    public static FieldInfo[] GetInstanceFields(this Type type)
    {
      FieldInfo[] fieldInfos;
      
      FieldInfoCache.TryGetValue(type, out fieldInfos);
      if (fieldInfos == null)
      {
        fieldInfos = type.GetFields(BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic);
        FieldInfoCache.Add(type, fieldInfos);
      }

      return fieldInfos;
    }


    public static PropertyInfo[] GetInstanceProperties(this Type type)
    {
      PropertyInfo[] propertyInfos;
      PropertyInfoCache.TryGetValue(type, out propertyInfos);
      if (propertyInfos == null)
      {
        propertyInfos = type.GetProperties(BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic);
        PropertyInfoCache.Add(type, propertyInfos);
      }
      return propertyInfos;
    }

    public static PropertyInfo[] GetPublicInstanceProperties(this Type type)
    {
        PropertyInfo[] propertyInfos;
        PropertyInfoCache.TryGetValue(type, out propertyInfos);
        if (propertyInfos == null)
        {
            propertyInfos = type.GetProperties(BindingFlags.Instance | BindingFlags.Public);
            PropertyInfoCache.Add(type, propertyInfos);
        }
        return propertyInfos;
    }

    public static string[] GetPublicInstancePropertyNames(this Type type)
    {
      if (type == null) throw new ArgumentNullException("type");

      return type.GetProperties(BindingFlags.Public | BindingFlags.Instance).Select(l => l.Name).ToArray();
    }

    public static Type GetFirstGenericInterfaceArgumentType(this Type type)
    {
      if (type == null) 
          throw new ArgumentNullException("type");

      var interfaces = GetGenericInterfaces(type);
      var firstInterface = interfaces.FirstOrDefault();
      if (firstInterface == null) 
          throw new ArgumentException(string.Format( "The type {0} has not a generic interface",type.FullName), "type");

      var firstType = firstInterface.GetGenericArguments().FirstOrDefault();

      return firstType;
    }

    public static String GetFirstGenericInterfaceArgumentTypeFullName(this Type type)
    {
      if (type == null) throw new ArgumentNullException("type");

      return type.GetFirstGenericInterfaceArgumentType().FullName;
    }

    public static string GetBaseGenericArgumentFullName(this Type type)
    {
      if (type == null) 
          throw new ArgumentNullException("type");
      if (type.BaseType==null) 
          throw new ArgumentException(String.Format("Type {0} has no base type",type.FullName),"type");

      return type.BaseType.GetGenericArguments().First().FullName;
    }


    public static IEnumerable<Type> GetAllGenericImplementedInterfacesGenericArguments(this Type type)
    {
      if (type == null) throw new ArgumentNullException("type");

      var interfaces = GetGenericInterfaces(type);
      return interfaces.SelectMany(@interface => @interface.GetGenericArguments());
    }

    public static MethodInfo GetMethodWithSingleParameterOfType(this Type type, string methodName, Type parameterType)
    {
      if (type == null)
          throw new ArgumentNullException("type");
      if (methodName == null) 
          throw new ArgumentNullException("methodName");
      if (parameterType == null) 
          throw new ArgumentNullException("parameterType");


      var methodInfo =
        type.GetMethods(BindingFlags.Instance | BindingFlags.Public).FirstOrDefault(t => t.Name == methodName && t.GetParameters().Count() == 1 && MethodHasAParameterOfType(t, parameterType));
      return methodInfo;
    }

    public static bool ImplementsInterface(this Type type, Type interfaceType)
    {
      if (interfaceType == null) throw new ArgumentNullException("interfaceType");
      if (!interfaceType.IsInterface) throw new ArgumentException("interfaceType is not an interface");
      if (interfaceType.IsGenericType)
        return type.GetInterfaces().Any(l => l.IsGenericType && l.GetGenericTypeDefinition() == interfaceType);
      return type.GetInterfaces().Contains(interfaceType);
    }

    public static bool HasAttribute(this Type type,Type attributeType)
    {
      if (type == null) throw new ArgumentNullException("type");
      if (attributeType == null) throw new ArgumentNullException("attributeType");
      if (attributeType.BaseType==null) throw new ArgumentException( String.Format("{0} is not an attribute",attributeType.FullName));
      return Attribute.IsDefined(type, attributeType);
      //return type.GetCustomAttributes(attributeType, true).Length > 0;
    }

    public static IEnumerable<Type> GetAllImplementationsInAssembly(this Type type, IEnumerable<Type> typesToExclude)
    {
      return type.Assembly.GetTypes().Where(p => type.IsAssignableFrom(p) && !p.IsInterface).Where(presenterType => !typesToExclude.Any(t => t == presenterType));
    }

    public static PropertyInfo[] GetInstancePropertiesImplementingBaseType(this Type type,Type baseType)
    {
        var propertyInfos = type.GetInstanceProperties();

        return propertyInfos.Where(t => baseType.IsAssignableFrom(t.PropertyType)).ToArray();
    }

    public static string GenericTypeFullNameToString(this Type type)
    {
      if (!type.IsGenericType)
        return type.FullName;

      string value = type.FullName.Substring(0, type.FullName.IndexOf('`')) + "<";
      Type[] genericArgs = type.GetGenericArguments();
      List<string> list = new List<string>();
      for (int i = 0; i < genericArgs.Length; i++)
      {
        value += "{" + i + "},";
        string s = GenericTypeFullNameToString(genericArgs[i]);
        list.Add(s);
      }
      value = value.TrimEnd(',');
      value += ">";
      value = string.Format(value, list.ToArray<string>());
      return value;

    }

    public static IEnumerable<FieldInfo> GetAllFields(this Type t)
    {
        if (t == null)
            return Enumerable.Empty<FieldInfo>();

        var flags = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.Instance | BindingFlags.DeclaredOnly;
        return t.GetFields(flags).Union(GetAllFields(t.BaseType));
    }
  }
}