﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace GestionConcesionarios.Presentacion.Base
{
  /// <summary>
  /// Marcador para el registro de clases en el contenedor IoC
  /// </summary>
  public interface IViewConfigurator
  {
    void FireAllPropertiesChanged(bool isModelValid);
  }

  /// <summary>
  /// Clase base para los configuradores de vista (clases que determinan el estado de los 
  /// controles de las vistas)
  /// </summary>
  /// <typeparam name="TPresenter">Clase presenter asociada al configurador de vista.</typeparam>
  public abstract class ViewConfiguratorBase<TPresenter> : INotifyPropertyChanged, IViewConfigurator
  {
    #region Eventos

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    #region Atributos

    protected string[] PropertyNames;

    #endregion

    #region Constructores

    protected ViewConfiguratorBase(TPresenter presenter)
    {
      Presenter = presenter;
      RegisterPresenterNotification();
    }

    #endregion

    #region Metodos Privados

    private void RegisterPresenterNotification()
    {
      if (Presenter is INotifyPropertyChanged)
      {
        var notify = Presenter as INotifyPropertyChanged;
        notify.PropertyChanged += PresenterPropertyChanged;
      }
    }

    void PresenterPropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      FireAllPropertiesChanged(false);
    }

    protected void FireAllPropertiesChanged()
    {
      foreach (var propertyName in GetPropertyNames())
      {
        OnPropertyChanged(propertyName);
      }
    }

    protected abstract IEnumerable<string> GetPropertyNames();

    protected void OnPropertyChanged(string propertyName)
    {
      if (propertyName == null) throw new ArgumentNullException("propertyName");

      if (PropertyChanged != null)
      {
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }

    #endregion

    #region Propiedades

    protected TPresenter Presenter { get; private set; }

    protected bool IsModelValid { get; set; }

    #endregion

    #region Metodos Publicos

    public void FireAllPropertiesChanged(bool isModelValid)
    {
      IsModelValid = isModelValid;
      FireAllPropertiesChanged();
    }

    #endregion
  }
}