﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulosAlmacen;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Almacen
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormEntradaAlAlmacen)]
  [RegisterInMediator(MediatorEnumerator.BuscarArticulo)]
  public class EntradaAlAlmacenPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private EntradaAlAlmacenViewModel _model;
    private readonly EntradaAlAlmacenViewModelFactory _modelFactory;

    private EntradaAlAlmacenViewModelReadOnly _modelReadOnly;
    private readonly EntradaAlAlmacenViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly IGestionArticuloAlmacenService _gestionArticuloAlmacenService;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructores

    public EntradaAlAlmacenPresenter(EntradaAlAlmacenViewModelFactory modelFactory,
                                EntradaAlAlmacenViewModelReadOnlyFactory modelReadOnlyFactory,
                                IMediatorManager mediatorManager,
                                IGestionArticuloAlmacenService gestionArticuloAlmacenService,
                                InformacionUsuario informacionUsuario)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _gestionArticuloAlmacenService = gestionArticuloAlmacenService;
      _informacionUsuario = informacionUsuario;
    }

    #endregion

    #region Propiedades

    public EntradaAlAlmacenViewModel Model
    {
      get { return _model; }
    }

    public EntradaAlAlmacenViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {

    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateEntradaAlAlmacenViewModelReadOnly();
      NotifyPropertyChanged<EntradaAlAlmacenPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevaEntradaAlAlmacen()
    {
      _model = _modelFactory.CrearEntradaAlAlmacenViewModel();
      NotifyPropertyChanged<EntradaAlAlmacenPresenter>(t => t.Model);
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarEntradaAlAlmacen()
    {
      foreach (var articuloAlmacen in _model.LineasDeLaEntradaAlAlmacen)
      {
        Guid idArticuloAlmacen;
        if (!articuloAlmacen.HasBeenPersisted())
        {
          idArticuloAlmacen = Guid.NewGuid();
          _gestionArticuloAlmacenService.AddArticuloAlmacen(idArticuloAlmacen, _model.IdEmpleado, _model.FechaEntrada,
                                                            articuloAlmacen.StockMinimo, articuloAlmacen.StockMaximo,
                                                            (articuloAlmacen.StockActual ?? 0) + articuloAlmacen.CantidadEntrada,
                                                            articuloAlmacen.Ubicacion,
                                                            articuloAlmacen.IdArticulo,
                                                            _informacionUsuario.CodigoTaller);
        }
        else
        {
          idArticuloAlmacen = articuloAlmacen.IdArticuloAlmacen;
          if (articuloAlmacen.Borrar)
            _gestionArticuloAlmacenService.RemoveArticuloAlmacen(idArticuloAlmacen);
          else
            _gestionArticuloAlmacenService.ChangeArticuloAlmacen(idArticuloAlmacen, _model.IdEmpleado,
                                                                 _model.FechaEntrada,
                                                                 articuloAlmacen.StockMinimo,
                                                                 articuloAlmacen.StockMaximo,
                                                                 (articuloAlmacen.StockActual ?? 0) + articuloAlmacen.CantidadEntrada,
                                                                 articuloAlmacen.Ubicacion,
                                                                 articuloAlmacen.IdArticulo,
                                                                 _informacionUsuario.CodigoTaller);
        }
      }
      MessageBox.Show(Resources.EntradaAlAlmacenPresenter_GuardarEntradaAlAlmacen_Entradas_al_almacén_guardadas_con_éxito,
                      Resources.EntradaAlAlmacenPresenter_GuardarEntradaAlAlmacen_Guardar_entradas_al_almacén, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Muestra la vista de artículos para añadir un nuevo artículo
    /// </summary>
    public void MostrarFormArticulos()
    {
      AlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormArticulos));
    }


    /// <summary>
    /// Añade líneas de artículos al paquete de servicio
    /// </summary>
    /// <param name="buscarArticuloViewModel">Líneas de artículos a añadir.</param>
    [MediatorMessageSink(MediatorMessages.BuscarArticuloArticuloSeleccionado, ParameterType = typeof(BuscarArticuloViewModel))]
    public void AñadirLineasArticuloALaOrdenDeReparacion(BuscarArticuloViewModel buscarArticuloViewModel)
    {
      foreach (var articulo in buscarArticuloViewModel.ListaBuscarArticulos)
      {
        if (articulo.Seleccionar)
          _model.AñadirLineaEntradaAlAlmacen(articulo);
      }
      NotifyPropertyChanged<EntradaAlAlmacenPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade un artículo a las entradas del almacén
    /// </summary>
    /// <param name="buscarArticuloListItemModel">Artículo a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirArticuloALasEntradasAlAlmacen, ParameterType = typeof(BuscarArticuloListItemModel))]
    public void AñadirArticuloALasEntradasAlAlmacen(BuscarArticuloListItemModel buscarArticuloListItemModel)
    {
      _model.AñadirLineaEntradaAlAlmacen(buscarArticuloListItemModel);
      NotifyPropertyChanged<EntradaAlAlmacenPresenter>(t => t.Model);
    }

    /// <summary>
    /// Elimina un artículo de las entradas del almacén
    /// </summary>
    /// <param name="lineaEntradaAlAlmacenViewModel">Línea de entrada al almacén a eliminar.</param>
    public void EliminarArticuloDeLasEntradasAlAlmacen(EntradaAlAlmacenViewModel.LineaEntradaAlAlmacenViewModel lineaEntradaAlAlmacenViewModel)
    {
      _model.EliminarLineaEntradaAlAlmacen(lineaEntradaAlAlmacenViewModel);
      NotifyPropertyChanged<EntradaAlAlmacenPresenter>(t => t.Model);
    }

    #endregion

    #region Metodos Privados

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de la búsqueda de artículos
    /// que indique que el artículo seleccionado en dicho presenter se añada
    /// a la lista de entradas de artículos al almacén
    /// </summary>
    private void AlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen()
    {
      _timerAlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen = new Timer();
      _timerAlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen.Tick += TimerAlAsignarArticuloAñadirloALasEntradasAlAlmacen_Tick;
      _timerAlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen.Interval = 1000;
      _timerAlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen.Enabled = true;
    }
    Timer _timerAlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen = new Timer();
    private void TimerAlAsignarArticuloAñadirloALasEntradasAlAlmacen_Tick(object sender, EventArgs e)
    {
      _timerAlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormArticulos].NotifyColleagues(MediatorMessages.AlAsignarArticuloDesdeFormArticulosAñadirloALasEntradasAlAlmacen);
    }

    #endregion
  }
}
