﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo;
using GestionConcesionarios.Presentacion.CasosDeUso.Proveedores;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Articulos
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormArticulo)]
  public class ArticuloPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;

    private ArticuloViewModel _model;
    private readonly ArticuloViewModelFactory _modelFactory;

    private ArticuloViewModelReadOnly _modelReadOnly;
    private readonly ArticuloViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarArticuloPresenter _buscarArticuloPresenter;
    private readonly IGestionArticuloService _gestionArticuloService;
    //private readonly IGestionModeloService _gestionModeloService;
    private bool _alGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion;

    #endregion

    #region Constructores

    public ArticuloPresenter(ArticuloViewModelFactory modelFactory,
                             IMediatorManager mediatorManager,
                             BuscarArticuloPresenter buscarArticuloPresenter,
                             IGestionArticuloService gestionArticuloService,
                             ArticuloViewModelReadOnlyFactory modelReadOnlyFactory)//,
      //IGestionModeloService gestionModeloService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _buscarArticuloPresenter = buscarArticuloPresenter;
      _gestionArticuloService = gestionArticuloService;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _mediator = mediatorManager[MediatorEnumerator.FormArticulo];
      //_gestionModeloService = gestionModeloService;
    }

    #endregion

    #region Propiedades

    public ArticuloViewModel Model
    {
      get { return _model; }
    }

    public ArticuloViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarArticuloListItemModel> ListaBuscarArticuloViewModel
    {
      get { return _buscarArticuloPresenter.Model.ListaBuscarArticulos; }
    }

    public bool AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion
    {
      get { return _alGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.CosteActualPorUnidad).Do(x => ActualizarCosteAnterior());
    }
    
    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateArticuloViewModelReadOnly();
      NotifyPropertyChanged<ArticuloPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Muestra la vista de proveedores para crear un nuevo proveedor o buscar uno existente
    /// </summary>
    public void MostrarFormProveedorConNuevoProveedor()
    {
      AlGuardarElProveedorOSeleccionarloDesdeFormProveedorAñadirloAlArticulo();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormProveedor));
    }

    /// <summary>
    /// Añade un proveedor al artículo
    /// </summary>
    /// <param name="proveedorViewModel">Proveedor a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirProveedorAlArticulo, ParameterType = typeof(ProveedorViewModel))]
    public void AñadirProveedorAlArticulo(ProveedorViewModel proveedorViewModel)
    {
      SetModelReadOnly();
      _model.IdProveedor = proveedorViewModel.Id;
      NotifyPropertyChanged<ArticuloPresenter>(t => t.Model);
    }

    /// <summary>
    /// Pone el flag _alGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = true, de forma
    /// que cuando se guarde un nuevo artículo o se seleccione uno existente, se envíe un 
    /// mensaje al presenter de la orden de reparación para añadir el artículo a la orden.
    /// </summary>
    [MediatorMessageSink(MediatorMessages.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion)]
    public void SetAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion()
    {
      _alGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = true;
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    //[MediatorMessageSink(MediatorMessages.NuevoArticulo)]
    public void NuevoArticulo()
    {
      _model = _modelFactory.CrearArticuloViewModel();
      NotifyPropertyChanged<ArticuloPresenter>(t => t.Model);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id pasado por parámetro.
    /// </summary>
    /// <param name="articuloId">Id del artículo con el se se creará un nuevo modelo de vista.</param>
    //[MediatorMessageSink(MediatorMessages.EditarArticulo, ParameterType = typeof(Guid))]
    public void EditarArticulo(Guid articuloId)
    {
      SetArticulo(articuloId);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaArticulo()
    {
      _model = _modelFactory.CrearArticuloViewModel();
      NotifyPropertyChanged<ArticuloPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de artículos
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarArticulo()
    {
      _buscarArticuloPresenter.Buscar(_model.Nombre);
      NotifyPropertyChanged<ArticuloPresenter>(t => t.ListaBuscarArticuloViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarArticulo()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.ArticuloPresenter_EliminarArticulo_, Resources.ArticuloPresenter_EliminarArticulo_Eliminar_artículo, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionArticuloService.RemoveArticulo(_model.Id);
          NuevoArticulo();
          MessageBox.Show(Resources.ArticuloPresenter_EliminarArticulo_Articulo_eliminado_con_éxito,
                          Resources.ArticuloPresenter_EliminarArticulo_Eliminar_artículo, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.ArticuloPresenter_EliminarArticulo_No_se_puede_eliminar_el_artículo_porque_todavía_no_se_ha_guardado_nada,
                          Resources.ArticuloPresenter_EliminarArticulo_Eliminar_artículo, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    /// <remarks>
    /// Si el flag _alGuardarElProveedorOSeleccionarloAñadirloAlArticulo = True, se envía un mensaje
    /// al presenter del artículo para añadir el proveedor al artículo
    /// </remarks>
    public void GuardarArticulo()
    {
      if (!_model.HasBeenPersisted())
        _gestionArticuloService.AddArticulo(_model.Nombre, _model.Descripcion, _model.IdProveedor, _model.IdCategoriaArticulo, _model.CosteActualPorUnidad,
                                            _model.CosteAnteriorPorUnidad);
      else
      {
        _gestionArticuloService.ChangeArticulo(_model.Id, _model.Nombre, _model.Descripcion, _model.IdProveedor, _model.IdCategoriaArticulo, _model.CosteActualPorUnidad,
                                            _model.CosteAnteriorPorUnidad);
      }
      MessageBox.Show(Resources.ArticuloPresenter_GuardarArticulo_Artículo_guardado_con_éxito,
                      Resources.ArticuloPresenter_GuardarArticulo_Guardar_artículo, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);

      if (_alGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion)
        //_mediatorManager[MediatorEnumerator.FormOrdenDeReparacion].NotifyColleagues(MediatorMessages.AñadirArticuloALaOrdenDeReparacion, _model);
        _mediator.NotifyColleagues(MediatorMessages.AñadirArticuloALaOrdenDeReparacion, _model);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="articuloId">Id de la entidad a cargar</param>
    public void SetArticulo(Guid articuloId)
    {
      _model = _modelFactory.CrearArticuloViewModel(articuloId);
      NotifyPropertyChanged<ArticuloPresenter>(t => t.Model);

      if (_alGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion)
        //_mediatorManager[MediatorEnumerator.FormOrdenDeReparacion].NotifyColleagues(MediatorMessages.AñadirArticuloALaOrdenDeReparacion, _model);
        _mediator.NotifyColleagues(MediatorMessages.AñadirArticuloALaOrdenDeReparacion, _model);
    }
    #endregion

    #region Metodos Privados

    /// <summary>
    /// Actualiza el coste anterior del artículo.
    /// </summary>
    private void ActualizarCosteAnterior()
    {
      if (Model.CosteActualPorUnidad != Model.CosteActualBuffer)
      {
        Model.CosteAnteriorPorUnidad = Model.CosteActualBuffer;
        Model.CosteActualBuffer = Model.CosteActualPorUnidad;
        NotifyPropertyChanged<ArticuloPresenter>(t => t.Model);
      }
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los proveedores
    /// que indique que el nuevo proveedor creado o seleccionado en dicho presenter se añada
    /// como proveedor al artículo
    /// </summary>
    private void AlGuardarElProveedorOSeleccionarloDesdeFormProveedorAñadirloAlArticulo()
    {
      _timerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo = new Timer();
      _timerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo.Tick += TimerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo_Tick;
      _timerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo.Interval = 1000;
      _timerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo.Enabled = true;
    }
    Timer _timerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo = new Timer();
    private void TimerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormProveedor].NotifyColleagues(MediatorMessages.AlGuardarElProveedorOSeleccionarloAñadirloAlArticulo);
    }

    #endregion
  }
}
