﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Articulos
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista del artículo,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class ArticuloViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<ArticuloViewModel> _crearViewModel;
    private readonly IGestionArticuloService _gestionArticuloService;

    #endregion

    #region Constructores

    public ArticuloViewModelFactory(Func<ArticuloViewModel> crearViewModel, IGestionArticuloService gestionArticuloService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionArticuloService = gestionArticuloService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo ArticuloViewModel vacío</returns>
    public ArticuloViewModel CrearArticuloViewModel()
    {
      var articuloViewModel = _crearViewModel();

      articuloViewModel.Id = Guid.Empty;
      articuloViewModel.Nombre = string.Empty;
      articuloViewModel.Descripcion = string.Empty;
      articuloViewModel.IdProveedor = Guid.Empty;
      articuloViewModel.IdCategoriaArticulo = string.Empty;
      articuloViewModel.CosteActualPorUnidad = 0;
      articuloViewModel.CosteActualBuffer = 0;
      articuloViewModel.CosteAnteriorPorUnidad = 0;
      
      return articuloViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del artículo articuloId.
    /// </summary>
    /// <param name="articuloId">Id del artículo a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo ArticuloViewModel con los datos del artículo con id=articuloId</returns>
    public ArticuloViewModel CrearArticuloViewModel(Guid articuloId)
    {
      var lista = _gestionArticuloService.BuscarArticuloPorId(articuloId);

      var articuloViewModel = CrearArticuloViewModelDesdeListaDeArticulos(lista);

      return articuloViewModel;
    }

    #endregion

    #region Metodos Privados

    /// <summary>
    /// Crea un artículo view model desde lista de articulos. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de artículos en un único artículo.
    /// </summary>
    /// <param name="lista">Lista de articulos.</param>
    /// <returns>Un modelo ArticuloViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo ArticuloViewModel vacío</returns>
    private ArticuloViewModel CrearArticuloViewModelDesdeListaDeArticulos(IEnumerable<Articulo> lista)
    {
      var articuloViewModel = _crearViewModel();

      foreach (var articulo in lista)
      {
        articuloViewModel.Id = articulo.Id;
        articuloViewModel.Nombre = articulo.Nombre;
        articuloViewModel.Descripcion = articulo.Descripcion;
        articuloViewModel.IdProveedor = articulo.IdProveedor;
        articuloViewModel.IdCategoriaArticulo = articulo.IdCategoria;
        articuloViewModel.CosteActualPorUnidad = articulo.CosteActual;
        articuloViewModel.CosteActualBuffer = articulo.CosteActual;
        articuloViewModel.CosteAnteriorPorUnidad = articulo.CosteAnterior;

        return articuloViewModel;
      }
      return articuloViewModel;
    }

    #endregion
  }
}
