﻿using System;
using System.Windows.Forms;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Articulos;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.BuscarArticulo)]
  public class BuscarArticuloPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private readonly IMediator _mediator;
    private readonly BuscarArticuloViewModelFactory _buscarArticuloViewModelFactory;
    private readonly BuscarArticuloViewModelReadOnlyFactory _buscarArticuloViewModelReadOnlyFactory;

    #endregion

    #region Constructor

    public BuscarArticuloPresenter(IMediatorManager mediatorManager, BuscarArticuloViewModelFactory buscarArticuloViewModelFactory, BuscarArticuloViewModelReadOnlyFactory buscarArticuloViewModelReadOnlyFactory)
      : base(mediatorManager)
    {
      if (mediatorManager == null) throw new ArgumentNullException("mediatorManager");

      _buscarArticuloViewModelFactory = buscarArticuloViewModelFactory;
      _buscarArticuloViewModelReadOnlyFactory = buscarArticuloViewModelReadOnlyFactory;
      Model = _buscarArticuloViewModelFactory.CrearBuscarArticuloViewModelFactory();
      _mediator = mediatorManager[MediatorEnumerator.BuscarArticulo];
    }

    #endregion

    #region Propiedades

    public BuscarArticuloViewModel Model { get; set; }
    public BuscarArticuloViewModelReadOnly ModelReadOnly { get; set; }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevaBusqueda()
    {
      Model = _buscarArticuloViewModelFactory.CrearBuscarArticuloViewModelFactory();
      NotifyPropertyChanged<BuscarArticuloPresenter>(t => t.Model);
    }

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.Nombre).Do(x =>
                                             {
                                               Buscar(Model.Nombre);
                                               NotifyPropertyChanged<BuscarArticuloPresenter>(t => t.Model);
                                             });
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      ModelReadOnly = _buscarArticuloViewModelReadOnlyFactory.CreateArticuloViewModelReadOnly();
      NotifyPropertyChanged<BuscarArticuloPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Muestra la vista de artículos para añadir un nuevo artículo
    /// </summary>
    public void MostrarFormArticulo()
    {
      AlGuardarElArticuloOSeleccionarloDesdeFormArticuloAñadirloALaOrdenDeReparacion();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormArticulo));
    }

    /// <summary>
    /// Notifica que se han seleccionado artículos
    /// </summary>
    public void AsignarArticulos()
    {
      _mediator.NotifyColleagues(MediatorMessages.BuscarArticuloArticuloSeleccionado, Model);
    }

    /// <summary>
    /// Ejecuta la búsqueda de artículos en función de los valores pasados por parámetro
    /// </summary>
    /// <param name="nombre">Nombre del artículo a buscar.</param>
    public void Buscar(string nombre)
    {
      Model = _buscarArticuloViewModelFactory.CrearBuscarArticuloViewModelFactory(nombre);
      NotifyPropertyChanged<BuscarArticuloPresenter>(t => t.Model);
    }
    
    #endregion

    #region Métodos Privados

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los artículos
    /// que indique que el nuevo artículo creado o seleccionado en dicho presenter se añada
    /// a la lista de artículos de la orden de reparación
    /// </summary>
    private void AlGuardarElArticuloOSeleccionarloDesdeFormArticuloAñadirloALaOrdenDeReparacion()
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Tick += TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Interval = 1000;
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = true;
    }
    Timer _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion = new Timer();
    private void TimerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormArticulo].NotifyColleagues(MediatorMessages.AlGuardarElArticuloOSeleccionarloAñadirloALaOrdenDeReparacion);
    }

    #endregion
  }
}
