﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionArticulos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarArticulo
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de artículos,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarArticuloViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<BuscarArticuloViewModel> _crearViewModel;
    private readonly IGestionArticuloService _gestionArticuloService;
    
    #endregion

    #region Constructor

    public BuscarArticuloViewModelFactory(Func<BuscarArticuloViewModel> crearViewModel, IGestionArticuloService gestionArticuloService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionArticuloService = gestionArticuloService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarArticuloViewModel vacío</returns>
    public BuscarArticuloViewModel CrearBuscarArticuloViewModelFactory()
    {
      var buscarArticuloViewModel = _crearViewModel();

      buscarArticuloViewModel.ListaBuscarArticulos = new BindingList<BuscarArticuloListItemModel>();
      
      return buscarArticuloViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="nombre">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarArticuloViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarArticuloViewModel CrearBuscarArticuloViewModelFactory(string nombre)
    {
      var buscarArticuloViewModel = _crearViewModel();

      var lista = _gestionArticuloService.BuscarArticuloPorNombre(nombre);
      buscarArticuloViewModel.ListaBuscarArticulos = CrearListaBuscarArticulos(lista);

      return buscarArticuloViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarArticuloListItemModel> CrearListaBuscarArticulos(IEnumerable<Articulo> listaArticulos)
    {
      var listaBuscarArticulos = new BindingList<BuscarArticuloListItemModel>();

      foreach (var clienteListItemModel in listaArticulos.Select(articulo => new BuscarArticuloListItemModel
                                                                             {
                                                                               ArticuloId = articulo.Id,
                                                                               Nombre = articulo.Nombre,
                                                                               Descripcion = articulo.Descripcion,
                                                                               IdCategoria = articulo.IdCategoria,
                                                                               IdProveedor = articulo.IdProveedor,
                                                                               CosteActual = articulo.CosteActual
                                                                             }))
      {
        listaBuscarArticulos.Add(clienteListItemModel);
      }

      return listaBuscarArticulos;
    }

    #endregion
  }
}
