﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarCliente
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de clientes,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarClienteViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<BuscarClienteViewModel> _crearViewModel;
    private readonly IGestionClienteService _gestionClienteService;

    #endregion

    #region Constructor

    public BuscarClienteViewModelFactory(IGestionClienteService gestionClienteService, Func<BuscarClienteViewModel> crearViewModel)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _gestionClienteService = gestionClienteService;
      _crearViewModel = crearViewModel;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarClienteViewModel vacío</returns>
    public BuscarClienteViewModel CrearBuscarClienteViewModelFactory()
    {
      var buscarClienteViewModel = _crearViewModel();

      buscarClienteViewModel.ListaBuscarClientes = new BindingList<BuscarClienteListItemModel>();

      return buscarClienteViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="nombre">Criterios de búsqueda a partir de los cuales se creará el modelo. </param>
    /// <param name="primerApellido">Criterios de búsqueda a partir de los cuales se creará el modelo. </param>
    /// <param name="segundoApellido">Criterios de búsqueda a partir de los cuales se creará el modelo. </param>
    /// <returns>Un modelo BuscarClienteViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarClienteViewModel CrearBuscarClienteViewModelFactory(string nombre, string primerApellido, string segundoApellido)
    {
      var lista = _gestionClienteService.BuscarCliente(nombre, primerApellido, segundoApellido);

      var buscarClienteViewModel = _crearViewModel();
      buscarClienteViewModel.ListaBuscarClientes = CrearListaBuscarClientes(lista);

      return buscarClienteViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarClienteListItemModel> CrearListaBuscarClientes(IEnumerable<Cliente> listaClientes)
    {
      var listaBuscarClientes = new BindingList<BuscarClienteListItemModel>();

      foreach (var clienteListItemModel in listaClientes.Select(cliente => new BuscarClienteListItemModel
                                                                             {
                                                                               ClienteId = cliente.Id,
                                                                               Nombre = cliente.Nombre,
                                                                               PrimerApellido = cliente.PrimerApellido,
                                                                               SegundoApellido = cliente.SegundoApellido,
                                                                               Direccion = cliente.Direccion,
                                                                               Observaciones = cliente.Observaciones,
                                                                               Telefonos = cliente.Telefonos,
                                                                               CorreoElectronico = cliente.CorreoElectronico
                                                                             }))
      {
        listaBuscarClientes.Add(clienteListItemModel);
      }

      return listaBuscarClientes;
    }

    #endregion
  }
}
