﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionManosDeObra;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarManoDeObra
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de manos de obra,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarManoDeObraViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<BuscarManoDeObraViewModel> _crearViewModel;
    private readonly IGestionManoDeObraService _gestionManoDeObraService;

    #endregion

    #region Constructor

    public BuscarManoDeObraViewModelFactory(IGestionManoDeObraService gestionManoDeObraService, Func<BuscarManoDeObraViewModel> crearViewModel)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _gestionManoDeObraService = gestionManoDeObraService;
      _crearViewModel = crearViewModel;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarManoDeObraViewModel vacío</returns>
    public BuscarManoDeObraViewModel CrearBuscarManoDeObraViewModelFactory()
    {
      var buscarManoDeObraViewModel = _crearViewModel();

      buscarManoDeObraViewModel.ListaBuscarManosDeObra = new BindingList<BuscarManoDeObraListItemModel>();

      return buscarManoDeObraViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="descripcion">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarManoDeObraViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarManoDeObraViewModel CrearBuscarManoDeObraViewModelFactory(string descripcion)
    {
      var lista = _gestionManoDeObraService.BuscarManoDeObraPorDescripcion(descripcion);

      var buscarManoDeObraViewModel = _crearViewModel();
      buscarManoDeObraViewModel.ListaBuscarManosDeObra = CrearListaBuscarManosDeObra(lista);

      return buscarManoDeObraViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarManoDeObraListItemModel> CrearListaBuscarManosDeObra(IEnumerable<ManoDeObra> listaManosDeObra)
    {
      var listaBuscarManosDeObra = new BindingList<BuscarManoDeObraListItemModel>();

      foreach (var clienteListItemModel in listaManosDeObra.Select(manoDeObra => new BuscarManoDeObraListItemModel
                                                                             {
                                                                               ManoDeObraId = manoDeObra.Id,
                                                                               Descripcion = manoDeObra.Descripcion,
                                                                               Duracion = manoDeObra.Duracion,
                                                                               PrecioPorHora = manoDeObra.PrecioPorHora
                                                                             }))
      {
        listaBuscarManosDeObra.Add(clienteListItemModel);
      }

      return listaBuscarManosDeObra;
    }

    #endregion
  }
}
