﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarOrdenDeReparacion
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de ordenes de reparación,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarOrdenDeReparacionViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;

    #endregion

    #region Constructor

    public BuscarOrdenDeReparacionViewModelFactory(IGestionOrdenDeReparacionService gestionOrdenDeReparacionService)
    {
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarOrdenDeReparacionViewModel vacío</returns>
    public BuscarOrdenDeReparacionViewModel CrearBuscarOrdenDeReparacionViewModelFactory()
    {
      return new BuscarOrdenDeReparacionViewModel { ListaBuscarOrdenesDeReparacion = new BindingList<BuscarOrdenDeReparacionListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaOrdenDeReparacion">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarOrdenDeReparacionViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarOrdenDeReparacionViewModel CrearBuscarOrdenDeReparacionViewModelFactory(CriteriosBusquedaOrdenDeReparacion criteriosBusquedaOrdenDeReparacion)
    {
      List<OrdenReparacion> lista;
      if (criteriosBusquedaOrdenDeReparacion.HayNumeroOrden)
        lista = _gestionOrdenDeReparacionService.BuscarOrdenDeReparacionPorNumeroOrden(criteriosBusquedaOrdenDeReparacion.NumeroOrden);
      else if (criteriosBusquedaOrdenDeReparacion.HayMatricula)
        lista = _gestionOrdenDeReparacionService.BuscarOrdenDeReparacionPorMatricula(criteriosBusquedaOrdenDeReparacion.Matricula);
      else
      {
        lista = _gestionOrdenDeReparacionService.BuscarOrdenDeReparacionPorNombreCliente(criteriosBusquedaOrdenDeReparacion.NombreCliente);
      }
      var buscarOrdenDeReparacionViewModel = new BuscarOrdenDeReparacionViewModel { ListaBuscarOrdenesDeReparacion = CrearListaBuscarOrdenesDeReparacion(lista) };

      return buscarOrdenDeReparacionViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarOrdenDeReparacionListItemModel> CrearListaBuscarOrdenesDeReparacion(IEnumerable<OrdenReparacion> listaOrdenesDeReparacion)
    {
      var listaBuscarOrdenesDeReparacion = new BindingList<BuscarOrdenDeReparacionListItemModel>();

      foreach (var ordenReparacionListItemModel in listaOrdenesDeReparacion.Select(ordenReparacion => new BuscarOrdenDeReparacionListItemModel
      {
        OrdenDeReparacionId = ordenReparacion.Id,
        NumeroOrden = ordenReparacion.NumeroOrden,
        IdCliente = ordenReparacion.Recepcion.Cliente.Id,
        Cliente = ordenReparacion.Recepcion.Cliente.Nombre,
        Matricula = ordenReparacion.Vehiculo.Matricula,
        Estado = ordenReparacion.IdEstado.ToString(),
        FechaHoraRecepcion = ordenReparacion.Recepcion.FechaHoraRecepcion,
        FechaHoraEntrega = ordenReparacion.FechaHoraEntrega
      }))
      {
        listaBuscarOrdenesDeReparacion.Add(ordenReparacionListItemModel);
      }
      return listaBuscarOrdenesDeReparacion;
    }

    #endregion
  }
}
