﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetesDeServicio;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarPaqueteDeServicio
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de paquetes de servicio,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarPaqueteDeServicioViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<BuscarPaqueteDeServicioViewModel> _crearViewModel;
    private readonly IGestionPaqueteDeServicioService _gestionPaqueteDeServicioService;

    #endregion

    #region Constructor

    public BuscarPaqueteDeServicioViewModelFactory(IGestionPaqueteDeServicioService gestionPaqueteDeServicioService, Func<BuscarPaqueteDeServicioViewModel> crearViewModel)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _gestionPaqueteDeServicioService = gestionPaqueteDeServicioService;
      _crearViewModel = crearViewModel;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarPaqueteDeServicioViewModel vacío</returns>
    public BuscarPaqueteDeServicioViewModel CrearBuscarPaqueteDeServicioViewModelFactory()
    {
      var buscarPaqueteDeServicioViewModel = _crearViewModel();

      buscarPaqueteDeServicioViewModel.ListaBuscarPaquetesDeServicio = new BindingList<BuscarPaqueteDeServicioListItemModel>();

      return buscarPaqueteDeServicioViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="descripcion">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarPaqueteDeServicioViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarPaqueteDeServicioViewModel CrearBuscarPaqueteDeServicioViewModelFactory(string descripcion)
    {
      var lista = _gestionPaqueteDeServicioService.BuscarPaqueteDeServicioPorDescripcion(descripcion);

      var buscarPaqueteDeServicioViewModel = _crearViewModel();
      buscarPaqueteDeServicioViewModel.ListaBuscarPaquetesDeServicio = CrearListaBuscarPaquetesDeServicio(lista);

      return buscarPaqueteDeServicioViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarPaqueteDeServicioListItemModel> CrearListaBuscarPaquetesDeServicio(IEnumerable<PaqueteDeServicio> listaPaquetesDeServicio)
    {
      var listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId =
        CrearListaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId(listaPaquetesDeServicio).ToList();

      var listaBuscarPaquetesDeServicio = new BindingList<BuscarPaqueteDeServicioListItemModel>();

      foreach (var buscarPaqueteDeServicioListItemModel in listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId.Select(listaPaqueteDeServicio => new BuscarPaqueteDeServicioListItemModel
                                                                                                                                                               {
                                                                                                                                                                 PaqueteDeServicioId = listaPaqueteDeServicio[0].Id,
                                                                                                                                                                 DuracionSugerida = listaPaqueteDeServicio[0].DuracionSugerida,
                                                                                                                                                                 Descripcion = listaPaqueteDeServicio[0].Descripcion,
                                                                                                                                                                 Precio = listaPaqueteDeServicio[0].Precio
                                                                                                                                                               }))
      {
        listaBuscarPaquetesDeServicio.Add(buscarPaqueteDeServicioListItemModel);
      }

      return listaBuscarPaquetesDeServicio;
    }

    private IEnumerable<List<PaqueteDeServicio>> CrearListaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId(IEnumerable<PaqueteDeServicio> listaPaquetesDeServicio)
    {
      var listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId = new List<List<PaqueteDeServicio>>();

      foreach (var paqueteDeServicio in listaPaquetesDeServicio)
      {
        var listaDePaquetesDeServicio =
          ObtenerListaDePaquetesDeServicioConIdPaqueteDeServicio(
            listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId, paqueteDeServicio.Id);

        if (listaDePaquetesDeServicio == null)
        {
          listaDePaquetesDeServicio = new List<PaqueteDeServicio>();
          listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId.Add(listaDePaquetesDeServicio);
        }

        listaDePaquetesDeServicio.Add(paqueteDeServicio);
      }

      return listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId;
    }

    private List<PaqueteDeServicio> ObtenerListaDePaquetesDeServicioConIdPaqueteDeServicio(IEnumerable<List<PaqueteDeServicio>> listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId, Guid id)
    {
      return
        listaDeListasDePaquetesDeServicioAgrupadosPorPaqueteDeServicioId.FirstOrDefault(
          listaDePaquetesDeServicio =>
          listaDePaquetesDeServicio.Any(paqueteDeServicio => paqueteDeServicio.Id.Equals(id)));
    }

    #endregion
  }
}
