﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPresupuestos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarPresupuesto
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de presupuestos,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarPresupuestoViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionPresupuestoService _gestionPresupuestoService;

    #endregion

    #region Constructor

    public BuscarPresupuestoViewModelFactory(IGestionPresupuestoService gestionPresupuestoService)
    {
      _gestionPresupuestoService = gestionPresupuestoService;
    }

    #endregion

    #region Métodos Públicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarPresupuestoViewModel vacío</returns>
    public BuscarPresupuestoViewModel CrearBuscarPresupuestoViewModelFactory()
    {
      return new BuscarPresupuestoViewModel { ListaBuscarPresupuestos = new BindingList<BuscarPresupuestoListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaPresupuesto">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarPresupuestoViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarPresupuestoViewModel CrearBuscarPresupuestoViewModelFactory(CriteriosBusquedaPresupuesto criteriosBusquedaPresupuesto)
    {
      List<Presupuesto> lista;
      if (HayNumeroPresupuesto(criteriosBusquedaPresupuesto))
        lista = _gestionPresupuestoService.BuscarPresupuestoPorNumero(criteriosBusquedaPresupuesto.Numero);
      else
      {
        lista = _gestionPresupuestoService.BuscarPresupuestoPorClienteYFecha(criteriosBusquedaPresupuesto.Cliente,
                                                                          criteriosBusquedaPresupuesto.Fecha);
      }
      var buscarPresupuestoViewModel = new BuscarPresupuestoViewModel { ListaBuscarPresupuestos = CrearListaBuscarPresupuestos(lista) };

      return buscarPresupuestoViewModel;
    }

    #endregion

    #region Metodos Privados

    private bool HayNumeroPresupuesto(CriteriosBusquedaPresupuesto criteriosBusquedaPresupuesto)
    {
      return !string.IsNullOrWhiteSpace(criteriosBusquedaPresupuesto.Numero);
    }

    private BindingList<BuscarPresupuestoListItemModel> CrearListaBuscarPresupuestos(IEnumerable<Presupuesto> listaPresupuestos)
    {
      var listaBuscarPresupuestos = new BindingList<BuscarPresupuestoListItemModel>();

      foreach (var presupuestoListItemModel in listaPresupuestos.Select(presupuesto => new BuscarPresupuestoListItemModel
      {
        PresupuestoId = presupuesto.Id,
        NumeroPresupuesto = presupuesto.Numero,
        Fecha = presupuesto.Fecha,
        Cliente = presupuesto.Cliente != null ? presupuesto.Cliente.Nombre : string.Empty,
        Empleado = presupuesto.Empleado != null ? presupuesto.Empleado.Nombre : string.Empty,
        Subtotal = presupuesto.BaseImponible,
        ImporteIva = presupuesto.BaseImponible.HasValue && presupuesto.PorcentajeIva.HasValue ? (presupuesto.BaseImponible * presupuesto.PorcentajeIva) / 100 : 0,
        ImporteTotal = presupuesto.BaseImponible.HasValue && presupuesto.PorcentajeIva.HasValue ? presupuesto.BaseImponible + ((presupuesto.BaseImponible * presupuesto.PorcentajeIva) / 100) : 0
      }))
      {
        listaBuscarPresupuestos.Add(presupuestoListItemModel);
      }
      return listaBuscarPresupuestos;
    }

    #endregion
  }
}
