﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProveedores;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarProveedor
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de proveedores,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarProveedorViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionProveedorService _gestionProveedorService;

    #endregion

    #region Constructor

    public BuscarProveedorViewModelFactory(IGestionProveedorService gestionProveedorService)
    {
      _gestionProveedorService = gestionProveedorService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarProveedorViewModel vacío</returns>
    public BuscarProveedorViewModel CrearBuscarProveedorViewModelFactory()
    {
      return new BuscarProveedorViewModel { ListaBuscarProveedores = new BindingList<BuscarProveedorListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaProveedor">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarProveedorViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarProveedorViewModel CrearBuscarProveedorViewModelFactory(CriteriosBusquedaProveedor criteriosBusquedaProveedor)
    {
      List<Proveedor> lista;
      if (criteriosBusquedaProveedor.HayCodigo)
        lista = _gestionProveedorService.BuscarProveedorPorCodigo(criteriosBusquedaProveedor.Codigo);
      else if (criteriosBusquedaProveedor.HayCifDni)
        lista = _gestionProveedorService.BuscarProveedorPorCifDni(criteriosBusquedaProveedor.CifDni);
      else
      {
        lista = _gestionProveedorService.BuscarProveedorPorRazonSocial(criteriosBusquedaProveedor.RazonSocial);
      }
      var buscarProveedorViewModel = new BuscarProveedorViewModel { ListaBuscarProveedores = CrearListaBuscarProveedores(lista) };

      return buscarProveedorViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarProveedorListItemModel> CrearListaBuscarProveedores(IEnumerable<Proveedor> listaProveedores)
    {
      var listaBuscarProveedores = new BindingList<BuscarProveedorListItemModel>();

      foreach (var proveedorListItemModel in listaProveedores.Select(proveedor => new BuscarProveedorListItemModel
      {
        ProveedorId = proveedor.Id,
        Codigo = proveedor.Codigo,
        RazonSocial = proveedor.RazonSocial,
        CifDni = proveedor.CifDni,
      }))
      {
        listaBuscarProveedores.Add(proveedorListItemModel);
      }
      return listaBuscarProveedores;
    }

    #endregion
  }
}
