﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarReparaciones
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de reparaciones,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarReparacionViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;

    #endregion

    #region Constructor

    public BuscarReparacionViewModelFactory(IGestionOrdenDeReparacionService gestionOrdenDeReparacionService)
    {
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarReparacionViewModel vacío</returns>
    public BuscarReparacionViewModel CrearBuscarReparacionViewModelFactory()
    {
      return new BuscarReparacionViewModel { ListaBuscarReparaciones = new BindingList<BuscarReparacionListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaReparacion">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarReparacionViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarReparacionViewModel CrearBuscarReparacionViewModelFactory(CriteriosBusquedaReparacion criteriosBusquedaReparacion)
    {
      List<OrdenReparacion> lista;
      if (criteriosBusquedaReparacion.HayOrdenDeReparacion)
        lista = _gestionOrdenDeReparacionService.BuscarOrdenReparacionPorId(criteriosBusquedaReparacion.IdOrdenDeReparacion);
      else if (criteriosBusquedaReparacion.HayMatricula)
        lista = _gestionOrdenDeReparacionService.BuscarOrdenDeReparacionPorMatricula(criteriosBusquedaReparacion.Matricula);
      else
      {
        lista = _gestionOrdenDeReparacionService.BuscarPorEmpleadoYEstado(criteriosBusquedaReparacion.IdEmpleado, EstadoOrdenReparacion.EnProceso);
      }
      var buscarReparacionViewModel = new BuscarReparacionViewModel { ListaBuscarReparaciones = CrearListaBuscarReparaciones(lista) };

      return buscarReparacionViewModel;
    }

    #endregion

    #region Metodos Privados

    private BindingList<BuscarReparacionListItemModel> CrearListaBuscarReparaciones(IEnumerable<OrdenReparacion> listaOrdenesDeReparacion)
    {
      var listaBuscarReparaciones = new BindingList<BuscarReparacionListItemModel>();

      foreach (var ordenReparacionListItemModel in listaOrdenesDeReparacion.Select(ordenReparacion => new BuscarReparacionListItemModel
      {
        OrdenDeReparacionId = ordenReparacion.Id,
        NumeroOrden = ordenReparacion.NumeroOrden,
        IdCliente = ordenReparacion.Recepcion.Cliente.Id,
        Cliente = ordenReparacion.Recepcion.Cliente.Nombre,
        Matricula = ordenReparacion.Vehiculo.Matricula,
        Estado = ordenReparacion.IdEstado.ToString(),
        FechaHoraRecepcion = ordenReparacion.Recepcion.FechaHoraRecepcion,
        FechaHoraEntrega = ordenReparacion.FechaHoraEntrega
      }))
      {
        listaBuscarReparaciones.Add(ordenReparacionListItemModel);
      }
      return listaBuscarReparaciones;
    }

    #endregion
  }
}
