﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarVehiculo
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de vehículos,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarVehiculoViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionVehiculoService _gestionVehiculoService;

    #endregion

    #region Constructor

    public BuscarVehiculoViewModelFactory(IGestionVehiculoService gestionVehiculoService)
    {
      _gestionVehiculoService = gestionVehiculoService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarVehiculoViewModel vacío</returns>
    public BuscarVehiculoViewModel CrearBuscarVehiculoViewModelFactory()
    {
      return new BuscarVehiculoViewModel { ListaBuscarVehiculos = new BindingList<BuscarVehiculoListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaVehiculo">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarVehiculoViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarVehiculoViewModel CrearBuscarVehiculoViewModelFactory(CriteriosBusquedaVehiculo criteriosBusquedaVehiculo)
    {
      List<Vehiculo> lista;
      if (HayMatricula(criteriosBusquedaVehiculo))
        lista = _gestionVehiculoService.BuscarVehiculoPorMatricula(criteriosBusquedaVehiculo.Matricula);
      else
      {
        lista = _gestionVehiculoService.BuscarVehiculoPorMarcaYModelo(criteriosBusquedaVehiculo.Marca,
                                                                          criteriosBusquedaVehiculo.Modelo);
      }
      var buscarVehiculoViewModel = new BuscarVehiculoViewModel { ListaBuscarVehiculos = CrearListaBuscarVehiculos(lista) };

      return buscarVehiculoViewModel;
    }

    #endregion

    #region Metodos Privados

    private bool HayMatricula(CriteriosBusquedaVehiculo criteriosBusquedaVehiculo)
    {
      return !string.IsNullOrWhiteSpace(criteriosBusquedaVehiculo.Matricula);
    }

    private BindingList<BuscarVehiculoListItemModel> CrearListaBuscarVehiculos(IEnumerable<Vehiculo> listaVehiculos)
    {
      var listaBuscarVehiculos = new BindingList<BuscarVehiculoListItemModel>();

      foreach (var vehiculoListItemModel in listaVehiculos.Select(vehiculo => new BuscarVehiculoListItemModel
      {
        VehiculoId = vehiculo.Id,
        Matricula = vehiculo.Matricula,
        Marca = vehiculo.Marca,
        Modelo = vehiculo.Modelo,
        Color = vehiculo.Color,
        Año = vehiculo.Fecha
      }))
      {
        listaBuscarVehiculos.Add(vehiculoListItemModel);
      }
      return listaBuscarVehiculos;
    }

    #endregion
  }
}
