﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarVehiculoVenta
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de vehículos de venta,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarVehiculoVentaViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionVehiculoVentaService _gestionVehiculoVentaService;

    #endregion

    #region Constructor

    public BuscarVehiculoVentaViewModelFactory(IGestionVehiculoVentaService gestionVehiculoVentaService)
    {
      _gestionVehiculoVentaService = gestionVehiculoVentaService;
    }

    #endregion

    #region Métodos Públicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarVehiculoVentaViewModel vacío</returns>
    public BuscarVehiculoVentaViewModel CrearBuscarVehiculoVentaViewModelFactory()
    {
      return new BuscarVehiculoVentaViewModel { ListaBuscarVehiculosVenta = new BindingList<BuscarVehiculoVentaListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaVehiculoVenta">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarVehiculoVentaViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarVehiculoVentaViewModel CrearBuscarVehiculoVentaViewModelFactory(CriteriosBusquedaVehiculoVenta criteriosBusquedaVehiculoVenta)
    {
      List<VehiculoVenta> lista;
      if (HayMatricula(criteriosBusquedaVehiculoVenta))
        lista = _gestionVehiculoVentaService.BuscarVehiculoVentaPorMatricula(criteriosBusquedaVehiculoVenta.Matricula);
      else
      {
        lista = _gestionVehiculoVentaService.BuscarVehiculoVentaPorMarcaYModelo(criteriosBusquedaVehiculoVenta.Marca,
                                                                          criteriosBusquedaVehiculoVenta.Modelo);
      }
      var buscarVehiculoVentaViewModel = new BuscarVehiculoVentaViewModel { ListaBuscarVehiculosVenta = CrearListaBuscarVehiculosVenta(lista) };

      return buscarVehiculoVentaViewModel;
    }

    #endregion

    #region Metodos Privados

    private bool HayMatricula(CriteriosBusquedaVehiculoVenta criteriosBusquedaVehiculoVenta)
    {
      return !string.IsNullOrWhiteSpace(criteriosBusquedaVehiculoVenta.Matricula);
    }

    private BindingList<BuscarVehiculoVentaListItemModel> CrearListaBuscarVehiculosVenta(IEnumerable<VehiculoVenta> listaVehiculosVenta)
    {
      var listaBuscarVehiculosVenta = new BindingList<BuscarVehiculoVentaListItemModel>();

      foreach (var vehiculoListItemModel in listaVehiculosVenta.Select(vehiculoVenta => new BuscarVehiculoVentaListItemModel
      {
        VehiculoId = vehiculoVenta.Id,
        Matricula = vehiculoVenta.Matricula,
        Marca = vehiculoVenta.Marca,
        Modelo = vehiculoVenta.Modelo,
        Color = vehiculoVenta.Color,
        Año = vehiculoVenta.FechaDeEntrada
      }))
      {
        listaBuscarVehiculosVenta.Add(vehiculoListItemModel);
      }
      return listaBuscarVehiculosVenta;
    }

    #endregion
  }
}
