﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVentas;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.BuscarVenta
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de búsqueda de facturas,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class BuscarFacturaViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly IGestionVentaService _gestionFacturaService;

    #endregion

    #region Constructor

    public BuscarFacturaViewModelFactory(IGestionVentaService gestionFacturaService)
    {
      _gestionFacturaService = gestionFacturaService;
    }

    #endregion

    #region Métodos Públicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo BuscarFacturaViewModel vacío</returns>
    public BuscarFacturaViewModel CrearBuscarFacturaViewModelFactory()
    {
      return new BuscarFacturaViewModel { ListaBuscarFacturas = new BindingList<BuscarFacturaListItemModel>() };
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos que coincidan con los criterios de búsqueda.
    /// </summary>
    /// <param name="criteriosBusquedaFactura">Criterios de búsqueda a partir de los cuales se creará el modelo.</param>
    /// <returns>Un modelo BuscarFacturaViewModel con los datos que coincidan con el criterio de búsqueda</returns>
    public BuscarFacturaViewModel CrearBuscarFacturaViewModelFactory(CriteriosBusquedaFactura criteriosBusquedaFactura)
    {
      List<Venta> lista;
      if (HayNumeroFactura(criteriosBusquedaFactura))
        lista = _gestionFacturaService.BuscarVentaPorNumero(criteriosBusquedaFactura.Numero);
      else
      {
        lista = _gestionFacturaService.BuscarVentaPorClienteYFecha(criteriosBusquedaFactura.Cliente,
                                                                          criteriosBusquedaFactura.Fecha);
      }
      var buscarFacturaViewModel = new BuscarFacturaViewModel { ListaBuscarFacturas = CrearListaBuscarFacturas(lista) };

      return buscarFacturaViewModel;
    }

    #endregion

    #region Metodos Privados

    private bool HayNumeroFactura(CriteriosBusquedaFactura criteriosBusquedaFactura)
    {
      return !string.IsNullOrWhiteSpace(criteriosBusquedaFactura.Numero);
    }

    private BindingList<BuscarFacturaListItemModel> CrearListaBuscarFacturas(IEnumerable<Venta> listaFacturas)
    {
      var listaBuscarFacturas = new BindingList<BuscarFacturaListItemModel>();

      foreach (var facturaListItemModel in listaFacturas.Select(factura => new BuscarFacturaListItemModel
      {
        FacturaId = factura.Id,
        NumeroFactura = factura.Numero,
        Fecha = factura.Fecha,
        Cliente = factura.Cliente != null ? factura.Cliente.Nombre : string.Empty,
        Empleado = factura.Empleado != null ? factura.Empleado.Nombre : string.Empty,
        Subtotal = factura.BaseImponible,
        ImporteIva = factura.BaseImponible.HasValue && factura.PorcentajeIva.HasValue ? (factura.BaseImponible * (decimal)factura.PorcentajeIva) / 100 : 0,
        ImporteTotal = factura.BaseImponible.HasValue && factura.PorcentajeIva.HasValue ? factura.BaseImponible + ((factura.BaseImponible * (decimal)factura.PorcentajeIva) / 100) : 0
      }))
      {
        listaBuscarFacturas.Add(facturaListItemModel);
      }
      return listaBuscarFacturas;
    }

    #endregion
  }
}
