﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPoblaciones;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarCliente;
using GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Clientes
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormCliente)]
  [RegisterInMediator(MediatorEnumerator.FormVehiculos)]
  public class ClientePresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private ClienteViewModel _model;
    private readonly ClienteViewModelFactory _modelFactory;

    private ClienteViewModelReadOnly _modelReadOnly;
    private readonly ClienteViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarClientePresenter _buscarClientePresenter;

    private readonly IGestionClienteService _gestionClienteService;
    private readonly IGestionPoblacionService _gestionPoblacionService;
    private readonly IGestionVehiculoService _gestionVehiculoService;

    private bool _alGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura;
    private bool _alGuardarElClienteOSeleccionarloAñadirloALaRecepcion;

    #endregion

    #region Constructores

    public ClientePresenter(ClienteViewModelFactory modelFactory,
                            ClienteViewModelReadOnlyFactory modelReadOnlyFactory,
                            IMediatorManager mediatorManager,
                            BuscarClientePresenter buscarClientePresenter,
                            IGestionClienteService gestionClienteService,
                            IGestionPoblacionService gestionPoblacionService, 
                            IGestionVehiculoService gestionVehiculoService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _buscarClientePresenter = buscarClientePresenter;
      _gestionClienteService = gestionClienteService;
      _gestionPoblacionService = gestionPoblacionService;
      _gestionVehiculoService = gestionVehiculoService;
    }

    #endregion

    #region Propiedades

    public ClienteViewModel Model
    {
      get { return _model; }
    }

    public ClienteViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarClienteListItemModel> ListaBuscarClienteViewModel
    {
      get { return _buscarClientePresenter.Model.ListaBuscarClientes; }
    }

    public bool AlGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura
    {
      get { return _alGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura; }
    }
    public bool AlGuardarElClienteOSeleccionarloAñadirloALaRecepcion
    {
      get { return _alGuardarElClienteOSeleccionarloAñadirloALaRecepcion; }
    }
    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.IdProvincia).Do(x =>
                                                   {
                                                     ModelReadOnly.SetPoblaciones(_gestionPoblacionService.BuscarListaPoblacionPorIdProvincia(Model.IdProvincia));
                                                     NotifyPropertyChanged<ClientePresenter>(t => t.ModelReadOnly);
                                                   });
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateClienteViewModelReadOnly();
      NotifyPropertyChanged<ClientePresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Pone el flag _alGuardarElClienteOSeleccionarloAñadirloAlPresupuesto = true, de forma
    /// que cuando se guarde un nuevo cliente o se seleccione uno existente, se envíe un 
    /// mensaje al presenter del presupuesto/factura para añadir el cliente al presupuesto.
    /// </summary>
    [MediatorMessageSink(MediatorMessages.AlGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura)]
    public void SetAlGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura()
    {
      _alGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura = true;
    }
    /// <summary>
    /// Pone el flag _alGuardarElClienteOSeleccionarloAñadirloALaRecepcion = true, de forma
    /// que cuando se guarde un nuevo cliente o se seleccione uno existente, se envíe un 
    /// mensaje al presenter de la recepción del vehículo para añadir el cliente a la misma.
    /// </summary>
    [MediatorMessageSink(MediatorMessages.AlGuardarElClienteOSeleccionarloAñadirloALaRecepcion)]
    public void SetAlGuardarElClienteOSeleccionarloAñadirloALaRecepcion()
    {
      _alGuardarElClienteOSeleccionarloAñadirloALaRecepcion = true;
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevoCliente()
    {
      _model = _modelFactory.CrearClienteViewModel();
      NotifyPropertyChanged<ClientePresenter>(t => t.Model);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaCliente()
    {
      _model = _modelFactory.CrearClienteViewModel();
      NotifyPropertyChanged<ClientePresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de clientes
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarCliente()
    {
      _buscarClientePresenter.Buscar(_model.Nombre, _model.PrimerApellido, _model.SegundoApellido);
      NotifyPropertyChanged<ClientePresenter>(t => t.ListaBuscarClienteViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarCliente()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.ClientePresenter_EliminarCliente_, Resources.ClientePresenter_EliminarCliente_Eliminar_cliente, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionClienteService.RemoveCliente(_model.Id);
          NuevoCliente();
          MessageBox.Show(Resources.ClientePresenter_EliminarCliente_Cliente_eliminado_con_éxito,
                          Resources.ClientePresenter_EliminarCliente_Eliminar_cliente, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.ClientePresenter_EliminarCliente_No_se_puede_eliminar_el_cliente_porque_todavía_no_se_ha_guardado_nada,
                          Resources.ClientePresenter_EliminarCliente_Eliminar_cliente, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarCliente()
    {
      Guid idCliente;
      if (!_model.HasBeenPersisted())
      {
        _model.Id = idCliente = Guid.NewGuid();
        _gestionClienteService.AddCliente(idCliente, _model.Nombre, _model.PrimerApellido, _model.SegundoApellido,
                                          _model.CorreoElectronico, _model.CuentaBancaria, _model.Direccion,
                                          _model.ListaTelefonos, _model.NifCif, _model.NombreComercial,
                                          _model.Observaciones, _model.FormaDePago, _model.IdCodigoPostal);
      }
      else
      {
        idCliente = _model.Id;
        _gestionClienteService.ChangeCliente(idCliente, _model.Nombre, _model.PrimerApellido, _model.SegundoApellido,
                                            _model.CorreoElectronico, _model.CuentaBancaria, _model.Direccion,
                                            _model.ListaTelefonos, _model.NifCif, _model.NombreComercial,
                                            _model.Observaciones, _model.FormaDePago, _model.IdCodigoPostal);
      }
      EliminarVehiculos(_model.ObtenerListaVehiculos(_model.VehiculosDelClienteEliminados));
      GuardarVehiculos(idCliente, _model.ObtenerListaVehiculos(_model.VehiculosDelCliente));
      MessageBox.Show(Resources.ClientePresenter_GuardarCliente_Cliente_guardado_con_éxito,
                      Resources.ClientePresenter_GuardarCliente_Información, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);

      SiEsNecesarioNotificarCliente();
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="clienteId">Id de la entidad a cargar</param>
    public void SetCliente(Guid clienteId)
    {
      _model = _modelFactory.CrearClienteViewModel(clienteId);
      NotifyPropertyChanged<ClientePresenter>(t => t.Model);

      SiEsNecesarioNotificarCliente();
    }


    /// <summary>
    /// Muestra la vista de vehículos para crear un nuevo vehículo
    /// </summary>
    public void MostrarFormVehiculoConNuevoVehiculo()
    {
      AlGuardarElVehiculoOSeleccionarloDesdeFormVehiculosAñadirloAlCliente();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormVehiculos));
    }

    /// <summary>
    /// Muestra la vista de vehículos con el vehículo -vehiculoId- cargado
    /// </summary>
    public void MostrarFormVehiculoConVehiculoSeleccionado(Guid vehiculoId)
    {
      AlGuardarElVehiculoOSeleccionarloDesdeFormVehiculosAñadirloAlCliente();
      var formWithParameter = new FormWithParameterAndMediator
      {
        FormType = typeof(FormVehiculos),
        Mediator = _mediatorManager[MediatorEnumerator.FormVehiculos],
        MediatorMessage = MediatorMessages.EditarVehiculo,
        Parameter = vehiculoId
      };
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenFormWithParameter, formWithParameter);
    }

    /// <summary>
    /// Añade un vehículo al cliente
    /// </summary>
    /// <param name="vehiculoViewModel">Vehículo a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirVehiculo, ParameterType = typeof(VehiculoViewModel))]
    public void AñadirVehiculoAlCliente(VehiculoViewModel vehiculoViewModel)
    {
      _model.AñadirVehiculo(vehiculoViewModel);
      NotifyPropertyChanged<ClientePresenter>(t => t.Model);
    }

    /// <summary>
    /// Elimina un vehículo al cliente
    /// </summary>
    /// <param name="vehiculoViewModel">Vehículo a eliminar.</param>
    public void EliminarVehiculoAlCliente(ClienteViewModel.VehiculoViewModel vehiculoViewModel)
    {
      _model.EliminarVehiculo(vehiculoViewModel);
      NotifyPropertyChanged<ClientePresenter>(t => t.Model);
    }
    #endregion

    #region Metodos Privados

    /// <summary>
    /// En caso de ser necesario se notifica el cliente seleccionado o guardado a otras pantallas.
    /// </summary>
    private void SiEsNecesarioNotificarCliente()
    {
      if (_alGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura)
      {
        _mediatorManager[MediatorEnumerator.FormPresupuesto].NotifyColleagues(
          MediatorMessages.AñadirClienteAlPresupuesto, _model);
        _mediatorManager[MediatorEnumerator.FormFactura].NotifyColleagues(
          MediatorMessages.AñadirClienteALaFactura, _model);
      }
      if (_alGuardarElClienteOSeleccionarloAñadirloALaRecepcion)
      {
        _mediatorManager[MediatorEnumerator.FormRecepcionVehiculo].NotifyColleagues(
          MediatorMessages.AñadirClienteALaRecepcion, _model);
      }
    }

    /// <summary>
    /// Modifica los vehículos visualizados en la vista de forma que asocia a los mismos
    /// el cliente actualmente cargado
    /// </summary>
    /// <param name="idCliente">Id del cliente cargado en la vista.</param>
    /// <param name="listaVehiculos">Lista de vehículos visualizados en la vista.</param>
    private void GuardarVehiculos(Guid idCliente, IEnumerable<Vehiculo> listaVehiculos)
    {
      foreach (var vehiculo in listaVehiculos)
      {
        vehiculo.IdCliente = idCliente;
        _gestionVehiculoService.ChangeVehiculo(vehiculo);
      }
    }

    /// <summary>
    /// Modifica los vehículos que se han eliminado en la vista de forma que desasocia a los mismos
    /// del cliente actualmente cargado
    /// </summary>
    /// <param name="listaVehiculos">Lista de vehículos eliminados en la vista.</param>
    private void EliminarVehiculos(IEnumerable<Vehiculo> listaVehiculos)
    {
      foreach (var vehiculo in listaVehiculos)
      {
        vehiculo.IdCliente = Guid.Empty;
        _gestionVehiculoService.ChangeVehiculo(vehiculo);
      }
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los vehiculos
    /// que indique que el nuevo vehículo creado o seleccionado en dicho presenter se añada
    /// a la lista de vehículos del cliente
    /// </summary>
    private void AlGuardarElVehiculoOSeleccionarloDesdeFormVehiculosAñadirloAlCliente()
    {
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente = new Timer();
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente.Tick += TimerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente_Tick;
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente.Interval = 1000;
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente.Enabled = true;
    }
    Timer _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente = new Timer();
    private void TimerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlCliente.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormVehiculos].NotifyColleagues(MediatorMessages.AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista);
    }
    #endregion
  }
}
