﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Clientes
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista del cliente,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class ClienteViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<ClienteViewModel> _crearViewModel;
    private readonly IGestionClienteService _gestionClienteService;

    #endregion

    #region Constructors

    public ClienteViewModelFactory(Func<ClienteViewModel> crearViewModel, IGestionClienteService gestionClienteService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionClienteService = gestionClienteService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo ClienteViewModel vacío</returns>
    public ClienteViewModel CrearClienteViewModel()
    {
      var clienteViewModel = _crearViewModel();

      clienteViewModel.Id = Guid.Empty;
      clienteViewModel.Nombre = string.Empty;
      clienteViewModel.PrimerApellido = string.Empty;
      clienteViewModel.SegundoApellido = string.Empty;
      clienteViewModel.NifCif = string.Empty;
      clienteViewModel.Observaciones = string.Empty;
      clienteViewModel.ListaTelefonos = string.Empty;
      clienteViewModel.IdCodigoPostal = 0;
      clienteViewModel.CodigoPostal = string.Empty;
      //clienteViewModel.Provincia =
      clienteViewModel.IdPoblacion = 0;
      clienteViewModel.Direccion = string.Empty;
      clienteViewModel.CorreoElectronico = string.Empty;

      return clienteViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del cliente clienteId.
    /// </summary>
    /// <param name="clienteId">Id del cliente a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo ClienteViewModel con los datos del cliente con id=clienteId</returns>
    public ClienteViewModel CrearClienteViewModel(Guid clienteId)
    {
      var lista = _gestionClienteService.BuscarClientePorId(clienteId);

      var clienteViewModel = CrearClienteViewModelDesdeListaDeClientes(lista);

      return clienteViewModel;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Crea un cliente view model desde lista de clientes. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de clientes en un único cliente.
    /// </summary>
    /// <param name="lista">Lista de clientes.</param>
    /// <returns>Un modelo ClienteViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo ClienteViewModel vacío</returns>
    private ClienteViewModel CrearClienteViewModelDesdeListaDeClientes(IEnumerable<Cliente> lista)
    {
      var clienteViewModel = _crearViewModel();

      foreach (var cliente in lista)
      {
        clienteViewModel.Id = cliente.Id;
        clienteViewModel.Nombre = cliente.Nombre;
        clienteViewModel.PrimerApellido = cliente.PrimerApellido;
        clienteViewModel.SegundoApellido = cliente.SegundoApellido;
        clienteViewModel.NifCif = cliente.NifCif;
        clienteViewModel.Observaciones = cliente.Observaciones;
        clienteViewModel.ListaTelefonos = cliente.Telefonos;
        if (cliente.CodigoPostal != null)
        {
          clienteViewModel.IdCodigoPostal = cliente.CodigoPostal.Id;
          clienteViewModel.CodigoPostal = cliente.CodigoPostal.Codigo;
          clienteViewModel.IdProvincia = cliente.CodigoPostal.Poblacion.IdProvincia;
          clienteViewModel.IdPoblacion = cliente.CodigoPostal.IdPoblacion;
        }
        clienteViewModel.Direccion = cliente.Direccion;
        clienteViewModel.CorreoElectronico = cliente.CorreoElectronico;

        clienteViewModel.FormaDePago = cliente.FormaDePago;
        clienteViewModel.CuentaBancaria = cliente.CuentaBancaria;
        clienteViewModel.NombreComercial = cliente.NombreComercial;

        clienteViewModel.SetVehiculosDelCliente(ObtenerVehiculosDelCliente(cliente));

        return clienteViewModel;
      }

      return clienteViewModel;
    }

    /// <summary>
    /// Obtiene todos los vehículos del cliente.
    /// </summary>
    /// <param name="cliente">Cliente del cual se desean obtener los vehículos.</param>
    /// <returns>Lista de vehículos del cliente como modelo de vista</returns>
    private BindingList<ClienteViewModel.VehiculoViewModel> ObtenerVehiculosDelCliente(Cliente cliente)
    {
      var listaVehiculosDelCliente = new BindingList<ClienteViewModel.VehiculoViewModel>();
      foreach (var vehiculoCliente in cliente.Vehiculos.Select(vehiculo => new ClienteViewModel.VehiculoViewModel
                                                                             {
                                                                               Id = vehiculo.Id,
                                                                               Matricula = vehiculo.Matricula,
                                                                               Marca = vehiculo.Marca,
                                                                               Modelo = vehiculo.Modelo,
                                                                               Año = vehiculo.Fecha,
                                                                               Color = vehiculo.Color,
                                                                               Bastidor = vehiculo.Bastidor
                                                                             }))
      {
        listaVehiculosDelCliente.Add(vehiculoCliente);
      }
      return listaVehiculosDelCliente;
    }

    #endregion
  }
}
