﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;

namespace GestionConcesionarios.Presentacion.CasosDeUso.FinalizarReparaciones
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de finalizar reparaciones,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class FinalizarReparacionesViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<FinalizarReparacionesViewModel> _crearViewModel;
    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructors

    public FinalizarReparacionesViewModelFactory(Func<FinalizarReparacionesViewModel> crearViewModel, IGestionOrdenDeReparacionService gestionOrdenDeReparacionService, InformacionUsuario informacionUsuario)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
      _informacionUsuario = informacionUsuario;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo FinalizarReparacionesViewModel vacío</returns>
    public FinalizarReparacionesViewModel CrearFinalizarReparacionesViewModel()
    {
      var finalizarReparacionesViewModel = _crearViewModel();

      finalizarReparacionesViewModel.IdOrdenDeReparacion = Guid.Empty;
      finalizarReparacionesViewModel.Empleado = _informacionUsuario.Usuario;
      finalizarReparacionesViewModel.Matricula = string.Empty;

      return finalizarReparacionesViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos de la ordenDeReparacion ordenDeReparacionId.
    /// </summary>
    /// <param name="ordenDeReparacionId">Id de la ordenDeReparacion a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo FinalizarReparacionesViewModel con los datos de la ordenDeReparacion con id=ordenDeReparacionId</returns>
    public FinalizarReparacionesViewModel CrearFinalizarReparacionesViewModel(Guid ordenDeReparacionId)
    {
      var lista = _gestionOrdenDeReparacionService.BuscarOrdenReparacionPorId(ordenDeReparacionId);

      var finalizarReparacionesViewModel = CrearFinalizarReparacionesViewModelDesdeListaDeOrdenesDeReparacion(lista);

      return finalizarReparacionesViewModel;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Crea un finalizar reparaciones view model desde lista de ordenDeReparaciones. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de ordenesDeReparacion en una única ordenDeReparacion.
    /// </summary>
    /// <param name="lista">Lista de ordenDeReparacions.</param>
    /// <returns>Un modelo FinalizarReparacionesViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo FinalizarReparacionesViewModel vacío</returns>
    private FinalizarReparacionesViewModel CrearFinalizarReparacionesViewModelDesdeListaDeOrdenesDeReparacion(IEnumerable<OrdenReparacion> lista)
    {
      var finalizarReparacionesViewModel = _crearViewModel();

      foreach (var ordenDeReparacion in lista)
      {
        finalizarReparacionesViewModel.IdOrdenDeReparacion = ordenDeReparacion.Id;
        finalizarReparacionesViewModel.Empleado = ordenDeReparacion.Empleado.Nombre;
        finalizarReparacionesViewModel.Matricula = ordenDeReparacion.Vehiculo.Matricula;

        finalizarReparacionesViewModel.SetLineasDeFinalizarReparaciones(ObtenerLineasDeFinalizarReparacionesDeLaOrdenDeReparacion(ordenDeReparacion));

        return finalizarReparacionesViewModel;
      }
      return finalizarReparacionesViewModel;
    }

    /// <summary>
    /// Obtiene todas las lineas de finalizar reparaciones de la ordenDeReparacion.
    /// </summary>
    /// <param name="ordenDeReparacion">OrdenDeReparacion del cual se desean obtener las lineas de finalizar reparaciones.</param>
    /// <returns>Lista de linea finalizar reparaciones como modelo de vista</returns>
    private BindingList<FinalizarReparacionesViewModel.LineaFinalizarReparacionesViewModel> ObtenerLineasDeFinalizarReparacionesDeLaOrdenDeReparacion(OrdenReparacion ordenDeReparacion)
    {
      var listaLineaFinalizarReparacionesViewModel = new BindingList<FinalizarReparacionesViewModel.LineaFinalizarReparacionesViewModel>();

      // Obtener manos de obra
      foreach (var lineaManoDeObraOrdenDeReparacion in ordenDeReparacion.Reparacion.SelectMany(reparacion => (from contenidoReparacion in reparacion.ContenidoReparacion
                                                                                                              where contenidoReparacion.ManoDeObra != null
                                                                                                              select new FinalizarReparacionesViewModel.LineaFinalizarReparacionesViewModel
                                                                                                              {
                                                                                                                IdReparacion = reparacion.Id,
                                                                                                                NumeroOrden = reparacion.OrdenReparacion.NumeroOrden,
                                                                                                                Matricula = reparacion.OrdenReparacion.Vehiculo.Matricula,
                                                                                                                Descripcion = contenidoReparacion.ManoDeObra.Descripcion,
                                                                                                                DuracionReal = reparacion.DuracionReal,
                                                                                                                DuracionSugerida = reparacion.DuracionSugerida,
                                                                                                                Estado = reparacion.Estado
                                                                                                              })))
      {
        listaLineaFinalizarReparacionesViewModel.Add(lineaManoDeObraOrdenDeReparacion);
      }

      // Obtener paquetes de servicio
      foreach (var lineaPaqueteDeServicioOrdenDeReparacion in ordenDeReparacion.Reparacion.SelectMany(reparacion => (from contenidoReparacion in reparacion.ContenidoReparacion
                                                                                                                     where contenidoReparacion.PaqueteDeServicio != null
                                                                                                                     select new FinalizarReparacionesViewModel.LineaFinalizarReparacionesViewModel
                                                                                                                     {
                                                                                                                       IdReparacion = reparacion.Id,
                                                                                                                       NumeroOrden = reparacion.OrdenReparacion.NumeroOrden,
                                                                                                                       Matricula = reparacion.OrdenReparacion.Vehiculo.Matricula,
                                                                                                                       Descripcion = contenidoReparacion.PaqueteDeServicio.Descripcion,
                                                                                                                       DuracionReal = reparacion.DuracionReal,
                                                                                                                       DuracionSugerida = reparacion.DuracionSugerida,
                                                                                                                       Estado = reparacion.Estado
                                                                                                                     })))
      {
        listaLineaFinalizarReparacionesViewModel.Add(lineaPaqueteDeServicioOrdenDeReparacion);
      }

      return listaLineaFinalizarReparacionesViewModel;
    }
    #endregion
  }
}
