﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionManosDeObra;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.ManosDeObra
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de la mano de obra,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class ManoDeObraViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<ManoDeObraViewModel> _crearViewModel;
    private readonly IGestionManoDeObraService _gestionManoDeObraService;

    #endregion

    #region Constructores

    public ManoDeObraViewModelFactory(Func<ManoDeObraViewModel> crearViewModel, IGestionManoDeObraService gestionManoDeObraService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionManoDeObraService = gestionManoDeObraService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo ManoDeObraViewModel vacío</returns>
    public ManoDeObraViewModel CrearManoDeObraViewModel()
    {
      var manoDeObraViewModel = _crearViewModel();

      manoDeObraViewModel.Id = Guid.Empty;
      manoDeObraViewModel.Descripcion = string.Empty;
      manoDeObraViewModel.Duracion = 0;
      manoDeObraViewModel.PrecioPorHora = 0;

      return manoDeObraViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos de la mano de obra manoDeObraId.
    /// </summary>
    /// <param name="manoDeObraId">Id de la mano de obra a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo ManoDeObraViewModel con los datos de la mano de obra con id=manoDeObraId</returns>
    public ManoDeObraViewModel CrearManoDeObraViewModel(Guid manoDeObraId)
    {
      var lista = _gestionManoDeObraService.BuscarManoDeObraPorId(manoDeObraId);

      var manoDeObraViewModel = CrearManoDeObraViewModelDesdeListaDeManosDeObra(lista);

      return manoDeObraViewModel;
    }

    #endregion

    #region Metodos Privados

    /// <summary>
    /// Crea una mano de obra view model desde la lista de manos de obra. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de manos de obra en una única mano de obra.
    /// </summary>
    /// <param name="lista">Lista de manos de obra.</param>
    /// <returns>Un modelo ManoDeObraViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo ManoDeObraViewModel vacío</returns>
    private ManoDeObraViewModel CrearManoDeObraViewModelDesdeListaDeManosDeObra(IEnumerable<ManoDeObra> lista)
    {
      var manoDeObraViewModel = _crearViewModel();

      foreach (var manoDeObra in lista)
      {
        manoDeObraViewModel.Id = manoDeObra.Id;
        manoDeObraViewModel.Descripcion = manoDeObra.Descripcion;
        manoDeObraViewModel.Duracion = manoDeObra.Duracion;
        manoDeObraViewModel.PrecioPorHora = manoDeObra.PrecioPorHora;

        return manoDeObraViewModel;
      }
      return manoDeObraViewModel;
    }

    #endregion
  }
}
