﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionOrdenesReparacion;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.CasosDeUso.RecepcionVehiculo;

namespace GestionConcesionarios.Presentacion.CasosDeUso.OrdenesDeReparacion
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de la orden de reparación,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class OrdenDeReparacionViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<OrdenDeReparacionViewModel> _crearViewModel;
    private readonly IGestionOrdenDeReparacionService _gestionOrdenDeReparacionService;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructores

    public OrdenDeReparacionViewModelFactory(Func<OrdenDeReparacionViewModel> crearViewModel,
                                             IGestionOrdenDeReparacionService gestionOrdenDeReparacionService,
                                             InformacionUsuario informacionUsuario)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionOrdenDeReparacionService = gestionOrdenDeReparacionService;
      _informacionUsuario = informacionUsuario;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo OrdenDeReparacionViewModel vacío</returns>
    public OrdenDeReparacionViewModel CrearOrdenDeReparacionViewModel()
    {
      var ordenDeReparacionViewModel = _crearViewModel();

      ordenDeReparacionViewModel.Id = Guid.Empty;
      ordenDeReparacionViewModel.NumeroOrden = string.Empty;
      ordenDeReparacionViewModel.FechaRecepcion = null;
      ordenDeReparacionViewModel.HoraRecepcion = null;
      ordenDeReparacionViewModel.FechaEntrega = null;
      ordenDeReparacionViewModel.HoraEntrega = null;
      ordenDeReparacionViewModel.IdEstado = 0;
      ordenDeReparacionViewModel.IdPersonalAsignado = Guid.Empty;
      ordenDeReparacionViewModel.SubTotal = 0;
      ordenDeReparacionViewModel.ImporteIva = 0;
      ordenDeReparacionViewModel.ImporteTotal = 0;
      ordenDeReparacionViewModel.IdCliente = Guid.Empty;
      ordenDeReparacionViewModel.NifCif = string.Empty;
      ordenDeReparacionViewModel.Nombre = string.Empty;
      ordenDeReparacionViewModel.ListaTelefonos = string.Empty;
      ordenDeReparacionViewModel.Observaciones = string.Empty;
      ordenDeReparacionViewModel.IdRecepcion = Guid.Empty;
      ordenDeReparacionViewModel.IdVehiculo = Guid.Empty;
      ordenDeReparacionViewModel.Matricula = string.Empty;
      ordenDeReparacionViewModel.Marca = string.Empty;
      ordenDeReparacionViewModel.Modelo = string.Empty;
      ordenDeReparacionViewModel.Color = string.Empty;
      ordenDeReparacionViewModel.Año = null;
      ordenDeReparacionViewModel.Bastidor = string.Empty;
      ordenDeReparacionViewModel.Prediagnostico = string.Empty;
      ordenDeReparacionViewModel.Diagnostico = string.Empty;

      return ordenDeReparacionViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos de la ordenDeReparacion ordenDeReparacionId.
    /// </summary>
    /// <param name="ordenDeReparacionId">Id de la ordenDeReparacion a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo OrdenDeReparacionViewModel con los datos de la ordenDeReparacion con id=ordenDeReparacionId</returns>
    public OrdenDeReparacionViewModel CrearOrdenDeReparacionViewModel(Guid ordenDeReparacionId)
    {
      var lista = _gestionOrdenDeReparacionService.BuscarOrdenReparacionPorId(ordenDeReparacionId);

      var ordenDeReparacionViewModel = CrearOrdenDeReparacionViewModelDesdeListaDeOrdenesDeReparacion(lista);

      return ordenDeReparacionViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos de la ordenDeReparacion con recepcionId.
    /// </summary>
    /// <param name="recepcionId">Id de la recepcion a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo OrdenDeReparacionViewModel con los datos de la ordenDeReparacion con idRecepcion=recepcionId</returns>
    public OrdenDeReparacionViewModel CrearOrdenDeReparacionViewModelPorRecepcionId(Guid recepcionId)
    {
      var lista = _gestionOrdenDeReparacionService.BuscarOrdenReparacionPorRecepcionId(recepcionId);

      var ordenDeReparacionViewModel = CrearOrdenDeReparacionViewModelDesdeListaDeOrdenesDeReparacion(lista);

      return ordenDeReparacionViewModel;
    }

    /// <summary>
    /// Crea un modelo de vista de la orden de reparación a partir del modelo de vista de la recepción
    /// de un vehíuculo.
    /// </summary>
    /// <param name="recepcionVehiculoViewModel">El modelo de vista de la recepcion del vehículo.</param>
    /// <returns>El modelo de vista de la orden de reparación</returns>
    public OrdenDeReparacionViewModel CrearOrdenDeReparacionViewModel(RecepcionVehiculoViewModel recepcionVehiculoViewModel)
    {
      var ordenDeReparacionViewModel = _crearViewModel();

      ordenDeReparacionViewModel.NumeroOrden = CrearNumeroDeOrden();
      ordenDeReparacionViewModel.FechaEntrega = recepcionVehiculoViewModel.FechaHoraEstimadaEntrega;
      ordenDeReparacionViewModel.IdEstado = 0;
      ordenDeReparacionViewModel.IdPersonalAsignado = recepcionVehiculoViewModel.IdPersonalAsignado;
      ordenDeReparacionViewModel.SubTotal = recepcionVehiculoViewModel.SubTotal;
      ordenDeReparacionViewModel.ImporteIva = recepcionVehiculoViewModel.ImporteTotal;
      ordenDeReparacionViewModel.Prediagnostico = recepcionVehiculoViewModel.PreDiagnostico;
      ordenDeReparacionViewModel.IdRecepcion = recepcionVehiculoViewModel.Id;
      ordenDeReparacionViewModel.IdVehiculo = recepcionVehiculoViewModel.IdVehiculo;

      return ordenDeReparacionViewModel;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Crea un ordenDeReparacion view model desde lista de ordenDeReparacions. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de ordenDeReparacions en un único ordenDeReparacion.
    /// </summary>
    /// <param name="lista">Lista de ordenDeReparacions.</param>
    /// <returns>Un modelo OrdenDeReparacionViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo OrdenDeReparacionViewModel vacío</returns>
    private OrdenDeReparacionViewModel CrearOrdenDeReparacionViewModelDesdeListaDeOrdenesDeReparacion(IEnumerable<OrdenReparacion> lista)
    {
      var ordenDeReparacionViewModel = _crearViewModel();

      foreach (var ordenDeReparacion in lista)
      {
        ordenDeReparacionViewModel.Id = ordenDeReparacion.Id;
        ordenDeReparacionViewModel.NumeroOrden = ordenDeReparacion.NumeroOrden;
        ordenDeReparacionViewModel.FechaRecepcion = ordenDeReparacion.Recepcion.FechaHoraRecepcion;
        ordenDeReparacionViewModel.HoraRecepcion = ordenDeReparacion.Recepcion.FechaHoraRecepcion;
        ordenDeReparacionViewModel.FechaEntrega = ordenDeReparacion.FechaHoraEntrega;
        ordenDeReparacionViewModel.HoraEntrega = ordenDeReparacion.FechaHoraEntrega;
        ordenDeReparacionViewModel.IdEstado = ordenDeReparacion.IdEstado;
        ordenDeReparacionViewModel.IdPersonalAsignado = ordenDeReparacion.IdPersonalAsignado;
        ordenDeReparacionViewModel.SubTotal = ordenDeReparacion.SubTotal;
        ordenDeReparacionViewModel.ImporteIva = ordenDeReparacion.ImporteIva;
        ordenDeReparacionViewModel.ImporteTotal = ordenDeReparacion.SubTotal + ordenDeReparacion.ImporteIva;
        ordenDeReparacionViewModel.IdCliente = ordenDeReparacion.Recepcion.IdCliente;
        ordenDeReparacionViewModel.NifCif = ordenDeReparacion.Recepcion.Cliente.NifCif;
        ordenDeReparacionViewModel.Nombre = ordenDeReparacion.Recepcion.Cliente.Nombre;
        ordenDeReparacionViewModel.ListaTelefonos = ordenDeReparacion.Recepcion.Cliente.Telefonos;
        ordenDeReparacionViewModel.Observaciones = ordenDeReparacion.Recepcion.Cliente.Observaciones;
        ordenDeReparacionViewModel.IdRecepcion = ordenDeReparacion.Recepcion.Id;
        ordenDeReparacionViewModel.IdVehiculo = ordenDeReparacion.IdVehiculo;
        ordenDeReparacionViewModel.Matricula = ordenDeReparacion.Vehiculo.Matricula;
        ordenDeReparacionViewModel.Marca = ordenDeReparacion.Vehiculo.Marca;
        ordenDeReparacionViewModel.Modelo = ordenDeReparacion.Vehiculo.Modelo;
        ordenDeReparacionViewModel.Color = ordenDeReparacion.Vehiculo.Color;
        ordenDeReparacionViewModel.Año = ordenDeReparacion.Vehiculo.Fecha;
        ordenDeReparacionViewModel.Bastidor = ordenDeReparacion.Vehiculo.Bastidor;
        ordenDeReparacionViewModel.Prediagnostico = ordenDeReparacion.Recepcion.Prediagnostico;
        ordenDeReparacionViewModel.Diagnostico = ordenDeReparacion.Diagnostico;

        ordenDeReparacionViewModel.SetLineasArticuloDeLaOrdenDeReparacion(ObtenerLineasArticuloDeLaOrdenDeReparacion(ordenDeReparacion));
        ordenDeReparacionViewModel.SetLineasManoDeObraDeLaOrdenDeReparacion(ObtenerLineasManoDeObraDeLaOrdenDeReparacion(ordenDeReparacion));
        ordenDeReparacionViewModel.SetLineasPaqueteDeServicioDeLaOrdenDeReparacion(ObtenerLineasPaqueteDeServicioDeLaOrdenDeReparacion(ordenDeReparacion));

        return ordenDeReparacionViewModel;
      }

      return ordenDeReparacionViewModel;
    }

    /// <summary>
    /// Obtiene todos los artículos de la ordenDeReparacion.
    /// </summary>
    /// <param name="ordenDeReparacion">OrdenDeReparacion del cual se desean obtener los artículos.</param>
    /// <returns>Lista de artículos de la ordenDeReparacion como modelo de vista</returns>
    private BindingList<OrdenDeReparacionViewModel.LineaArticuloViewModel> ObtenerLineasArticuloDeLaOrdenDeReparacion(OrdenReparacion ordenDeReparacion)
    {
      var listaLineaArticuloDeLaOrdenDeReparacion = new BindingList<OrdenDeReparacionViewModel.LineaArticuloViewModel>();

      foreach (var lineaArticuloOrdenDeReparacion in ordenDeReparacion.Reparacion.SelectMany(reparacion => (from contenidoReparacion in reparacion.ContenidoReparacion
                                                                                                            where contenidoReparacion.Articulo != null
                                                                                                            select new OrdenDeReparacionViewModel.LineaArticuloViewModel
                                                                                                                     {
                                                                                                                       Id = reparacion.Id,
                                                                                                                       Descripcion = contenidoReparacion.Articulo.Descripcion,
                                                                                                                       Cantidad = contenidoReparacion.Cantidad,
                                                                                                                       PrecioUnidad = contenidoReparacion.Articulo.CosteActual,
                                                                                                                       ImporteLinea = contenidoReparacion.Articulo.CosteActual *
                                                                                                                                      contenidoReparacion.Cantidad
                                                                                                                     })))
      {
        listaLineaArticuloDeLaOrdenDeReparacion.Add(lineaArticuloOrdenDeReparacion);
      }
      return listaLineaArticuloDeLaOrdenDeReparacion;
    }

    /// <summary>
    /// Obtiene todos las manos de obra de la ordenDeReparacion.
    /// </summary>
    /// <param name="ordenDeReparacion">OrdenDeReparacion del cual se desean obtener las manos de obra.</param>
    /// <returns>Lista de manos de obra de la ordenDeReparacion como modelo de vista</returns>
    private BindingList<OrdenDeReparacionViewModel.LineaManoDeObraViewModel> ObtenerLineasManoDeObraDeLaOrdenDeReparacion(OrdenReparacion ordenDeReparacion)
    {
      var listaLineaManoDeObraDeLaOrdenDeReparacion = new BindingList<OrdenDeReparacionViewModel.LineaManoDeObraViewModel>();

      foreach (var lineaManoDeObraOrdenDeReparacion in ordenDeReparacion.Reparacion.SelectMany(reparacion => (from contenidoReparacion in reparacion.ContenidoReparacion
                                                                                                              where contenidoReparacion.ManoDeObra != null
                                                                                                              select new OrdenDeReparacionViewModel.LineaManoDeObraViewModel
                                                                                                                       {
                                                                                                                         Id = reparacion.Id,
                                                                                                                         Descripcion = contenidoReparacion.ManoDeObra.Descripcion,
                                                                                                                         Cantidad = contenidoReparacion.Cantidad,
                                                                                                                         PrecioUnidad = contenidoReparacion.ManoDeObra.PrecioPorHora,
                                                                                                                         ImporteLinea = contenidoReparacion.ManoDeObra.PrecioPorHora *
                                                                                                                                        contenidoReparacion.Cantidad
                                                                                                                       })))
      {
        listaLineaManoDeObraDeLaOrdenDeReparacion.Add(lineaManoDeObraOrdenDeReparacion);
      }
      return listaLineaManoDeObraDeLaOrdenDeReparacion;
    }

    /// <summary>
    /// Obtiene todos los paquetes de servicio de la ordenDeReparacion.
    /// </summary>
    /// <param name="ordenDeReparacion">OrdenDeReparacion del cual se desean obtener los paquetes de servicio.</param>
    /// <returns>Lista de paquetes de servicio de la ordenDeReparacion como modelo de vista</returns>
    private BindingList<OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel> ObtenerLineasPaqueteDeServicioDeLaOrdenDeReparacion(OrdenReparacion ordenDeReparacion)
    {
      var listaLineaPaqueteDeServicioDeLaOrdenDeReparacion = new BindingList<OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel>();

      foreach (var lineaPaqueteDeServicioOrdenDeReparacion in ordenDeReparacion.Reparacion.SelectMany(reparacion => (from contenidoReparacion in reparacion.ContenidoReparacion
                                                                                                                     where contenidoReparacion.PaqueteDeServicio != null
                                                                                                                     select new OrdenDeReparacionViewModel.LineaPaqueteDeServicioViewModel
                                                                                                                              {
                                                                                                                                Id = reparacion.Id,
                                                                                                                                Descripcion = contenidoReparacion.PaqueteDeServicio.Descripcion,
                                                                                                                                Cantidad = contenidoReparacion.Cantidad,
                                                                                                                                PrecioUnidad = contenidoReparacion.PaqueteDeServicio.Precio,
                                                                                                                                ImporteLinea = contenidoReparacion.PaqueteDeServicio.Precio *
                                                                                                                                               contenidoReparacion.Cantidad
                                                                                                                              })))
      {
        listaLineaPaqueteDeServicioDeLaOrdenDeReparacion.Add(lineaPaqueteDeServicioOrdenDeReparacion);
      }
      return listaLineaPaqueteDeServicioDeLaOrdenDeReparacion;
    }

    /// <summary>
    /// Crea un numero de orden a partir del codigo de taller y la fecha/hora actual.
    /// </summary>
    /// <returns>Un nuevo número de orden</returns>
    private string CrearNumeroDeOrden()
    {
      return _informacionUsuario.CodigoTaller + DateTime.Now.Ticks.ToString(CultureInfo.InvariantCulture);
    }
    #endregion
  }
}
