﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPaquetesDeServicio;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.PaquetesDeServicio
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de los paquetes de servicio,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class PaqueteDeServicioViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<PaqueteDeServicioViewModel> _crearViewModel;
    private readonly IGestionPaqueteDeServicioService _gestionPaqueteDeServicioService;

    #endregion

    #region Constructores

    public PaqueteDeServicioViewModelFactory(Func<PaqueteDeServicioViewModel> crearViewModel, IGestionPaqueteDeServicioService gestionPaqueteDeServicioService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionPaqueteDeServicioService = gestionPaqueteDeServicioService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo PaqueteDeServicioViewModel vacío</returns>
    public PaqueteDeServicioViewModel CrearPaqueteDeServicioViewModel()
    {
      var paqueteDeServicioViewModel = _crearViewModel();

      paqueteDeServicioViewModel.Id = Guid.Empty;
      paqueteDeServicioViewModel.Descripcion = string.Empty;
      paqueteDeServicioViewModel.Precio = 0;
      paqueteDeServicioViewModel.SetLineasArticuloDelPaqueteDeServicio(new BindingList<PaqueteDeServicioViewModel.LineaArticuloViewModel>());
      paqueteDeServicioViewModel.SetLineasManoDeObraDelPaqueteDeServicio(new BindingList<PaqueteDeServicioViewModel.LineaManoDeObraViewModel>());

      return paqueteDeServicioViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del paquete de servicio paqueteDeServicioId.
    /// </summary>
    /// <param name="paqueteDeServicioId">Id del paquete de servicio a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo PaqueteDeServicioViewModel con los datos del paquete de servicio con id=paqueteDeServicioId</returns>
    public PaqueteDeServicioViewModel CrearPaqueteDeServicioViewModel(Guid paqueteDeServicioId)
    {
      var lista = _gestionPaqueteDeServicioService.BuscarPaqueteDeServicioPorId(paqueteDeServicioId);

      var paqueteDeServicioViewModel = CrearPaqueteDeServicioViewModelDesdeListaDePaquetesDeServicio(lista);

      return paqueteDeServicioViewModel;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Crea un paquete de servicio view model desde lista de paquetes de servicio. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de paquetes de servicio en un único paquete de servicio.
    /// </summary>
    /// <param name="lista">Lista de paquetes de servicio.</param>
    /// <returns>Un modelo PaqueteDeServicioViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo PaqueteDeServicioViewModel vacío</returns>
    private PaqueteDeServicioViewModel CrearPaqueteDeServicioViewModelDesdeListaDePaquetesDeServicio(IEnumerable<PaqueteDeServicio> lista)
    {
      var paqueteDeServicioViewModel = _crearViewModel();

      foreach (var paqueteDeServicio in lista)
      {
        paqueteDeServicioViewModel.Id = paqueteDeServicio.Id;
        paqueteDeServicioViewModel.Descripcion = paqueteDeServicio.Descripcion;

        paqueteDeServicioViewModel.SetLineasArticuloDelPaqueteDeServicio(ObtenerLineasArticuloDelPaqueteDeServicio(paqueteDeServicio));
        paqueteDeServicioViewModel.SetLineasManoDeObraDelPaqueteDeServicio(ObtenerLineasManoDeObraDelPaqueteDeServicio(paqueteDeServicio));

        paqueteDeServicioViewModel.Precio = paqueteDeServicio.Precio;

        return paqueteDeServicioViewModel;
      }
      return paqueteDeServicioViewModel;
    }

    /// <summary>
    /// Obtiene todos los artículos del paquete de servicio.
    /// </summary>
    /// <param name="paqueteDeServicio">PaqueteDeServicio del cual se desean obtener los artículos.</param>
    /// <returns>Lista de artículos del paquete de servicio como modelo de vista</returns>
    private BindingList<PaqueteDeServicioViewModel.LineaArticuloViewModel> ObtenerLineasArticuloDelPaqueteDeServicio(PaqueteDeServicio paqueteDeServicio)
    {
      var listaLineaArticuloDelPaqueteDeServicio = new BindingList<PaqueteDeServicioViewModel.LineaArticuloViewModel>();

      foreach (var lineaArticuloPaqueteDeServicio in from paquete in paqueteDeServicio.Paquete
                                                     where paquete.Articulo != null
                                                     select new PaqueteDeServicioViewModel.LineaArticuloViewModel
                                                              {
                                                                Id = paquete.Articulo.Id,
                                                                Descripcion = paquete.Articulo.Descripcion,
                                                                Cantidad = 1,
                                                                PrecioUnidad = paquete.Articulo.CosteActual,
                                                                ImporteLinea = paquete.Articulo.CosteActual
                                                              })
      {
        listaLineaArticuloDelPaqueteDeServicio.Add(lineaArticuloPaqueteDeServicio);
      }
      return listaLineaArticuloDelPaqueteDeServicio;
    }

    /// <summary>
    /// Obtiene todas las manos de obra del paquete de servicio.
    /// </summary>
    /// <param name="paqueteDeServicio">PaqueteDeServicio del cual se desean obtener las manos de obra.</param>
    /// <returns>Lista de manos de obra del paquete de servicio como modelo de vista</returns>
    private BindingList<PaqueteDeServicioViewModel.LineaManoDeObraViewModel> ObtenerLineasManoDeObraDelPaqueteDeServicio(PaqueteDeServicio paqueteDeServicio)
    {
      var listaLineaManoDeObraDelPaqueteDeServicio = new BindingList<PaqueteDeServicioViewModel.LineaManoDeObraViewModel>();

      foreach (var lineaManoDeObraPaqueteDeServicio in from paquete in paqueteDeServicio.Paquete
                                                     where paquete.ManoDeObra != null
                                                     select new PaqueteDeServicioViewModel.LineaManoDeObraViewModel
                                                              {
                                                       Id = paquete.ManoDeObra.Id,
                                                       Descripcion = paquete.ManoDeObra.Descripcion,
                                                       Cantidad = 1,
                                                       PrecioUnidad = paquete.ManoDeObra.PrecioPorHora,
                                                       ImporteLinea = paquete.ManoDeObra.PrecioPorHora
                                                     })
      {
        listaLineaManoDeObraDelPaqueteDeServicio.Add(lineaManoDeObraPaqueteDeServicio);
      }
      return listaLineaManoDeObraDelPaqueteDeServicio;
    }

    #endregion
  }
}
