﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPresupuestos;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPoblaciones;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarPresupuesto;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta;
using GestionConcesionarios.Presentacion.CasosDeUso.Ventas;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormPresupuesto)]
  public class PresupuestoPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private PresupuestoViewModel _model;
    private readonly PresupuestoViewModelFactory _modelFactory;

    private PresupuestoViewModelReadOnly _modelReadOnly;
    private readonly PresupuestoViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarPresupuestoPresenter _buscarPresupuestoPresenter;

    private readonly IGestionPresupuestoService _gestionPresupuestoService;
    private readonly IGestionPoblacionService _gestionPoblacionService;
    private readonly IGestionVehiculoVentaService _gestionVehiculoVentaService;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructores

    public PresupuestoPresenter(PresupuestoViewModelFactory modelFactory,
                                PresupuestoViewModelReadOnlyFactory modelReadOnlyFactory,
                                IMediatorManager mediatorManager,
                                BuscarPresupuestoPresenter buscarPresupuestoPresenter,
                                IGestionPresupuestoService gestionPresupuestoService,
                                IGestionPoblacionService gestionPoblacionService, 
                                IGestionVehiculoVentaService gestionVehiculoVentaService,
                                InformacionUsuario informacionUsuario)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _buscarPresupuestoPresenter = buscarPresupuestoPresenter;
      _gestionPresupuestoService = gestionPresupuestoService;
      _gestionPoblacionService = gestionPoblacionService;
      _gestionVehiculoVentaService = gestionVehiculoVentaService;
      _informacionUsuario = informacionUsuario;
      _model = _modelFactory.CrearPresupuestoViewModel();
    }

    #endregion

    #region Propiedades

    public PresupuestoViewModel Model
    {
      get { return _model; }
    }

    public PresupuestoViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarPresupuestoListItemModel> ListaBuscarPresupuestoViewModel
    {
      get { return _buscarPresupuestoPresenter.Model.ListaBuscarPresupuestos; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreatePresupuestoViewModelReadOnly();
      NotifyPropertyChanged<PresupuestoPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevoPresupuesto()
    {
      _model = _modelFactory.CrearPresupuestoViewModel();
      NotifyPropertyChanged<PresupuestoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaPresupuesto()
    {
      _model = _modelFactory.CrearPresupuestoViewModel();
      _model.Numero = string.Empty;
      NotifyPropertyChanged<PresupuestoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de presupuestos
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarPresupuesto()
    {
      _buscarPresupuestoPresenter.Buscar(_model.Numero, _model.Fecha, _model.IdCliente);
      NotifyPropertyChanged<PresupuestoPresenter>(t => t.ListaBuscarPresupuestoViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarPresupuesto()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.PresupuestoPresenter_EliminarPresupuesto_, Resources.PresupuestoPresenter_EliminarPresupuesto_Eliminar_presupuesto, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionPresupuestoService.RemovePresupuesto(_model.Id);
          NuevoPresupuesto();
          MessageBox.Show(Resources.PresupuestoPresenter_EliminarPresupuesto_Presupuesto_eliminado_con_éxito,
                          Resources.PresupuestoPresenter_EliminarPresupuesto_Eliminar_presupuesto, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.PresupuestoPresenter_EliminarPresupuesto_No_se_puede_eliminar_el_presupuesto_porque_todavía_no_se_ha_guardado_nada,
                          Resources.PresupuestoPresenter_EliminarPresupuesto_Eliminar_presupuesto, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarPresupuesto()
    {
      Guid idPresupuesto;
      if (!_model.HasBeenPersisted())
      {
        idPresupuesto = Guid.NewGuid();
        _gestionPresupuestoService.AddPresupuesto(idPresupuesto, 
                                                  _model.Numero,
                                                  _model.Fecha,
                                                  _model.BaseImponible,
                                                  _model.PorcentajeIva,
                                                  _model.IdEmpleado,
                                                  _model.IdCliente,
                                                  _informacionUsuario.CodigoTaller);
      }
      else
      {
        idPresupuesto = _model.Id;
        _gestionPresupuestoService.ChangePresupuesto(idPresupuesto,
                                                    _model.Numero,
                                                    _model.Fecha,
                                                    _model.BaseImponible,
                                                    _model.PorcentajeIva,
                                                    _model.IdEmpleado,
                                                    _model.IdCliente,
                                                    _informacionUsuario.CodigoTaller);
      }
      GuardarLineasPresupuesto(idPresupuesto, _model.ObtenerLineasDelPresupuesto(_model.LineasDelPresupuesto));
      MessageBox.Show(Resources.PresupuestoPresenter_GuardarPresupuesto_Presupuesto_guardado_con_éxito,
                      Resources.PresupuestoPresenter_GuardarPresupuesto_Guardar_Presupuesto, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="presupuestoId">Id de la entidad a cargar</param>
    public void SetPresupuesto(Guid presupuestoId)
    {
      _model = _modelFactory.CrearPresupuestoViewModel(presupuestoId);
      NotifyPropertyChanged<PresupuestoPresenter>(t => t.Model);
    }

    public void ConvertirPresupuestoEnFactura()
    {
      var formWithParameter = new FormWithParameterAndMediator
      {
        Context = null,
        FormType = typeof(FormFactura),
        Mediator = _mediatorManager[MediatorEnumerator.FormFactura],
        MediatorMessage = MediatorMessages.ConvertirPresupuestoEnFactura,
        Parameter = _model
      };
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenFormWithParameter, formWithParameter);
    }

    public void MostrarFormClienteConNuevoCliente()
    {
      AlGuardarElClienteOSeleccionarloDesdeFormClienteAñadirloAlPresupuesto();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormClientes));
    }

    /// <summary>
    /// Muestra la vista de vehículos para crear un nuevo vehículo
    /// </summary>
    public void MostrarFormVehiculoVentaConNuevoVehiculo()
    {
      AlGuardarElVehiculoVentaOSeleccionarloDesdeFormVehiculosVentaAñadirloAlPresupuesto();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormVehículoVenta));
    }
    /*
    /// <summary>
    /// Muestra la vista de vehículos con el vehículo -vehiculoId- cargado
    /// </summary>
    public void MostrarFormVehiculoVentaConVehiculoSeleccionado(Guid vehiculoId)
    {
      AlGuardarElVehiculoOSeleccionarloDesdeFormVehiculosAñadirloAlPresupuesto();
      var formWithParameter = new FormWithParameterAndMediator
      {        
        Context = null,
        FormType = typeof(FormVehiculos),
        Mediator = _mediatorManager[MediatorEnumerator.FormVehículoVenta],
        MediatorMessage = MediatorMessages.EditarVehiculo,
        Parameter = vehiculoId
      };
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenFormWithParameter, formWithParameter);
    }
         */ 

    /// <summary>
    /// Añade un vehículo de venta (línea) al presupuesto
    /// </summary>
    /// <param name="vehiculoVentaViewModel">Vehículo de venta a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirVehiculoVentaAlPresupuesto, ParameterType = typeof(VehiculoVentaViewModel))]
    public void AñadirVehiculoVentaAlPresupuesto(VehiculoVentaViewModel vehiculoVentaViewModel)
    {
      _model.AñadirLineaPresupuesto(vehiculoVentaViewModel);
      NotifyPropertyChanged<PresupuestoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Elimina un vehículo de venta (línea) al presupuesto
    /// </summary>
    /// <param name="lineaPresupuestoViewModel">Línea de presupuesto (Vehículo de venta) a eliminar.</param>
    public void EliminarVehiculoVentaAlPresupuesto(PresupuestoViewModel.LineaPresupuestoViewModel lineaPresupuestoViewModel)
    {
      _model.EliminarLineaPresupuesto(lineaPresupuestoViewModel);
      NotifyPropertyChanged<PresupuestoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade un cliente al presupuesto
    /// </summary>
    /// <param name="clienteViewModel">Cliente a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirClienteAlPresupuesto, ParameterType = typeof(ClienteViewModel))]
    public void AñadirClienteAlPresupuesto(ClienteViewModel clienteViewModel)
    {
      _model.AñadirCliente(clienteViewModel);
      NotifyPropertyChanged<ClientePresenter>(t => t.Model);
    }

    public void ImprimirPresupuesto()
    {
      MessageBox.Show("Esta funcionalidad no está implementada", "Imprimir");
    }
    #endregion

    #region Metodos Privados

    /// <summary>
    /// Modifica los vehículos de venta visualizados en la vista de forma que asocia a los mismos
    /// al presupuesto actualmente cargado
    /// </summary>
    /// <param name="idPresupuesto">Id del presupuesto cargado en la vista.</param>
    /// <param name="listaVehiculosVenta">Lista de vehículos de venta (líneas de presupuesto) visualizados en la vista.</param>
    private void GuardarLineasPresupuesto(Guid idPresupuesto, IEnumerable<VehiculoVenta> listaVehiculosVenta)
    {
      foreach (var vehiculoVenta in listaVehiculosVenta)
      {
        vehiculoVenta.IdPresupuesto = idPresupuesto;
        _gestionVehiculoVentaService.ChangeVehiculoVenta(vehiculoVenta);
      }
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los vehiculos de venta
    /// que indique que el nuevo vehículo creado o seleccionado en dicho presenter se añada
    /// a la lista de vehículos de venta (líneas) del presupuesto
    /// </summary>
    private void AlGuardarElVehiculoVentaOSeleccionarloDesdeFormVehiculosVentaAñadirloAlPresupuesto()
    {
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto = new Timer();
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto.Tick += TimerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto_Tick;
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto.Interval = 1000;
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto.Enabled = true;
    }
    Timer _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto = new Timer();
    private void TimerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlPresupuesto.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormVehiculosVenta].NotifyColleagues(MediatorMessages.AlGuardarElVehiculoDeVentaOSeleccionarloAñadirloAlPresupuestoOFactura);
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los clientes
    /// que indique que el nuevo cliente creado o seleccionado en dicho presenter se añada
    /// como cliente del presupuesto
    /// </summary>
    private void AlGuardarElClienteOSeleccionarloDesdeFormClienteAñadirloAlPresupuesto()
    {
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto = new Timer();
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto.Tick += TimerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto_Tick;
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto.Interval = 1000;
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto.Enabled = true;
    }
    Timer _timerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto = new Timer();
    private void TimerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlPresupuesto.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormCliente].NotifyColleagues(MediatorMessages.AlGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura);
    }
    #endregion
  }
}
