﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;
using GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista de los presupuestos
  /// </summary>
  public class PresupuestoViewModel : ModelBase, IPersistentModel
  {
    #region Modelos Auxiliares
    public class LineaPresupuestoViewModel : ModelBase
    {
      public Guid IdVehiculoVenta { get; set; }
      public string Codigo { get; set; }
      public string Marca { get; set; }
      public string Modelo { get; set; }
      public decimal? Precio { get; set; }
    }

    #endregion

    #region Atributos

    private readonly IGestionVehiculoVentaService _gestionVehiculoVentaService;
    private BindingList<LineaPresupuestoViewModel> _lineasDelPresupuesto = new BindingList<LineaPresupuestoViewModel>();

    #endregion

    #region Constructores
    public PresupuestoViewModel(IGestionVehiculoVentaService gestionVehiculoVentaService)
    {
      _gestionVehiculoVentaService = gestionVehiculoVentaService;
    }

    #endregion

    #region Propiedades

    public virtual Guid Id { get; set; }
    public virtual string Numero { get; set; }
    public virtual DateTime? Fecha { get; set; }
    public virtual decimal BaseImponible { get; set; }
    public virtual decimal PorcentajeIva { get; set; }
    public virtual decimal ImporteIva { get; set; }
    public virtual decimal ImporteTotal { get; set; }
    public virtual Guid? IdEmpleado { get; set; }
    public virtual Guid? IdCliente { get; set; }
    public virtual string NombreCliente { get; set; }
    public virtual string DireccionCliente { get; set; }
    public virtual string TelefonosCliente { get; set; }
    public virtual string CodigoTaller { get; set; }

    public BindingList<LineaPresupuestoViewModel> LineasDelPresupuesto { get { return _lineasDelPresupuesto; } }

    #endregion

    #region Métodos Públicos

    public List<VehiculoVenta> ObtenerLineasDelPresupuesto(IEnumerable<LineaPresupuestoViewModel> lineasDelPresupuestoViewModel)
    {
      return
        lineasDelPresupuestoViewModel.Select(
          lineaPresupuestoViewModel => _gestionVehiculoVentaService.BuscarVehiculoVentaPorId(lineaPresupuestoViewModel.IdVehiculoVenta).First()).ToList();
    }

    public void SetLineasDelPresupuesto(BindingList<LineaPresupuestoViewModel> lineasDelPresupuesto)
    {
      _lineasDelPresupuesto = lineasDelPresupuesto;
    }

    public void EliminarLineaPresupuesto(LineaPresupuestoViewModel lineaPresupuestoViewModel)
    {
      LineasDelPresupuesto.Remove(lineaPresupuestoViewModel);
      ActualizarImportes();
    }

    public void AñadirLineaPresupuesto(VehiculoVentaViewModel lineaPresupuestoViewModel)
    {
      var nuevaLineaPresupuesto = new LineaPresupuestoViewModel
                            {
                              IdVehiculoVenta = lineaPresupuestoViewModel.Id,
                              Codigo = lineaPresupuestoViewModel.Codigo,
                              Marca = lineaPresupuestoViewModel.Marca,
                              Modelo = lineaPresupuestoViewModel.Modelo,
                              Precio = lineaPresupuestoViewModel.Precio
                            };
      _lineasDelPresupuesto.Add(nuevaLineaPresupuesto);
      ActualizarImportes();
    }

    public void AñadirCliente(ClienteViewModel clienteViewModel)
    {
      IdCliente = clienteViewModel.Id;
      NombreCliente = new StringBuilder(clienteViewModel.Nombre).Append(" ")
                                .Append(clienteViewModel.PrimerApellido).Append(" ")
                                .Append(clienteViewModel.SegundoApellido).ToString();
      DireccionCliente = clienteViewModel.Direccion;
      TelefonosCliente = clienteViewModel.ListaTelefonos;
    }

    /// <summary>
    /// Determina si el modelo [ha estado persistido].
    /// </summary>
    /// <returns>
    ///   <c>true</c> Si [ha estado persistido]; de otra forma, <c>false</c>.
    /// </returns>
    public bool HasBeenPersisted()
    {
      return !Id.Equals(Guid.Empty);
    }
    #endregion

    #region Métodos privados

    public void ActualizarImportes()
    {
      var baseImponible =
        _lineasDelPresupuesto.Sum(lineaPresupuestoViewModel => lineaPresupuestoViewModel.Precio ?? 0);

      BaseImponible = baseImponible;
      ImporteIva = (baseImponible * PorcentajeIva) / 100;
      ImporteTotal = baseImponible + ImporteIva;
    }

    #endregion
  }
}
