﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPresupuestos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de presupuestos,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class PresupuestoViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<PresupuestoViewModel> _crearViewModel;
    private readonly IGestionPresupuestoService _gestionPresupuestoService;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructors

    public PresupuestoViewModelFactory(Func<PresupuestoViewModel> crearViewModel, IGestionPresupuestoService gestionPresupuestoService, InformacionUsuario informacionUsuario)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionPresupuestoService = gestionPresupuestoService;
      _informacionUsuario = informacionUsuario;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo PresupuestoViewModel vacío</returns>
    public PresupuestoViewModel CrearPresupuestoViewModel()
    {
      var presupuestoViewModel = _crearViewModel();

      presupuestoViewModel.Id = Guid.Empty;
      presupuestoViewModel.Numero = CrearNumeroDePresupuesto();
      presupuestoViewModel.Fecha = null;
      presupuestoViewModel.BaseImponible = 0;
      presupuestoViewModel.PorcentajeIva = InformacionUsuario.PorcentajeIva;
      presupuestoViewModel.ImporteIva = 0;
      presupuestoViewModel.ImporteTotal = 0;
      presupuestoViewModel.IdEmpleado = Guid.Empty;
      presupuestoViewModel.IdCliente = Guid.Empty;
      presupuestoViewModel.NombreCliente = string.Empty;
      presupuestoViewModel.DireccionCliente = string.Empty;
      presupuestoViewModel.TelefonosCliente = string.Empty;
      presupuestoViewModel.CodigoTaller = string.Empty;

      presupuestoViewModel.SetLineasDelPresupuesto(new BindingList<PresupuestoViewModel.LineaPresupuestoViewModel>());

      return presupuestoViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del presupuesto presupuestoId.
    /// </summary>
    /// <param name="presupuestoId">Id del presupuesto a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo PresupuestoViewModel con los datos del presupuesto con id=presupuestoId</returns>
    public PresupuestoViewModel CrearPresupuestoViewModel(Guid presupuestoId)
    {
      var lista = _gestionPresupuestoService.BuscarPresupuestoPorId(presupuestoId);

      var presupuestoViewModel = CrearPresupuestoViewModelDesdeListaDePresupuestos(lista);
      presupuestoViewModel.ActualizarImportes();

      return presupuestoViewModel;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Crea un presupuesto view model desde lista de presupuestos. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de presupustos en un único presupuesto.
    /// </summary>
    /// <param name="lista">Lista de presupuestos.</param>
    /// <returns>Un modelo PresupuestoViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo PresupuestoViewModel vacío</returns>
    private PresupuestoViewModel CrearPresupuestoViewModelDesdeListaDePresupuestos(IEnumerable<Presupuesto> lista)
    {
      var presupuestoViewModel = _crearViewModel();

      foreach (var presupuesto in lista)
      {
        presupuestoViewModel.Id = presupuesto.Id;
        presupuestoViewModel.Numero = presupuesto.Numero;
        presupuestoViewModel.Fecha = presupuesto.Fecha;
        if (presupuesto.BaseImponible != null) presupuestoViewModel.BaseImponible = presupuesto.BaseImponible.Value;
        if (presupuesto.PorcentajeIva != null) presupuestoViewModel.PorcentajeIva = presupuesto.PorcentajeIva.Value;
        presupuestoViewModel.IdEmpleado = presupuesto.IdEmpleado;
        presupuestoViewModel.IdCliente = presupuesto.IdCliente;
        presupuestoViewModel.NombreCliente = presupuesto.Cliente.Nombre;
        presupuestoViewModel.DireccionCliente = presupuesto.Cliente.Direccion;
        presupuestoViewModel.TelefonosCliente = presupuesto.Cliente.Telefonos;
        presupuestoViewModel.CodigoTaller = presupuesto.CodigoTaller;

        presupuestoViewModel.SetLineasDelPresupuesto(ObtenerLineasDelPresupuesto(presupuesto));

        return presupuestoViewModel;
      }
      return presupuestoViewModel;
    }

    /// <summary>
    /// Obtiene todas las líneas del presupuesto.
    /// </summary>
    /// <param name="presupuesto">Presupuesto del cual se desean obtener las líneas.</param>
    /// <returns>Líneas del presupuesto como modelo de vista</returns>
    private BindingList<PresupuestoViewModel.LineaPresupuestoViewModel> ObtenerLineasDelPresupuesto(Presupuesto presupuesto)
    {
      var lineasDelPresupuesto = new BindingList<PresupuestoViewModel.LineaPresupuestoViewModel>();
      foreach (var vehiculoVenta in presupuesto.VehiculoVenta.Select(vehiculo => new PresupuestoViewModel.LineaPresupuestoViewModel
                                                                             {
                                                                               IdVehiculoVenta = vehiculo.Id,
                                                                               Marca = vehiculo.Marca,
                                                                               Modelo = vehiculo.Modelo,
                                                                               Codigo = vehiculo.Codigo,
                                                                               Precio = vehiculo.Precio
                                                                             }))
      {
        lineasDelPresupuesto.Add(vehiculoVenta);
      }
      return lineasDelPresupuesto;
    }

    /// <summary>
    /// Crea un numero de presupuesto a partir del codigo de taller y la fecha/hora actual.
    /// </summary>
    /// <returns>Un nuevo número de presupuesto</returns>
    private string CrearNumeroDePresupuesto()
    {
      return _informacionUsuario.CodigoTaller +
             DateTime.Now.Year.ToString() +
             DateTime.Now.Month.ToString() +
             DateTime.Now.Day.ToString() +
             DateTime.Now.Hour.ToString() +
             DateTime.Now.Minute.ToString() +
             DateTime.Now.Second.ToString();
    }
    #endregion
  }
}
