﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionPoblaciones;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProveedores;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarProveedor;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Proveedores
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormProveedor)]
  public class ProveedorPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private ProveedorViewModel _model;
    private readonly ProveedorViewModelFactory _modelFactory;

    private ProveedorViewModelReadOnly _modelReadOnly;
    private readonly ProveedorViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarProveedorPresenter _buscarProveedorPresenter;
    private readonly IGestionProveedorService _gestionProveedorService;
    private readonly IGestionPoblacionService _gestionPoblacionService;
    private bool _alGuardarElProveedorOSeleccionarloAñadirloAlArticulo;

    #endregion

    #region Constructores

    public ProveedorPresenter(ProveedorViewModelFactory modelFactory,
                             IMediatorManager mediatorManager,
                             BuscarProveedorPresenter buscarProveedorPresenter,
                             IGestionProveedorService gestionProveedorService,
                             ProveedorViewModelReadOnlyFactory modelReadOnlyFactory,
                             IGestionPoblacionService gestionPoblacionService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _buscarProveedorPresenter = buscarProveedorPresenter;
      _gestionProveedorService = gestionProveedorService;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _gestionPoblacionService = gestionPoblacionService;
    }

    #endregion

    #region Propiedades

    public ProveedorViewModel Model
    {
      get { return _model; }
    }

    public ProveedorViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarProveedorListItemModel> ListaBuscarProveedorViewModel
    {
      get { return _buscarProveedorPresenter.Model.ListaBuscarProveedores; }
    }
    
    public bool AlGuardarElProveedorOSeleccionarloAñadirloAlArticulo
    {
      get { return _alGuardarElProveedorOSeleccionarloAñadirloAlArticulo; }
    }
    
    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.IdProvincia).Do(x =>
      {
        ModelReadOnly.SetPoblaciones(_gestionPoblacionService.BuscarListaPoblacionPorIdProvincia(Model.IdProvincia));
        NotifyPropertyChanged<ProveedorPresenter>(t => t.ModelReadOnly);
      });
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateProveedorViewModelReadOnly();
      NotifyPropertyChanged<ProveedorPresenter>(t => t.ModelReadOnly);
    }
    
    /// <summary>
    /// Pone el flag _alGuardarElProveedorOSeleccionarloAñadirloAlArticulo = true, de forma
    /// que cuando se guarde un nuevo vehículo o se seleccione uno existente, se envíe un 
    /// mensaje al presenter del cliente para añadir el vehículo al cliente.
    /// </summary>
    [MediatorMessageSink(MediatorMessages.AlGuardarElProveedorOSeleccionarloAñadirloAlArticulo)]
    public void SetAlGuardarElProveedorOSeleccionarloAñadirloAlArticulo()
    {
      _alGuardarElProveedorOSeleccionarloAñadirloAlArticulo = true;
    }
    
    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    //[MediatorMessageSink(MediatorMessages.NuevoProveedor)]
    public void NuevoProveedor()
    {
      _model = _modelFactory.CrearProveedorViewModel();
      NotifyPropertyChanged<ProveedorPresenter>(t => t.Model);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id pasado por parámetro.
    /// </summary>
    /// <param name="proveedorId">Id del proveedor con el se se creará un nuevo modelo de vista.</param>
    //[MediatorMessageSink(MediatorMessages.EditarProveedor, ParameterType = typeof(Guid))]
    public void EditarProveedor(Guid proveedorId)
    {
      SetProveedor(proveedorId);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaProveedor()
    {
      _model = _modelFactory.CrearProveedorViewModel();
      NotifyPropertyChanged<ProveedorPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de proveedores
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarProveedor()
    {
      _buscarProveedorPresenter.Buscar(_model.Codigo, _model.RazonSocial, _model.CifDni);
      NotifyPropertyChanged<ProveedorPresenter>(t => t.ListaBuscarProveedorViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarProveedor()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.ProveedorPresenter_EliminarProveedor_, Resources.ProveedorPresenter_EliminarProveedor_Eliminar_proveedor, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionProveedorService.RemoveProveedor(_model.Id);
          NuevoProveedor();
          MessageBox.Show(Resources.ProveedorPresenter_EliminarProveedor_Proveedor_eliminado_con_éxito,
                          Resources.ProveedorPresenter_EliminarProveedor_Eliminar_proveedor, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.ProveedorPresenter_EliminarProveedor_No_se_puede_eliminar_el_proveedor_porque_todavía_no_se_ha_guardado_nada,
                          Resources.ProveedorPresenter_EliminarProveedor_Eliminar_proveedor, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    /// <remarks>
    /// Si el flag _alGuardarElProveedorOSeleccionarloAñadirloAlCliente = True, se envía un mensaje
    /// al presenter del cliente para añadir el vehículo al cliente
    /// </remarks>
    public void GuardarProveedor()
    {
      if (!_model.HasBeenPersisted())
        _gestionProveedorService.AddProveedor(Guid.NewGuid(), _model.Codigo, _model.RazonSocial, _model.Direccion,
                                              _model.IdCodigoPostal, _model.CifDni,
                                              _model.Telefonos, _model.Observaciones, _model.CorreoElectronico,
                                              _model.PaginaWeb);
      else
      {
        _gestionProveedorService.ChangeProveedor(_model.Id, _model.Codigo, _model.RazonSocial, _model.Direccion,
                                              _model.IdCodigoPostal, _model.CifDni,
                                              _model.Telefonos, _model.Observaciones, _model.CorreoElectronico,
                                              _model.PaginaWeb);
      }
      MessageBox.Show(Resources.ProveedorPresenter_GuardarProveedor_Proveedor_guardado_con_éxito,
                      Resources.ProveedorPresenter_GuardarProveedor_Guardar_proveedor, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
      
      if (_alGuardarElProveedorOSeleccionarloAñadirloAlArticulo)
        _mediatorManager[MediatorEnumerator.FormArticulo].NotifyColleagues(MediatorMessages.AñadirProveedorAlArticulo, _model);
      
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="proveedorId">Id de la entidad a cargar</param>
    public void SetProveedor(Guid proveedorId)
    {
      _model = _modelFactory.CrearProveedorViewModel(proveedorId);
      NotifyPropertyChanged<ProveedorPresenter>(t => t.Model);
      
      if (_alGuardarElProveedorOSeleccionarloAñadirloAlArticulo)
        _mediatorManager[MediatorEnumerator.FormArticulo].NotifyColleagues(MediatorMessages.AñadirProveedorAlArticulo, _model);
      
    }
    #endregion
  }
}
