﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionProveedores;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Proveedores
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista del proveedor,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class ProveedorViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<ProveedorViewModel> _crearViewModel;
    private readonly IGestionProveedorService _gestionProveedorService;

    #endregion

    #region Constructores

    public ProveedorViewModelFactory(Func<ProveedorViewModel> crearViewModel, IGestionProveedorService gestionProveedorService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionProveedorService = gestionProveedorService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo ProveedorViewModel vacío</returns>
    public ProveedorViewModel CrearProveedorViewModel()
    {
      var proveedorViewModel = _crearViewModel();

      proveedorViewModel.Id = Guid.Empty;
      proveedorViewModel.Codigo = string.Empty;
      proveedorViewModel.RazonSocial = string.Empty;
      proveedorViewModel.Direccion = string.Empty;
      proveedorViewModel.IdCodigoPostal = 0;
      proveedorViewModel.CodigoPostal = string.Empty;
      proveedorViewModel.IdProvincia = string.Empty;
      proveedorViewModel.IdPoblacion = 0;
      proveedorViewModel.CifDni = string.Empty;
      proveedorViewModel.Observaciones = string.Empty;
      proveedorViewModel.Telefonos = string.Empty;
      proveedorViewModel.PaginaWeb = string.Empty;
      proveedorViewModel.CorreoElectronico = string.Empty;

      return proveedorViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del proveedor proveedorId.
    /// </summary>
    /// <param name="proveedorId">Id del proveedor a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo ProveedorViewModel con los datos del proveedor con id=proveedorId</returns>
    public ProveedorViewModel CrearProveedorViewModel(Guid proveedorId)
    {
      var lista = _gestionProveedorService.BuscarProveedorPorId(proveedorId);

      var proveedorViewModel = CrearProveedorViewModelDesdeListaDeProveedores(lista);

      return proveedorViewModel;
    }

    #endregion

    #region Metodos Privados

    /// <summary>
    /// Crea un vehículo view model desde lista de proveedores. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de vehículos en un único vehículo.
    /// </summary>
    /// <param name="lista">Lista de proveedores.</param>
    /// <returns>Un modelo ProveedorViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo ProveedorViewModel vacío</returns>
    private ProveedorViewModel CrearProveedorViewModelDesdeListaDeProveedores(IEnumerable<Proveedor> lista)
    {
      var proveedorViewModel = _crearViewModel();

      foreach (var proveedor in lista)
      {
        proveedorViewModel.Id = proveedor.Id;
        proveedorViewModel.Codigo = proveedor.Codigo;
        proveedorViewModel.RazonSocial = proveedor.RazonSocial;
        proveedorViewModel.Direccion = proveedor.Direccion;
        if (proveedor.CodigoPostal != null)
        {
          proveedorViewModel.IdCodigoPostal = proveedor.CodigoPostal.Id;
          proveedorViewModel.CodigoPostal = proveedor.CodigoPostal.Codigo;
          //proveedorViewModel.Provincia =
          proveedorViewModel.IdPoblacion = proveedor.CodigoPostal.IdPoblacion;
        }
        proveedorViewModel.CifDni = proveedor.CifDni;
        proveedorViewModel.Observaciones = proveedor.Observaciones;
        proveedorViewModel.Telefonos = proveedor.Telefonos;
        proveedorViewModel.PaginaWeb = proveedor.PaginaWeb;
        proveedorViewModel.CorreoElectronico = proveedor.CorreoElectronico;

        return proveedorViewModel;
      }
      return proveedorViewModel;
    }

    #endregion
  }
}
