﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionRecepciones;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.RecepcionVehiculo
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista del cliente,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class RecepcionVehiculoViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<RecepcionVehiculoViewModel> _crearViewModel;
    private readonly IGestionRecepcionVehiculoService _gestionRecepcionVehiculoService;

    #endregion

    #region Constructors

    public RecepcionVehiculoViewModelFactory(Func<RecepcionVehiculoViewModel> crearViewModel, IGestionRecepcionVehiculoService gestionRecepcionVehiculoService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionRecepcionVehiculoService = gestionRecepcionVehiculoService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo RecepcionVehiculoViewModel vacío</returns>
    public RecepcionVehiculoViewModel CrearRecepcionVehiculoViewModel()
    {
      var recepcionVehiculoViewModel = _crearViewModel();

      recepcionVehiculoViewModel.Id = Guid.Empty;
      recepcionVehiculoViewModel.IdEmpleadoRecibe = Guid.Empty;
      recepcionVehiculoViewModel.Fecha = null;

      recepcionVehiculoViewModel.IdCliente = Guid.Empty;
      recepcionVehiculoViewModel.NombreCliente = string.Empty;
      recepcionVehiculoViewModel.DireccionCliente = string.Empty;
      recepcionVehiculoViewModel.TelefonosCliente = string.Empty;
      recepcionVehiculoViewModel.CorreoElectronicoCliente = string.Empty;
      recepcionVehiculoViewModel.ObservacionesSobreElCliente = string.Empty;
      recepcionVehiculoViewModel.NombreContactoAdicional = string.Empty;
      recepcionVehiculoViewModel.TelefonoContactoAdicional = string.Empty;

      recepcionVehiculoViewModel.IdVehiculo = Guid.Empty;
      recepcionVehiculoViewModel.InformacionSobreLosVehiculosDelCliente = string.Empty;
      recepcionVehiculoViewModel.Marca = string.Empty;
      recepcionVehiculoViewModel.Modelo = string.Empty;
      recepcionVehiculoViewModel.Año = null;
      recepcionVehiculoViewModel.Color = string.Empty;

      recepcionVehiculoViewModel.Kilometraje = 0;
      recepcionVehiculoViewModel.PreDiagnostico = string.Empty;
      recepcionVehiculoViewModel.ContenidoAdicional = string.Empty;
      recepcionVehiculoViewModel.MedidorCombustible = 0;
      recepcionVehiculoViewModel.InformacionMedidorCombustible = string.Empty;

      recepcionVehiculoViewModel.FechaHoraEstimadaEntrega = null;
      recepcionVehiculoViewModel.ObservacionesRecepcion = string.Empty;
      recepcionVehiculoViewModel.IdPersonalAsignado = Guid.Empty;

      recepcionVehiculoViewModel.SubTotal = 0;
      recepcionVehiculoViewModel.ImporteIva = 0;
      recepcionVehiculoViewModel.ImporteTotal = 0;

      //recepcionVehiculoViewModel.LineasRecepcionVehiculo = new BindingList<RecepcionVehiculoViewModel.LineaRecepcionVehiculoViewModel>();

      return recepcionVehiculoViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos de la recepcion recepcionVehiculoId.
    /// </summary>
    /// <param name="recepcionVehiculoId">Id de la recepción a partir de la cual se creará el modelo.</param>
    /// <returns>Un modelo RecepcionVehiculoViewModel con los datos de la recepción con id=recepcionVehiculoId</returns>
    public RecepcionVehiculoViewModel CrearRecepcionVehiculoViewModel(Guid recepcionVehiculoId)
    {
      var lista = _gestionRecepcionVehiculoService.BuscarRecepcionPorId(recepcionVehiculoId);

      var recepcionVehiculoViewModel = CrearRecepcionVehiculoViewModelDesdeListaDeRecepcionesVehiculos(lista);

      return recepcionVehiculoViewModel;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Crea una recepción de vehículo view model desde lista de recepciones. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de recepciones en una única recepción.
    /// </summary>
    /// <param name="lista">Lista de recepciones.</param>
    /// <returns>Un modelo RecepcionVehiculoViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo RecepcionVehiculoViewModel vacío</returns>
    private RecepcionVehiculoViewModel CrearRecepcionVehiculoViewModelDesdeListaDeRecepcionesVehiculos(IEnumerable<Recepcion> lista)
    {
      var recepcionVehiculoViewModel = _crearViewModel();

      foreach (var recepcionVehiculo in lista)
      {
        recepcionVehiculoViewModel.Id = recepcionVehiculo.Id;
        recepcionVehiculoViewModel.IdEmpleadoRecibe = recepcionVehiculo.IdEmpleado;
        recepcionVehiculoViewModel.Fecha = recepcionVehiculo.FechaHoraRecepcion;

        recepcionVehiculoViewModel.IdCliente = recepcionVehiculo.IdCliente;
        recepcionVehiculoViewModel.NombreCliente = recepcionVehiculo.Cliente.Nombre;
        recepcionVehiculoViewModel.DireccionCliente = recepcionVehiculo.Cliente.Direccion;
        recepcionVehiculoViewModel.TelefonosCliente = recepcionVehiculo.Cliente.Telefonos;
        recepcionVehiculoViewModel.CorreoElectronicoCliente = recepcionVehiculo.Cliente.CorreoElectronico;
        recepcionVehiculoViewModel.ObservacionesSobreElCliente = recepcionVehiculo.ObservacionesSobreElCliente;
        recepcionVehiculoViewModel.NombreContactoAdicional = recepcionVehiculo.NombreContactoAdicional;
        recepcionVehiculoViewModel.TelefonoContactoAdicional = recepcionVehiculo.TelefonoContactoAdicional;

        recepcionVehiculoViewModel.IdVehiculo = recepcionVehiculo.IdVehiculo;
        recepcionVehiculoViewModel.Marca = recepcionVehiculo.Vehiculo.Marca;
        recepcionVehiculoViewModel.Modelo = recepcionVehiculo.Vehiculo.Modelo;
        recepcionVehiculoViewModel.Año = recepcionVehiculo.Vehiculo.Fecha;
        recepcionVehiculoViewModel.Color = recepcionVehiculo.Vehiculo.Color;
        recepcionVehiculoViewModel.SetVehiculosDelCliente(recepcionVehiculo.IdCliente);

        recepcionVehiculoViewModel.Kilometraje = recepcionVehiculo.Kilometraje;
        recepcionVehiculoViewModel.PreDiagnostico = recepcionVehiculo.Prediagnostico;
        recepcionVehiculoViewModel.ContenidoAdicional = recepcionVehiculo.ContenidoAdicional;
        recepcionVehiculoViewModel.MedidorCombustible = recepcionVehiculo.Combustible;
        recepcionVehiculoViewModel.SetInformacionMedidorCombustible();

        recepcionVehiculoViewModel.FechaHoraEstimadaEntrega = recepcionVehiculo.FechaHoraEntrega;
        recepcionVehiculoViewModel.ObservacionesRecepcion = recepcionVehiculo.Observaciones;
        recepcionVehiculoViewModel.IdPersonalAsignado = recepcionVehiculo.IdEmpleado;

        recepcionVehiculoViewModel.SubTotal = recepcionVehiculo.SubTotal;
        recepcionVehiculoViewModel.ImporteIva = recepcionVehiculo.ImporteIva;
        recepcionVehiculoViewModel.ImporteTotal = recepcionVehiculo.SubTotal + recepcionVehiculo.ImporteIva;

        //recepcionVehiculoViewModel.SetLineasRecepcionVehiculo(ObtenerLineasDeRecepcionDeLaRecepcionDelVehiculo(recepcionVehiculo));

        return recepcionVehiculoViewModel;
      }

      return recepcionVehiculoViewModel;
    }
    /*
    /// <summary>
    /// Obtiene todos los paquetes de servicio de la recepción del vehículo.
    /// </summary>
    /// <param name="recepcion">RecepcionVehiculo de la cual se desean obtener los paquetes.</param>
    /// <returns>Lineas de la recepción (paquetes de servicio) como modelo de vista</returns>
    private BindingList<RecepcionVehiculoViewModel.LineaRecepcionVehiculoViewModel> ObtenerLineasDeRecepcionDeLaRecepcionDelVehiculo(Recepcion recepcion)
    {
      var lineasRecepcionDeLaRecepcionVehiculo = new BindingList<RecepcionVehiculoViewModel.LineaRecepcionVehiculoViewModel>();
      foreach (var lineaRecepcionVehiculo in recepcion.PaquetesDeServicio.Select(paqueteDeServicio => new RecepcionVehiculoViewModel.LineaRecepcionVehiculoViewModel
                                                                                                        {
                                                                                                          IdPaqueteDeServicio = paqueteDeServicio.Id,
                                                                                                          Cantidad = 1,
                                                                                                          Descripcion = paqueteDeServicio.Descripcion,
                                                                                                          Precio = paqueteDeServicio.Precio,
                                                                                                          Importe = paqueteDeServicio.Precio
                                                                                                        }))
      {
        lineasRecepcionDeLaRecepcionVehiculo.Add(lineaRecepcionVehiculo);
      }
      return lineasRecepcionDeLaRecepcionVehiculo;
    }
    */
    #endregion
  }
}
