﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarVehiculo;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormVehiculos)]
  public class VehiculoPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private VehiculoViewModel _model;
    private readonly VehiculoViewModelFactory _modelFactory;

    private VehiculoViewModelReadOnly _modelReadOnly;
    private readonly VehiculoViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarVehiculoPresenter _buscarVehiculoPresenter;
    private readonly IGestionVehiculoService _gestionVehiculoService;
    private readonly IGestionModeloService _gestionModeloService;
    private bool _alGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista;

    #endregion

    #region Constructores

    public VehiculoPresenter(VehiculoViewModelFactory modelFactory,
                             IMediatorManager mediatorManager,
                             BuscarVehiculoPresenter buscarVehiculoPresenter,
                             IGestionVehiculoService gestionVehiculoService,
                             VehiculoViewModelReadOnlyFactory modelReadOnlyFactory,
                             IGestionModeloService gestionModeloService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _buscarVehiculoPresenter = buscarVehiculoPresenter;
      _gestionVehiculoService = gestionVehiculoService;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _gestionModeloService = gestionModeloService;
    }

    #endregion

    #region Propiedades

    public VehiculoViewModel Model
    {
      get { return _model; }
    }

    public VehiculoViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarVehiculoListItemModel> ListaBuscarVehiculoViewModel
    {
      get { return _buscarVehiculoPresenter.Model.ListaBuscarVehiculos; }
    }

    public bool AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista
    {
      get { return _alGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista; }
    }
    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.Marca).Do(x =>
                                             {
                                               ModelReadOnly.SetModelos(_gestionModeloService.BuscarListaModeloPorIdMarca(Model.Marca));
                                               NotifyPropertyChanged<VehiculoPresenter>(t => t.ModelReadOnly);
                                             });
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateVehiculoViewModelReadOnly();
      NotifyPropertyChanged<VehiculoPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Pone el flag _alGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista = true, de forma
    /// que cuando se guarde un nuevo vehículo o se seleccione uno existente, se envíe un 
    /// mensaje al presenter que este suscrito para que añada el vehículo.
    /// </summary>
    [MediatorMessageSink(MediatorMessages.AlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista)]
    public void SetAlGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista()
    {
      _alGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista = true;
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    [MediatorMessageSink(MediatorMessages.NuevoVehiculo)]
    public void NuevoVehiculo()
    {
      _model = _modelFactory.CrearVehiculoViewModel();
      NotifyPropertyChanged<VehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id pasado por parámetro.
    /// </summary>
    /// <param name="vehiculoId">Id del vehículo con el se se creará un nuevo modelo de vista.</param>
    [MediatorMessageSink(MediatorMessages.EditarVehiculo, ParameterType = typeof(Guid))]
    public void EditarVehiculo(Guid vehiculoId)
    {
      SetVehiculo(vehiculoId);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaVehiculo()
    {
      _model = _modelFactory.CrearVehiculoViewModel();
      NotifyPropertyChanged<VehiculoPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de vehículos
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarVehiculo()
    {
      _buscarVehiculoPresenter.Buscar(_model.Matricula, _model.Marca, _model.Modelo);
      NotifyPropertyChanged<VehiculoPresenter>(t => t.ListaBuscarVehiculoViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarVehiculo()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.VehiculoPresenter_EliminarVehiculo_, Resources.VehiculoPresenter_EliminarVehiculo_Eliminar_vehículo, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionVehiculoService.RemoveVehiculo(_model.Id);
          NuevoVehiculo();
          MessageBox.Show(Resources.VehiculoPresenter_EliminarVehiculo_Vehículo_eliminado_con_éxito,
                          Resources.VehiculoPresenter_EliminarVehiculo_Eliminar_vehículo, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.VehiculoPresenter_EliminarVehiculo_No_se_puede_eliminar_el_vehículo_porque_todavía_no_se_ha_guardado_nada,
                          Resources.VehiculoPresenter_EliminarVehiculo_Eliminar_vehículo, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    /// <remarks>
    /// Si el flag _alGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista = True, se envía un mensaje
    /// al presenter del cliente para añadir el vehículo al cliente
    /// </remarks>
    public void GuardarVehiculo()
    {
      if (!_model.HasBeenPersisted())
      {
        _model.Id = Guid.NewGuid();
        _gestionVehiculoService.AddVehiculo(_model.Id, _model.Matricula, _model.Marca, _model.Modelo, _model.Color, _model.Año,
                                            _model.Bastidor);
      }
      else
      {
        _gestionVehiculoService.ChangeVehiculo(_model.Id, _model.Matricula, _model.Marca, _model.Modelo, _model.Color,
                                               _model.Año,
                                               _model.Bastidor);
      }
      MessageBox.Show(Resources.VehiculoPresenter_GuardarVehiculo_Vehículo_guardado_con_éxito,
                      Resources.VehiculoPresenter_GuardarVehiculo_Guardar_vehículo, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);

      if (_alGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista)
        _mediatorManager[MediatorEnumerator.FormVehiculos].NotifyColleagues(MediatorMessages.AñadirVehiculo, _model);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="vehiculoId">Id de la entidad a cargar</param>
    public void SetVehiculo(Guid vehiculoId)
    {
      _model = _modelFactory.CrearVehiculoViewModel(vehiculoId);
      NotifyPropertyChanged<VehiculoPresenter>(t => t.Model);

      if (_alGuardarElVehiculoOSeleccionarloNotificarYCerrarLaVista)
        _mediatorManager[MediatorEnumerator.FormVehiculos].NotifyColleagues(MediatorMessages.AñadirVehiculo, _model);
    }
    #endregion
  }
}
