﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculos;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Vehiculos
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista del vehículo,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class VehiculoViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<VehiculoViewModel> _crearViewModel;
    private readonly IGestionVehiculoService _gestionVehiculoService;

    #endregion

    #region Constructores

    public VehiculoViewModelFactory(Func<VehiculoViewModel> crearViewModel, IGestionVehiculoService gestionVehiculoService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionVehiculoService = gestionVehiculoService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo VehiculoViewModel vacío</returns>
    public VehiculoViewModel CrearVehiculoViewModel()
    {
      var vehiculoViewModel = _crearViewModel();

      vehiculoViewModel.Id = Guid.Empty;
      vehiculoViewModel.Marca = string.Empty;
      vehiculoViewModel.Modelo = string.Empty;
      vehiculoViewModel.Matricula = string.Empty;
      vehiculoViewModel.Color = string.Empty;
      vehiculoViewModel.Año = null;
      vehiculoViewModel.Bastidor = string.Empty;
      
      return vehiculoViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del vehículo vehiculoId.
    /// </summary>
    /// <param name="vehiculoId">Id del vehículo a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo VehiculoViewModel con los datos del vehículo con id=vehiculoId</returns>
    public VehiculoViewModel CrearVehiculoViewModel(Guid vehiculoId)
    {
      var lista = _gestionVehiculoService.BuscarVehiculoPorId(vehiculoId);

      var vehiculoViewModel = CrearVehiculoViewModelDesdeListaDeVehiculos(lista);

      return vehiculoViewModel;
    }

    #endregion

    #region Metodos Privados

    /// <summary>
    /// Crea un vehículo view model desde lista de vehiculos. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de vehículos en un único vehículo.
    /// </summary>
    /// <param name="lista">Lista de vehiculos.</param>
    /// <returns>Un modelo VehiculoViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo VehiculoViewModel vacío</returns>
    private VehiculoViewModel CrearVehiculoViewModelDesdeListaDeVehiculos(IEnumerable<Vehiculo> lista)
    {
      var vehiculoViewModel = _crearViewModel();

      foreach (var vehiculo in lista)
      {
        vehiculoViewModel.Id = vehiculo.Id;
        vehiculoViewModel.Marca = vehiculo.Marca;
        vehiculoViewModel.Modelo = vehiculo.Modelo;
        vehiculoViewModel.Matricula = vehiculo.Matricula;
        vehiculoViewModel.Color = vehiculo.Color;
        vehiculoViewModel.Año = vehiculo.Fecha;
        vehiculoViewModel.Bastidor = vehiculo.Bastidor;

        return vehiculoViewModel;
      }
      return vehiculoViewModel;
    }

    #endregion
  }
}
