﻿using System;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionModelos;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarVehiculoVenta;
using GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos;
using GestionConcesionarios.Presentacion.CasosDeUso.Ventas;
using GestionConcesionarios.Presentacion.Infraestructura;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormVehiculosVenta)]
  public class VehiculoVentaPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private VehiculoVentaViewModel _model;
    private readonly VehiculoVentaViewModelFactory _modelFactory;

    private VehiculoVentaViewModelReadOnly _modelReadOnly;
    private readonly VehiculoVentaViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarVehiculoVentaPresenter _buscarVehiculoVentaPresenter;
    private readonly IGestionVehiculoVentaService _gestionVehiculoVentaService;
    private readonly IGestionModeloService _gestionModeloService;

    private bool _alGuardarElVehiculoVentaOSeleccionarloAñadirloAlPresupuestoOFactura;

    #endregion

    #region Constructores

    public VehiculoVentaPresenter(VehiculoVentaViewModelFactory modelFactory,
                             IMediatorManager mediatorManager,
                             BuscarVehiculoVentaPresenter buscarVehiculoVentaPresenter,
                             IGestionVehiculoVentaService gestionVehiculoVentaService,
                             VehiculoVentaViewModelReadOnlyFactory modelReadOnlyFactory,
                             IGestionModeloService gestionModeloService)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _buscarVehiculoVentaPresenter = buscarVehiculoVentaPresenter;
      _gestionVehiculoVentaService = gestionVehiculoVentaService;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _gestionModeloService = gestionModeloService;
    }

    #endregion

    #region Propiedades

    public VehiculoVentaViewModel Model
    {
      get { return _model; }
    }

    public VehiculoVentaViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarVehiculoVentaListItemModel> ListaBuscarVehiculoVentaViewModel
    {
      get { return _buscarVehiculoVentaPresenter.Model.ListaBuscarVehiculosVenta; }
    }

    public bool AlGuardarElVehiculoVentaOSeleccionarloAñadirloAlPresupuestoOFactura
    {
      get { return _alGuardarElVehiculoVentaOSeleccionarloAñadirloAlPresupuestoOFactura; }
    }
    #endregion

    #region Métodos Públicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      Model.OnChanged(c => Model.Marca).Do(x =>
      {
        ModelReadOnly.SetModelos(_gestionModeloService.BuscarListaModeloPorIdMarca(Model.Marca));
        NotifyPropertyChanged<VehiculoVentaPresenter>(t => t.ModelReadOnly);
      });
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateVehiculoVentaViewModelReadOnly();
      NotifyPropertyChanged<VehiculoVentaPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Pone el flag _alGuardarElVehiculoVentaOSeleccionarloAñadirloAlPresupuesto = true, de forma
    /// que cuando se guarde un nuevo vehiculo de venta o se seleccione uno existente, se envía un 
    /// mensaje al presenter del presupuesto/factura para añadir el vehículo de venta al presupuesto.
    /// </summary>
    [MediatorMessageSink(MediatorMessages.AlGuardarElVehiculoDeVentaOSeleccionarloAñadirloAlPresupuestoOFactura)]
    public void SetAlGuardarElVehiculoDeVentaOSeleccionarloAñadirloAlPresupuestoOFactura()
    {
      _alGuardarElVehiculoVentaOSeleccionarloAñadirloAlPresupuestoOFactura = true;
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    [MediatorMessageSink(MediatorMessages.NuevoVehiculoVenta)]
    public void NuevoVehiculoVenta()
    {
      _model = _modelFactory.CrearVehiculoVentaViewModel();
      NotifyPropertyChanged<VehiculoVentaPresenter>(t => t.Model);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id pasado por parámetro.
    /// </summary>
    /// <param name="vehiculoVentaId">Id del vehículo de venta con el se se creará un nuevo modelo de vista.</param>
    [MediatorMessageSink(MediatorMessages.EditarVehiculoVenta, ParameterType = typeof(Guid))]
    public void EditarVehiculoVenta(Guid vehiculoVentaId)
    {
      SetVehiculoVenta(vehiculoVentaId);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaVehiculoVenta()
    {
      _model = _modelFactory.CrearVehiculoVentaViewModel();
      NotifyPropertyChanged<VehiculoVentaPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de vehículos
    /// de venta que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarVehiculoVenta()
    {
      _buscarVehiculoVentaPresenter.Buscar(_model.Matricula, _model.Marca, _model.Modelo);
      NotifyPropertyChanged<VehiculoVentaPresenter>(t => t.ListaBuscarVehiculoVentaViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarVehiculoVenta()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.VehiculoPresenter_EliminarVehiculo_, Resources.VehiculoPresenter_EliminarVehiculo_Eliminar_vehículo, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionVehiculoVentaService.RemoveVehiculoVenta(_model.Id);
          NuevoVehiculoVenta();
          MessageBox.Show(Resources.VehiculoPresenter_EliminarVehiculo_Vehículo_eliminado_con_éxito,
                          Resources.VehiculoPresenter_EliminarVehiculo_Eliminar_vehículo, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.VehiculoPresenter_EliminarVehiculo_No_se_puede_eliminar_el_vehículo_porque_todavía_no_se_ha_guardado_nada,
                          Resources.VehiculoPresenter_EliminarVehiculo_Eliminar_vehículo, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarVehiculoVenta()
    {
      if (!_model.HasBeenPersisted())
      {
        _model.Id = Guid.NewGuid();
        _gestionVehiculoVentaService.AddVehiculoVenta(_model.Id,
                                                      _model.Codigo,
                                                      _model.FechaDeEntrada,
                                                      _model.Estado,
                                                      _model.Matricula,
                                                      _model.Marca,
                                                      _model.Modelo,
                                                      _model.Kilometros,
                                                      _model.Color,
                                                      _model.Tipo,
                                                      _model.FechaMatriculacion,
                                                      _model.Cilindrada,
                                                      _model.Potencia,
                                                      _model.Combustible,
                                                      _model.Bastidor,
                                                      _model.Itv,
                                                      _model.UltimaInspeccion,
                                                      _model.ProximaInspeccion,
                                                      _model.Observaciones,
                                                      _model.Precio,
                                                      _model.FechaInicioGarantia,
                                                      _model.MesesGarantia);
      }
      else
      {
        _gestionVehiculoVentaService.ChangeVehiculoVenta(_model.Id,
                                                        _model.Codigo,
                                                        _model.FechaDeEntrada,
                                                        _model.Estado,
                                                        _model.Matricula,
                                                        _model.Marca,
                                                        _model.Modelo,
                                                        _model.Kilometros,
                                                        _model.Color,
                                                        _model.Tipo,
                                                        _model.FechaMatriculacion,
                                                        _model.Cilindrada,
                                                        _model.Potencia,
                                                        _model.Combustible,
                                                        _model.Bastidor,
                                                        _model.Itv,
                                                        _model.UltimaInspeccion,
                                                        _model.ProximaInspeccion,
                                                        _model.Observaciones,
                                                        _model.Precio,
                                                        _model.FechaInicioGarantia,
                                                        _model.MesesGarantia);
      }
      MessageBox.Show(Resources.VehiculoPresenter_GuardarVehiculo_Vehículo_guardado_con_éxito,
                      Resources.VehiculoPresenter_GuardarVehiculo_Guardar_vehículo, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);

      SiEsNecesarioNotificarVehiculoDeVenta();
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="vehiculoVentaId">Id de la entidad a cargar</param>
    public void SetVehiculoVenta(Guid vehiculoVentaId)
    {
      _model = _modelFactory.CrearVehiculoVentaViewModel(vehiculoVentaId);
      NotifyPropertyChanged<VehiculoVentaPresenter>(t => t.Model);

      SiEsNecesarioNotificarVehiculoDeVenta();
    }

    /// <summary>
    /// Duplica el vehiculo de venta actualmente cargado, para crear uno nuevo a partir
    /// de los datos del cargado
    /// </summary>
    public void DuplicarVehiculoVenta()
    {
      _model = _modelFactory.CrearVehiculoVentaViewModel(_model);
      NotifyPropertyChanged<VehiculoVentaPresenter>(t => t.Model);

      MessageBox.Show(Resources.VehiculoVentaPresenter_DuplicarVehiculoVenta_Vehículo_de_venta_duplicado_con_éxito,
                      Resources.VehiculoVentaPresenter_DuplicarVehiculoVenta_Duplicar_vehículo_de_venta,
                      MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Crear un presupuesto para el vehiculo de venta.
    /// </summary>
    public void CrearPresupuestoParaVehiculoVenta()
    {
      var formWithParameter = new FormWithParameterAndMediator
      {
        Context = null,
        FormType = typeof(FormPresupuesto),
        Mediator = _mediatorManager[MediatorEnumerator.FormPresupuesto],
        MediatorMessage = MediatorMessages.AñadirVehiculoVentaAlPresupuesto,
        Parameter = _model
      };
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenFormWithParameter, formWithParameter);
    }

    /// <summary>
    /// Crear una venta para el vehiculo de venta.
    /// </summary>
    public void CrearVentaParaVehiculoVenta()
    {
      var formWithParameter = new FormWithParameterAndMediator
      {
        Context = null,
        FormType = typeof(FormFactura),
        Mediator = _mediatorManager[MediatorEnumerator.FormFactura],
        MediatorMessage = MediatorMessages.AñadirVehiculoVentaALaFactura,
        Parameter = _model
      };
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenFormWithParameter, formWithParameter);
    }
    #endregion

    #region Métodos Privados

    /// <summary>
    /// En caso de ser necesario se notifica el vehículo de venta seleccionado o 
    /// guardado a otras pantallas.
    /// </summary>
    private void SiEsNecesarioNotificarVehiculoDeVenta()
    {
      if (_alGuardarElVehiculoVentaOSeleccionarloAñadirloAlPresupuestoOFactura)
      {
        _mediatorManager[MediatorEnumerator.FormPresupuesto].NotifyColleagues(
          MediatorMessages.AñadirVehiculoVentaAlPresupuesto, _model);
        _mediatorManager[MediatorEnumerator.FormFactura].NotifyColleagues(
          MediatorMessages.AñadirVehiculoVentaALaFactura, _model);
      }
    }

    #endregion
  }
}
