﻿using System;
using System.Collections.Generic;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista del vehículo de venta,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class VehiculoVentaViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<VehiculoVentaViewModel> _crearViewModel;
    private readonly IGestionVehiculoVentaService _gestionVehiculoVentaService;

    #endregion

    #region Constructores

    public VehiculoVentaViewModelFactory(Func<VehiculoVentaViewModel> crearViewModel, IGestionVehiculoVentaService gestionVehiculoVentaService)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionVehiculoVentaService = gestionVehiculoVentaService;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo vehiculoVentaViewModel vacío</returns>
    public VehiculoVentaViewModel CrearVehiculoVentaViewModel()
    {
      var vehiculoVentaViewModel = _crearViewModel();

      vehiculoVentaViewModel.Id = Guid.Empty;
      vehiculoVentaViewModel.Marca = string.Empty;
      vehiculoVentaViewModel.Modelo = string.Empty;
      vehiculoVentaViewModel.Matricula = string.Empty;
      vehiculoVentaViewModel.Color = string.Empty;
      vehiculoVentaViewModel.Bastidor = string.Empty;
      vehiculoVentaViewModel.Codigo = string.Empty;
      vehiculoVentaViewModel.FechaDeEntrada = null;
      vehiculoVentaViewModel.Estado = 0;
      vehiculoVentaViewModel.Kilometros = 0;
      vehiculoVentaViewModel.Tipo = string.Empty;
      vehiculoVentaViewModel.FechaMatriculacion = null;
      vehiculoVentaViewModel.Cilindrada = 0;
      vehiculoVentaViewModel.Potencia = 0;
      vehiculoVentaViewModel.Combustible = string.Empty;
      vehiculoVentaViewModel.Itv = 0;
      vehiculoVentaViewModel.UltimaInspeccion = null;
      vehiculoVentaViewModel.ProximaInspeccion = null;
      vehiculoVentaViewModel.Observaciones = string.Empty;
      vehiculoVentaViewModel.Precio = 0;
      vehiculoVentaViewModel.FechaInicioGarantia = null;
      vehiculoVentaViewModel.MesesGarantia = 0;
      vehiculoVentaViewModel.FechaFinGarantia = null;
      vehiculoVentaViewModel.CodigoTaller = string.Empty;

      vehiculoVentaViewModel.IdVenta = Guid.Empty;
      vehiculoVentaViewModel.VendidoPor = string.Empty;
      vehiculoVentaViewModel.NumeroFactura = string.Empty;
      vehiculoVentaViewModel.FechaVenta = null;

      return vehiculoVentaViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del vehículo de venta vehiculoVentaId.
    /// </summary>
    /// <param name="vehiculoVentaId">Id del vehículo de venta a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo VehiculoVentaViewModel con los datos del vehículo de venta con id=vehiculoVentaId</returns>
    public VehiculoVentaViewModel CrearVehiculoVentaViewModel(Guid vehiculoVentaId)
    {
      var lista = _gestionVehiculoVentaService.BuscarVehiculoVentaPorId(vehiculoVentaId);

      var vehiculoVentaViewModel = CrearVehiculoVentaViewModelDesdeListaDeVehiculos(lista);

      return vehiculoVentaViewModel;
    }

    #endregion

    #region Metodos Privados

    /// <summary>
    /// Crea un vehículo de venta view model desde lista de vehiculos de venta. Es una función 
    /// puente para transformar la lista devuelta por el servicio de gestión de vehículos de venta
    /// en un único vehículo de venta.
    /// </summary>
    /// <param name="lista">Lista de vehiculos de venta.</param>
    /// <returns>Un modelo VehiculoVentaViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo VehiculoVentaViewModel vacío</returns>
    private VehiculoVentaViewModel CrearVehiculoVentaViewModelDesdeListaDeVehiculos(IEnumerable<VehiculoVenta> lista)
    {
      var vehiculoVentaViewModel = _crearViewModel();

      foreach (var vehiculoVenta in lista)
      {
        vehiculoVentaViewModel.Id = vehiculoVenta.Id;
        vehiculoVentaViewModel.Marca = vehiculoVenta.Marca;
        vehiculoVentaViewModel.Modelo = vehiculoVenta.Modelo;
        vehiculoVentaViewModel.Matricula = vehiculoVenta.Matricula;
        vehiculoVentaViewModel.Color = vehiculoVenta.Color;
        vehiculoVentaViewModel.Bastidor = vehiculoVenta.Bastidor;
        vehiculoVentaViewModel.Codigo = vehiculoVenta.Codigo;
        vehiculoVentaViewModel.FechaDeEntrada = vehiculoVenta.FechaDeEntrada;
        vehiculoVentaViewModel.Estado = vehiculoVenta.Estado;
        vehiculoVentaViewModel.Kilometros = vehiculoVenta.Kilometros;
        vehiculoVentaViewModel.Tipo = vehiculoVenta.Tipo;
        vehiculoVentaViewModel.FechaMatriculacion = vehiculoVenta.FechaMatriculacion;
        vehiculoVentaViewModel.Cilindrada = vehiculoVenta.Cilindrada;
        vehiculoVentaViewModel.Potencia = vehiculoVenta.Potencia;
        vehiculoVentaViewModel.Combustible = vehiculoVenta.Combustible;
        vehiculoVentaViewModel.Itv = vehiculoVenta.Itv;
        vehiculoVentaViewModel.UltimaInspeccion = vehiculoVenta.UltimaInspeccion;
        vehiculoVentaViewModel.ProximaInspeccion = vehiculoVenta.ProximaInspeccion;
        vehiculoVentaViewModel.Observaciones = vehiculoVenta.Observaciones;
        vehiculoVentaViewModel.Precio = vehiculoVenta.Precio;
        vehiculoVentaViewModel.FechaInicioGarantia = vehiculoVenta.FechaInicioGarantia;
        vehiculoVentaViewModel.MesesGarantia = vehiculoVenta.MesesGarantia;
        if (vehiculoVenta.MesesGarantia != null)
          if (vehiculoVenta.Venta!=null && vehiculoVenta.Venta.Fecha != null)
            vehiculoVentaViewModel.FechaFinGarantia = vehiculoVenta.Venta.Fecha.Value.AddMonths((int) vehiculoVenta.MesesGarantia);
        vehiculoVentaViewModel.CodigoTaller = vehiculoVenta.CodigoTaller;

        vehiculoVentaViewModel.IdVenta = vehiculoVenta.IdVenta;
        if (vehiculoVenta.Venta != null)
        {
          if (vehiculoVenta.Venta.Empleado != null)
            vehiculoVentaViewModel.VendidoPor = vehiculoVenta.Venta.Empleado.Nombre;
          vehiculoVentaViewModel.NumeroFactura = vehiculoVenta.Venta.Numero;
          vehiculoVentaViewModel.FechaVenta = vehiculoVenta.Venta.Fecha;
        }

        return vehiculoVentaViewModel;
      }
      return vehiculoVentaViewModel;
    }

    #endregion

    /// <summary>
    /// Crea un vehículo de venta view model a partir del view model pasado por parámetro
    /// </summary>
    /// <param name="vehiculoVenta">View model del vehículo de venta actualmente cargado</param>
    /// <returns>Una duplicación del modelo VehiculoVentaViewModel pasado por parámetro</returns>
    public VehiculoVentaViewModel CrearVehiculoVentaViewModel(VehiculoVentaViewModel vehiculoVenta)
    {
      var vehiculoVentaViewModel = _crearViewModel();

      vehiculoVentaViewModel.Id = Guid.Empty;
      vehiculoVentaViewModel.Codigo = string.Empty;
      vehiculoVentaViewModel.Marca = vehiculoVenta.Marca;
      vehiculoVentaViewModel.Modelo = vehiculoVenta.Modelo;
      vehiculoVentaViewModel.Matricula = string.Empty;
      vehiculoVentaViewModel.Color = vehiculoVenta.Color;
      vehiculoVentaViewModel.Bastidor = string.Empty;
      vehiculoVentaViewModel.FechaDeEntrada = vehiculoVenta.FechaDeEntrada;
      vehiculoVentaViewModel.Estado = vehiculoVenta.Estado;
      vehiculoVentaViewModel.Kilometros = 0;
      vehiculoVentaViewModel.Tipo = vehiculoVenta.Tipo;
      vehiculoVentaViewModel.FechaMatriculacion = null;
      vehiculoVentaViewModel.Cilindrada = vehiculoVenta.Cilindrada;
      vehiculoVentaViewModel.Potencia = vehiculoVenta.Potencia;
      vehiculoVentaViewModel.Combustible = vehiculoVenta.Combustible;
      vehiculoVentaViewModel.Itv = 0;
      vehiculoVentaViewModel.UltimaInspeccion = null;
      vehiculoVentaViewModel.ProximaInspeccion = null;
      vehiculoVentaViewModel.Observaciones = vehiculoVenta.Observaciones;
      vehiculoVentaViewModel.Precio = vehiculoVentaViewModel.Precio;
      vehiculoVentaViewModel.FechaInicioGarantia = null;
      vehiculoVentaViewModel.MesesGarantia = 0;
      //vehiculoVentaViewModel.FechaFinGarantia = null;
      vehiculoVentaViewModel.CodigoTaller = vehiculoVenta.CodigoTaller;

      vehiculoVentaViewModel.IdVenta = Guid.Empty;
      //vehiculoVentaViewModel.VendidoPor = string.Empty;
      //vehiculoVentaViewModel.NumeroFactura = string.Empty;
      //vehiculoVentaViewModel.FechaVenta = null;

      return vehiculoVentaViewModel;
    }
  }
}
