﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVentas;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.BuscarVenta;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos;
using GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Properties;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Ventas
{
  /// <summary>
  /// Esta clase implementa el comportamiento de la vista 
  /// </summary>
  [RegisterInMediator(MediatorEnumerator.FormFactura)]
  public class FacturaPresenter : PresenterWithMediatorBase
  {
    #region Atributos

    private FacturaViewModel _model;
    private readonly FacturaViewModelFactory _modelFactory;

    private FacturaViewModelReadOnly _modelReadOnly;
    private readonly FacturaViewModelReadOnlyFactory _modelReadOnlyFactory;

    private readonly BuscarFacturaPresenter _buscarFacturaPresenter;

    private readonly IGestionVentaService _gestionFacturaService;
    private readonly IGestionVehiculoVentaService _gestionVehiculoVentaService;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructores

    public FacturaPresenter(FacturaViewModelFactory modelFactory,
                                FacturaViewModelReadOnlyFactory modelReadOnlyFactory,
                                IMediatorManager mediatorManager,
                                BuscarFacturaPresenter buscarFacturaPresenter,
                                IGestionVentaService gestionFacturaService,
                                IGestionVehiculoVentaService gestionVehiculoVentaService,
                            InformacionUsuario informacionUsuario)
      : base(mediatorManager)
    {
      _modelFactory = modelFactory;
      _modelReadOnlyFactory = modelReadOnlyFactory;
      _buscarFacturaPresenter = buscarFacturaPresenter;
      _gestionFacturaService = gestionFacturaService;
      _gestionVehiculoVentaService = gestionVehiculoVentaService;
      _informacionUsuario = informacionUsuario;
      _model = _modelFactory.CrearFacturaViewModel();
    }

    #endregion

    #region Propiedades

    public FacturaViewModel Model
    {
      get { return _model; }
    }

    public FacturaViewModelReadOnly ModelReadOnly
    {
      get { return _modelReadOnly; }
    }

    public BindingList<BuscarFacturaListItemModel> ListaBuscarFacturaViewModel
    {
      get { return _buscarFacturaPresenter.Model.ListaBuscarFacturas; }
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Se suscriben acciones (comportamientos) a cambios en propiedades del modelo de vista.
    /// </summary>
    public void SubscribirAccionesACambiosEnElModelo()
    {
      
    }

    /// <summary>
    /// Pone el modelo de solo lectura (valores de listas desplegables, etc) y lo notifica
    /// </summary>
    public void SetModelReadOnly()
    {
      _modelReadOnly = _modelReadOnlyFactory.CreateFacturaViewModelReadOnly();
      NotifyPropertyChanged<FacturaPresenter>(t => t.ModelReadOnly);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista
    /// </summary>
    public void NuevaFactura()
    {
      _model = _modelFactory.CrearFacturaViewModel();
      NotifyPropertyChanged<FacturaPresenter>(t => t.Model);
    }

    /// <summary>
    /// Inicializa el modelo de vista para poder realizar una nueva búsqueda
    /// </summary>
    public void NuevaBusquedaFactura()
    {
      _model = _modelFactory.CrearFacturaViewModel();
      _model.Numero = string.Empty;
      NotifyPropertyChanged<FacturaPresenter>(t => t.Model);
    }

    /// <summary>
    /// Envía un mensaje al presenter de búsquedas para crear una lista de ventas
    /// que coincida con los criterios de búsqueda
    /// </summary>
    public void BuscarFactura()
    {
      _buscarFacturaPresenter.Buscar(_model.Numero, _model.Fecha, _model.IdCliente);
      NotifyPropertyChanged<FacturaPresenter>(t => t.ListaBuscarFacturaViewModel);
    }

    /// <summary>
    /// Elimina la entidad cargada en la vista, previa confirmación del usuario, siempre y cuando
    /// la misma ya haya sido persistida.
    /// </summary>
    public void EliminarFactura()
    {
      if (_model.HasBeenPersisted())
      {
        if (MessageBox.Show(Resources.FacturaPresenter_EliminarFactura_, Resources.FacturaPresenter_EliminarFactura_Eliminar_factura, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
        {
          _gestionFacturaService.RemoveVenta(_model.Id);
          NuevaFactura();
          MessageBox.Show(Resources.FacturaPresenter_EliminarFactura_Factura_eliminada_con_éxito,
                          Resources.FacturaPresenter_EliminarFactura_Eliminar_factura, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
        }
      }
      else
      {
        MessageBox.Show(Resources.FacturaPresenter_EliminarFactura_No_se_puede_eliminar_la_factura_porque_todavía_no_se_ha_guardado_nada,
                          Resources.FacturaPresenter_EliminarFactura_Eliminar_factura, MessageBoxButtons.OK,
                          MessageBoxIcon.Information);
      }
    }

    /// <summary>
    /// Se persiste la entidad del modelo de vista
    /// </summary>
    public void GuardarFactura()
    {
      Guid idFactura;
      if (!_model.HasBeenPersisted())
      {
        idFactura = Guid.NewGuid();
        _gestionFacturaService.AddVenta(idFactura, 
                                                  _model.Numero,
                                                  _model.Fecha,
                                                  _model.BaseImponible,
                                                  _model.PorcentajeIva,
                                                  _model.IdEmpleado,
                                                  _model.IdCliente,
                                                  _informacionUsuario.CodigoTaller,
                                                  _model.FormaDePago,
                                                  _model.CuentaBancaria);
      }
      else
      {
        idFactura = _model.Id;
        _gestionFacturaService.ChangeVenta(idFactura,
                                                    _model.Numero,
                                                    _model.Fecha,
                                                    _model.BaseImponible,
                                                    _model.PorcentajeIva,
                                                    _model.IdEmpleado,
                                                    _model.IdCliente,
                                                    _informacionUsuario.CodigoTaller,
                                                    _model.FormaDePago,
                                                    _model.CuentaBancaria);
      }
      GuardarLineasFactura(idFactura, _model.ObtenerLineasDeLaFactura(_model.LineasDeLaFactura));
      MessageBox.Show(Resources.FacturaPresenter_GuardarFactura_Factura_guardada_con_éxito,
                      Resources.FacturaPresenter_GuardarFactura_Guardar_Factura, MessageBoxButtons.OK,
                      MessageBoxIcon.Information);
    }

    /// <summary>
    /// Crea un nuevo modelo de vista a partir del Id de la entidad pasada por parámetro
    /// </summary>
    /// <param name="facturaId">Id de la entidad a cargar</param>
    public void SetFactura(Guid facturaId)
    {
      _model = _modelFactory.CrearFacturaViewModel(facturaId);
      NotifyPropertyChanged<FacturaPresenter>(t => t.Model);
    }

    [MediatorMessageSink(MediatorMessages.ConvertirPresupuestoEnFactura, ParameterType = typeof(PresupuestoViewModel))]
    private void ConvertirPresupuestoEnFactura(PresupuestoViewModel presupuestoViewModel)
    {
      _model = _modelFactory.CrearFacturaViewModel(presupuestoViewModel);
      NotifyPropertyChanged<FacturaPresenter>(t => t.Model);
    }

    public void MostrarFormClienteConNuevoCliente()
    {
      AlGuardarElClienteOSeleccionarloDesdeFormClienteAñadirloAlFactura();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormClientes));
    }

    /// <summary>
    /// Muestra la vista de vehículos para crear un nuevo vehículo
    /// </summary>
    public void MostrarFormVehiculoVentaConNuevoVehiculo()
    {
      AlGuardarElVehiculoVentaOSeleccionarloDesdeFormVehiculosVentaAñadirloAlFactura();
      _mediatorManager[MediatorEnumerator.FormManager].NotifyColleagues(MediatorMessages.OpenForm, typeof(FormVehículoVenta));
    }
    
    /// <summary>
    /// Añade un vehículo de venta (línea) a la venta
    /// </summary>
    /// <param name="vehiculoVentaViewModel">Vehículo de venta a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirVehiculoVentaALaFactura, ParameterType = typeof(VehiculoVentaViewModel))]
    public void AñadirVehiculoVentaALaFactura(VehiculoVentaViewModel vehiculoVentaViewModel)
    {
      _model.AñadirLineaFactura(vehiculoVentaViewModel);
      NotifyPropertyChanged<FacturaPresenter>(t => t.Model);
    }

    /// <summary>
    /// Elimina un vehículo de venta (línea) a la venta
    /// </summary>
    /// <param name="lineaFacturaViewModel">Línea de venta (Vehículo de venta) a eliminar.</param>
    public void EliminarVehiculoVentaALaFactura(FacturaViewModel.LineaFacturaViewModel lineaFacturaViewModel)
    {
      _model.EliminarLineaFactura(lineaFacturaViewModel);
      NotifyPropertyChanged<FacturaPresenter>(t => t.Model);
    }

    /// <summary>
    /// Añade un cliente a la venta
    /// </summary>
    /// <param name="clienteViewModel">Cliente a añadir.</param>
    [MediatorMessageSink(MediatorMessages.AñadirClienteALaFactura, ParameterType = typeof(ClienteViewModel))]
    public void AñadirClienteALaFactura(ClienteViewModel clienteViewModel)
    {
      _model.AñadirCliente(clienteViewModel);
      NotifyPropertyChanged<ClientePresenter>(t => t.Model);
    }

    public void ImprimirFactura()
    {
      MessageBox.Show("Esta funcionalidad no esta implementada", "Imprimir factura");
    }
    #endregion

    #region Metodos Privados

    /// <summary>
    /// Modifica los vehículos de venta visualizados en la vista de forma que asocia a los mismos
    /// a la venta actualmente cargada
    /// </summary>
    /// <param name="idFactura">Id de la venta cargada en la vista.</param>
    /// <param name="listaVehiculosVenta">Lista de vehículos de venta (líneas de venta) visualizados en la vista.</param>
    private void GuardarLineasFactura(Guid idFactura, IEnumerable<VehiculoVenta> listaVehiculosVenta)
    {
      foreach (var vehiculoVenta in listaVehiculosVenta)
      {
        vehiculoVenta.IdVenta = idFactura;
        _gestionVehiculoVentaService.ChangeVehiculoVenta(vehiculoVenta);
      }
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los vehiculos de venta
    /// que indique que el nuevo vehículo creado o seleccionado en dicho presenter se añada
    /// a la lista de vehículos de venta (líneas) de la venta
    /// </summary>
    private void AlGuardarElVehiculoVentaOSeleccionarloDesdeFormVehiculosVentaAñadirloAlFactura()
    {
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura = new Timer();
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura.Tick += TimerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura_Tick;
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura.Interval = 1000;
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura.Enabled = true;
    }
    Timer _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura = new Timer();
    private void TimerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElVehiculoOSeleccionarloAñadirloAlFactura.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormVehiculosVenta].NotifyColleagues(MediatorMessages.AlGuardarElVehiculoDeVentaOSeleccionarloAñadirloAlPresupuestoOFactura);
    }

    /// <summary>
    /// Creación de una llamada paralela para enviar un mensaje al presenter de los clientes
    /// que indique que el nuevo cliente creado o seleccionado en dicho presenter se añada
    /// como cliente de la venta
    /// </summary>
    private void AlGuardarElClienteOSeleccionarloDesdeFormClienteAñadirloAlFactura()
    {
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlFactura = new Timer();
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlFactura.Tick += TimerAlGuardarElClienteOSeleccionarloAñadirloAlFactura_Tick;
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlFactura.Interval = 1000;
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlFactura.Enabled = true;
    }
    Timer _timerAlGuardarElClienteOSeleccionarloAñadirloAlFactura = new Timer();
    private void TimerAlGuardarElClienteOSeleccionarloAñadirloAlFactura_Tick(object sender, EventArgs e)
    {
      _timerAlGuardarElClienteOSeleccionarloAñadirloAlFactura.Enabled = false;
      _mediatorManager[MediatorEnumerator.FormCliente].NotifyColleagues(MediatorMessages.AlGuardarElClienteOSeleccionarloAñadirloAlPresupuestoOFactura);
    }
    #endregion
  }
}
