﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVehiculosVenta;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Clientes;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos;
using GestionConcesionarios.Presentacion.CasosDeUso.VehiculosDeVenta;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Ventas
{
  /// <summary>
  /// Clase que representa la información a mostrar en la vista de los presupuestos
  /// </summary>
  public class FacturaViewModel : ModelBase, IPersistentModel
  {
    #region Modelos Auxiliares
    public class LineaFacturaViewModel : ModelBase
    {
      public Guid IdVehiculoVenta { get; set; }
      public string Codigo { get; set; }
      public string Marca { get; set; }
      public string Modelo { get; set; }
      public decimal? Precio { get; set; }
    }

    #endregion

    #region Atributos

    private readonly IGestionVehiculoVentaService _gestionVehiculoVentaService;
    private BindingList<LineaFacturaViewModel> _lineasDeLaFactura = new BindingList<LineaFacturaViewModel>();

    #endregion

    #region Constructores
    public FacturaViewModel(IGestionVehiculoVentaService gestionVehiculoVentaService)
    {
      _gestionVehiculoVentaService = gestionVehiculoVentaService;
    }

    #endregion

    #region Propiedades

    public virtual Guid Id { get; set; }
    public virtual string Numero { get; set; }
    public virtual DateTime? Fecha { get; set; }
    public virtual decimal BaseImponible { get; set; }
    public virtual decimal PorcentajeIva { get; set; }
    public virtual decimal ImporteIva { get; set; }
    public virtual decimal ImporteTotal { get; set; }
    public virtual Guid? IdEmpleado { get; set; }
    public virtual Guid? IdCliente { get; set; }
    public virtual string NombreCliente { get; set; }
    public virtual string DireccionCliente { get; set; }
    public virtual string TelefonosCliente { get; set; }
    public virtual string CodigoTaller { get; set; }
    public virtual string CuentaBancaria { get; set; }
    public virtual int? FormaDePago { get; set; }

    public BindingList<LineaFacturaViewModel> LineasDeLaFactura { get { return _lineasDeLaFactura; } }

    #endregion

    #region Métodos Públicos

    public List<VehiculoVenta> ObtenerLineasDeLaFactura(IEnumerable<LineaFacturaViewModel> lineasDeLaFacturaViewModel)
    {
      return
        lineasDeLaFacturaViewModel.Select(
          lineaFacturaViewModel => _gestionVehiculoVentaService.BuscarVehiculoVentaPorId(lineaFacturaViewModel.IdVehiculoVenta).First()).ToList();
    }

    public void SetLineasDeLaFactura(BindingList<LineaFacturaViewModel> lineasDeLaFactura)
    {
      _lineasDeLaFactura = lineasDeLaFactura;
    }

    public void SetLineasDeLaFactura(BindingList<PresupuestoViewModel.LineaPresupuestoViewModel> lineasDeLaFactura)
    {
      _lineasDeLaFactura=new BindingList<LineaFacturaViewModel>();
      foreach (var nuevaLineaFactura in lineasDeLaFactura.Select(linea => new LineaFacturaViewModel
                                                                            {
                                                                              IdVehiculoVenta = linea.IdVehiculoVenta,
                                                                              Codigo = linea.Codigo,
                                                                              Marca = linea.Marca,
                                                                              Modelo = linea.Modelo,
                                                                              Precio = linea.Precio
                                                                            }))
      {
        _lineasDeLaFactura.Add(nuevaLineaFactura);
      }
    }

    public void EliminarLineaFactura(LineaFacturaViewModel lineaFacturaViewModel)
    {
      LineasDeLaFactura.Remove(lineaFacturaViewModel);
      ActualizarImportes();
    }

    public void AñadirLineaFactura(VehiculoVentaViewModel lineaFacturaViewModel)
    {
      var nuevaLineaFactura = new LineaFacturaViewModel
                            {
                              IdVehiculoVenta = lineaFacturaViewModel.Id,
                              Codigo = lineaFacturaViewModel.Codigo,
                              Marca = lineaFacturaViewModel.Marca,
                              Modelo = lineaFacturaViewModel.Modelo,
                              Precio = lineaFacturaViewModel.Precio
                            };
      _lineasDeLaFactura.Add(nuevaLineaFactura);
      ActualizarImportes();
    }

    public void AñadirCliente(ClienteViewModel clienteViewModel)
    {
      IdCliente = clienteViewModel.Id;
      NombreCliente = new StringBuilder(clienteViewModel.Nombre).Append(" ")
                                .Append(clienteViewModel.PrimerApellido).Append(" ")
                                .Append(clienteViewModel.SegundoApellido).ToString();
      DireccionCliente = clienteViewModel.Direccion;
      TelefonosCliente = clienteViewModel.ListaTelefonos;
    }

    /// <summary>
    /// Determina si el modelo [ha estado persistido].
    /// </summary>
    /// <returns>
    ///   <c>true</c> Si [ha estado persistido]; de otra forma, <c>false</c>.
    /// </returns>
    public bool HasBeenPersisted()
    {
      return !Id.Equals(Guid.Empty);
    }
    #endregion

    #region Métodos privados

    public void ActualizarImportes()
    {
      var baseImponible =
        _lineasDeLaFactura.Sum(lineaFacturaViewModel => lineaFacturaViewModel.Precio ?? 0);

      BaseImponible = baseImponible;
      ImporteIva = (baseImponible * PorcentajeIva) / 100;
      ImporteTotal = baseImponible + ImporteIva;
    }

    #endregion
  }
}
