﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Linq;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionClientes;
using GestionConcesionarios.Aplicacion.ModuloPrincipal.GestionVentas;
using GestionConcesionarios.Infraestructura.Data.ModuloPrincipal.Modelo;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.CasosDeUso.Login;
using GestionConcesionarios.Presentacion.CasosDeUso.Presupuestos;

namespace GestionConcesionarios.Presentacion.CasosDeUso.Ventas
{
  /// <summary>
  /// Clase factoria para construir la información a mostrar en 
  /// la vista de facturas,
  /// por ejemplo los valores de las listas desplegables, etc.
  /// </summary>
  public class FacturaViewModelFactory : IModelFactory
  {
    #region Atributos

    private readonly Func<FacturaViewModel> _crearViewModel;
    private readonly IGestionVentaService _gestionFacturaService;
    private readonly IGestionClienteService _gestionClienteService;
    private readonly InformacionUsuario _informacionUsuario;

    #endregion

    #region Constructors

    public FacturaViewModelFactory(Func<FacturaViewModel> crearViewModel, IGestionVentaService gestionFacturaService, IGestionClienteService gestionClienteService, InformacionUsuario informacionUsuario)
    {
      if (crearViewModel == null) throw new ArgumentNullException("crearViewModel");

      _crearViewModel = crearViewModel;
      _gestionFacturaService = gestionFacturaService;
      _gestionClienteService = gestionClienteService;
      _informacionUsuario = informacionUsuario;
    }

    #endregion

    #region Metodos Publicos

    /// <summary>
    /// Crea el modelo para la vista vacío.
    /// </summary>
    /// <returns>Un modelo FacturaViewModel vacío</returns>
    public FacturaViewModel CrearFacturaViewModel()
    {
      var facturaViewModel = _crearViewModel();

      facturaViewModel.Id = Guid.Empty;
      facturaViewModel.Numero = CrearNumeroDeVenta();
      facturaViewModel.Fecha = null;
      facturaViewModel.BaseImponible = 0;
      facturaViewModel.PorcentajeIva = InformacionUsuario.PorcentajeIva;
      facturaViewModel.ImporteIva = 0;
      facturaViewModel.ImporteTotal = 0;
      facturaViewModel.IdEmpleado = Guid.Empty;
      facturaViewModel.IdCliente = Guid.Empty;
      facturaViewModel.NombreCliente = string.Empty;
      facturaViewModel.DireccionCliente = string.Empty;
      facturaViewModel.TelefonosCliente = string.Empty;
      facturaViewModel.CodigoTaller = string.Empty;
      facturaViewModel.FormaDePago = 0;
      facturaViewModel.CuentaBancaria = string.Empty;

      facturaViewModel.ActualizarImportes();

      facturaViewModel.SetLineasDeLaFactura(new BindingList<FacturaViewModel.LineaFacturaViewModel>());

      return facturaViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del factura facturaId.
    /// </summary>
    /// <param name="facturaId">Id del factura a partir del cual se creará el modelo.</param>
    /// <returns>Un modelo FacturaViewModel con los datos del factura con id=facturaId</returns>
    public FacturaViewModel CrearFacturaViewModel(Guid facturaId)
    {
      var lista = _gestionFacturaService.BuscarVentaPorId(facturaId);

      var facturaViewModel = CrearFacturaViewModelDesdeListaDeFacturas(lista);
      facturaViewModel.ActualizarImportes();

      return facturaViewModel;
    }

    /// <summary>
    /// Crea un nuevo modelo para la vista con los datos del presupuesto pasado por parámetro.
    /// </summary>
    /// <param name="presupuestoViewModel">Datos del presupuesto a partir del cual se creará la factura.</param>
    /// <returns>Un modelo FacturaViewModel con los datos del presupuesto presupuestoViewModel</returns>
    public FacturaViewModel CrearFacturaViewModel(PresupuestoViewModel presupuestoViewModel)
    {
      var facturaViewModel = _crearViewModel();

      facturaViewModel.Id = Guid.Empty;
      facturaViewModel.Numero = CrearNumeroDeVenta();
      facturaViewModel.Fecha = presupuestoViewModel.Fecha;
      facturaViewModel.BaseImponible = presupuestoViewModel.BaseImponible;
      facturaViewModel.PorcentajeIva = presupuestoViewModel.PorcentajeIva;
      facturaViewModel.IdEmpleado = presupuestoViewModel.IdEmpleado;
      facturaViewModel.IdCliente = presupuestoViewModel.IdCliente;
      facturaViewModel.NombreCliente = presupuestoViewModel.NombreCliente;
      facturaViewModel.DireccionCliente = presupuestoViewModel.DireccionCliente;
      facturaViewModel.TelefonosCliente = presupuestoViewModel.TelefonosCliente;
      facturaViewModel.CodigoTaller = presupuestoViewModel.CodigoTaller;

      var cliente = _gestionClienteService.BuscarClientePorId(presupuestoViewModel.IdCliente);
      if (cliente != null && cliente[0] != null)
      {
        facturaViewModel.FormaDePago = cliente[0].FormaDePago;
        facturaViewModel.CuentaBancaria = cliente[0].CuentaBancaria;
      }

      facturaViewModel.SetLineasDeLaFactura(presupuestoViewModel.LineasDelPresupuesto);

      facturaViewModel.ActualizarImportes();

      return facturaViewModel;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Crea un factura view model desde lista de facturas. Es una función puente para transformar
    /// la lista devuelta por el servicio de gestión de presupustos en un único factura.
    /// </summary>
    /// <param name="lista">Lista de facturas.</param>
    /// <returns>Un modelo FacturaViewModel con el primer elemento de la lista o si la lista
    /// está vacía un modelo FacturaViewModel vacío</returns>
    private FacturaViewModel CrearFacturaViewModelDesdeListaDeFacturas(IEnumerable<Venta> lista)
    {
      var facturaViewModel = _crearViewModel();

      foreach (var factura in lista)
      {
        facturaViewModel.Id = factura.Id;
        facturaViewModel.Numero = factura.Numero;
        facturaViewModel.Fecha = factura.Fecha;
        if (factura.BaseImponible != null) facturaViewModel.BaseImponible = factura.BaseImponible.Value;
        if (factura.PorcentajeIva != null) facturaViewModel.PorcentajeIva = (decimal)factura.PorcentajeIva.Value;
        facturaViewModel.IdEmpleado = factura.IdEmpleado;
        facturaViewModel.IdCliente = factura.IdCliente;
        facturaViewModel.NombreCliente = factura.Cliente.Nombre;
        facturaViewModel.DireccionCliente = factura.Cliente.Direccion;
        facturaViewModel.TelefonosCliente = factura.Cliente.Telefonos;
        facturaViewModel.CodigoTaller = factura.CodigoTaller;
        facturaViewModel.FormaDePago = factura.FormaDePago;
        facturaViewModel.CuentaBancaria = factura.CuentaBancaria;

        facturaViewModel.SetLineasDeLaFactura(ObtenerLineasDeLaFactura(factura));

        facturaViewModel.ActualizarImportes();

        return facturaViewModel;
      }
      return facturaViewModel;
    }

    /// <summary>
    /// Obtiene todas las líneas del factura.
    /// </summary>
    /// <param name="factura">Factura del cual se desean obtener las líneas.</param>
    /// <returns>Líneas del factura como modelo de vista</returns>
    private BindingList<FacturaViewModel.LineaFacturaViewModel> ObtenerLineasDeLaFactura(Venta factura)
    {
      var lineasDeLaFactura = new BindingList<FacturaViewModel.LineaFacturaViewModel>();
      foreach (var vehiculoVenta in factura.VehiculoVenta.Select(vehiculo => new FacturaViewModel.LineaFacturaViewModel
                                                                             {
                                                                               IdVehiculoVenta = vehiculo.Id,
                                                                               Marca = vehiculo.Marca,
                                                                               Modelo = vehiculo.Modelo,
                                                                               Codigo = vehiculo.Codigo,
                                                                               Precio = vehiculo.Precio
                                                                             }))
      {
        lineasDeLaFactura.Add(vehiculoVenta);
      }
      return lineasDeLaFactura;
    }

    /// <summary>
    /// Crea un numero de venta a partir del codigo de taller y la fecha/hora actual.
    /// </summary>
    /// <returns>Un nuevo número de venta</returns>
    private string CrearNumeroDeVenta()
    {
      return _informacionUsuario.CodigoTaller + DateTime.Now.Ticks.ToString(CultureInfo.InvariantCulture);
    }
    #endregion
  }
}
