﻿using System;
using System.Linq;
using System.Reflection;
using Autofac;
using AutofacContrib.DynamicProxy2;
using Castle.DynamicProxy;
using FluentValidation;
using GestionConcesionarios.Infraestructura.Helpers;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Interceptors;
using Module = Autofac.Module;

namespace GestionConcesionarios.Presentacion.Infraestructura.Container
{
  public class ModelModule : Module
  {
    #region Atributos

    private readonly Assembly[] _assemblies;
    private readonly string[] _namespaces;

    #endregion

    #region Métodos Privados

    private bool IsNamespaceToScan(string @namespace)
    {
      if (@namespace == null) return false;
      return  _namespaces.Length == 0 || _namespaces.Any(@namespace.Contains);
    }

    #endregion

    #region Constructores

    public ModelModule(Assembly[] assemblies,string[] namespaces)
    {
      if (assemblies == null) throw new ArgumentNullException("assemblies");
      if (namespaces == null) throw new ArgumentNullException("namespaces");

      if (assemblies.Length==0) throw new ArgumentException("no hay assemblies para escanear","assemblies");

      _assemblies = assemblies;
      _namespaces = namespaces;
    }

    #endregion

    #region Métodos Protegidos

    protected override void Load(ContainerBuilder builder)
    {
      if (builder == null ) throw new ArgumentNullException("builder");

      builder.RegisterType<ProxyCreator>().InstancePerLifetimeScope();
      
      foreach (Assembly assembly in _assemblies)
      {
        //Interceptores
        builder.RegisterAssemblyTypes(assembly).Where(t => t.ImplementsInterface(typeof(IInterceptor))).InstancePerLifetimeScope();

        //Validadores
        builder.RegisterAssemblyTypes(assembly).Where(t => IsNamespaceToScan(t.Namespace) && t.ImplementsInterface(typeof(IValidator))).Keyed<IValidator>(t => t.Name).AsImplementedInterfaces().
          SingleInstance();

        //Clases Factoria de los modelos de vista
        builder.RegisterAssemblyTypes(assembly).Where(t => IsNamespaceToScan(t.Namespace) && t.ImplementsInterface(typeof(IModelFactory))).InstancePerLifetimeScope();

        //Modelos de vista
        builder.RegisterAssemblyTypes(assembly).Where(
          t => IsNamespaceToScan(t.Namespace) && t.ImplementsInterface(typeof (IModel))).InstancePerLifetimeScope().EnableClassInterceptors().
          InterceptedBy(typeof (NotifyPropertyChangedInterceptor)).
          InterceptedBy(typeof (DataErrorInfoInterceptor));
      }
    }

    #endregion
  }
}