﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Autofac;
using GestionConcesionarios.Infraestructura.Helpers;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using Module = Autofac.Module;

namespace GestionConcesionarios.Presentacion.Infraestructura.Container
{
  public class PresenterModule : Module
  {
    #region Atributos

    private readonly Assembly[] _presenterAssemblies;
    private readonly string[] _namespaces;
    private readonly string _domainPrefix;

    #endregion

    #region Private Methods

    private bool IsNamespaceToScan(string @namespace)
    {
      if (@namespace == null) return false;
      return _namespaces.Length == 0 || _namespaces.Any(@namespace.Contains);
    }

    private IEnumerable<MediatorEnumerator> MediatorsUsedByPresentersInAssemblies()
    {
      var mediatorEnumerators = new List<MediatorEnumerator>();

      foreach (Assembly assembly in _presenterAssemblies)
      {
        foreach (Type type in assembly.GetTypes().Where(t => IsNamespaceToScan(t.Namespace) && t.ImplementsInterface(typeof(IPresenter))))
        {
          var attributes = type.GetCustomAttributes(typeof(RegisterInMediatorAttribute), true).Cast<RegisterInMediatorAttribute>();
          mediatorEnumerators.AddRange(attributes.Select(l => l.Mediator));
        }

      }
      return mediatorEnumerators.Distinct();
    }

    #endregion

    #region Constructors

    public PresenterModule(Assembly[] presenterAssemblies, string[] namespaces)
    {
      if (presenterAssemblies == null) throw new ArgumentNullException("presenterAssemblies");
      if (namespaces == null) throw new ArgumentNullException("namespaces");
      if (presenterAssemblies.Length == 0) throw new ArgumentException("no presenterAssemblies to scan", "presenterAssemblies");

      _presenterAssemblies = presenterAssemblies;
      _namespaces = namespaces;
    }

    #endregion

    protected override void Load(ContainerBuilder builder)
    {
      //Mesage Log Presenter
      //builder.RegisterType<MessageLogPresenter>().InstancePerLifetimeScope();
      //Mediator
      builder.RegisterType<Mediator.Mediator>().As<IMediator>().InstancePerDependency();
      //Mediator Manager
      builder.RegisterType<MediatorManager>().As<IMediatorManager>().SingleInstance();

      foreach (Assembly assembly in _presenterAssemblies)
      {
        //Presenters
        builder.RegisterAssemblyTypes(assembly).Where(
          l => IsNamespaceToScan(l.Namespace) && l.ImplementsInterface(typeof(IPresenter))).InstancePerLifetimeScope()
          .OnActivated(obj => Console.WriteLine("Activating: {0}", obj.Instance.GetType()))
          .OnActivated(t =>
                           {
                             var mediatorManager = t.Context.Resolve<IMediatorManager>();
                             mediatorManager.Register(t.Instance);
                           })
          .AsImplementedInterfaces().AsSelf();
        /*
        //Search Criteria
        builder.RegisterAssemblyTypes(assembly).Where(l => IsNamespaceToScan(l.Namespace) && l.ImplementsInterface(typeof(ISearchCriteria))).EnableClassInterceptors().
          InterceptedBy(typeof(NotifyPropertyChangedInterceptor)).InterceptedBy(typeof(DataErrorInfoInterceptor)).AsSelf();
        */
      }
    }
  }
}