﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using Autofac;
using Autofac.Core;
using Castle.DynamicProxy;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.Infraestructura.Container
{
  /// <summary>
  /// Clase que sirve para crear una nueva clase proxy a partir de otra.
  /// </summary>
  public class ProxyCreator
  {
    #region Constantes

    const string InterceptorsPropertyName = "AutofacContrib.DynamicProxy2.RegistrationExtensions.InterceptorsPropertyName";

    #endregion

    #region Atributos

    private readonly ILifetimeScope _lifetimeScope;
    private readonly ProxyGenerator _proxyGen;

    #endregion

    #region Constructores

    public ProxyCreator(ILifetimeScope lifetimeScope)
    {
      _lifetimeScope = lifetimeScope;
      _proxyGen = new ProxyGenerator();
    }

    #endregion

    #region Métodos Públicos

    /// <summary>
    /// Crea una clase proxy con las interfaces INotifyPropertyChanged y IDataErrorInfo
    /// implementadas a partir de la instancia pasada por parámetro
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <param name="instance">La instancia a partir de la que se crea el proxy.</param>
    /// <returns>Retorna una clase con las interfaces INotifyPropertyChanged y IDataErrorInfo
    /// implementadas </returns>
    public T CreateProxyFromInstance<T>(T instance) where T : ModelBase
    {
      var interceptors = GetInterceptors<T>();

      var proxy = _proxyGen.CreateClassProxyWithTarget(typeof(T), new Type[] { typeof(INotifyPropertyChanged), typeof(IDataErrorInfo) }, instance, interceptors);

      return proxy as T;
    }

    #endregion

    #region Métodos Privados

    private IInterceptor[] GetInterceptors<T>()
    {
      //Obtenemos los registros del tipo T en el contenedor IoC
      var registration = _lifetimeScope.ComponentRegistry.Registrations.Where(t => t.Services.FirstOrDefault().Description == typeof(T).FullName).FirstOrDefault();

      //Obtenemos los interceptores
      var servicesList = new List<Service>();
      if (registration != null)
      {
        object services;
        registration.Metadata.TryGetValue(InterceptorsPropertyName, out services);
        if (services != null)
        {
          servicesList.AddRange((IEnumerable<Service>)services);
        }
      }

      //Instanciamos los interceptores y los retornamos
      var result = new List<IInterceptor>();
      foreach (var service in servicesList)
      {
        result.Add(_lifetimeScope.ResolveService(service) as IInterceptor);
      }

      return result.ToArray();
    }

    #endregion
  }
}
