﻿using System;
using System.Reflection;
using Autofac;
using System.Linq;
using GestionConcesionarios.Infraestructura.Helpers;
using GestionConcesionarios.Presentacion.Base;
using GestionConcesionarios.Presentacion.Infraestructura.Mediator;
using GestionConcesionarios.Presentacion.Infraestructura.MessageService;
using Kent.Boogaart.Truss;
using Module = Autofac.Module;

namespace GestionConcesionarios.Presentacion.Infraestructura.Container
{
  public class ViewModule : Module
  {
    #region Atributos
    private readonly Assembly[] _assemblies;
    private readonly string[] _namespaces;
    #endregion

    #region Private Methods

    private bool IsNamespaceToScan(string @namespace)
    {
      if (@namespace == null) return false;
      return _namespaces.Length == 0 || _namespaces.Any(@namespace.Contains);
    }

    #endregion

    #region Constructor
    public ViewModule(Assembly[] assemblies, string[] namespaces)
    {
      if (assemblies == null) throw new ArgumentNullException("assemblies");
      if (namespaces == null) throw new ArgumentNullException("namespaces");

      if (assemblies.Length == 0) throw new ArgumentException("no assemblies to scan", "assemblies");

      _assemblies = assemblies;
      _namespaces = namespaces;
    }
    #endregion

    #region Implementacion Module
    protected override void Load(ContainerBuilder builder)
    {
      builder.RegisterType<WindowsFormsMessageService>().SingleInstance();
      builder.RegisterType<Mediator.Mediator>().Keyed<IMediator>(MediatorEnumerator.FormManager).SingleInstance();
      builder.RegisterType<FormManager>().OnActivated(ContainerRegisterHelper.RegisterInMediator).SingleInstance();
      builder.RegisterGeneric(typeof(FormContext<>));

      //Form Context Factory
      builder.Register<Func<Type, IFormContext>>(c =>
                                                   {
                                                     var componentContext = c.Resolve<IComponentContext>();
                                                     return type =>
                                                              {
                                                                var concreteType = typeof(FormContext<>).MakeGenericType(type);
                                                                return (IFormContext)componentContext.Resolve(concreteType);
                                                              };
                                                   }
        );

      foreach (Assembly assembly in _assemblies)
      {
        //Views
        builder.RegisterAssemblyTypes(assembly).Where(l => IsNamespaceToScan(l.Namespace) && l.ImplementsInterface(typeof(IView)));
        // View Configurators
        builder.RegisterAssemblyTypes(assembly).Where(l => IsNamespaceToScan(l.Namespace) && l.ImplementsInterface(typeof(IViewConfigurator)) && !l.IsAbstract)
          //.OnActivated(ContainerRegisterHelper.SubscribeApplicationEventHandlerMethodsToFormBus)
          .Keyed<IViewConfigurator>(t =>
          {
            var lastGenericArgument = t.BaseType.GetGenericArguments().LastOrDefault();
            return lastGenericArgument.FullName;
          }).AsImplementedInterfaces().InstancePerLifetimeScope().AsSelf();
      }
    }
    #endregion
  }
}
