﻿using System;
using System.ComponentModel;
using System.Linq;
using Autofac.Features.Indexed;
using Castle.DynamicProxy;
using FluentValidation;
using GestionConcesionarios.Presentacion.Base;

namespace GestionConcesionarios.Presentacion.Infraestructura.Interceptors
{
  public class NotifyPropertyChangedInterceptor : IInterceptor
  {
    #region Atributos
    private readonly IIndex<string, IValidator> _validatorIndex;
    private readonly IIndex<string, IViewConfigurator> _viewConfiguratorIndex;
    private PropertyChangedEventHandler _subscribers = delegate { };
    #endregion

    #region Private Methods

    private bool IsObjectValid(IInvocation invocation)
    {
      if (invocation == null) throw new ArgumentNullException("invocation");
        
      var validator = GetValidator(invocation);
      if (validator != null)
      {
        var modelBase = invocation.InvocationTarget as ModelBase;
        if (modelBase == null)
          return true;
        var validationResult = validator.Validate(invocation.InvocationTarget);
        modelBase.IsValid= !validationResult.Errors.Any();
        Console.WriteLine("Validando {0} : resulado {1}",modelBase.GetType().Name,modelBase.IsValid.ToString());
        return modelBase.IsValid;
      }
      return true;
    }

    private IValidator GetValidator(IInvocation invocation)
    {
      if (invocation == null) throw new ArgumentNullException("invocation");

      IValidator validator;
      _validatorIndex.TryGetValue(GetValidatorServiceKey(invocation), out validator);
      return validator;
    }


    private static string GetValidatorServiceKey(IInvocation invocation)
    {
      return invocation.TargetType.Name + "Validator";
    }


    private IViewConfigurator GetViewConfigurator(IInvocation invocation)
    {
      IViewConfigurator viewConfigurator;
      _viewConfiguratorIndex.TryGetValue(invocation.TargetType.FullName, out viewConfigurator);
      return viewConfigurator;
    }

    private void TrackPropertyChange(IInvocation invocation, string propertyName)
    {

      var modelPropertyChangeTracker = invocation.InvocationTarget as IModel;
      if (modelPropertyChangeTracker != null)
      {
        modelPropertyChangeTracker.TrackPropertyChange(propertyName);
      }

    }

    private void ConfigureView(IInvocation invocation)
    {
      if (invocation == null) throw new ArgumentNullException("invocation");

      var objectIsValid = IsObjectValid(invocation);
        var viewConfigurator = GetViewConfigurator(invocation);
        if (viewConfigurator != null)
        {
          Console.WriteLine("Firing all properties changed {0}",viewConfigurator.GetType().Name);
          viewConfigurator.FireAllPropertiesChanged(objectIsValid);
        }
    }

    #endregion

    #region Constructors

    public NotifyPropertyChangedInterceptor(IIndex<string, IValidator> validatorIndex, IIndex<string, IViewConfigurator> viewConfiguratorIndex)
    {
      if (validatorIndex == null) throw new ArgumentNullException("validatorIndex");
      if (viewConfiguratorIndex == null) throw new ArgumentNullException("viewConfiguratorIndex");

      _validatorIndex = validatorIndex;
      _viewConfiguratorIndex = viewConfiguratorIndex;
    }

    #endregion

    #region Metodos Publicos

    public void Intercept(IInvocation invocation)
    {
        
      if (invocation.Method.Name.StartsWith("add_PropertyChanged"))
      {
        var propertyChangedEventHandler = (PropertyChangedEventHandler) invocation.Arguments[0];
        _subscribers += propertyChangedEventHandler;
        return;
      }
      if (invocation.Method.Name.StartsWith("remove_PropertyChanged"))
      {
        var propertyChangedEventHandler = (PropertyChangedEventHandler) invocation.Arguments[0];
        _subscribers -= propertyChangedEventHandler;
        return;
      }

      invocation.Proceed();
      if (invocation.Method.Name.StartsWith("set_"))
      {
        var propertyName = invocation.Method.Name.Substring(4);
        _subscribers(invocation.InvocationTarget, new PropertyChangedEventArgs(propertyName));

        TrackPropertyChange(invocation, propertyName);
        ConfigureView(invocation);
      }
    }

    #endregion
  }
}