﻿using System;
using System.Collections.Generic;
using System.Reflection;

namespace GestionConcesionarios.Presentacion.Infraestructura.Mediator
{
  /// <summary>
  /// A message mediator allowing disconnected ViewModels to send and
  /// receive messages. This interface should not be referenced directly 
  /// by the end system.
  /// </summary>
  public interface IMediatorBase
  {
    /// <summary>
    /// Register a ViewModel so that it can receive notifications from the mediator.
    /// </summary>
    /// <param name="target">The instance to register.</param>
    IDisposable Register(object target);

    /// <summary>
    /// Notify any registered parties that a specific message has been broadcast.
    /// </summary>
    /// <typeparam name="T">The type of the parameter being passed.</typeparam>
    /// <param name="message">The message that is being broadcast.</param>
    /// <param name="parameter">The parameter to pass with the message.</param>
    void NotifyColleagues<T>(string message, T parameter);

    /// <summary>
    /// Notify any 
    /// registered parties that a specific message has been broadcast.
    /// </summary>
    /// <param name="message">The message that is being broadcast.</param>
    void NotifyColleagues(string message);
  }

  /// <summary>
  /// A message mediator allowing disconnected ViewModels to send and
  /// receive messages. This interface should not be referenced directly 
  /// by the end system.
  /// </summary>
  public interface IMediator : IMediatorBase
  {

    /// <summary>
    /// Register a ViewModel so that it can receive notifications from the mediator.
    /// </summary>
    /// <param name="message">The message to register.</param>
    /// <param name="callback">The callback.</param>
    void Register(string message, Delegate callback);

    void UnRegister(object obj);
  }


  public class Mediator : MediatorBase, IMediator
  {
    /// <summary>
    /// Register a ViewModel so that it can receive notifications from the mediator.
    /// </summary>
    /// <param name="message">The message to register.</param>
    /// <param name="callback">The callback.</param>
    public void Register(string message, Delegate callback)
    {
      ParameterInfo[] parameters = callback.Method.GetParameters();

      if (parameters != null && parameters.Length > 1)
        throw new InvalidOperationException("The registered delegate should only have 0 or 1 parameter since the Mediator has up to 1 argument to pass");
      Type paramType = parameters[0].ParameterType;
      InvocationList.AddAction(message, callback.Target, callback.Method, paramType);
    }

    public void UnRegister(object target)
    {
      InvocationList.RemoveAction(target);
    }
  }


  public abstract class MediatorBase : IMediatorBase
  {
    private MessageToActionMap _invocationList = new MessageToActionMap();

    public IDisposable Register(object target)
    {
      if (target == null)
        throw new ArgumentNullException("target");

      //  _invocationList = new MessageToActionMap();
      List<KeyValuePair<string, WeakAction>> registrations = new List<KeyValuePair<string, WeakAction>>();

      foreach (var methodInfo in target.GetType().GetMethods(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance))
      {
        foreach (MediatorMessageSinkAttribute attribute in methodInfo.GetCustomAttributes(typeof(MediatorMessageSinkAttribute), true))
        {
          if (methodInfo.GetParameters().Length > 1)
            throw new InvalidOperationException("The registered method should less than 2 parameters since the Mediator has a maximum 1 argument to pass");

          registrations.Add(new KeyValuePair<string, WeakAction>(attribute.Message,
                                                               _invocationList.AddAction(attribute.Message, target, methodInfo, attribute.ParameterType)));
        }
      }

      return new MediatorRegistration(_invocationList, registrations);
    }

    public void NotifyColleagues<T>(string message, T parameter)
    {
      var actions = _invocationList.GetActions(message);

      if (actions != null)
      {
        actions.ForEach(action =>
        {
          try
          {
            action.DynamicInvoke(parameter);
          }
          catch (Exception ode)
          {
            //_logger.Warn(string.Format("(MediatorBase) NotifyColleagues exception {0}", ode.Message));
          }
        });
      }
    }

    public void NotifyColleagues(string message)
    {
      var actions = _invocationList.GetActions(message);

      if (actions != null)
      {
        actions.ForEach(action => 
        {
          try
          {
            action.DynamicInvoke();
          }
          catch (Exception ode)
          {
            //_logger.Warn(string.Format("(MediatorBase) NotifyColleagues exception {0}", ode.Message));
          }
        });
      }
    }

    protected MessageToActionMap InvocationList
    {
      get { return _invocationList; }
    }

  }

  internal class MediatorRegistration : IDisposable
  {
    private MessageToActionMap _invocationList;
    private IEnumerable<KeyValuePair<string, WeakAction>> _actionRegistereds;

    public MediatorRegistration(MessageToActionMap invocationList, IEnumerable<KeyValuePair<string, WeakAction>> actionRegistereds)
    {
      _invocationList = invocationList;
      _actionRegistereds = actionRegistereds;
    }

    #region Implementation of IDisposable

    /// <summary>
    /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
    /// </summary>
    public void Dispose()
    {
      foreach (var actionRegistered in _actionRegistereds)
        _invocationList.RemoveAction(actionRegistered.Key, actionRegistered.Value);

      _actionRegistereds = null;
      _invocationList = null;
    }

    #endregion
  }
}