﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq.Expressions;
using System.Reflection;
using GestionConcesionarios.Infraestructura.Helpers;

namespace GestionConcesionarios.Presentacion.Infraestructura
{
  public static class ObservablePropertyChanged
  {
    public static ManyPropertiesChangedSubscriber<TSource> OnChanged<TSource>(this TSource source, params Expression<Func<TSource, object>>[] properties)
      where TSource : class, INotifyPropertyChanged
    {
      return new ManyPropertiesChangedSubscriber<TSource>(source, properties);
    }

    public static AnyPropertyChangedSubscriber<TSource> OnAnyProperyChanged<TSource>(this TSource source)
        where TSource : class, INotifyPropertyChanged
    {
      return new AnyPropertyChangedSubscriber<TSource>(source);
    }

    public static AnyPropertyChangedWithExceptionSubscriber<TSource> OnAnyProperyChanged<TSource>(this TSource source, params Expression<Func<TSource, object>>[] exceptionProperties)
        where TSource : class, INotifyPropertyChanged
    {
      return new AnyPropertyChangedWithExceptionSubscriber<TSource>(source, exceptionProperties);
    }
  }

  public class ManyPropertiesChangedSubscriber<TSource> : IDisposable where TSource : class ,INotifyPropertyChanged
  {
    private readonly TSource _source;
    private Action<TSource> _onChange;
    private readonly List<string> _propertiesToListen = new List<string>();
    private Func<TSource, bool> _fireEventCondition = (c => true);
    public ManyPropertiesChangedSubscriber(TSource source, params Expression<Func<TSource, object>>[] properties)
    {
      _source = source;
      _source.PropertyChanged += SourcePropertyChanged;
      SetPropertiesToListen(properties);
    }

    public ManyPropertiesChangedSubscriber<TSource> When(Func<TSource, bool> condition)
    {
      if (condition == null) throw new ArgumentNullException("condition");
      _fireEventCondition = condition;
      return this;
    }

    private bool CanFireEvent(TSource source)
    {
      return _fireEventCondition(source);
    }
    private void SourcePropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (!CanFireEvent(sender as TSource)) return;
      if (IsPropertyToListen(e.PropertyName)) _onChange(sender as TSource);
    }

    public IDisposable Do(Action<TSource> onChanged)
    {
      _onChange = onChanged;
      return this;
    }

    public void DoOnce(Action<TSource> onChanged)
    {
      Action<TSource> dispose = x => Dispose();
      _onChange = (Action<TSource>)Delegate.Combine(onChanged, dispose);
    }


    public void Dispose()
    {
      _source.PropertyChanged -= SourcePropertyChanged;
    }

    private void SetPropertiesToListen(IEnumerable<Expression<Func<TSource, object>>> exceptionProperties)
    {
      foreach (var exceptionProperty in exceptionProperties)
      {
        var propertyName = ExpressionHelper.GetPropertyName(exceptionProperty);
        if (propertyName.Length > 0)
          _propertiesToListen.Add(propertyName);
      }
    }

    private bool IsPropertyToListen(string propertyName)
    {
      return _propertiesToListen.Contains(propertyName);
    }

  }

  public class AnyPropertyChangedWithExceptionSubscriber<TSource> : IDisposable where TSource : class ,INotifyPropertyChanged
  {
    private readonly TSource _source;
    private Action<TSource> _onChange;
    private readonly List<string> _propertyExceptions = new List<string>();

    public AnyPropertyChangedWithExceptionSubscriber(TSource source, params Expression<Func<TSource, object>>[] exceptionProperties)
    {
      _source = source;
      _source.PropertyChanged += SourcePropertyChanged;
      SetPropertyExceptions(exceptionProperties);
    }

    private void SourcePropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (!IsPropertyException(e.PropertyName)) _onChange(sender as TSource);
    }

    public IDisposable Do(Action<TSource> onChanged)
    {
      _onChange = onChanged;
      return this;
    }

    public void DoOnce(Action<TSource> onChanged)
    {
      Action<TSource> dispose = x => Dispose();
      _onChange = (Action<TSource>)Delegate.Combine(onChanged, dispose);
    }


    public void Dispose()
    {
      _source.PropertyChanged -= SourcePropertyChanged;
    }

    private void SetPropertyExceptions(IEnumerable<Expression<Func<TSource, object>>> exceptionProperties)
    {
      foreach (var exceptionProperty in exceptionProperties)
      {
        var propertyName = ExpressionHelper.GetPropertyName(exceptionProperty);
        if (propertyName.Length > 0)
          _propertyExceptions.Add(propertyName);
      }
    }

    private bool IsPropertyException(string propertyName)
    {
      return _propertyExceptions.Contains(propertyName);
    }

  }


  public class AnyPropertyChangedSubscriber<TSource> : IDisposable where TSource : class ,INotifyPropertyChanged
  {
    private readonly TSource _source;
    private Action<TSource> _onChange;
    public AnyPropertyChangedSubscriber(TSource source)
    {
      _source = source;
      _source.PropertyChanged += SourcePropertyChanged;
    }

    private void SourcePropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      _onChange(sender as TSource);
    }

    public IDisposable Do(Action<TSource> onChanged)
    {
      _onChange = onChanged;
      return this;
    }

    public void DoOnce(Action<TSource> onChanged)
    {
      Action<TSource> dispose = x => Dispose();
      _onChange = (Action<TSource>)Delegate.Combine(onChanged, dispose);
    }


    public void Dispose()
    {
      _source.PropertyChanged -= SourcePropertyChanged;
    }
  }

  /// <summary>
  /// Shortcut to subscribe to PropertyChanged on an INotfiyPropertyChanged and executes an action when that happens
  /// </summary>
  /// <typeparam name="TSource">Must implement INotifyPropertyChanged</typeparam>
  /// <typeparam name="TProperty">Can be any type</typeparam>
  public class PropertyChangedSubscriber<TSource, TProperty>
      : IDisposable where TSource : class, INotifyPropertyChanged
  {
    private readonly Expression<Func<TSource, TProperty>> _propertyValidation;
    private readonly TSource _source;
    private Action<TSource> _onChange;

    public PropertyChangedSubscriber(TSource source, Expression<Func<TSource, TProperty>> property)
    {
      _propertyValidation = property;
      _source = source;
      source.PropertyChanged += SourcePropertyChanged;
    }

    private void SourcePropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (IsPropertyValid(e.PropertyName))
      {
        _onChange(sender as TSource);
      }
    }

    /// <summary>
    /// Executes the action and returns an IDisposable so that you can unregister 
    /// </summary>
    /// <param name="onChanged">The action to execute</param>
    /// <returns>The IDisposable so that you can unregister</returns>
    public IDisposable Do(Action<TSource> onChanged)
    {
      _onChange = onChanged;
      return this;
    }

    /// <summary>
    /// Executes the action only once and automatically unregisters
    /// </summary>
    /// <param name="onChanged">The action to be executed</param>
    public void DoOnce(Action<TSource> onChanged)
    {
      Action<TSource> dispose = x => Dispose();
      _onChange = (Action<TSource>)Delegate.Combine(onChanged, dispose);
    }

    private bool IsPropertyValid(string propertyName)
    {
      var propertyInfo = ((MemberExpression)_propertyValidation.Body).Member as PropertyInfo;
      if (propertyInfo == null)
      {
        throw new ArgumentException("The lambda expression 'property' should point to a valid Property");
      }

      return propertyInfo.Name == propertyName;
    }

    #region Implementation of IDisposable

    /// <summary>
    ///   Unregisters the property
    /// </summary>
    public void Dispose()
    {
      _source.PropertyChanged -= SourcePropertyChanged;
    }

    #endregion
  }

}
