//---------------------------------------------------------------------------------------
// DeriveNode sample.
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved.
// http://www.lassalle.com
// Author: Patrick Lassalle
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
// ---------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using Lassalle.Flow;

namespace DeriveNode
{
    public partial class DeriveNode : Form
    {
        string strProgName = null;
        string strFileName = null;

        public DeriveNode()
        {
            InitializeComponent();
        }

        private void DeriveNode_Load(object sender, EventArgs e)
        {
            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.AutoScroll = true;
            AddFlow1.BackColor = SystemColors.Window;
            AddFlow1.DefNodeProp.FillColor = Color.LightYellow;
            AddFlow1.DefNodeProp.Shape.Style = ShapeStyle.Document;

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            strFileName = "";
            AddFlow1.Nodes.Clear();
            AddFlow1.ResetUndoRedo();
            AddFlow1.SetChangedFlag(false);
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuItemEdit_Popup(object sender, EventArgs e)
        {
            bool isSelectedItems  = (AddFlow1.SelectedItems.Count > 0);

            MenuItemEditUndo.Enabled = AddFlow1.CanUndo;
            MenuItemEditRedo.Enabled = AddFlow1.CanRedo;
            MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count != 0);
            MenuItemEditCut.Enabled = isSelectedItems;
            MenuItemEditCopy.Enabled = isSelectedItems;
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat");
            MenuItemEditDelete.Enabled = isSelectedItems;
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            AddFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            AddFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, null);
            AddFlow1.WriteXml(writer, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                AddFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                AddFlow1.BeginAction(1003);

                XmlTextReader reader = new XmlTextReader(stream);
                reader.WhitespaceHandling = WhitespaceHandling.None;
                AddFlow1.ReadXml(reader, true, false, false);
                reader.Close();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = AddFlow1.Grid.Size.Width;
                float dy = AddFlow1.Grid.Size.Height;
                foreach (Item item in AddFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }

                AddFlow1.EndAction();
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            foreach (Item item in AddFlow1.Items)
                item.Selected = true;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            if (AddFlow1.MouseAction == MouseAction.Selection)
                AddFlow1.MouseAction = MouseAction.None;
            else
                AddFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction == MouseAction.Selection);
        }

        private void MenuItemData_Popup(object sender, EventArgs e)
        {
            MenuItemDataNodeCustomData.Enabled = AddFlow1.SelectedItem is Node;
        }

        private void MenuItemDataNodeCustomData_Click(object sender, EventArgs e)
        {
            EnterCustomData();
        }

        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: DeriveNode sample\r\nCopyright (c) 2003-2011 Lassalle Technologies");
        }

        // See the tutorial (paragraph "Derivation of Node and Link classes") for explanations
        private void AddFlow1_BeforeAddNode(object sender, BeforeAddNodeEventArgs e)
        {
            e.Cancel.Cancel = true;
            MyNode mynode = new MyNode(AddFlow1.DefNodeProp, null, null);
            mynode.Location = e.Location;
            mynode.Size = e.Size;
            AddFlow1.Nodes.Add(mynode);
            mynode.Selected = true;
        }

        // See the tutorial (paragraph "Derivation of Node and Link classes") for explanations
        private void AddFlow1_DoubleClick(object sender, EventArgs e)
        {
            EnterCustomData();
        }

        // See the tutorial (paragraph "Derivation of Node and Link classes") for explanations
        private void AddFlow1_BeforeReadXMLNode(object sender, BeforeReadXMLNodeEventArgs e)
        {
            if (e.Custom == "MyNode")
            {
                MyNode mynode = new MyNode(e.DefNode, null, null);
                e.Node = mynode;
            }
        }

        // See the tutorial (paragraph "Derivation of Node and Link classes") for explanations
        private void AddFlow1_ReadXMLNodeExtraData(object sender, ReadXMLNodeExtraDataEventArgs e)
        {
            if (e.Node is MyNode)
            {
                MyNode mynode = (MyNode)e.Node;
                if (e.Reader.Name == "Author")
                    mynode.Author = e.Reader.ReadElementContentAsString();
                else if (e.Reader.Name == "Comment")
                    mynode.Comment = e.Reader.ReadElementContentAsString();
            }
        }

        // See the tutorial (paragraph "Derivation of Node and Link classes") for explanations
        private void AddFlow1_WriteXMLNodeExtraData(object sender, WriteXMLNodeExtraDataEventArgs e)
        {
            if (e.Node is MyNode)
            {
                MyNode mynode = (MyNode)e.Node;
                if (mynode.Author != null && mynode.Author.Length > 0)
                    e.Writer.WriteElementString("Author", mynode.Author);
                if (mynode.Comment != null && mynode.Comment.Length > 0)
                    e.Writer.WriteElementString("Comment", mynode.Comment);
            }
        }

        // See the tutorial (paragraph "Derivation of Node and Link classes") for explanations
        private void AddFlow1_BeforeWriteXMLNode(object sender, BeforeWriteXMLNodeEventArgs e)
        {
            if (e.Node is MyNode)
                e.Custom = "MyNode";
        }

        private void EnterCustomData()
        {
            if (AddFlow1.SelectedItem is Node)
            {
                Node node = (Node)AddFlow1.SelectedItem;
                if (node is MyNode)
                {
                    MyNode mynode = (MyNode)node;
                    NodeData nd = new NodeData();
                    nd.MyNode = mynode;
                    if (nd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        AddFlow1.SubmitTask(new MyTask(mynode));
                        mynode.Author = nd.TextBoxAuthor.Text;
                        mynode.Comment = nd.textBoxComment.Text;
                    }
                }
            }
        }

        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1)
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            AddFlow1.ResetUndoRedo();
            AddFlow1.Nodes.Clear();
            XmlTextReader reader = new XmlTextReader(strFileName);
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
            AddFlow1.ReadXml(reader);
            reader.Close();
            AddFlow1.SetChangedFlag(false);
            this.strFileName = strFileName;
            MakeCaption();
        }

        void SaveFile()
        {
            XmlTextWriter writer = new XmlTextWriter(strFileName, null);
            writer.Formatting = Formatting.Indented;
            AddFlow1.WriteXml(writer);
            writer.Close();
            AddFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }

        string FileTitle()
        {
            if (strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }

        bool OkToTrash()
        {
            if (!AddFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch (dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }
    }

    // Create a new node class, derived from the Node class of AddFlow.
    // This new class has two new properties: Author and Comment.
    internal class MyNode : Node
    {
        private string m_author = null;
        private string m_comment = null;

        public MyNode(Node defnode, string author, string comment) : base(defnode)
        {
            m_author = author;
            m_comment = comment;
        }

        public string Author
        {
            get { return m_author; }
            set { m_author = value; }
        }

        public string Comment
        {
            get { return m_comment; }
            set { m_comment = value; }
        }
    }

    internal class MyTask : Task
    {
        MyNode m_mynode = null;
        string m_oldAuthor = null;
        string m_oldComment = null;

        private MyTask() { }

        internal MyTask(MyNode mynode)
        {
            this.Code = (Lassalle.Flow.Action)1001;
            m_mynode = mynode;
            m_oldAuthor = m_mynode.Author;
            m_oldComment = m_mynode.Comment;
        }

        public override void Redo()
        {
            Undo();
        }

        public override void Undo()
        {
            string author = m_mynode.Author;
            string comment = m_mynode.Comment;
            m_mynode.Author = m_oldAuthor;
            m_mynode.Comment = m_oldComment;
            m_oldAuthor = author;
            m_oldComment = comment;
        }
    }
}