//----------------------------------------------------------------------------------------
// AddFlow for WinForms: PropertyBag sample
// Copyright (c) 2003-2011 Lassalle Technologies. All Rights Reserved
// http://www.lassalle.com
// Author: Patrick Lassalle    
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
//----------------------------------------------------------------------------------------

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using Lassalle.Flow;

namespace PropertyBag
{
    public partial class PropertyBag : Form
    {
        string strProgName = null;
        string strFileName = null;

        public PropertyBag()
        {
            InitializeComponent();
        }

        private void PropertyBag_Load(object sender, EventArgs e)
        {
            AddFlow1.Dock = DockStyle.Fill;
            AddFlow1.BackColor = Color.White;
            AddFlow1.AutoScroll = true;

            strProgName = this.Text;
            strFileName = "";
            MakeCaption();
        }

        private void PropertyBag_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!OkToTrash())
                e.Cancel = true;
        }

        private void MenuItemFileNew_Click(object sender, EventArgs e)
        {
            if (!OkToTrash())
                return;
            strFileName = "";
            AddFlow1.Nodes.Clear();
            AddFlow1.ResetUndoRedo();
            AddFlow1.SetChangedFlag(false);
            MakeCaption();
        }

        private void MenuItemFileOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Application.StartupPath;
            ofd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            ofd.FileName = "*.xml";
            if (ofd.ShowDialog() == DialogResult.OK)
                LoadFile(ofd.FileName);
        }

        private void MenuItemFileSave_Click(object sender, EventArgs e)
        {
            if (strFileName.Length == 0)
                SaveFileDlg();
            else
                SaveFile();
        }

        private void MenuItemFileSaveAs_Click(object sender, EventArgs e)
        {
            SaveFileDlg();
        }

        private void MenuItemFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void MenuItemEdit_Popup(object sender, EventArgs e)
        {
            bool isSelectedItems = (AddFlow1.SelectedItems.Count > 0);

            MenuItemEditUndo.Enabled = AddFlow1.CanUndo;
            MenuItemEditRedo.Enabled = AddFlow1.CanRedo;
            MenuItemEditSelectAll.Enabled = (AddFlow1.Nodes.Count != 0);
            MenuItemEditCut.Enabled = isSelectedItems;
            MenuItemEditCopy.Enabled = isSelectedItems;
            MenuItemEditPaste.Enabled = Clipboard.GetDataObject().GetDataPresent("AddFlow.XMLFormat");
            MenuItemEditDelete.Enabled = isSelectedItems;
        }

        private void MenuItemEditUndo_Click(object sender, EventArgs e)
        {
            AddFlow1.Undo();
        }

        private void MenuItemEditRedo_Click(object sender, EventArgs e)
        {
            AddFlow1.Redo();
        }

        private void MenuItemEditCut_Click(object sender, EventArgs e)
        {
            MenuItemEditCopy_Click(sender, e);
            MenuItemEditDelete_Click(sender, e);
        }

        private void MenuItemEditCopy_Click(object sender, EventArgs e)
        {
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, null);
            AddFlow1.WriteXml(writer, true, false);
            DataObject data = new DataObject();
            data.SetData("AddFlow.XMLFormat", stream);
            Clipboard.SetDataObject(data, true);
        }

        private void MenuItemEditPaste_Click(object sender, EventArgs e)
        {
            IDataObject data = Clipboard.GetDataObject();
            if (data.GetDataPresent("AddFlow.XMLFormat"))
            {
                // We first unselect the selected items
                AddFlow1.SelectedItems.Clear();

                // Get the data from the clipboard
                MemoryStream stream = (MemoryStream)data.GetData("AddFlow.XMLFormat");

                // We paste the portion of the diagram in our AddFlow control.
                // We group all the load actions in only one action so that we could undo
                // the paste action in one time.
                AddFlow1.BeginAction(1003);

                XmlTextReader reader = new XmlTextReader(stream);
                reader.WhitespaceHandling = WhitespaceHandling.None;
                AddFlow1.ReadXml(reader, true, false, false);
                reader.Close();

                // We move a little each pasted node and link so that they do not recover
                // the original items.
                float dx = AddFlow1.Grid.Size.Width;
                float dy = AddFlow1.Grid.Size.Height;
                foreach (Item item in AddFlow1.SelectedItems)
                {
                    if (item is Node)
                    {
                        Node node = (Node)item;
                        node.Location = new PointF(node.Location.X + dx, node.Location.Y + dy);
                    }
                    else
                    {
                        PointF pt;
                        Link link = (Link)item;

                        if (link.AdjustOrg)
                        {
                            pt = link.Points[0];
                            link.Points[0] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        for (int k = 1; k < link.Points.Count - 1; k++)
                        {
                            pt = link.Points[k];
                            link.Points[k] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                        if (link.AdjustDst)
                        {
                            pt = link.Points[link.Points.Count - 1];
                            link.Points[link.Points.Count - 1] = new PointF(pt.X + dx, pt.Y + dy);
                        }
                    }
                }

                AddFlow1.EndAction();
            }
        }

        private void MenuItemEditDelete_Click(object sender, EventArgs e)
        {
            AddFlow1.DeleteSel();
        }

        private void MenuItemEditSelectAll_Click(object sender, EventArgs e)
        {
            foreach (Item item in AddFlow1.Items)
                item.Selected = true;
        }

        private void MenuItemEditSelectMode_Click(object sender, EventArgs e)
        {
            if (AddFlow1.MouseAction == MouseAction.Selection)
                AddFlow1.MouseAction = MouseAction.None;
            else
                AddFlow1.MouseAction = MouseAction.Selection;
            MenuItemEditSelectMode.Checked = (AddFlow1.MouseAction == MouseAction.Selection);
        }

        private void MenuItemData_Popup(object sender, EventArgs e)
        {
            MenuItemDataNodeCustomData.Enabled = AddFlow1.SelectedItem is Node;
        }

        private void MenuItemDataNodeCustomData_Click(object sender, EventArgs e)
        {
            EnterCustomData();
        }

        private void MenuItemHelpAbout_Click(object sender, EventArgs e)
        {
            MessageBox.Show("AddFlow for WinForms: PropertyBag Sample\r\nCopyright  2003-2011 Lassalle Technologies");
        }

        private void AddFlow1_AfterAddNode(object sender, AfterAddNodeEventArgs e)
        {
            e.Node.Properties["Comment"].Value = "Comment" + e.Node.Index.ToString();
            e.Node.Properties["Author"].Value = "Author...";
            e.Node.Properties["Comment"].Updating += new UpdatingEventHandler(Comment_Updating);
            e.Node.Properties["Author"].Updating += new UpdatingEventHandler(Author_Updating);
        }

        private void AddFlow1_DoubleClick(object sender, EventArgs e)
        {
            EnterCustomData();
        }

        // The WriteMLNodeExtraData event is fired when serializing a node to allow the 
        // serialization of custom data. 
        private void AddFlow1_WriteXMLNodeExtraData(object sender, WriteXMLNodeExtraDataEventArgs e)
        {
            string comment = (string)e.Node.Properties["Comment"].Value;
            string author = (string)e.Node.Properties["Author"].Value;
            if (comment != null && comment.Length > 0)
                e.Writer.WriteElementString("Comment", comment);
            if (author != null && author.Length > 0)
                e.Writer.WriteElementString("Author", author);
        }

        // The ReadXMLNodeExtraData event is fired when de-serializing a node and each time 
        // AddFlow encounters a custom XML element. In this example, there are two possible
        // custom elements: "Author" and "Comment".
        // Notice that each time something is read, the IsRead property is set to true to
        // inform AddFlow that it does not need to read the data itself.
        // If you do not do that, the node de-serialization will not be successfull.
        private void AddFlow1_ReadXMLNodeExtraData(object sender, ReadXMLNodeExtraDataEventArgs e)
        {
            if (e.Reader.Name == "Comment")
                e.Node.Properties["Comment"].Value = e.Reader.ReadElementContentAsString();
            else if (e.Reader.Name == "Author")
                e.Node.Properties["Author"].Value = e.Reader.ReadElementContentAsString();
        }

        private void Comment_Updating(object sender, UpdatingEventArgs e)
        {
            if (e.NewValue.ToString().Length == 0)
                e.Cancel.Cancel = true;
        }

        private void Author_Updating(object sender, UpdatingEventArgs e)
        {
            if (e.NewValue.ToString().Length == 0)
                e.Cancel.Cancel = true;
        }

        void EnterCustomData()
        {
            if (AddFlow1.SelectedItem is Node)
            {
                Node node = (Node)AddFlow1.SelectedItem;
                NodeData nd = new NodeData();
                nd.Node = node;
                if (nd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    AddFlow1.SubmitTask(new MyTask(node));
                    node.Properties["Author"].Value = nd.TextBoxAuthor.Text;
                    node.Properties["Comment"].Value = nd.textBoxComment.Text;
                }
            }
        }

        bool SaveFileDlg()
        {
            SaveFileDialog sfd = new SaveFileDialog();
            if (strFileName.Length > 1)
                sfd.FileName = strFileName;
            else
                sfd.FileName = "*.xml";
            sfd.Filter = "AddFlow Diagrams(*.xml)|*.xml|All Files(*.*)|*.*";
            sfd.InitialDirectory = Application.StartupPath;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                strFileName = sfd.FileName;
                SaveFile();
                MakeCaption();
                return true;
            }
            else
                return false;
        }

        void LoadFile(string strFileName)
        {
            AddFlow1.ResetUndoRedo();
            AddFlow1.Nodes.Clear();
            XmlTextReader reader = new XmlTextReader(strFileName);
            reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
            AddFlow1.ReadXml(reader);
            reader.Close();
            AddFlow1.SetChangedFlag(false);
            this.strFileName = strFileName;
            MakeCaption();
        }

        void SaveFile()
        {
            XmlTextWriter writer = new XmlTextWriter(strFileName, null);
            writer.Formatting = Formatting.Indented;
            AddFlow1.WriteXml(writer);
            writer.Close();
            AddFlow1.SetChangedFlag(false);
        }

        void MakeCaption()
        {
            this.Text = strProgName + " - " + FileTitle();
        }

        string FileTitle()
        {
            if (strFileName.Length > 1)
                return Path.GetFileName(strFileName);
            else
                return "Untitled";
        }

        bool OkToTrash()
        {
            if (!AddFlow1.IsChanged)
                return true;
            DialogResult dr = MessageBox.Show("The diagram in the " + FileTitle() + " file has changed." + "\r\n\r\n" + "Do you want to save the changes?", strProgName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
            switch (dr)
            {
                case DialogResult.Yes:
                    return SaveFileDlg();
                case DialogResult.No:
                    return true;
                case DialogResult.Cancel:
                default:
                    return false;
            }
        }
    }

    internal class MyTask : Task
    {
        Node m_node = null;
        string m_oldAuthor = null;
        string m_oldComment = null;

        private MyTask() { }

        internal MyTask(Node node)
        {       
            this.Code = (Lassalle.Flow.Action)1001;
            m_node = node;
            m_oldAuthor = (string)node.Properties["Author"].Value;
            m_oldComment = (string)node.Properties["Comment"].Value;
        }

        public override void Redo()
        {
            Undo();
        }

        public override void Undo()
        {
            string author = (string)m_node.Properties["Author"].Value;
            string comment = (string)m_node.Properties["Comment"].Value;
            m_node.Properties["Author"].Value = m_oldAuthor;
            m_node.Properties["Comment"].Value = m_oldComment;
            m_oldAuthor = author;
            m_oldComment = comment;
        }
    }
}