//---------------------------------------------------------------------------------------
// Copyright (c) 2003-2006 Lassalle Technologies. All Rights Reserved.
// http://www.lassalle.com
// Author: Patrick Lassalle
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
// ---------------------------------------------------------------------------------------

using System;
using System.Collections;
using System.ComponentModel;

namespace Lassalle.GraphAlgo
{
	/// <summary>Graph is a component that performs some algorithms on a graph.</summary>
	public class Graph : System.ComponentModel.Component
	{  
		private int connectedPartCount;
		private int biConnectedPartCount;
		private int postOrderNumber;
		private int visitedCount;
		private bool isRoot;
		private ArrayList aVertices;
		private ArrayList aEdges;
		private ArrayList aConnectedPartRootVertices;
		private ArrayList aArticulationVertices;

		private System.ComponentModel.Container components = null;

		/// <summary>
		/// Initializes a new instance of the Graph class with default values.
		/// </summary>
		public Graph()
		{
			InitializeComponent();
			Reset();
		}
		
		void Reset()
		{
			postOrderNumber = 0;
			connectedPartCount = 0;
			biConnectedPartCount = 0;
			visitedCount = 0;
			isRoot = false;
			aVertices = new ArrayList();
			aEdges = new ArrayList();
			aConnectedPartRootVertices = new ArrayList();
			aArticulationVertices = new ArrayList();
		}

		/// <summary></summary>
		protected override void Dispose(bool disposing)
		{
			base.Dispose(disposing);
		}
		
		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		/// <summary>Returns the collection of all the vertices.</summary>
		[
		Description("Returns the collection of all the vertices."),
		Browsable(false),
		]
		public VerticesCollection Vertices
		{ 
			get { return new VerticesCollection(aVertices); }
		}
		
		/// <summary>Returns the collection of all the edges.</summary>
		[
		Description("Returns the collection of all the edges."),
		Browsable(false),
		]
		public EdgesCollection Edges
		{ 
			get { return new EdgesCollection(aEdges); }
		}
		
		/// <summary>
		/// Create a vertex and add it to the vertices collection
		/// </summary>
		/// <param name="node">The corresponding "physical node"</param>
		/// <returns>The new created vertex</returns>
		public Vertex AddVertex(object node)
		{                 
			Vertex v = new Vertex();
			// Store a reference to the corresponding node in the vertex
			v.node = node; 
			// Add a reference to this vertex to the array of vertices  
			aVertices.Add(v);
			return v;
		}
		
		/// <summary>
		/// Create an edge and add it to the edges collection
		/// </summary>
		/// <param name="org">Origin vertex of the link to create</param>
		/// <param name="dst">Destination vertex of the link to create</param>
		/// <param name="link">The corresponding "physical link"</param>
		/// <returns>The new created edge</returns>
		public Edge AddEdge(Vertex org, Vertex dst, object link)
		{       
			if (org == null || dst == null)
				return null;   

			Edge e = new Edge(); 
			e.org = org;
			e.dst = dst;
			e.link = link;
    
			// Add the reference to this edge to:
			// - the edge collections of origin vertex
			// - the edge collections of destination vertex
			org.aEdges.Add(e);
			dst.aEdges.Add(e);
			aEdges.Add(e);
			return e;
		}
		
		/// <summary>
		/// Create an edge with a cost and add it to the edges collection
		/// </summary>
		/// <param name="org">Origin vertex of the link to create</param>
		/// <param name="dst">Destination vertex of the link to create</param>
		/// <param name="link">The corresponding "physical link"</param>
		/// <param name="cost">The cost associated to the link</param>
		/// <returns>The new created edge</returns>
		public Edge AddEdge(Vertex org, Vertex dst, object link, int cost)
		{
			Edge e = AddEdge(org, dst, link);
			e.cost = cost;
			return e;
		}
		
		/// <summary>Determine is the graph is directed acyclic</summary>
		/// <returns>True if the graph is directed acyclic</returns>
		public bool IsDirectedAcyclicGraph()
		{
			return !IsCycleDirected();
		}

		/// <summary>Determine is the graph is acyclic</summary>
		/// <returns>True if the graph is acyclic</returns>
		public bool IsAcyclicGraph()
		{
			bool acyclic = true;
			ResetGraph();
			foreach (Vertex v in aVertices)
			{ 
				if (!v.visited)
				{
					if (DFS2(v))
					{
						acyclic = false;
						break;
					}
				}
			}
			return acyclic;
		}

		// Check if a connected part of a graph (defined by one of its vertex) contains a cycle.
		bool IsCycle(Vertex v)
		{
			ResetGraph();
			return DFS2(v);
		}

		// "Depth-first search" algorithm (used for cycle search)
		bool DFS2(Vertex v)
		{
			bool cycle = false;
			v.visited = true;
			foreach (Edge e in v.aEdges)
			{   
				if (!e.visited)
				{
					e.visited = true;      
					Vertex u = (v == e.org) ? e.dst : e.org;
					cycle = u.visited ? true : DFS2(u);
					if (cycle)
						break;
				}
			}
			return cycle;
		}
		
		// Check if the graph contains a directed cycle.
		bool IsCycleDirected()
		{
			bool result = false;

			ResetGraph();

			// Assign postorder numbers
			postOrderNumber = 0;
			foreach (Vertex v in aVertices)
			{ 
				if (!v.visited)
					DFS(v);
			}

			foreach (Vertex v in aVertices)
			{ 
				foreach (Edge e in v.aEdges)
				{   
					if (e.dst.postOrderNumber > e.org.postOrderNumber)
					{
						result = true;
						break;
					}
				}
				if (result)
					break;
			}
			return result;
		}

		// "Depth-first search" algorithm (used for directed cycle search)
		void DFS(Vertex v)
		{
			v.visited = true;
			foreach (Edge e in v.aEdges)
			{   
				if (e.org == v && !e.visited)
				{
					e.visited = true;      
					Vertex u = e.dst;
					if (!u.visited)
						DFS(u);
				}
			}
			v.postOrderNumber = postOrderNumber++;
		}
		
		/// <summary>Separate the connected parts of the graph</summary>
		public void SeparateConnectedParts()
		{	
			ResetGraph();
			connectedPartCount = 0;
			while (visitedCount < aVertices.Count)
			{
				foreach (Vertex v in aVertices)
				{
					if (!v.visited)
					{
						connectedPartCount++;
						aConnectedPartRootVertices.Add(v);
						DFS3(v);
						break;
					}
				}
			}
		}

		// "Depth-first search" algorithm (used for connectivity)
		void DFS3(Vertex v)
		{
			v.visited = true;
			v.connectedPart = connectedPartCount;
			visitedCount++;
			foreach (Edge e in v.aEdges)
			{
				Vertex u = (v == e.org) ? e.dst : e.org;
				if (!u.visited)
					DFS3(u);
			}
		}
		
		/// <summary>Find the articulation points of a graph</summary>
		/// <remarks>
		/// An articulation point in a connected graph is a vertex that, if deleted,
		/// would break the graph into 2 or more pieces.
		/// If there is no articulation point, the graph is said to be biconnected.
		/// </remarks>
		public void ArticulationPoints()
		{
			aArticulationVertices.Clear();
			aConnectedPartRootVertices.Clear();
			SeparateConnectedParts();
			ResetGraph();
			foreach (Vertex root in aConnectedPartRootVertices)
			{
				visitedCount = 0;
				isRoot = true;
				VisitArticulation(root);
			}
		}

		int VisitArticulation(Vertex v)
		{
			bool isRoot2 = isRoot;
			isRoot = false;
			visitedCount++;
			v.visitOrderVertex = visitedCount;
    
			// For each descendant of the current vertex, determines the
			// highest point in the tree reachable from this descendant.
			int min = v.visitOrderVertex;
			int childs = 0;
			foreach (Edge e in v.aEdges)
			{
				Vertex u = (v == e.org) ? e.dst : e.org;
				if (u.visitOrderVertex == 0)
				{
					childs++;
					// If the descendant is not visited, visit it.
					int m = VisitArticulation(u);
					if (m < min)
						min = m;
					// If the minimum value reachable from this descendant is higher
					// up in the tree, then the current vertex is an articulation point.
					if (m >= v.visitOrderVertex)
					{
						v.articulation = true;
						aArticulationVertices.Add(v);
					}
				}
				else
				{
					if (u.visitOrderVertex < min) 
						min = u.visitOrderVertex;
				}
			}
			if (isRoot2 && childs <= 1) 
			{
				v.articulation = false;
				aArticulationVertices.Remove(v);
			}
			return min;
		}
		
		/// <summary>Separate the biconnected parts of the graph</summary>
		public void SeparateBiconnectedParts()
		{
			foreach (Vertex v in aVertices)
				v.visited = false;
			foreach (Edge e in aEdges)
				e.visited = false;
			visitedCount = 0;
			biConnectedPartCount = 0;	

			while (visitedCount < aVertices.Count - aArticulationVertices.Count)
			{
				foreach (Vertex v in aVertices)
				{
					if (!v.articulation && !v.visited)
					{
						biConnectedPartCount++;
						DFS4(v);
						break;
					}
				}
			}
		}
			
		// "Depth-first search" algorithm (used for connectivity)
		void DFS4(Vertex v)
		{
			visitedCount++;
			v.visited = true;
			v.biConnectedPart = biConnectedPartCount;
			foreach (Edge e in v.aEdges)
			{
				Vertex u = (v == e.org) ? e.dst : e.org;
				if (!u.articulation && !u.visited)
					DFS4(u);
			}
		}

		/// <summary>Find a minimal spanning tree for the graph.</summary>
		/// <param name="root">A vertex to start with.</param>
		public void FindSpanningTree(Vertex root)
		{
			if (root == null)
				return;

			ArrayList aCandidates = new ArrayList();
			root.marked = true;
			Vertex vertex = root; // We start with the root node

			do
			{
				// Add the latest vertex's edges to the list of candidates
				foreach (Edge e in vertex.aEdges)
				{
					if (!e.used)
					{
						aCandidates.Add(e);
						e.used = true;
					}
				}

				int best = -1;
				int minCost = int.MaxValue;
				int i = 0;
				while (i < aCandidates.Count)
				{
					Edge e = (Edge)aCandidates[i];
					Vertex v = e.org.marked ? e.dst : e.org;
					if (v.marked)
					{
						// If the edge connects two vertices already in the spanning tree...
						aCandidates.RemoveAt(i);
					}
					else
					{
						if (e.cost < minCost)
						{
							best = i;
							minCost = e.cost;
							vertex = v;
						}
						i++;
					}
				} 

				if (best < 0)
					break;

				Edge e2 = (Edge)aCandidates[best];
				e2.inSpanningTree = true;
				aCandidates.RemoveAt(best);
				vertex.marked = true;
			} while(true);
		}

		/// <summary>
		/// Find all the shortest paths from a single vertex to every other point in the graph
		/// </summary>
		/// <param name="root">The root vertex from which the shorted path tree is determined.</param>
		/// <remarks>This is a "label setting" algorithm.</remarks>
		public void FindShortestPath(Vertex root)
		{
			if (root == null)
				return;

			ArrayList aCandidates = new ArrayList();

			// Start with the roo. We place it in the candidate list.
			aCandidates.Add(root);
			root.dist = 0;
			root.inEdge = null;
			int newDist = 0;

			while (aCandidates.Count > 0)
			{
				int minDist = int.MaxValue;
				int best = -1;
				for (int i = 0; i < aCandidates.Count; i++)
				{
					newDist = ((Vertex)aCandidates[i]).dist;
					if (newDist < minDist)
					{
						best = i;
						minDist = newDist;
					}
				}

				// Add the vertex to the shorted path tree.
				Vertex vertex = (Vertex)aCandidates[best];
				aCandidates.RemoveAt(best);
				vertex.status = ShortedPathStatus.NowInList;

				// Add the latest vertex's edges to the list of candidates
				foreach (Edge e in vertex.aEdges)
				{
					Vertex v = (vertex == e.org) ? e.dst : e.org;
					if (v.status == ShortedPathStatus.NotInList)
					{
						aCandidates.Add(v);
						v.status = ShortedPathStatus.NowInList;
						v.dist = minDist + e.cost;
						v.inEdge = e;
					}
					else if (v.status == ShortedPathStatus.NowInList)
					{
						newDist = minDist + e.cost;
						if (newDist < v.dist)
						{
							v.dist = newDist;
							v.inEdge = e;
						}
					}
				}
			}
			foreach (Vertex v in aVertices)
				if (v.inEdge != null)
					v.inEdge.inPathTree = true;
		}
		
		void ResetGraph()
		{
			visitedCount = 0;
			foreach (Vertex v in aVertices)
			{
				v.visited = false;
				v.articulation = false;
				v.visitOrderVertex = 0;
				v.marked = false;
				v.status = ShortedPathStatus.NotInList;
				v.inEdge = null;
			}
			foreach (Edge e in aEdges)
			{
				e.visited = false;
				e.used = false;
				e.inSpanningTree = false;
				e.inPathTree = false;
			}
		}
	}
}