//---------------------------------------------------------------------------------------
// Copyright (c) 2003-2006 Lassalle Technologies. All Rights Reserved.
// http://www.lassalle.com
// Author: Patrick Lassalle
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
// ---------------------------------------------------------------------------------------

using System;
using System.Configuration;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Printing;
using System.Windows.Forms;
using System.ComponentModel;
using Lassalle.Flow;

namespace Lassalle.PrnFlow
{
	/// <summary>
	/// This component allows printing/print previewing of AddFlow diagrams.
	/// </summary>
	public class PrnFlow : System.ComponentModel.Component
	{
		#region declarations

		private System.ComponentModel.Container components = null;
		private Metafile metafile = null;
		private bool fitToPage = false;
		private int pageNumber = 0;
		private int pagesCount = 0;         
		private int horzPagesCount = 0; 
		private int vertPagesCount = 0;  
		private int startPageNumber = 0;
		private float xOffsetPrint = 0;
		private float yOffsetPrint = 0;
		private PrintDocument prndoc = new PrintDocument();           
		private PageSetupDialog setdlg = new PageSetupDialog();
		private PrintDialog prndlg = new PrintDialog();
		private PrintPreviewDialog predlg = new PrintPreviewDialog();
        
		#endregion // declarations

		#region constructors

		/// <summary>
		/// Initializes a new instance of the PrnFlow class with default values.
		/// </summary>
		public PrnFlow()
		{
			InitializeComponent();
			Reset();
		}

		#endregion //constructors

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		#region Properties
		/// <summary>Determines whether all the diagram is printed 
		/// in one page or several pages.</summary>
		[
		Category("Behavior"),
		Description("Determines whether all the diagram is printed in one page or several pages."),
		DefaultValue(false),
		]
		public bool FitToPage
		{
			set { fitToPage = value; }
			get { return fitToPage; }
		}

		#endregion //Properties

		#region Public Methods

		/// <summary>Print method</summary>
		/// <param name="addflow">A reference to an AddFlow control</param>
		public void Print(AddFlow addflow)
		{
			metafile = addflow.ExportMetafile(false, false, true, true);
			if (fitToPage)
			{
				pagesCount = 1;
				prndlg.AllowSomePages = false;
				if (prndlg.ShowDialog() == DialogResult.OK)
					prndoc.Print();
			}
			else
			{
				pagesCount = GetPageCount(prndoc, metafile);
				prndlg.PrinterSettings.ToPage = pagesCount;
				if (prndlg.ShowDialog() == DialogResult.OK)
				{
					pageNumber = 1;
					xOffsetPrint = 0;
					yOffsetPrint = 0;
					switch (prndlg.PrinterSettings.PrintRange)
					{
						case PrintRange.AllPages:
							startPageNumber = 1;
							break;
						case PrintRange.SomePages:
							startPageNumber = prndlg.PrinterSettings.FromPage;
							pagesCount = prndlg.PrinterSettings.ToPage - startPageNumber + 1;
							break;
					}
					prndoc.Print();
				}
			}
		}

		/// <summary>Preview method.</summary>
		/// <param name="addflow">A reference to an AddFlow control</param>
		public void Preview(AddFlow addflow)
		{                           
			// Export the AddFlow diagram in metafile
			metafile = addflow.ExportMetafile(false, false, true);

			if (!fitToPage)
			{
				// Get the number of pages
				pagesCount = GetPageCount(prndoc, metafile);

				// Initialize some variables
				pageNumber = 1;
				startPageNumber = 1;
				xOffsetPrint = 0;
				yOffsetPrint = 0;
			}

			// Display preview dialog box
			predlg.ShowDialog();
		}

		/// <summary>Page setup method</summary>
		public void PageSetup()
		{
			Margins savedMargins = setdlg.PageSettings.Margins;
			if (IsMetric())
			{
				// Unit conversion (the member Display means hundredths of an inch)
				// This is necessary to fix a bug in the PageSetup dialog box:
				// if you click ok then show again the dialog box, the margins are
				// are changed (10 mm becomes 3.9 mm). Obviously this has something to 
				// do with millimeters to inches conversion (10 / 2.54 = 3.937). 
				setdlg.PageSettings.Margins = PrinterUnitConvert.Convert(
					setdlg.PageSettings.Margins, 
					PrinterUnit.Display, 
					PrinterUnit.TenthsOfAMillimeter);
			}

			if (setdlg.ShowDialog() != DialogResult.OK)
				setdlg.PageSettings.Margins = savedMargins;
		}

		/// <summary>Returns the number of pages</summary>
		/// <param name="addflow">A reference to an AddFlow control</param>
		/// <returns>The number of pages</returns>
		[
		Description("Returns the number of pages."),
		]
		public int PageCount(AddFlow addflow)
		{
			metafile = addflow.ExportMetafile(false, false, true, true);
			return GetPageCount(prndoc, metafile);
		}

		/// <summary>Returns the number of pages and obtain also 
		/// the horizontal and vertical numbers of pages.</summary>
		/// <param name="addflow">A reference to an AddFlow control</param>
		/// <param name="horzPages">the horizontal number of pages</param>
		/// <param name="vertPages">the vertical number of pages</param>
		/// <returns>The number of pages</returns>
		[
		Description("Returns the horizontal and vertical numbers of pages."),
		]
		public int GetHorzVertPageCount(AddFlow addflow, ref int horzPages, ref int vertPages)
		{
			metafile = addflow.ExportMetafile(false, false, true);
			int n = GetPageCount(prndoc, metafile);
			horzPages = horzPagesCount;
			vertPages = vertPagesCount;
			return n;    
		}

		/// <summary>Returns the page size</summary>
		/// <param name="addflow">A reference to an AddFlow control</param>
		[
		Description("Returns the page size."),
		]
		public SizeF GetPageSize(AddFlow addflow)
		{
			metafile = addflow.ExportMetafile(false, false, true, true);
			return GetPageSize(prndoc, metafile);
		}

		#endregion // Public Methods

		#region Implementation
        
		void Reset()
		{
			prndoc.PrintPage += new PrintPageEventHandler(OnPrintPage); 
			setdlg.Document = prndoc;
			predlg.Document = prndoc;
			prndlg.Document = prndoc;
			prndlg.AllowSomePages = true;
			prndlg.PrinterSettings.FromPage = 1;
			prndlg.PrinterSettings.ToPage = prndlg.PrinterSettings.MaximumPage;
			fitToPage = false;
		}

		void OnPrintPage(object obj, PrintPageEventArgs ppea)
		{
			if (fitToPage)
			{
				Graphics grfx = ppea.Graphics;
				RectangleF rcPrintArea;

				if (grfx.VisibleClipBounds.X < 0)
					rcPrintArea = ppea.MarginBounds;
				else
					// Calculation of the display rectangle taking account of the 
					// margins in hundredths of an inch.
					rcPrintArea = new RectangleF(
						ppea.MarginBounds.Left -
						(ppea.PageBounds.Width - grfx.VisibleClipBounds.Width) / 2,
						ppea.MarginBounds.Top -
						(ppea.PageBounds.Height - grfx.VisibleClipBounds.Height) / 2,
						ppea.MarginBounds.Width,
						ppea.MarginBounds.Height);

				ScaleImageIsotropically(grfx, metafile, rcPrintArea);

				// Draw borders
				Rectangle borderRect = Rectangle.Round(rcPrintArea);
				borderRect.Inflate(1, 1);
				Pen pen = new Pen(Color.Black);
				pen.DashStyle = DashStyle.Dash;
				grfx.DrawRectangle(pen, borderRect);

				ppea.HasMorePages = false;
			}
			else
			{
				float dpix = metafile.GetMetafileHeader().DpiX;
				float dpiy = metafile.GetMetafileHeader().DpiY;
				if (pageNumber >= startPageNumber)
				{
					Graphics grfx = ppea.Graphics;
					RectangleF rcPrintArea;

					if (grfx.VisibleClipBounds.X < 0)
						rcPrintArea = ppea.MarginBounds;
					else
						// Calculation of the display rectangle taking account of the 
						// margins in hundredths of an inch.
						rcPrintArea = new RectangleF(
							ppea.MarginBounds.Left -
							(ppea.PageBounds.Width - grfx.VisibleClipBounds.Width) / 2,
							ppea.MarginBounds.Top -
							(ppea.PageBounds.Height - grfx.VisibleClipBounds.Height) / 2,
							ppea.MarginBounds.Width,
							ppea.MarginBounds.Height);

					// Rectangular portion of the metafile to print in pixels
					RectangleF rcSrc = new RectangleF(xOffsetPrint, yOffsetPrint,
						rcPrintArea.Width, rcPrintArea.Height);
					rcSrc.X = rcSrc.X * dpix / 100;
					rcSrc.Y = rcSrc.Y * dpiy / 100;
					rcSrc.Width = rcSrc.Width * dpix / 100;
					rcSrc.Height = rcSrc.Height * dpiy / 100;

					// Draw portion of metafile
					grfx.DrawImage(metafile, rcPrintArea, rcSrc, GraphicsUnit.Pixel);

					// Draw borders
					Rectangle borderRect = Rectangle.Round(rcPrintArea);
					borderRect.Inflate(1, 1);
					Pen pen = new Pen(Color.Black);
					pen.DashStyle = DashStyle.Dash;
					grfx.DrawRectangle(pen, borderRect);

					// Print page number
					StringFormat strfmt = new StringFormat();
					strfmt.Alignment = StringAlignment.Center;
					string sPage = "- page " + pageNumber + " -";
					Font font = new Font("Times New Roman", 12);
					grfx.DrawString(sPage, font, Brushes.Black,
						borderRect.Left + borderRect.Width / 2, borderRect.Bottom + 4, strfmt);
				}

				pageNumber++;
				ppea.HasMorePages = (pageNumber < startPageNumber + pagesCount);
				if (ppea.HasMorePages)
				{
					int widthMfInch = (int)(metafile.Width * 100 / dpix);
					int heightMfInch = (int)(metafile.Height * 100 / dpiy);
					if (xOffsetPrint + ppea.MarginBounds.Width < widthMfInch)
					{
						xOffsetPrint += ppea.MarginBounds.Width;
					}
					else if (yOffsetPrint + ppea.MarginBounds.Height < heightMfInch)
					{
						xOffsetPrint = 0;
						yOffsetPrint += ppea.MarginBounds.Height;
					}
				}
				else
				{
					pageNumber = 1;
					startPageNumber = 1;
					xOffsetPrint = 0;
					yOffsetPrint = 0;
					pagesCount = GetPageCount(prndoc, metafile);
				}
			}
		}

		int GetPageCount(PrintDocument prndoc, Metafile metafile)
		{
			horzPagesCount = 0; 
			vertPagesCount = 0;                 
			float dpix = metafile.GetMetafileHeader().DpiX;
			float dpiy = metafile.GetMetafileHeader().DpiY;
			int widthMfInch = (int)(metafile.Width * 100 / dpix);
			int heightMfInch = (int)(metafile.Height * 100 / dpiy);
			Margins margins = setdlg.PageSettings.Margins;
			Rectangle bounds = setdlg.PageSettings.Bounds;
			int cx = bounds.Width - margins.Left - margins.Right;
			int cy = bounds.Height - margins.Top - margins.Bottom;
			if (cx != 0 && cy != 0)
			{
				int x = widthMfInch / cx;
				if (widthMfInch % cx > 0)
					x++;
				int y = heightMfInch / cy;
				if (heightMfInch % cy > 0)
					y++;
				horzPagesCount = x;
				vertPagesCount = y;
			}
			return horzPagesCount * vertPagesCount;
		}

		SizeF GetPageSize(PrintDocument prndoc, Metafile metafile)
		{
			Margins margins = setdlg.PageSettings.Margins;
			Rectangle bounds = setdlg.PageSettings.Bounds;

			// Page size (in 1/100 inch)
			float cx = (float)(bounds.Width - margins.Left - margins.Right);
			float cy = (float)(bounds.Height - margins.Top - margins.Bottom);

			// Page size in pixels
			float dpix = metafile.GetMetafileHeader().DpiX;
			float dpiy = metafile.GetMetafileHeader().DpiY;
			cx = cx * dpix / 100;
			cy = cy * dpiy / 100;

			return new SizeF(cx, cy);
		}
		
		private void ScaleImageIsotropically(Graphics grfx, Metafile metafile, RectangleF rc)
		{
			SizeF size = new SizeF(metafile.Width, metafile.Height);
			float scale = Math.Min(rc.Width/size.Width, rc.Height/size.Height);
			size.Width *= scale;
			size.Height *= scale;
			grfx.DrawImage(metafile, rc.X + (rc.Width - size.Width)/2,
				rc.Y + (rc.Height - size.Height)/2,
				size.Width, size.Height);
		}

		// Returns true is the user is currently working in metric measurement system 
		// and false for US system. 
		private static bool IsMetric()
		{
			// Get the measurement system for the current user. We have to go through
			// API here, because the System.Globalization.RegionInfo.CurrentRegion.IsMetric returns
			// a value for the "whole" region, but the user could specificaly set measurement
			// system to other than the default for the given region.
			System.Text.StringBuilder info = new System.Text.StringBuilder(10);
			int result = GetLocaleInfo(GetUserDefaultLCID(), LOCALE_IMEASURE, info, info.Capacity);
			if (result == 0)
				return System.Globalization.RegionInfo.CurrentRegion.IsMetric;
			else
				return info.ToString() == "0";
		}

		private const int LOCALE_IMEASURE = 0; // 0 = metric, 1 = US

		[System.Runtime.InteropServices.DllImport("Kernel32")] 
		private extern static int GetUserDefaultLCID();

		[System.Runtime.InteropServices.DllImport("Kernel32", CharSet=System.Runtime.InteropServices.CharSet.Auto, SetLastError=true)]
		private extern static int GetLocaleInfo(int lcid, int lctype, System.Text.StringBuilder lcdata, int cchData);

		#endregion // Implementation
	}
}
