//---------------------------------------------------------------------------------------
// XMLFlow component.
// Copyright (c) 2003-2006 Lassalle Technologies. All Rights Reserved.
// http://www.lassalle.com
// Author: Patrick Lassalle
// THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND EITHER EXPRESSED OR IMPLIED.
// ---------------------------------------------------------------------------------------

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Collections;
using System.Windows.Forms;
using System.Xml;
using Lassalle.Flow;

namespace Lassalle.XMLFlow
{
    /// <summary>
    /// This class allows to save/load an AddFlow diagram in a XML stream. 
    /// It has just 2 static methods, each coming in several versions.
    /// </summary>
    public class Serial
    {
        private Serial() {}

        #region Public Methods

        /// <summary>
        /// Save an AddFlow diagram in a XML stream.
        /// </summary>
        /// <param name="tw">Represents a XML writer</param>
        /// <param name="addflow">A reference to the AddFlow control</param>
        /// <remarks>All the nodes and the links of the diagram are saved.
        /// However the collection of images is not saved.</remarks>
        public static void FlowToXML(XmlTextWriter tw, AddFlow addflow)
        {
            FlowToXML2(tw, addflow, false, false);
        }

        /// <summary>
        /// Save an AddFlow diagram or a portion of it in a XML stream.
        /// </summary>
        /// <param name="tw">Represents a XML writer</param>
        /// <param name="addflow">A reference to the AddFlow control</param>
        /// <param name="selected">Indicates if only the selected items are saved</param>
        /// <param name="saveImages">Indicates if the collection of images is saved</param>
        public static void FlowToXML(XmlTextWriter tw, AddFlow addflow, bool selected, 
                                                                        bool saveImages)
        {       
            FlowToXML2(tw, addflow, selected, saveImages);
        }        

        /// <summary>
        /// Load an AddFlow diagram from a XML stream.
        /// </summary>
        /// <param name="tr">Represents a XML reader</param>
        /// <param name="addflow">A reference to the AddFlow control</param>
        /// <remarks>
        /// <list type="number">
        /// <item>Here no image is loaded. The Images collection of AddFlow 
        /// will not change.</item>
        /// <item>The default values for the node and the link properties defined in the 
        /// XML file will be also the default values of the AddFlow control...</item>
        /// </list>
        /// </remarks>
        public static void XMLToFlow(XmlTextReader tr, AddFlow addflow)
        {
            XMLToFlow2(tr, addflow, false, false, true);
        }

        /// <summary>
        /// Load an AddFlow diagram from a XML stream.
        /// </summary>
        /// <param name="tr">Represents a XML reader</param>
        /// <param name="addflow">A reference to the AddFlow control</param>
        /// <param name="selected">Indicates if the loaded items will be selected</param>
        /// <param name="loadImages">Indicates if the collection of images used
        ///  by AddFlow is stored in the XML file</param>
        /// <remarks>The default values for the node and the link properties defined in the 
        /// XML file will be also the default values of the AddFlow control...</remarks>
        public static void XMLToFlow(XmlTextReader tr, AddFlow addflow, 
                                                bool selected, bool loadImages)
        {     
            XMLToFlow2(tr, addflow, selected, loadImages, true);
        }

        /// <summary>
        /// Load an AddFlow diagram from a XML stream.
        /// </summary>
        /// <param name="tr">Represents a XML reader</param>
        /// <param name="addflow">A reference to the AddFlow control</param>
        /// <param name="selected">Indicates if the loaded items will be selected</param>
        /// <param name="loadImages">Indicates if the collection of images used
        ///  by AddFlow is stored in the XML file</param>
        /// <param name="acceptDefaultValues">Indicates if the default values for the node 
        /// and the link properties defined in the XML file will be also the default values 
        /// of the AddFlow control, or if these default values are just used during the
        /// load operation.</param>
        public static void XMLToFlow(XmlTextReader tr, AddFlow addflow, 
            bool selected, bool loadImages, bool acceptDefaultValues)
        {     
            XMLToFlow2(tr, addflow, selected, loadImages, acceptDefaultValues);
        }

        #endregion // Public Methods

        #region Saving to XML

        static void FlowToXML2(XmlTextWriter tw, AddFlow af, bool selected, bool saveImages)
        {      
            tw.Formatting = Formatting.Indented;
            tw.WriteStartDocument();
            tw.WriteComment("AddFlow.net diagram");
            tw.WriteStartElement("AddFlow");
                        
            // Fill the header with the number of nodes and the number of links
            // (This information will not used when loading the diagram)
            FillHeader(tw, af, selected);

            // Version number of AddFlow
            // (This information will not used when loading the diagram)
            tw.WriteElementString("Version", af.ProductVersion);

            // 1. Default node properties
			if (af.ShouldSerializeDefNodeProp())
				DefaultNodeToXML(tw, af.DefNodeProp);
            
			// 2. Default link properties
			if (af.ShouldSerializeDefLinkProp())
				DefaultLinkToXML(tw, af.DefLinkProp);

            // 3) Pictures collection
            if (saveImages)
            {
                foreach (FlowImage image in af.Images)
                    ImageToXML(tw, image.Image);
            }

            // 4) Nodes and links
            // First we create and fill an array with all the involved items
            // Then we XMLize all items.
            IList items;
            if (selected)
                items = af.SelectedItems;
            else
                items = af.Items;

            // XMLize all items
            foreach (Item item in items)
            {
                int index = items.IndexOf(item);
                if (item is Node)
                {
                    Node node = (Node)item;
                    int indexParent = (node.Parent != null) ? items.IndexOf(node.Parent) : -1;
                    NodeToXML(tw, node, index, indexParent);
                }
                else if (item is Link)
                {
                    Link link = (Link)item;
                    int idOrg = items.IndexOf(link.Org);
                    int idDst = items.IndexOf(link.Dst);
                    if (idOrg >= 0 && idDst >= 0)
                        LinkToXML(tw, link, idOrg, idDst, index);
                }
            }

            tw.WriteEndElement();
            tw.WriteEndDocument();
            tw.Flush();
        }

        static void FillHeader(XmlTextWriter tw, AddFlow af, bool selected)
        {            
            int nodes = 0;
            int links = 0;
            if (selected)
            {
                foreach (Item item in af.SelectedItems)
                {
                    if (item is Node)
                        nodes++;
                    else if (item is Link)
                    {
                        Link link = (Link)item;
                        if (link.Org.Selected && link.Dst.Selected)
                            links++;
                    }
                }
            }
            else
            {
                nodes = af.Nodes.Count; 
                foreach (Node node in af.Nodes)
                    links += node.OutLinks.Count;
            }
            tw.WriteAttributeString("Nodes", XmlConvert.ToString(nodes));
            tw.WriteAttributeString("Links", XmlConvert.ToString(links));
        }

        // DEFAULT PROPERTIES
        // The following function saves the default property values of the control.
        // This is useful to avoid saving all property values of each node and each link.
        //
        // For instance, it will not be necessary to save for each node the value of
        // its Hidden property except if it does not follow the default behaviour.
        // If by default, nodes are not hidden, then if a node is hidden, we will add 
        // <Hidden>True</Hidden> in the xml description of that node.
        // If the node is not hidden, we will not need to add <Hidden>False</Hidden> 
        // since this is the default behaviour in that case.
		//
		// Moreover, we will save the default property values only if necessary (only if 
		// distinct from the default values)

        static void DefaultNodeToXML(XmlTextWriter tw, Node dnp)
        {
			tw.WriteStartElement("DefaultNode");
			if (dnp.ShouldSerializeFont())
				FontToXML(tw, dnp.Font);
			if (dnp.ShouldSerializeFillColor())
				tw.WriteElementString("FillColor", dnp.FillColor.ToArgb().ToString());
			if (dnp.ShouldSerializeDrawColor())
				tw.WriteElementString("DrawColor", dnp.DrawColor.ToArgb().ToString());
			if (dnp.ShouldSerializeTextColor())
				tw.WriteElementString("TextColor", dnp.TextColor.ToArgb().ToString());
			if (dnp.ShouldSerializeGradientColor())
				tw.WriteElementString("GradientColor", dnp.GradientColor.ToArgb().ToString());
			if (dnp.ShouldSerializeShape())
			{
				tw.WriteStartElement("Shape");
				tw.WriteAttributeString("Type", dnp.Shape.Style.ToString());
				tw.WriteAttributeString("Orientation", dnp.Shape.Orientation.ToString());
				tw.WriteEndElement();
			}
			if (dnp.ShouldSerializeShadow())
			{
				tw.WriteStartElement("Shadow");
				tw.WriteAttributeString("Style", dnp.Shadow.Style.ToString());
				tw.WriteAttributeString("Color", dnp.Shadow.Color.ToArgb().ToString());
				tw.WriteAttributeString("Width", dnp.Shadow.Size.Width.ToString());
				tw.WriteAttributeString("Height", dnp.Shadow.Size.Height.ToString());
				tw.WriteEndElement();
			}
			if (dnp.AutoSize != AutoSize.None)
				tw.WriteElementString("AutoSize", dnp.AutoSize.ToString());
			if (dnp.Alignment != Alignment.CenterMIDDLE)
				tw.WriteElementString("Alignment", dnp.Alignment.ToString());
			if (dnp.DrawWidth != 1)
				tw.WriteElementString("DrawWidth", dnp.DrawWidth.ToString());
			if (dnp.DashStyle != DashStyle.Solid)
                tw.WriteElementString("DashStyle", dnp.DashStyle.ToString());
            if (dnp.AttachmentStyle != AttachmentStyle.Item)
                tw.WriteElementString("AttachmentStyle", dnp.AttachmentStyle.ToString());
            if (dnp.Dock != DockStyle.None)
                tw.WriteElementString("Dock", dnp.Dock.ToString());
			if (dnp.ShouldSerializeTextMargin())
			{
				tw.WriteStartElement("TextMargin"); 
				tw.WriteAttributeString("Width", dnp.TextMargin.Width.ToString());
				tw.WriteAttributeString("Height", dnp.TextMargin.Height.ToString());
				tw.WriteEndElement(); 
			}
			if (dnp.BackMode != BackMode.Transparent)
				tw.WriteElementString("BackMode", dnp.BackMode.ToString()); 
			if (dnp.Trimming != StringTrimming.None)
				tw.WriteElementString("Trimming", dnp.Trimming.ToString());
			if (dnp.GradientMode != LinearGradientMode.BackwardDiagonal)
				tw.WriteElementString("GradientMode", dnp.GradientMode.ToString());
			if (dnp.Hidden)
				tw.WriteElementString("Hidden", "True");
			if (dnp.Transparent)
				tw.WriteElementString("Transparent", "True");
			if (dnp.Gradient)
				tw.WriteElementString("Gradient", "True");
			if (!dnp.Logical)
				tw.WriteElementString("Logical", "False");
			if (!dnp.Selectable)
				tw.WriteElementString("Selectable", "False");
			if (dnp.OwnerDraw)
				tw.WriteElementString("OwnerDraw", "True");
			if (!dnp.XMoveable)
				tw.WriteElementString("XMoveable", "False");
			if (!dnp.YMoveable)
				tw.WriteElementString("YMoveable", "False");
			if (!dnp.XSizeable)
				tw.WriteElementString("XSizeable", "False");
			if (!dnp.YSizeable)
				tw.WriteElementString("YSizeable", "False");
			if (!dnp.InLinkable)
				tw.WriteElementString("InLinkable", "False");
			if (!dnp.OutLinkable)
				tw.WriteElementString("OutLinkable", "False");
			if (!dnp.LabelEdit)
                tw.WriteElementString("LabelEdit", "False");
            if (!dnp.HighlightChildren)
                tw.WriteElementString("HighlightChildren", "False");
            if (!dnp.RemoveChildren)
                tw.WriteElementString("RemoveChildren", "False");
			if (dnp.ImageIndex != -1)
				tw.WriteElementString("ImageIndex", dnp.ImageIndex.ToString());
			if (dnp.ShouldSerializeImageLocation())
			{
				tw.WriteStartElement("ImageLocation");  
				tw.WriteAttributeString("X", XmlConvert.ToString(dnp.ImageLocation.X));
				tw.WriteAttributeString("Y", XmlConvert.ToString(dnp.ImageLocation.Y));
				tw.WriteEndElement();
			}
			if (dnp.Tooltip != null && dnp.Tooltip.Length > 0)
				tw.WriteElementString("Tooltip", dnp.Tooltip);
			if (dnp.Text != null && dnp.Text.Length > 0)
				tw.WriteElementString("Text", dnp.Text);
			if (dnp.Url != null && dnp.Url.Length > 0)
				tw.WriteElementString("Url", dnp.Url);
            tw.WriteEndElement();
        }
        
        static void DefaultLinkToXML(XmlTextWriter tw, Link dlp)
        {
            tw.WriteStartElement("DefaultLink");
			if (dlp.ShouldSerializeFont())
				FontToXML(tw, dlp.Font);
			if (dlp.ShouldSerializeDrawColor())
				tw.WriteElementString("DrawColor", dlp.DrawColor.ToArgb().ToString());
			if (dlp.ShouldSerializeTextColor())
				tw.WriteElementString("TextColor", dlp.TextColor.ToArgb().ToString());

            // Styles                
            if (dlp.ConnectionStyleOrg != ConnectionStyle.EveryWhere)
                tw.WriteElementString("ConnectionStyleOrg", dlp.ConnectionStyleOrg.ToString());
            if (dlp.ConnectionStyleDst != ConnectionStyle.EveryWhere)
                tw.WriteElementString("ConnectionStyleDst", dlp.ConnectionStyleDst.ToString());
			if (dlp.Jump != Jump.None)
				tw.WriteElementString("Jump", dlp.Jump.ToString());
			if (dlp.BackMode != BackMode.Transparent)
				tw.WriteElementString("BackMode", dlp.BackMode.ToString()); 
			if (dlp.DrawWidth != 1)
				tw.WriteElementString("DrawWidth", dlp.DrawWidth.ToString());
			if (dlp.DashStyle != DashStyle.Solid)
				tw.WriteElementString("DashStyle", dlp.DashStyle.ToString()); 
			if (dlp.StartCap != LineCap.Flat)
				tw.WriteElementString("StartCap", dlp.StartCap.ToString());
			if (dlp.EndCap != LineCap.Flat)
                tw.WriteElementString("EndCap", dlp.EndCap.ToString());
            if (dlp.ShouldSerializeShadow())
            {
                tw.WriteStartElement("Shadow");
                tw.WriteAttributeString("Style", dlp.Shadow.Style.ToString());
                tw.WriteAttributeString("Color", dlp.Shadow.Color.ToArgb().ToString());
                tw.WriteAttributeString("Width", dlp.Shadow.Size.Width.ToString());
                tw.WriteAttributeString("Height", dlp.Shadow.Size.Height.ToString());
                tw.WriteEndElement();
            } 
			if  (dlp.ShouldSerializeLine())
			{
				tw.WriteStartElement("Line"); 
				tw.WriteAttributeString("Style", dlp.Line.Style.ToString());
				tw.WriteAttributeString("OrthogonalDynamic", dlp.Line.OrthogonalDynamic ? 
					"True" : "False");
				tw.WriteAttributeString("RoundedCorner", dlp.Line.RoundedCorner ? 
					"True" : "False");
				tw.WriteEndElement();
			}             
			if (dlp.ShouldSerializeArrowDst())
			{
				tw.WriteStartElement("ArrowDst");
				tw.WriteAttributeString("Head", dlp.ArrowDst.Style.ToString());
				tw.WriteAttributeString("Size", dlp.ArrowDst.Size.ToString());
				tw.WriteAttributeString("Angle", dlp.ArrowDst.Angle.ToString());
				tw.WriteAttributeString("Filled", dlp.ArrowDst.Filled ? "True" : "False");
				tw.WriteEndElement();       
			}
			if (dlp.ShouldSerializeArrowOrg())
			{
				tw.WriteStartElement("ArrowOrg");
				tw.WriteAttributeString("Head", dlp.ArrowOrg.Style.ToString());
				tw.WriteAttributeString("Size", dlp.ArrowOrg.Size.ToString());
				tw.WriteAttributeString("Angle", dlp.ArrowOrg.Angle.ToString());
				tw.WriteAttributeString("Filled", dlp.ArrowOrg.Filled ? "True" : "False");
				tw.WriteEndElement();    
			}
			if (dlp.ShouldSerializeArrowMid())
			{
				tw.WriteStartElement("ArrowMid");
				tw.WriteAttributeString("Head", dlp.ArrowMid.Style.ToString());
				tw.WriteAttributeString("Size", dlp.ArrowMid.Size.ToString());
				tw.WriteAttributeString("Angle", dlp.ArrowMid.Angle.ToString());
				tw.WriteAttributeString("Filled", dlp.ArrowMid.Filled ? "True" : "False");
				tw.WriteEndElement();
			}
			if (dlp.Hidden)
				tw.WriteElementString("Hidden", "True");
			if (!dlp.Logical)
				tw.WriteElementString("Logical", "False");
			if (!dlp.Selectable)
				tw.WriteElementString("Selectable", "False");
			if (dlp.OwnerDraw)
				tw.WriteElementString("OwnerDraw", "True");
			if (dlp.OrientedText)
				tw.WriteElementString("OrientedText", "True");
			if (dlp.Line.OrthogonalDynamic)
				tw.WriteElementString("OrthogonalDynamic", "True");
			if (dlp.Line.RoundedCorner)
				tw.WriteElementString("RoundedCorner", "True");
			if (dlp.Rigid)
				tw.WriteElementString("Rigid", "True");
			if (dlp.AdjustOrg)
				tw.WriteElementString("AdjustOrg", "True");
			if (dlp.AdjustDst)
				tw.WriteElementString("AdjustDst", "True");
			if (!dlp.Stretchable)
                tw.WriteElementString("Stretchable", "False");
            if (!dlp.HighlightChildren)
                tw.WriteElementString("HighlightChildren", "False");
            if (!dlp.RemoveChildren)
                tw.WriteElementString("RemoveChildren", "False");
			if (dlp.Tooltip != null && dlp.Tooltip.Length > 0)
				tw.WriteElementString("Tooltip", dlp.Tooltip);
			if (dlp.Text != null && dlp.Text.Length > 0)
				tw.WriteElementString("Text", dlp.Text);
			if (dlp.Url != null && dlp.Url.Length > 0)
				tw.WriteElementString("Url", dlp.Url);
            tw.WriteEndElement();
        }

        static void NodeToXML(XmlTextWriter tw, Node node, int index, int indexParent)
        {
            tw.WriteStartElement("Node");
            RectangleF rc = node.Rect;

            // Attributes
            tw.WriteAttributeString("Index", XmlConvert.ToString(index));
            tw.WriteAttributeString("Left", XmlConvert.ToString(rc.Left));
            tw.WriteAttributeString("Top", XmlConvert.ToString(rc.Top));
            tw.WriteAttributeString("Width", XmlConvert.ToString(rc.Width));
            tw.WriteAttributeString("Height", XmlConvert.ToString(rc.Height));
            
            // Flags
            if (node.ShouldSerializeHidden())
                tw.WriteElementString("Hidden", node.Hidden ? "True" : "False");
            if (node.ShouldSerializeSelectable())
                tw.WriteElementString("Selectable", node.Selectable ? "True" : "False");
            if (node.ShouldSerializeLogical())
                tw.WriteElementString("Logical", node.Logical ? "True" : "False");
            if (node.ShouldSerializeOwnerDraw())
                tw.WriteElementString("OwnerDraw", node.OwnerDraw ? "True" : "False");
            if (node.ShouldSerializeTransparent())
				tw.WriteElementString("Transparent", node.Transparent ? "True" : "False");
			if (node.ShouldSerializeGradient())
				tw.WriteElementString("Gradient", node.Gradient ? "True" : "False");
            if (node.ShouldSerializeXMoveable())
                tw.WriteElementString("XMoveable", node.XMoveable ? "True" : "False");
            if (node.ShouldSerializeYMoveable())
                tw.WriteElementString("YMoveable", node.YMoveable ? "True" : "False");
            if (node.ShouldSerializeXSizeable())
                tw.WriteElementString("XSizeable", node.XSizeable ? "True" : "False");
            if (node.ShouldSerializeYSizeable())
				tw.WriteElementString("YSizeable", node.YSizeable ? "True" : "False");
			if (node.ShouldSerializeInLinkable())
				tw.WriteElementString("InLinkable", node.InLinkable ? "True" : "False");
			if (node.ShouldSerializeOutLinkable())
				tw.WriteElementString("OutLinkable", node.OutLinkable ? "True" : "False");
            if (node.ShouldSerializeLabelEdit())
                tw.WriteElementString("LabelEdit", node.LabelEdit ? "True" : "False");
            if (node.ShouldSerializeHighlightChildren())
                tw.WriteElementString("HighlightChildren", node.HighlightChildren ? "True" : "False");
            if (node.ShouldSerializeRemoveChildren())
                tw.WriteElementString("RemoveChildren", node.RemoveChildren ? "True" : "False");  
            if (node.ShouldSerializeBackMode())
                tw.WriteElementString("BackMode", node.BackMode.ToString()); 

            // Styles                 
            if (node.ShouldSerializeShadow())
            {
                tw.WriteStartElement("Shadow"); 
                tw.WriteAttributeString("Style", node.Shadow.Style.ToString());
                tw.WriteAttributeString("Color", node.Shadow.Color.ToArgb().ToString());
                tw.WriteAttributeString("Width", node.Shadow.Size.Width.ToString());
                tw.WriteAttributeString("Height", node.Shadow.Size.Height.ToString());
                tw.WriteEndElement();
            }
            if  (node.ShouldSerializeShape())
            {
                tw.WriteStartElement("Shape"); 
                tw.WriteAttributeString("Style", node.Shape.Style.ToString());
                tw.WriteAttributeString("Orientation", node.Shape.Orientation.ToString());
                tw.WriteEndElement();
            }           
            if (node.ShouldSerializeAutoSize())
                tw.WriteElementString("AutoSize", node.AutoSize.ToString());
            if (node.ShouldSerializeAlignment())
                tw.WriteElementString("Alignment", node.Alignment.ToString());
            if (node.ShouldSerializeImagePosition())
                tw.WriteElementString("ImagePosition", node.ImagePosition.ToString());
            if (node.ShouldSerializeTrimming())
				tw.WriteElementString("Trimming", node.Trimming.ToString());
			if (node.ShouldSerializeGradientMode())
				tw.WriteElementString("GradientMode", node.GradientMode.ToString());
            if (node.ShouldSerializeDashStyle())
                tw.WriteElementString("DashStyle", node.DashStyle.ToString());
            if (node.ShouldSerializeDrawWidth())
                tw.WriteElementString("DrawWidth", node.DrawWidth.ToString());
            if (node.ShouldSerializeAttachmentStyle())
                tw.WriteElementString("AttachmentStyle", node.AttachmentStyle.ToString());
            if (node.ShouldSerializeDock())
                tw.WriteElementString("Dock", node.Dock.ToString());
            if (node.ShouldSerializeTextMargin())
            {
                tw.WriteStartElement("TextMargin"); 
                tw.WriteAttributeString("Width", node.TextMargin.Width.ToString());
                tw.WriteAttributeString("Height", node.TextMargin.Height.ToString());
                tw.WriteEndElement();
            }           

            // Colors
            if (node.ShouldSerializeFillColor())
                tw.WriteElementString("FillColor", node.FillColor.ToArgb().ToString());
            if (node.ShouldSerializeDrawColor())
                tw.WriteElementString("DrawColor", node.DrawColor.ToArgb().ToString());
            if (node.ShouldSerializeTextColor())
				tw.WriteElementString("TextColor", node.TextColor.ToArgb().ToString());
			if (node.ShouldSerializeGradientColor())
				tw.WriteElementString("GradientColor", node.GradientColor.ToArgb().ToString());

            // User data
            if (node.Text != null && node.Text.Length > 0)
				tw.WriteElementString("Text", node.Text);
			if (node.Tooltip != null && node.Tooltip.Length > 0)
				tw.WriteElementString("Tooltip", node.Tooltip);
			if (node.Url != null && node.Url.Length > 0)
				tw.WriteElementString("Url", node.Url);

            // Font
            if (node.ShouldSerializeFont())
                FontToXML(tw, node.Font);

            // ImageIndex            
            if (node.ImageIndex >= 0 && node.ShouldSerializeImageIndex())
                tw.WriteElementString("ImageIndex", node.ImageIndex.ToString());

			// ImageLocation
			if (node.ShouldSerializeImageLocation())
			{
				tw.WriteStartElement("ImageLocation");  
				tw.WriteAttributeString("X", XmlConvert.ToString(node.ImageLocation.X));
				tw.WriteAttributeString("Y", XmlConvert.ToString(node.ImageLocation.Y));
				tw.WriteEndElement();
			}

            // Parent link
            if (indexParent != -1)
                tw.WriteElementString("Parent", XmlConvert.ToString(indexParent));

            tw.WriteEndElement();
        }
        
        static void LinkToXML(XmlTextWriter tw, Link link, int idOrg, int idDst, int index)
        {
            tw.WriteStartElement("Link");

            // Attributes
            tw.WriteAttributeString("Index", XmlConvert.ToString(index));
            tw.WriteAttributeString("Org", idOrg.ToString());
            tw.WriteAttributeString("Dst", idDst.ToString());

            // Flags
            if (link.ShouldSerializeHidden())
                tw.WriteElementString("Hidden", link.Hidden ? "True" : "False");
            if (link.ShouldSerializeSelectable())
                tw.WriteElementString("Selectable", link.Selectable ? "True" : "False");
            if (link.ShouldSerializeLogical())
                tw.WriteElementString("Logical", link.Logical ? "True" : "False");
            if (link.ShouldSerializeOwnerDraw())
                tw.WriteElementString("OwnerDraw", link.OwnerDraw ? "True" : "False");
            if (link.ShouldSerializeOrientedText())
                tw.WriteElementString("OrientedText", link.OrientedText ? "True" : "False");
            if (link.ShouldSerializeRigid())
                tw.WriteElementString("Rigid", link.Rigid ? "True" : "False");
            if (link.ShouldSerializeAdjustOrg())
                tw.WriteElementString("AdjustOrg", link.AdjustOrg ? "True" : "False");            
            if (link.ShouldSerializeAdjustDst())
                tw.WriteElementString("AdjustDst", link.AdjustDst ? "True" : "False");           
            if (link.ShouldSerializeStretchable())
                tw.WriteElementString("Stretchable", link.Stretchable ? "True" : "False");
            if (link.ShouldSerializeHighlightChildren())
                tw.WriteElementString("HighlightChildren", link.HighlightChildren ? "True" : "False");
            if (link.ShouldSerializeRemoveChildren())
                tw.WriteElementString("RemoveChildren", link.RemoveChildren ? "True" : "False");


            // Styles
            if (link.ShouldSerializeShadow())
            {
                tw.WriteStartElement("Shadow");
                tw.WriteAttributeString("Style", link.Shadow.Style.ToString());
                tw.WriteAttributeString("Color", link.Shadow.Color.ToArgb().ToString());
                tw.WriteAttributeString("Width", link.Shadow.Size.Width.ToString());
                tw.WriteAttributeString("Height", link.Shadow.Size.Height.ToString());
                tw.WriteEndElement();
            }
            if  (link.ShouldSerializeLine())
            {
                tw.WriteStartElement("Line"); 
                tw.WriteAttributeString("Style", link.Line.Style.ToString());
                tw.WriteAttributeString("OrthogonalDynamic", link.Line.OrthogonalDynamic ? 
                    "True" : "False");
                tw.WriteAttributeString("RoundedCorner", link.Line.RoundedCorner ? 
                    "True" : "False");
                tw.WriteEndElement();
            }                    

            if  (link.ShouldSerializeArrowDst())
            {
                tw.WriteStartElement("ArrowDst"); 
                tw.WriteAttributeString("Head", link.ArrowDst.Style.ToString());
                tw.WriteAttributeString("Size", link.ArrowDst.Size.ToString());
                tw.WriteAttributeString("Angle", link.ArrowDst.Angle.ToString());
                tw.WriteAttributeString("Filled", link.ArrowDst.Filled ? "True" : "False");
                tw.WriteEndElement();
            }         
            if  (link.ShouldSerializeArrowOrg())
            {
                tw.WriteStartElement("ArrowOrg");
                tw.WriteAttributeString("Head", link.ArrowOrg.Style.ToString());
                tw.WriteAttributeString("Size", link.ArrowOrg.Size.ToString());
                tw.WriteAttributeString("Angle", link.ArrowOrg.Angle.ToString());
                tw.WriteAttributeString("Filled", link.ArrowOrg.Filled ? "True" : "False");
                tw.WriteEndElement();
            }           
            if  (link.ShouldSerializeArrowMid())
            {
                tw.WriteStartElement("ArrowMid");
                tw.WriteAttributeString("Head", link.ArrowMid.Style.ToString());
                tw.WriteAttributeString("Size", link.ArrowMid.Size.ToString());
                tw.WriteAttributeString("Angle", link.ArrowMid.Angle.ToString());
                tw.WriteAttributeString("Filled", link.ArrowMid.Filled ? "True" : "False");
                tw.WriteEndElement();
            }   
            if (link.ShouldSerializeStartCap())
                tw.WriteElementString("StartCap", link.StartCap.ToString());
            if (link.ShouldSerializeEndCap())
                tw.WriteElementString("EndCap", link.EndCap.ToString());
            if (link.ShouldSerializeJump())
                tw.WriteElementString("Jump", link.Jump.ToString()); 
            if (link.ShouldSerializeBackMode())
                tw.WriteElementString("BackMode", link.BackMode.ToString());  
            if (link.ShouldSerializeDashStyle())
                tw.WriteElementString("DashStyle", link.DashStyle.ToString());
            if (link.ShouldSerializeDrawWidth())
                tw.WriteElementString("DrawWidth", link.DrawWidth.ToString());
            if (link.ShouldSerializeConnectionStyleOrg())
                tw.WriteElementString("ConnectionStyleOrg", link.ConnectionStyleOrg.ToString());
            if (link.ShouldSerializeConnectionStyleDst())
                tw.WriteElementString("ConnectionStyleDst", link.ConnectionStyleDst.ToString());


            // Colors
            if (link.ShouldSerializeDrawColor())
                tw.WriteElementString("DrawColor", link.DrawColor.ToArgb().ToString());
            if (link.ShouldSerializeTextColor())
                tw.WriteElementString("TextColor", link.TextColor.ToArgb().ToString());

            // User data
            if (link.Text != null && link.Text.Length > 0)
				tw.WriteElementString("Text", link.Text);
			if (link.Tooltip != null && link.Tooltip.Length > 0)
				tw.WriteElementString("Tooltip", link.Tooltip);
			if (link.Url != null && link.Url.Length > 0)
				tw.WriteElementString("Url", link.Url);

            // Font
            if (link.ShouldSerializeFont())
                FontToXML(tw, link.Font);

            // Collection of link points
            if (link.Points.Count > 2 || link.AdjustOrg || link.AdjustDst) 
                PointsToXML(tw, link);

            tw.WriteEndElement();
        }

        // LINK POINTS
        // When a link is created and if its link style is Polyline or Spline,
        // then it is created with one segment, therefore with 2 points (*).
        // Therefore its Points collection contains 2 points.
        //
        // The position of each of these 2 points is calculated automatically:
        // the origin (destination) point is positioned at the intersection
        // between the origin (destination) node border and a line containing the
        // center of the origin node and the center of the destination node.
        //
        // Then, other points can be added (programmaticaly or interactively) between
        // the origin point and the destination point, resulting in a link composed
        // of several segments. Let us call "extra points" these added points.
        //
        // However this basic behaviour can be modified if the AdjustOrg or AdjustDst
        // properties are true.
        //
        // If AdjustOrg (AdjustDst) is true, then the origin (destination) point can
        // be modified interactively by the user or also programmaticaly.
        //
        // Here, we have choosen to store only the points that are not calculated 
        // automatically: if a link has just one segment and if its AdjustOrg
        // and AdjustDst properties are false, we store nothing since its 2 points 
        // are calculated automatically.
        //
        // Therefore, when creating the link points, we have to take account of 
        // AdjustOrg and AdjustDst properties.
        //
        // For instance, if AdjustOrg is true, then the first point is actually the 
        // first link point, otherwise it is the first "extra point".
        // If there are n points, we can distinguish the following cases:
        // * if AdjustOrg and AdjustDst are both true, then
        //       we have only n-2 points to add
        //       we have to alter the position of the first and last points
        //       the number of link points is n
        // * if AdjustOrg or (exclusive) AdjustDst is true, then
        //       we have n-1 points to add
        //       if AdjustOrg is true, we have to alter the position of the first point
        //       if AdjustDst is true, we have to alter the position of the last point
        //       In both cases, the number of link points is n+1.
        // * if AdjustOrg and AdjustDst are both false
        //       we have n points to add
        //       the number of link points is n+2
        //
        // (*) There is just an exception to this rule: if the link is reflexive, then
        // it is created with 4 points.

        static void PointsToXML(XmlTextWriter tw, Link link)
        {
            if (link.AdjustOrg)
            {
                tw.WriteStartElement("Point");                
                PointF pt = link.Points[0];
                tw.WriteAttributeString("X", XmlConvert.ToString(pt.X));
                tw.WriteAttributeString("Y", XmlConvert.ToString(pt.Y));
                tw.WriteEndElement();
            }
            for (int k = 1; k < link.Points.Count-1; k++)
            {
                tw.WriteStartElement("Point"); 
                PointF pt = link.Points[k];
                tw.WriteAttributeString("X", XmlConvert.ToString(pt.X));
                tw.WriteAttributeString("Y", XmlConvert.ToString(pt.Y));
                tw.WriteEndElement();
            }
            if (link.AdjustDst)
            {
                tw.WriteStartElement("Point"); 
                PointF pt = link.Points[link.Points.Count-1];
                tw.WriteAttributeString("X", XmlConvert.ToString(pt.X));
                tw.WriteAttributeString("Y", XmlConvert.ToString(pt.Y));
                tw.WriteEndElement();
            }
        }

        #endregion // Saving to XML
        
        #region Loading from XML

        // We create a new node class, derived from the Node class.
        // This new class has one custom field allowing to record the index of the  
        // link owner if any.
        private class MyNode : Node
        {
            internal int parentIndex;

            private MyNode() { }

            internal MyNode(Node dn)
                : base(dn)
            {
                parentIndex = -1;
            }
        }
                
        // We create a new link class, derived from the Link class.
        // This new class has 2 custom fields allowing to record the index of the origin 
        // node, the index of the destination node and the array of link points.
        private class MyLink : Link
        {
            internal int orgIndex;
            internal int dstIndex;
            internal ArrayList aptf;

            private MyLink(){}

            internal MyLink(Link dl) : base(dl)
            {
                orgIndex = 0;
                dstIndex = 0;
                aptf = new ArrayList();
            }            
        }

        private enum LoadState {Initial, AddFlow, DefNode, DefLink, Node, Link, Image};

        // For each node, we create a Node object and for each link, we create a Mylink 
        // object. The reference of each created Node and Mylink objects are placed in 
        // an array.
        static void XMLToFlow2(XmlTextReader tr, AddFlow af, bool selected, bool loadImages, 
            bool acceptDefaultValues)
        {            
            // Temporary variables used for many purposes
            bool b;
            int n;
            string s;
            Color clr;
            Size size;

            Node dn = new Node();
            Link dl = new Link();

            // This variable contains a reference to a new loaded Node object.
            // Just after its creation, it is added to an array of loaded objects.
            MyNode mynode = null;

            // This variable contains a reference to a new loaded MyLink object.
            // Just after its creation, it is added to an array of loaded objects.
            MyLink mylink = null;      

            // The array that will contain the references to the loaded 
            // Node and Mylink objects.
            ArrayList aTempItems = new ArrayList();             

            // Initial state
            LoadState state = LoadState.Initial;

            // If we decide to load the images contained in the file,
            // we delete the images already loaded in AddFlow. 
            // We consider that the images loaded from the file will replace the 
            // the Images collection of AddFlow.
            if (loadImages)
                af.Images.Clear();

            while (tr.Read())
            {
                if (tr.NodeType == XmlNodeType.Element)
                {
                    switch (tr.Name)
                    {
                        case "AddFlow":
                            state = LoadState.AddFlow;
                            // We read the number of nodes and the number of links
                            // However we will not need to use these values.
                            int nodes = XmlConvert.ToInt32(tr.GetAttribute(0));
                            int links = XmlConvert.ToInt32(tr.GetAttribute(1));
                            break;

                        case "DefaultNode":
                            state = LoadState.DefNode;
                            break;

                        case "DefaultLink":
                            state = LoadState.DefLink;
                            break;

                        case "Node":
                            state = LoadState.Node;
                            mynode = new MyNode(dn);

                            if (tr.AttributeCount == 5)
                            {
                                // The index is not used
                                int idnode = XmlConvert.ToInt32(tr.GetAttribute("Index"));
                            }
                            mynode.Rect = new RectangleF(
                                XmlConvert.ToSingle(tr.GetAttribute("Left")),
                                XmlConvert.ToSingle(tr.GetAttribute("Top")),
                                XmlConvert.ToSingle(tr.GetAttribute("Width")),
                                XmlConvert.ToSingle(tr.GetAttribute("Height")));
                            aTempItems.Add(mynode);
                            break;

                        case "Link":
                            state = LoadState.Link;
                            mylink = new MyLink(dl);
                            if (tr.AttributeCount == 3)
                            {
                                // The index is not used
                                int idlink = XmlConvert.ToInt32(tr.GetAttribute("Index"));
                            }
                            mylink.orgIndex = XmlConvert.ToInt32(tr.GetAttribute("Org"));
                            mylink.dstIndex = XmlConvert.ToInt32(tr.GetAttribute("Dst"));
                            aTempItems.Add(mylink);
                            break;

                        case "Parent":
                            mynode.parentIndex = XmlConvert.ToInt32(tr.ReadString());
                            break;

                        case "Image":
                            state = LoadState.Image;
                            if (loadImages)
                                af.Images.Add(XMLToImage(tr));
                            break;

                        case "Font":                            
                            Font font = XMLToFont(tr);
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.Font = font;
                                    break;
                                case LoadState.DefLink:
                                    dl.Font = font;
                                    break;
                                case LoadState.Node:
                                    mynode.Font = font;
                                    break;
                                case LoadState.Link:
                                    mylink.Font = font;
                                    break;
                            }
                            break;

                        case "DrawColor":
                            clr = Color.FromArgb(XmlConvert.ToInt32(tr.ReadElementString()));
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.DrawColor = clr;
                                    break;
                                case LoadState.DefLink:
                                    dl.DrawColor = clr;
                                    break;
                                case LoadState.Node:
                                    mynode.DrawColor = clr;
                                    break;
                                case LoadState.Link:
                                    mylink.DrawColor = clr;
                                    break;
                            }
                            break;

                        case "TextColor":
                            clr = Color.FromArgb(XmlConvert.ToInt32(tr.ReadElementString()));
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.TextColor = clr;
                                    break;
                                case LoadState.DefLink:
                                    dl.TextColor = clr;
                                    break;
                                case LoadState.Node:
                                    mynode.TextColor = clr;
                                    break;
                                case LoadState.Link:
                                    mylink.TextColor = clr;
                                    break;
                            }
                            break;

                        case "FillColor":
                            clr = Color.FromArgb(XmlConvert.ToInt32(tr.ReadElementString()));
                            if (state == LoadState.DefNode)
                                dn.FillColor = clr;
                            else if (state == LoadState.Node)
                                mynode.FillColor = clr;
                            break;

						case "GradientColor":
							clr = Color.FromArgb(XmlConvert.ToInt32(tr.ReadElementString()));
							if (state == LoadState.DefNode)
								dn.GradientColor = clr;
							else if (state == LoadState.Node)
								mynode.GradientColor = clr;
							break;

                        case "Point":
                            if (state == LoadState.Link)
                            {
                                float x = XmlConvert.ToSingle(tr.GetAttribute(0));
                                float y = XmlConvert.ToSingle(tr.GetAttribute(1));
                                mylink.aptf.Add(new PointF(x, y));
                            }
                            break;

                        case "DashStyle":
                            DashStyle ds = (DashStyle)Enum.Parse(typeof(DashStyle), 
                                                          tr.ReadElementString(), false);
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.DashStyle = ds;
                                    break;
                                case LoadState.DefLink:
                                    dl.DashStyle = ds;
                                    break;
                                case LoadState.Node:
                                    mynode.DashStyle = ds;
                                    break;
                                case LoadState.Link:
                                    mylink.DashStyle = ds;
                                    break;
                            }
                            break;

                        case "DrawWidth":
                            n = XmlConvert.ToInt32(tr.ReadElementString());
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.DrawWidth = n;
                                    break;
                                case LoadState.DefLink:
                                    dl.DrawWidth = n;
                                    break;
                                case LoadState.Node:
                                    mynode.DrawWidth = n;
                                    break;
                                case LoadState.Link:
                                    mylink.DrawWidth = n;
                                    break;
                            }
                            break;

                        case "TextMargin":
                            size = new Size(XmlConvert.ToInt32(tr.GetAttribute(0)),
                                                 XmlConvert.ToInt32(tr.GetAttribute(1)));
                            if (state == LoadState.DefNode)
                                dn.TextMargin = size;
                            else if (state == LoadState.Node)
                                mynode.TextMargin = size;
                            break;

                        case "BackMode":
                            BackMode bm = (BackMode)Enum.Parse(typeof(BackMode), 
                                tr.ReadElementString(), false);
                            switch(state)
                            {
                            case LoadState.DefNode:
                                dn.BackMode = bm;
                                break;
                            case LoadState.DefLink:
                                dl.BackMode = bm;
                                break;
                            case LoadState.Node:
                                mynode.BackMode = bm;
                                break;
                            case LoadState.Link:
                                mylink.BackMode = bm;
                                break;
                            }
                            break;

                        case "Shape":
                            ShapeStyle st = (ShapeStyle)Enum.Parse(typeof(ShapeStyle), 
                                                           tr.GetAttribute(0), false);
                            ShapeOrientation so = 
                                    (ShapeOrientation)Enum.Parse(typeof(ShapeOrientation), 
                                                           tr.GetAttribute(1), false);
                            if (state == LoadState.DefNode)
                            {
                                dn.Shape.Style = st;
                                dn.Shape.Orientation = so;
                            }
                            else if (state == LoadState.Node)
                            {
                                mynode.Shape.Style = st;
                                mynode.Shape.Orientation = so;
                            }
                            break;

                        case "Shadow":
                            ShadowStyle ss = (ShadowStyle)Enum.Parse(typeof(ShadowStyle), 
                                                                tr.GetAttribute(0), false);  
                            clr = Color.FromArgb(XmlConvert.ToInt32(tr.GetAttribute(1)));
                            size = new Size(XmlConvert.ToInt32(tr.GetAttribute(2)),
                                            XmlConvert.ToInt32(tr.GetAttribute(3)));
                            switch (state)
                            {
                                case LoadState.DefNode:
                                    dn.Shadow.Style = ss;
                                    dn.Shadow.Color = clr;
                                    dn.Shadow.Size = size;
                                    break;
                                case LoadState.DefLink:
                                    dl.Shadow.Style = ss;
                                    dl.Shadow.Color = clr;
                                    dl.Shadow.Size = size;
                                    break;
                                case LoadState.Node:
                                    mynode.Shadow.Style = ss;
                                    mynode.Shadow.Color = clr;
                                    mynode.Shadow.Size = size;
                                    break;
                                case LoadState.Link:
                                    mylink.Shadow.Style = ss;
                                    mylink.Shadow.Color = clr;
                                    mylink.Shadow.Size = size;
                                    break;
                            }
                            break;

                        case "AttachmentStyle":
                            AttachmentStyle at = (AttachmentStyle)Enum.Parse(typeof(AttachmentStyle),
                                                       tr.ReadElementString(), false);
                            if (state == LoadState.DefNode)
                                dn.AttachmentStyle = at;
                            else if (state == LoadState.Node)
                                mynode.AttachmentStyle = at;
                            break;

                        case "Dock":
                            DockStyle dock = (DockStyle)Enum.Parse(typeof(DockStyle),
                                                            tr.ReadElementString(), false);
                            if (state == LoadState.DefNode)
                                dn.Dock = dock;
                            else if (state == LoadState.Node)
                                mynode.Dock = dock;
                            break;

                        case "Alignment":
                            Alignment al = (Alignment)Enum.Parse(typeof(Alignment), 
                                                     tr.ReadElementString(), false);
                            if (state == LoadState.DefNode)
                                dn.Alignment = al;
                            else if (state == LoadState.Node)
                                mynode.Alignment = al;
                            break;

                        case "AutoSize":
                            AutoSize au = (AutoSize)Enum.Parse(typeof(AutoSize), 
                                                          tr.ReadElementString(), false);                            
                            if (state == LoadState.DefNode)
                                dn.AutoSize = au;
                            else if (state == LoadState.Node)
                                mynode.AutoSize = au;
                            break;

                        case "ImagePosition":
                            ImagePosition ip = 
                                    (ImagePosition)Enum.Parse(typeof(ImagePosition), 
                                                              tr.ReadElementString(), false);                           
                            if (state == LoadState.DefNode)
                                dn.ImagePosition = ip;
                            else if (state == LoadState.Node)
                                mynode.ImagePosition = ip;
                            break;

                        case "Trimming":
                            StringTrimming tri = 
                                   (StringTrimming)Enum.Parse(typeof(StringTrimming), 
                                                              tr.ReadElementString(), false);                          
                            if (state == LoadState.DefNode)
                                 dn.Trimming = tri;
                            else if (state == LoadState.Node)
                                 mynode.Trimming = tri;
                            break;

						case "GradientMode":
							LinearGradientMode lgm = 
								(LinearGradientMode)Enum.Parse(typeof(LinearGradientMode), 
								tr.ReadElementString(), false);                          
							if (state == LoadState.DefNode)
								dn.GradientMode = lgm;
							else if (state == LoadState.Node)
								mynode.GradientMode = lgm;
							break;

                        case "StartCap":
                            LineCap startcap = (LineCap)Enum.Parse(typeof(LineCap), 
                                                             tr.ReadElementString(), false);
                            if (state == LoadState.DefLink)
                                dl.StartCap = startcap;
                            else if (state == LoadState.Link)
                                mylink.StartCap = startcap;
                            break;

                        case "EndCap":
                            LineCap endcap = (LineCap)Enum.Parse(typeof(LineCap), 
                                                             tr.ReadElementString(), false);
                            if (state == LoadState.DefLink)
                                dl.EndCap = endcap;
                            else if (state == LoadState.Link)
                                mylink.EndCap = endcap;
                            break;

                        case "ArrowDst":
                            ArrowStyle ahd = (ArrowStyle)Enum.Parse(typeof(ArrowStyle), 
                                                              tr.GetAttribute(0), false);
                            ArrowSize azd = (ArrowSize)Enum.Parse(typeof(ArrowSize), 
                                                              tr.GetAttribute(1), false);
                            ArrowAngle aad = (ArrowAngle)Enum.Parse(typeof(ArrowAngle), 
                                                              tr.GetAttribute(2), false);
                            bool fd = (tr.GetAttribute(3) == "True");
                            if (state == LoadState.DefLink)
                            {
                                dl.ArrowDst.Style = ahd;
                                dl.ArrowDst.Size = azd;
                                dl.ArrowDst.Angle = aad;
                                dl.ArrowDst.Filled = fd;
                            }
                            else if (state == LoadState.Link)
                            {
                                mylink.ArrowDst.Style = ahd;
                                mylink.ArrowDst.Size = azd;
                                mylink.ArrowDst.Angle = aad;
                                mylink.ArrowDst.Filled = fd;
                            }
                            break;

                        case "ArrowOrg":
                            ArrowStyle aho = (ArrowStyle)Enum.Parse(typeof(ArrowStyle), 
                                                              tr.GetAttribute(0), false);
                            ArrowSize azo = (ArrowSize)Enum.Parse(typeof(ArrowSize), 
                                                              tr.GetAttribute(1), false);
                            ArrowAngle aao = (ArrowAngle)Enum.Parse(typeof(ArrowAngle), 
                                                              tr.GetAttribute(2), false);
                            bool fo = (tr.GetAttribute(3) == "True");
                            if (state == LoadState.DefLink)
                            {
                                dl.ArrowOrg.Style = aho;
                                dl.ArrowOrg.Size = azo;
                                dl.ArrowOrg.Angle = aao;
                                dl.ArrowOrg.Filled = fo;
                            }
                            else if (state == LoadState.Link)
                            {
                                mylink.ArrowOrg.Style = aho;
                                mylink.ArrowOrg.Size = azo;
                                mylink.ArrowOrg.Angle = aao;
                                mylink.ArrowOrg.Filled = fo;
                            }
                            break;
                            
                        case "ArrowMid":
                            ArrowStyle ahm = (ArrowStyle)Enum.Parse(typeof(ArrowStyle), 
                                                              tr.GetAttribute(0), false);
                            ArrowSize azm = (ArrowSize)Enum.Parse(typeof(ArrowSize), 
                                                              tr.GetAttribute(1), false);
                            ArrowAngle aam = (ArrowAngle)Enum.Parse(typeof(ArrowAngle), 
                                                              tr.GetAttribute(2), false);
                            bool fm = (tr.GetAttribute(3) == "True");
                            if (state == LoadState.DefLink)
                            {
                                dl.ArrowMid.Style = ahm;
                                dl.ArrowMid.Size = azm;
                                dl.ArrowMid.Angle = aam;
                                dl.ArrowMid.Filled = fm;
                            }
                            else if (state == LoadState.Link)
                            {
                                mylink.ArrowMid.Style = ahm;
                                mylink.ArrowMid.Size = azm;
                                mylink.ArrowMid.Angle = aam;
                                mylink.ArrowMid.Filled = fm;
                            }
                            break;

                        case "Line":
                            LineStyle ls = (LineStyle)Enum.Parse(typeof(LineStyle), 
                                tr.GetAttribute(0), false);
                            bool orthogonalDynamic = (tr.GetAttribute(1) == "True");
                            bool roundedCorner = (tr.GetAttribute(2) == "True");
                            if (state == LoadState.DefLink)
                            {
                                dl.Line.Style = ls;
                                dl.Line.OrthogonalDynamic = orthogonalDynamic;
                                dl.Line.RoundedCorner = roundedCorner;
                            }
                            else if (state == LoadState.Link)
                            {
                                mylink.Line.Style = ls;
                                mylink.Line.OrthogonalDynamic = orthogonalDynamic;
                                mylink.Line.RoundedCorner = roundedCorner;
                            }
                            break;

                        case "Jump":
                            Jump ju = (Jump)Enum.Parse(typeof(Jump), 
                                                              tr.ReadElementString(), false);
                            if (state == LoadState.Link)
                                mylink.Jump = ju;
                            else if (state == LoadState.DefLink)
                                dl.Jump = ju;
                            break;

                        case "ConnectionStyleDst":
                            ConnectionStyle csd = (ConnectionStyle)Enum.Parse(typeof(ConnectionStyle),
                                                              tr.ReadElementString(), false);
                            if (state == LoadState.Link)
                                mylink.ConnectionStyleDst = csd;
                            else if (state == LoadState.DefLink)
                                dl.ConnectionStyleDst = csd;
                            break;

                        case "ConnectionStyleOrg":
                            ConnectionStyle cso = (ConnectionStyle)Enum.Parse(typeof(ConnectionStyle),
                                                              tr.ReadElementString(), false);
                            if (state == LoadState.Link)
                                mylink.ConnectionStyleOrg = cso;
                            else if (state == LoadState.DefLink)
                                dl.ConnectionStyleOrg = cso;
                            break;

                        case "LabelEdit":
                            b = (tr.ReadElementString() == "True");          
                            if (state == LoadState.DefNode)
                                dn.LabelEdit = b;
                            else if (state == LoadState.Node)
                                mynode.LabelEdit = b;
                            break;

                        case "Hidden":
                            b = (tr.ReadElementString() == "True");
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.Hidden = b;
                                    break;
                                case LoadState.DefLink:
                                    dl.Hidden = b;
                                    break;
                                case LoadState.Node:
                                    mynode.Hidden = b;
                                    break;
                                case LoadState.Link:
                                    mylink.Hidden = b;
                                    break;
                            }
                            break;

                        case "Selectable":
                            b = (tr.ReadElementString() == "True");
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.Selectable = b;
                                    break;
                                case LoadState.DefLink:
                                    dl.Selectable = b;
                                    break;
                                case LoadState.Node:
                                    mynode.Selectable = b;
                                    break;
                                case LoadState.Link:
                                    mylink.Selectable = b;
                                    break;
                            }
                            break;

                        case "Logical":
                            b = (tr.ReadElementString() == "True");
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.Logical = b;
                                    break;
                                case LoadState.DefLink:
                                    dl.Logical = b;
                                    break;
                                case LoadState.Node:
                                    mynode.Logical = b;
                                    break;
                                case LoadState.Link:
                                    mylink.Logical = b;
                                    break;
                            }
                            break;

                        case "OwnerDraw":
                            b = (tr.ReadElementString() == "True");
                            switch(state)
                            {
                                case LoadState.DefNode:
                                    dn.OwnerDraw = b;
                                    break;
                                case LoadState.DefLink:
                                    dl.OwnerDraw = b;
                                    break;
                                case LoadState.Node:
                                    mynode.OwnerDraw = b;
                                    break;
                                case LoadState.Link:
                                    mylink.OwnerDraw = b;
                                    break;
                            }
                            break;

                        case "Transparent":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Node)
                                mynode.Transparent = b;
                            else if (state == LoadState.DefNode)
                                dn.Transparent = b;
                            break;

						case "Gradient":
							b = (tr.ReadElementString() == "True");
							if (state == LoadState.Node)
								mynode.Gradient = b;
							else if (state == LoadState.DefNode)
								dn.Gradient = b;
							break;

                        case "XMoveable":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Node)
                                mynode.XMoveable = b;
                            else if (state == LoadState.DefNode)
                                dn.XMoveable = b;
                            break;

                        case "YMoveable":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Node)
                                mynode.YMoveable = b;
                            else if (state == LoadState.DefNode)
                                dn.YMoveable = b;
                            break;

                        case "XSizeable":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Node)
                                mynode.XSizeable = b;
                            else if (state == LoadState.DefNode)
                                dn.XSizeable = b;
                            break;

                        case "YSizeable":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Node)
                                mynode.YSizeable = b;
                            else if (state == LoadState.DefNode)
                                dn.YSizeable = b;
                            break;

						case "InLinkable":
							b = (tr.ReadElementString() == "True");
							if (state == LoadState.Node)
								mynode.InLinkable = b;
							else if (state == LoadState.DefNode)
								dn.InLinkable = b;
							break;

						case "OutLinkable":
							b = (tr.ReadElementString() == "True");
							if (state == LoadState.Node)
								mynode.OutLinkable = b;
							else if (state == LoadState.DefNode)
								dn.OutLinkable = b;
							break;
                        case "AdjustDst":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Link)
                                mylink.AdjustDst = b;
                            else if (state == LoadState.DefLink)
                                dl.AdjustDst = b;
                            break;

                        case "AdjustOrg":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Link)
                                mylink.AdjustOrg= b;
                            else if (state == LoadState.DefLink)
                                dl.AdjustOrg= b;
                            break;

                        case "OrientedText":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Link)
                                mylink.OrientedText = b;
                            else if (state == LoadState.DefLink)
                                dl.OrientedText = b;
                            break;

                        case "Rigid":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Link)
                                mylink.Rigid = b;
                            else if (state == LoadState.DefLink)
                                dl.Rigid = b;
                            break;

                        case "Stretchable":
                            b = (tr.ReadElementString() == "True");
                            if (state == LoadState.Link)
                                mylink.Stretchable = b;
                            else if (state == LoadState.DefLink)
                                dl.Stretchable = b;
                            break;

                        case "HighlightChildren":
                            b = (tr.ReadElementString() == "True");
                            switch (state)
                            {
                                case LoadState.DefNode:
                                    dn.HighlightChildren = b;
                                    break;
                                case LoadState.DefLink:
                                    dl.HighlightChildren = b;
                                    break;
                                case LoadState.Node:
                                    mynode.HighlightChildren = b;
                                    break;
                                case LoadState.Link:
                                    mylink.HighlightChildren = b;
                                    break;
                            }
                            break;

                        case "RemoveChildren":
                            b = (tr.ReadElementString() == "True");
                            switch (state)
                            {
                                case LoadState.DefNode:
                                    dn.RemoveChildren = b;
                                    break;
                                case LoadState.DefLink:
                                    dl.RemoveChildren = b;
                                    break;
                                case LoadState.Node:
                                    mynode.RemoveChildren = b;
                                    break;
                                case LoadState.Link:
                                    mylink.RemoveChildren = b;
                                    break;
                            }
                            break;

                        case "ImageIndex":
                            if (state == LoadState.Node)
								mynode.ImageIndex = XmlConvert.ToInt32(tr.ReadElementString());
							else if (state == LoadState.DefNode)
								dn.ImageIndex = XmlConvert.ToInt32(tr.ReadElementString());
                            break;
							
						case "ImageLocation":
							PointF pt = new PointF(XmlConvert.ToInt32(tr.GetAttribute(0)),
								            XmlConvert.ToInt32(tr.GetAttribute(1)));
							if (state == LoadState.DefNode)
								dn.ImageLocation = pt;
							else if (state == LoadState.Node)
								mynode.ImageLocation = pt;
							break;

                        case "Text":
							s = tr.ReadElementString();
							switch(state)
							{
								case LoadState.DefNode:
									dn.Text = s;
									break;
								case LoadState.DefLink:
									dl.Text = s;
									break;
								case LoadState.Node:
									mynode.Text = s;
									break;
								case LoadState.Link:
									mylink.Text = s;
									break;
							}
                            break;
							
						case "Tooltip":
							s = tr.ReadElementString();
							switch(state)
							{
								case LoadState.DefNode:
									dn.Tooltip = s;
									break;
								case LoadState.DefLink:
									dl.Tooltip = s;
									break;
								case LoadState.Node:
									mynode.Tooltip = s;
									break;
								case LoadState.Link:
									mylink.Tooltip = s;
									break;
							}
							break;
							
						case "Url":
							s = tr.ReadElementString();
							switch(state)
							{
								case LoadState.DefNode:
									dn.Url = s;
									break;
								case LoadState.DefLink:
									dl.Url = s;
									break;
								case LoadState.Node:
									mynode.Url = s;
									break;
								case LoadState.Link:
									mylink.Url = s;
									break;
							}
							break;
                    }
                }
            }

            // Needed to accelerate the load
            af.BeginUpdate();

            // Create nodes
            foreach (object o in aTempItems)
            {
                if (o is MyNode)
                {
                    Node node = (Node)o;
                    af.Nodes.Add(node);
                }
            }

            // Create links
            foreach (object o in aTempItems)
            {
                if (o is MyLink)
                {
                    mylink = (MyLink)o;
                    Node org = ((Node)aTempItems[mylink.orgIndex]);
                    Node dst = ((Node)aTempItems[mylink.dstIndex]);
                    org.OutLinks.Add((Link)mylink, dst);
                                        
                    // Link points
                    if (mylink.aptf.Count > 0)
                    {
                        int add = mylink.aptf.Count;
                        if (mylink.AdjustOrg)
                            add--;
                        if (mylink.AdjustDst)
                            add--;
                        if (mylink.Line.Style == LineStyle.Polyline || 
                            mylink.Line.Style == LineStyle.Spline)
                        {
                            // If the link is reflexive, then it has been created with 4 points 
                            // instead 2 therefore there are 2 points we don't need to create.
                            if (mylink.Org.Index == mylink.Dst.Index)
                                add -= 2;
                            for (int k = 0; k < add; k++)
                                mylink.Points.Add(new PointF(0, 0));
                        }
            
                        // Update points positions
                        int start = mylink.AdjustOrg? 0 : 1;
                        int end = mylink.AdjustDst ? mylink.Points.Count : mylink.Points.Count-1;
                        int i = 0;
                        for (int k = start; k < end; k++)
                            mylink.Points[k] = (PointF)mylink.aptf[i++];                        
                    }
                }
            }

            // Owner link for label nodes
            foreach (object o in aTempItems)
            {
                if (o is MyNode)
                {
                    mynode = (MyNode)o;
                    if (mynode.parentIndex >= 0)
                    {
                        Node node = (Node)o;
                        node.Parent = ((Item)aTempItems[mynode.parentIndex]);
                    }
                }
            }

            // Take account of the ZOrder of items
            for (int i = 0; i < aTempItems.Count; i++)
            {
                Object o = aTempItems[i];
                if (o is Node)
                {
                    Node node = (Node)o;
                    node.ZOrder = i;
                }
                else if (o is MyLink)
                {
                    Link link = (Link)o;
                    link.ZOrder = i;
                }
            }

            if (selected)
            {
                for (int i = 0; i < aTempItems.Count; i++)
                {
                    Item item = (Item)aTempItems[i];
                    item.Selected = true;
                }
            }

            // Display the loaded diagram
            af.EndUpdate();

            // If we decide that the default values defined in the XML file will be
            // also the default values of the AddFlow control...
            if (acceptDefaultValues)
            {
                af.DefNodeProp = dn;
                af.DefLinkProp = dl;
            }
        }

        #endregion // Loading from XML
        
        #region Font

        // Write a font object to XML
        static void FontToXML(XmlTextWriter tw, Font font)
        {
            tw.WriteStartElement("Font");
            tw.WriteAttributeString("Name", font.Name);
            tw.WriteAttributeString("Size", XmlConvert.ToString(font.Size));
            tw.WriteAttributeString("Bold", font.Bold ? "True" : "False");
            tw.WriteAttributeString("Italic", font.Italic ? "True" : "False");
            tw.WriteAttributeString("Strikeout", font.Strikeout ? "True" : "False");
            tw.WriteAttributeString("Underline", font.Underline ? "True" : "False");
            tw.WriteEndElement();
        }

        static Font XMLToFont(XmlTextReader tr)
        {         
            string familyName = tr.GetAttribute(0);
            float size = XmlConvert.ToSingle(tr.GetAttribute(1));
            FontStyle style = FontStyle.Regular;
            bool bold = (tr.GetAttribute(2) == "True");
            if (bold)
                style = style | FontStyle.Bold;
            bool italic = (tr.GetAttribute(3) == "True");
            if (italic)
                style = style | FontStyle.Italic;
            bool strikeout = (tr.GetAttribute(4) == "True");
            if (strikeout)
                style = style | FontStyle.Strikeout;
            bool underline = (tr.GetAttribute(5) == "True");
            if (underline)
                style = style | FontStyle.Underline;
            return new Font(familyName, size, style);
        }

        #endregion //Font
        
        #region Image

        // Write an image object to XML
        static void ImageToXML(XmlTextWriter tw, Image image)
        {   
            tw.WriteStartElement("Image");

            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            Image image1 = new Bitmap(image);
            image1.Save(stream, image.RawFormat); 
            stream.Position = 0;

            byte[] buffer = new byte[1000];
            int readByte = 0;
            System.IO.BinaryReader br = new System.IO.BinaryReader(stream);
            do 
            {
                readByte = br.Read(buffer, 0, 1000);
                tw.WriteBase64(buffer, 0, readByte);
            } while (1000 <= readByte);

            stream.Flush();
            stream.Close();

            tw.WriteEndElement();
        }
        
        // Load an image object from its XML description.
        static Image XMLToImage(XmlTextReader tr)
        {
            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            byte[] base64 = new byte[1000];   
            int base64len = tr.ReadBase64(base64, 0, 1000);
            while (0 != base64len)
            {   
                stream.Write(base64, 0, base64len);
                base64len = tr.ReadBase64(base64, 0, 1000); 
            } 
            return Image.FromStream(stream);
        }

        #endregion //Image
    }
}